/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/_docbdk.h-arc  $
 * 
 *    Rev 1.2   Apr 19 2005 02:44:42   oris
 * - Replaced BDK_PAGE_SIZE with BDK_SECTORS_PER_PAGE,
 *   and therefore moved it from docbdk.h
 * - Added hidden customization of the buffer size used for recovery
 *   from flash write failures (BDK_SECTORS_PER_WRITE).
 * 
 *    Rev 1.1   Apr 04 2005 20:14:40   oris
 * - Added noOfUsableBytes field.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.2   Mar 27 2005 09:15:44   oris
 * - Removed unnecessary declarations
 * - Added more ram buffers for dynamic bad block support
 * 
 *    Rev 1.1   Mar 09 2005 02:12:42   oris
 * Changed comments and spelling
 * 
 *    Rev 1.0   Dec 09 2004 18:24:34   oris
 * Initial revision.
 * 
 *    Rev 1.8   Oct 17 2004 14:55:42   OriS
 * - Removed duplicated definition of FL_MAX_NO_OF_FLOORS
 * 
 *    Rev 1.7   Jan 27 2004 08:14:04   oris
 * Changed comments
 * 
 *    Rev 1.6   Jan 14 2004 22:29:34   oris
 * Replaced old (5.x) TrueFFS variable types
 * 
 *    Rev 1.5   Sep 30 2003 17:55:22   oris
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.4   Sep 01 2003 19:03:12   oris
 * - removed BLOCK,MBYTE,KBYTE definition.
 * 
 *    Rev 1.3   May 11 2003 08:27:06   OriS
 * Changed REMOVE_ERASE_BDK_IMAGE to BDK_REMOVE_ERASE_BDK_IMAGE
 * Changed REMOVE_CREATE_BDK_IMAGE to BDK_CREATE_ERASE_BDK_IMAGE
 * 
 *    Rev 1.2   May 06 2003 11:52:20   OriS
 * Changed NO_BDK_PARTIAL_PAGE_ACCESS to FL_NO_BDK_PARTIAL_PAGE_ACCESS
 * 
 *    Rev 1.1   Apr 27 2003 09:00:52   Oris
 * Added unused fields to the media header.
 * 
 *    Rev 1.0   Apr 09 2003 12:16:00   OriS
 * Initial revision.
 * 
 */

/************************************************************************/
/* Caution: The BDK_ACCESS compilation flag is for M-SYSTEMS internal   */
/*          use ONLY. This flag is used by M-SYSTEMS drivers and        */
/*          therfore it is SHOULD NOT be used by this package           */
/************************************************************************/

#ifndef _DOCBDK_H
#define _DOCBDK_H

/*---------------------------------------------------------------------
 *
 *       Binary Development Kit Stand Alone Customization Area
 *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
/* Boundries of the memory location to look for the DiskOnChip         */
/*---------------------------------------------------------------------*/

#define DOC_LOW_ADDRESS                 0xC8000L
#define DOC_HIGH_ADDRESS                0xE0000L


/*---------------------------------------------------------------------
 *
 *     End of Binary Development Kit Stand Alone Customization Area
 *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
/* general constant                                                    */
/*---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
/* The maximum number of binary partitions                             */
/*---------------------------------------------------------------------*/

#ifndef FL_BINARY_PARTITIONS
#define FL_BINARY_PARTITIONS  (3 * FL_SOCKETS)	/* for backwards compatibility */
#endif

/*-------------------------- BDK Global Status Values --------------------*/
#define BDK_S_INIT          0    /* uninitialized binary partition record */
#define BDK_S_DOC_FOUND     0x01 /* DiskOnChip device was found           */
#define BDK_S_HEADER_FOUND  0X04 /* Partition information was found       */
#define BDK_S_INFO_FOUND    0x08 /* Sub partition information was found   */
/*------------------------------------------------------------------------*/

/*-------------------------- BDK General Constants --------------------*/
#define BDK_PAGE_SIZE                (BDK_SECTORS_PER_PAGE<<FL_SECTOR_SIZE_BITS)
#define BDK_PAGE_MASK                (BDK_PAGE_SIZE-1)
#define BDK_NO_OF_SECTORS_PER_PAGE   (BDK_PAGE_SIZE>>FL_SECTOR_SIZE_BITS)

#define SIGNATURE_LEN                   8
#define SIGNATURE_NUM                   3
#define BDK_INVALID_VOLUME_HANDLE       0xff
#define BDK_HEADERS_SPACING             4
#define BDK_NO_OF_MEDIA_HEADERS         2
#define BDK_MAX_SECTORS_PER_WRITE       128 /* TBD */
/*------------------------------------------------------------------------*/

/*-------------------------- SAFTL Volume Flags Constants ----------------*/
#define BDK_SAFTL_BINARY                0x02000000L
#define BDK_SAFTL_ADVANCED_MATCHING     0x00100000L
#define BDK_SAFTL_REL_PARTITION         0x00200000L
#define BDK_SAFTL_FAST_PARTITION        0x00400000L
#define BDK_SAFTL_NO_MATCHING           0x00800000L
/*------------------------------------------------------------------------*/

#if defined(BDK_ACCESS) || defined(MTD_STANDALONE)

/*------------------------------------------------------------------------*/
/* Global binary partition data structures                                */
/*------------------------------------------------------------------------*/

typedef struct { 
#ifndef FL_NO_BDK_PARTIAL_PAGE_ACCESS
   FLByte  readBuffer[FL_SECTOR_SIZE];
#endif /* FL_NO_BDK_PARTIAL_PAGE_ACCESS */
   FLByte  signBuffer[(BDK_SECTORS_PER_WRITE+1)*SIGNATURE_LEN];  /* signature of binary sub partition  */
#ifndef FL_READ_ONLY
   FLByte bSignatures[BDK_MAX_SECTORS_PER_WRITE * SIGNATURE_LEN];
#ifndef FL_NO_BDK_PARTIAL_PAGE_ACCESS
   FLByte  userCacheData[BDK_PAGE_SIZE];
   FLWord  userCacheLength;   
#endif /* FL_NO_BDK_PARTIAL_PAGE_ACCESS */   
   FLByte  dbbCopyBuffer[BDK_PAGE_SIZE];
   CardAddress curUpdateImageAddress; /* current address to write to         */
   FLDword curUpdateImageSector;  /* current sector to write to              */
   FLDword actualUpdateLen;       /* length needed to be write               */
   FLDword  noOfUsableBytes;       /* Good blocks of all the sub-partitions   */
   FLWord  curUpdateImageBlock;   /* current block number to write to        */
   FLByte  updateImageFlag;/* BDK_COMPLETE_IMAGE_UPDATE \ ERASE_BEFORE_WRITE */   
#endif /* FL_READ_ONLY */
#ifdef HW_PROTECTION
   FLByte  protectionArea;        /* protection area no protecting the volume */
   FLWord  protectionType;        /* PROTECTABLE , CHANGEABLE_PROTECTION      */
#endif /* HW_PROTECTION */
   FLByte  bdkGlobalStatus;       /* BDK global status variable         */
   FLWord  startPartitionBlock, endPartitionBlock; /* partition boundries     */
   FLWord  startImageBlock, endImageBlock;         /* sub partition boundries */
   FLWord  curReadImageBlock;   /* current block number to read from        */
   FLWord  lastDBBSpare;        /* last block number of spare block         */
   FLDword  curReadImageSector; /* current block number to read from        */
   FLWord  blockPerFloor;       /* Blocks per floor                         */
   FLWord  bdkSavedStartUnit;   /* Last Start Unit used for get info        */
   FLDword bootImageSize;       /* available sub binary partition size      */
   FLDword realBootImageSize;   /* size used by an image on the sub partition */
   FLDword actualReadLen;       /* length needed to be read                 */
   FLDword bdkDocWindow;        /* DiskOnChip window start address          */
   CardAddress curReadOffset;   /* current address to read from             */
   FLByte  sectorsInUnitBits;   /* number of sectors in unit shift          */
   FLDword sectorsInUnitMask;   /* Mask for the number of sector in a unit  */
   FLDword erasableBlockBits;   /* number of bits used to represent a block */
   FLDword MTDFlags;            /* Partition MTD flags                      */
   FLFlash * flash;             /* flash record representing the media      */
   FLWord firstUsableBlock[FL_MAX_NO_OF_FLOORS];
} BDKVol;

/*------------------------------------------------------------------------*/
/* Extern variables for low level operations.                             */
/*------------------------------------------------------------------------*/

extern BDKVol*  flBdkVol;


typedef struct {
  LEulong    bootRecordId[2];      /* "CNAND" */
  LEulong    noOfBootImageBlocks;  /* number of good blocks in the boot image area */
  LEulong    noOfBinaryPartitions; /* number of binary partitions */
  LEulong    noOfBDTLPartitions;   /* number of BDTL partitions   */
  LEulong    blockMultiplierBits;  /* number of bits used to represent the
                                      times a flash erasable block fits inside
                                      an INFTL erasable unit. */
  LEulong    formatFlags;          /* SAFTL_QUICK_MOUNT                        */
  LEulong    osakVersion;          /* version of osak used to format the media */
  LEulong    percentUsed;
  LEulong    headerUnits;          /* Number of units reserved in the begining */
                                   /* of each floor for the media header       */
  LEulong    floorNo;              /* Number of current floor                  */
  LEulong    unused1;
  LEulong    unused2;
  LEulong    unused3;
  LEulong    unused4;
} BootRecord;


/*------------------------------------------------------------------------*/
/* Different records used by the media header                             */
/*------------------------------------------------------------------------*/

typedef struct {
  LEulong  totalGoodUnits;  /* Total good unit (sum of the next 4 fields)     */
  LEulong  virtualUnits;    /* Virtual unit size exported by the TL           */
  LEulong  transferUnits;   /* Number of good units found in BB percentage    */
  LEulong  spareUnits;      /* Number of spare units requested by the user    */
  LEulong  quickMountUnits; /* Number of good units used by the quick mount   */  
  LEulong  firstUnit;       /* First unit of the partition                    */
  LEulong  lastUnit;        /* Last unit of the partition                     */
  LEulong  firstQuickMountUnit; /* First unit used for the quick mount data   */
  LEulong  flags;           /* See list above "Partition flags"               */
  LEulong  protectionArea;      /* Number of the partitions protection area */
  LEulong  unused1;
  LEulong  unused2;
  LEulong  unused3;
  LEulong  unused4;
} VolumeRecord;


/************************ Function Prototype Begin ************************/

#ifdef MTD_STANDALONE

/*************************/
/* BDK specific routines */
/*************************/

#ifdef FL_EXIT 
void     bdkExit                 (void);
#endif /* FL_EXIT */

FLStatus bdkSetBootPartitionNo   (FLByte partitionNo);
FLStatus bdkFindDiskOnChip       (CardAddress FAR2 *docAddress,
                 FLDword FAR2 *docSize );
FLStatus bdkCheckSignOffset      (FLByte FAR2 *signPtr );
FLStatus bdkPowerDownMode        (FLWord wState);
FLStatus bdkCopyBootArea         (FLByte FAR1 *startAddressPtr,
                 FLWord wStartUnit,
                 FLDword dwAreaLen,
                 FLByte FAR2 *bCheckSumPtr,
                 FLByte FAR2 *signPtr);

/**************************************************/
/* common functions which are exported by the BDK */
/**************************************************/

FLStatus bdkGetBootPartitionInfo (FLWord startUnit,
                 FLDword FAR2 *partitionSize,
                 FLDword FAR2 *realPartitionSize,
                 FLDword FAR2 *unitSize,
                 FLByte FAR2 *signature);

FLStatus bdkCopyBootAreaInit     (FLWord startUnit,
                 FLDword areaLen,
                 FLByte FAR2 *signature);
FLStatus bdkCopyBootAreaBlock    (FLByte FAR1 *buf ,
                 FLDword bufferLen,
                 FLByte FAR2 *checkSum);

#ifdef BDK_IMAGE_TO_FILE

FLStatus bdkCopyBootAreaFile     ( FLSByte FAR2 *fname,
                   FLWord startUnit,
                   FLDword areaLen,
                   FLByte FAR2 *checkSum,
                   FLByte FAR2 *signature );
#endif /* BDK_IMAGE_TO_FILE */

#ifndef FL_READ_ONLY

FLStatus bdkUpdateBootAreaInit   (FLWord  startUnit,
                 FLDword  areaLen,
                 FLByte updateFlag,
                 FLByte FAR2 *signature );
FLStatus bdkUpdateBootAreaBlock  (FLByte FAR1 *buf ,
                 FLDword bufferLen );

#ifndef BDK_REMOVE_ERASE_BDK_IMAGE
FLStatus bdkEraseBootArea        (FLWord startUnit,
                 FLWord noOfBlocks,
                 FLByte FAR2 * signature);
#endif /* BDK_REMOVE_ERASE_BDK_IMAGE */
#ifndef BDK_REMOVE_CREATE_BDK_IMAGE
FLStatus bdkCreateBootArea       (FLWord noOfBlocks,
                 FLByte FAR2 * oldSign,
                 FLByte FAR2 * newSign);
#endif /* BDK_REMOVE_CREATE_BDK_IMAGE */

#ifdef BDK_IMAGE_TO_FILE

FLStatus bdkUpdateBootAreaFile(FLSByte FAR2 *fname, FLWord startUnit,
                   FLDword areaLen, FLByte FAR2 *signature);
#endif /* BDK_IMAGE_TO_FILE */

#endif /* FL_READ_ONLY */

#ifdef HW_OTP
FLStatus bdkGetUniqueID(FLByte FAR1* buf);
FLStatus bdkReadOtp(FLWord offset,FLByte FAR1 * buffer,FLWord length);
FLStatus bdkGetOtpSize(FLDword FAR2* sectionSize, FLDword FAR2* usedSize,
               FLWord FAR2* locked);
#ifndef FL_READ_ONLY
FLStatus bdkWriteAndLockOtp(const FLByte FAR1 * buffer,FLWord length);
#endif /* FL_READ_ONLY */
#endif /* HW_OTP */

#ifdef HW_PROTECTION

FLStatus bdkGetProtectionType    (FLWord * protectionType);

FLStatus bdkInsertKey            (FLByte FAR1* key);

FLStatus bdkRemoveKey            (void);

FLStatus bdkStickyLock           (void);

#ifndef FL_READ_ONLY

FLStatus bdkSetProtectionType    (FLWord newType);

FLStatus bdkLockEnable           (FLByte enable);

FLStatus bdkChangeKey            (FLByte FAR1* key);

#ifndef NO_IPL_CODE
FLStatus bdkWriteIPL             (FLByte FAR1 * buf , FLDword bufLen , FLWord flags);
#endif /* NO_IPL_CODE */

#endif /* FL_READ_ONLY */

#endif /* HW_PROTECTION */

#else /* MTD_STANDALONE */

extern FLStatus bdkCall(FLFunctionNo functionNo,
                        IOreq FAR2 *ioreq, FLFlash* flash);

#endif /* MTD_STANDALONE */

/********************/
/* common functions */
/********************/

void     bdkInit( void );

/************************ Function Prototype End **************************/

#endif /* BDK_ACCESS || MTD_STANDALONE */
#endif /* _DOCBDK_H */




