/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/_flflash.h-arc  $
 * 
 *    Rev 1.7   Sep 12 2005 13:04:18   roneng
 * Add pointer to MultipleCOTP routines in flflash struct 
 * 
 *    Rev 1.6   Jul 18 2005 19:15:10   oris
 * - Added new H1 DiskOnChip types.
 * 
 *    Rev 1.5   Jun 07 2005 06:11:48   OriS
 * - Added 2 new flash sub types H1_SUBTYPE_TSB_512MB and H1_SUBTYPE_SAM_256MB.
 * 
 *    Rev 1.4   Apr 25 2005 02:00:14   oris
 * - Moved special API flags to flflash.h
 * 
 *    Rev 1.3   Apr 21 2005 02:19:36   oris
 * - Added FL_ to the following definitions:
 *   DPD_DISABLED, DPD_EXIT_RISE_EDGE,   DPD_EXIT_FALL_EDGE,
 *   DPD_ENTER_1_EXIT_0, DPD_ENTER_0_EXIT_1, 
 *   DMA_HW_ENABLED, DMA_HW_DISABLED
 *   DMA_REQ_EDGE, DMA_REQ_LEVEL
 *   NEGATED_1_ASSERTED_0, NEGATED_0_ASSERTED_1
 *   DMA_IRQ_ENABLED, DMA_IRQ_DISABLED
 *   DMA_IRQ_EDGE, DMA_IRQ_LEVEL
 * 
 *    Rev 1.2   Apr 05 2005 08:35:50   oris
 * - Bug fix:insert key routine was not supported when HW_PROTECTION compilation flag was not set, while writeIPL routine requires it
 * 
 *    Rev 1.1   Mar 29 2005 16:59:54   oris
 * Added definition of FLASH_BITS_IN_PAGEINFO_NO_DUMMY  (maximum number of bits in extra area without the dummy byte).
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.4   Mar 27 2005 09:15:54   oris
 * - More definitions to prevent the use of magic numbers
 * 
 *    Rev 1.3   Mar 07 2005 22:03:12   oris
 * -	Rearrange the order of the MTD internal and external modes flags
 * -	Added a new flag for reading data called MTD_HM_NOT_EXIST. This flag tells the MTD not to use the humming code to validate the BCH corrections.
 * -	Changed the FL_DMAREQ_TYPE definition to FL_DMA_TYPE.
 * -	Changed DMA flags values.
 * -	Removed complete operation routine from FLFlash record.
 * -	Added another argument to readIPL routine that returns the current written IPL size.
 * 
 *    Rev 1.2   Feb 01 2005 09:18:36   eugen
 * - new OTP prototypes, prepareForPowerChanges
 * 
 *    Rev 1.1   Jan 05 2005 21:03:06   eugen
 * Commentaries, headers
 * 
 *    Rev 1.0   Dec 09 2004 18:24:34   oris
 * Initial revision.
 * 
 *    Rev 1.7   Oct 18 2004 10:56:30   oris
 * - Added compilation flag to separate between 6.2.2 and down 
 *   MTD interface and the new 6.3 interfaces (FL_622_MTD_CODE). 
 *   Required for IPL routines
 * 
 *    Rev 1.6   Oct 17 2004 14:59:24   OriS
 * - New interface for protectionSet routine which reports the key 
 *   being used provided key was already inserted MTD_READ_KEY 4
 * - Added new MTD routine named BBManagment for marking 
 *   (MTD_MARK_BB) and retrieving (MTD_CHECK_BB) dynamic
 *   bad block status.
 * - Added new MTD routine named special for setting runtime MTD
 *   options, like interrupt modes, turbo mode, DPD exit modes 
 *   asynchronous access mode, only 2 DPS update mode. None 
 *   of these API's are implemented in the current MTD.
 * 
 *    Rev 1.5   Sep 22 2004 13:03:18   roneng
 * Add TFFS6.3 definitions for M512-G3 MTD basic functionality
 * 
 *    Rev 1.4   Jan 27 2004 08:14:04   oris
 * Changed comments
 * 
 *    Rev 1.3   Sep 01 2003 19:09:44   oris
 * - changed MTDS definition to FL_MTDS 
 * - Added FLASH_EXTRA_BUFFER_SIZE definition
 * 
 *    Rev 1.2   Aug 10 2003 13:36:54   roneng
 * FLASH_EXTRA_AREA_SIZE = 16
 * FLASH_SECTOR_TOTAL_SIZE = 528
 * 
 *    Rev 1.1   Jun 16 2003 01:17:50   oris
 * Added TFFS_DLL_API prefix.
 * 
 *    Rev 1.0   Apr 09 2003 12:16:02   OriS
 * Initial revision.
 * 
 */


#ifndef _FLFLASH_H
#define _FLFLASH_H


#ifdef MTD_STANDALONE

/* The following definition are supposed to replace the TrueFFS socket layer */
typedef enum {PowerOff, PowerGoingOff, PowerOn} PowerState;
typedef struct tSocket FLSocket;

struct tSocket
{
    unsigned        volNo;    /* Volume no. of socket             */
    PowerState      DPDState; /* DPD mode                         */
    struct {                       /* Window state                     */
       unsigned int  baseAddress;   /* Physical base as a 4K page       */
       void FAR0 *   base;          /* Pointer to window base           */
       FLSDword      size;          /* Window size (must by power of 2) */
       unsigned      busWidth;      /* 8 or 16 bits                     */
    } window;
};

/* Replacement for various TrueFFS typedefs */

#else /* MTD_STANDALONE */

#endif /* MTD_STANDALONE */

typedef FLStatus (*MTDCallBackRoutine)(void *MyArguments);
      
typedef struct 
{
  FLDword      OpFlags ;
  FLStatus     status ;
  FLByte       MaxBitErrors ;         /* Write only								  */
  FLByte       VerifyWriteMode ;      /* Write only								  */
  FLDword      bNoOfSectorsPassed ;   /* Write only							      */
  const FLByte FAR1 *writeMainBuf ;        
} LastOperationResults ;

/*----------------------------------------------------------------------*/
/*              MTD common paramete passing packet                      */
/* This structure contain the parameters that are passed for MTD        */
/* private routines. In order to minimize parameter passed through the  */
/* the stack and in order to boost performance, All MTD private         */
/* routines receive a single parameter for FLFlash structure.           */
/*  This structure is saved in FLFlash and passes all other needed      */
/* parameters for all MTD routines.                                     */
/*----------------------------------------------------------------------*/
typedef struct{
    FLDword       opFlags;
    FLDword       noOfSectors;
    FLDword       noOfSectorsPassed;
    FLDword       startSector;
    FLDword       freeSectorFound;
    FLDword       eraseCount;
    FLDword       partitionsFlags[FL_MAX_TL_PARTITIONS+1];
    LastOperationResults*   delayedStatus;
    void*         myargs;
    MTDCallBackRoutine      myRoutine;
    FLByte FAR1*  readMainBuf;
    const FLByte FAR1* writeMainBuf;
    FLByte FAR1 * extraBuf;
    FLWord        noOfUnits;
    FLWord        noOfUnitsPassed;
    FLWord        startUnit;
    FLWord        noOfUnitsArray[FL_MAX_TL_PARTITIONS+1];
    FLWord        noOfFirstUnitsArray[FL_MAX_TL_PARTITIONS+1];
    FLWord        partitionLogicalSize[FL_MAX_TL_PARTITIONS+1];
    FLSDword      unusedBlocks;
    FLByte        maxBitError;
    FLByte        verifyWriteMode;
    FLByte        noOfSkippedUnits;
    FLByte        activePartition;
    FLByte        noOfPartitions;
    FLWord        offset ;
    FLWord        length ;
}MTDArgumentPacket;

/* The following are flags for opFlags field - not all are supported */
/********************** E X T E R N A L *************************************/
#define MTD_DATA            0x00000001 /* Apply to main area of the sector  */
#define MTD_EXTRA           0x00000002 /* Apply to extra area of the sector */
#define MTD_RAW_MODE        0x00000004 /* Use RAW flash access              */
#define MTD_REL_MODE        0x00000010 /* Use the flash Reliable mode       */
#define MTD_FAST_MODE       0x00000020 /* Use the flash FAST mode (must be  */
                                       /* used with MTD_REL_MODE            */
#define MTD_SW_EDC          0x00000100 /* Add S/W EDC only                  */
/********************** N O T   U S E D   A N Y   M O R E *******************/
#define MTD_DO_NOT_WAIT     0x00001000 /* Return without waiting for the    */
                                       /* flash to finish the operation     */
#define MTD_NO_MATCHING     0x00000200 /* Do not match physical units into  */
                                       /* logical units                     */
#define MTD_CHECK_RB        0x00000800 /* Pole the R/B of last operation    */
#define MTD_ADD_ERASE_MARK  0x00010000 /* Place the erase mark after the    */
                                       /* erase operation                   */
#define MTD_ADD_ERASE_COUNT 0x00020000 /* Place the erase count after the   */
                                       /* erase operation                   */
/********************** I N T E R N A L *************************************/
#define MTD_532_BYTES       0x00040000 /* Page is 532 bytes instead of 528  */
#define MTD_BCH             0x00000008
#define MTD_WRITE_OP        0x00000040 /* This is a  write operation        */
#define MTD_ERASE_OP        0x00000080 /* This is an erase operation        */
#define MTD_NO_EDC          0x00000400 /* disable the EDC\ECC mechanism     */
#define MTD_HM_NOT_EXIST    0x00002000 /* In case of BCH error do not check */
                                       /* Hamming to improve probability of */
                                       /* BCH fail after power fail         */   

/* The following are flags for the protection set routine */
#define DO_NOT_COMMIT_PROTECTION 0 /* The new values will take affect only after reset  */
#define COMMIT_PROTECTION        1 /* The new values will take affect immediately       */
#define MTD_ON_DPS_ONLY          2 /* The new protection will be placed only on the DPS */
#define MTD_READ_KEY             4 /* Report the key that is being used                 */
                                   /* (provided key is inserted)                        */

/* The following values are for partitionFlags*/
#define MTD_SM              0   /* Use simple matching for partition*/
#define MTD_AM              1   /* Use advanced matching for partition*/

/* The following flags are used by the dwMode field in the BBManagment routine */
#define MTD_CHECK_BB        0x1
#define MTD_MARK_BB         0x2

/* Miscelenious */
#define MAX_ERROR_BIT_PER_PAGE      4 /* Maximum allowed number of bit failures per page */
#define MAX_NUMBER_OF_PLANES        4 /* Maximum allowed number of planes                */
#define MAX_DBBT_NONE_0_BITS        4 /* 4-8 from unused, 
                                       *    1 from ANAC,
                                       *    1 from VUN,
                                       */
#define MAX_DBBT_DISTANCE_FOR_MARK  40 /* about 1% of the block's data */
#define FLASH_SECTOR_DATA_SIZE      512
#define FLASH_SECTOR_PAGEINFO_SIZE  7
#define FLASH_SECTOR_HM_SIZE        1
#define FLASH_SECTOR_BCH_SIZE       7 
#define FLASH_SECTOR_DUMMY_SIZE     1
#define FLASH_EXTRA_BUFFER_SIZE     (FLASH_SECTOR_PAGEINFO_SIZE + FLASH_SECTOR_DUMMY_SIZE)
#define FLASH_EXTRA_AREA_SIZE       (FLASH_SECTOR_DUMMY_SIZE + FLASH_SECTOR_BCH_SIZE + FLASH_SECTOR_HM_SIZE + FLASH_SECTOR_PAGEINFO_SIZE) /* 16 */
#define FLASH_SECTOR_TOTAL_SIZE     (FLASH_EXTRA_AREA_SIZE + FLASH_SECTOR_DATA_SIZE) /* 528 */
#define FLASH_BCH_PROTECTED_SIZE    (FLASH_SECTOR_DATA_SIZE+FLASH_EXTRA_BUFFER_SIZE+FLASH_SECTOR_BCH_SIZE)
#define FLASH_PROTECTED_EXTRA_SIZE  (FLASH_SECTOR_BCH_SIZE + FLASH_SECTOR_HM_SIZE + FLASH_SECTOR_PAGEINFO_SIZE)

#define FLASH_BITS_IN_PAGEINFO_NO_DUMMY  ((FLASH_EXTRA_AREA_SIZE-1)<<3)

#define FLASH_SECTOR_DATA_SIZE_BITS  9
#define FLASH_EXTRA_BUFFER_SIZE_BITS 3

#define M512_SUBTYPE_G3                 0x00
#define M512_SUBTYPE_G3_LP              0x01

#define H1_SUBTYPE_TSB_512MB            0x00
#define H1_SUBTYPE_SAM_256MB            0x01
#define H1_SUBTYPE_H_256MB              0x02
#define H1_SUBTYPE_H_512MB              0x03
 
/*----------------------------------------------------------------------*/
/*                 Flash array identification structure                 */
/*                                                                      */
/* This structure contains a description of the Flash array and         */
/* routine pointers for the map, read, write & erase functions.         */
/*                                                                      */
/* The structure is initialized by the MTD that identifies the Flash    */
/* array.                                                               */
/* On entry to an MTD, the Flash structure contains default routines    */
/* for all operations. This routines are sufficient forread-only access */
/* to NOR Flash on a memory-mapped socket. The MTD should override the  */
/* default routines with MTD specific ones when appropriate.            */
/*----------------------------------------------------------------------*/

/* Flash array identification structure */

struct tFlash {
  FLBoolean wasIdentified;
  FlashType type;                   /* Flash device type (JEDEC id)           */
  FlashType subType;
  FLByte    flashTechnology;        /* Flash technology properties            */
                                    /* FL_SUPPORT_MLC , FL_SUPPORT_RELIABLE     */
                                    /* FL_SUPPORT_FAST , FL_SUPPORT_NO_MATCHING */
  FLByte    mediaType;              /* see media types obove                  */
  FLDword busAccessType;            /* saves bus access type                  */
  FLDword maxEraseCycles;           /* erase cycles limit per erase block     */
  FLByte changeableProtectedAreas;  /* areas capable of changing protection   */
                                 /* attribute with no danger of loosing       */
                                 /* the entire chip                           */
  FLByte     totalProtectedAreas;/* total number of protection arweas         */
  FLByte      if_cfg;  
  FLByte      bBusType;          /* Bus type                                  */
  FLDword     mediaSize;         /* chip size                                 */
  FLWord      unitsInFirstFloor;
  FLByte      noOfFloors;        /* no of controllers in array                */
  FLByte      noOfChips;         /* no of chips that the media contain.       */  
  FLByte      noOfBanks;         /* no of banks that the media contain.       */
  FLSWord     interleaving;      /* chip interleaving (The interleaving is    */
                                 /* defined as the address difference between */
                                 /* two consecutive bytes on a chip)          */
  FLByte      noOfSectorsPerPage;/* Number of logical sectors in physical page*/
  FLByte      sharedSectors;     /* Minimal number of sectors that can affect */
                                 /* one another when written.                 */
  FLByte      fastAreaSharedSectors; /* Minimal number of sectors that can affect */
                                     /* one another when written in fast mode.    */  
  FLByte      noOfPlanes;        /* Number of planes in the device.           */							   /* simultaniously.						    */
  FLDword     erasableBlockSize; /* Size of logical erasable unit             */
  FLDword     erasableBlockSizeBits;      /* Number of bits used to represent a logical*/
                                 /* erasable unit size.                       */
  FLDword     dwIPLSize;         /* Number of bytes of SRAM in the device.    */
  FLByte      evenPagePPP;       /* The maximum alowed PPP for even pages.    */
  FLByte      oddPagePPP;        /* The maximum alowed PPP for odd pages.     */
  FLDword     averageRead;       /* Average time for page read in microsec.   */
  FLDword     averageWrite;      /* Average time for page write in microsec.  */
  FLDword     averageErase;      /* Average time for page erase in microsec.  */
  FLByte      maxBadPercentage;  /* The flash vendor maximum bad blocks percentage*/ 
  FLWord      flags;             /* Special capabilities & options Bits 0-7   */
                                 /* may be used by FLite. Bits 8-15 are not   */
                                 /* used bt FLite and may beused by MTD's for */
                                 /* MTD-specific purposes.                    */
  void *    mtdVars;             /* Points to MTD private area for this socket.*/
                                 /* This field, if used by the MTD, is         */
                                 /* initialized bythe MTD identification       */
                                 /* routine.                                   */
  FLSocket * socket;             /* Socket of this drive. Note that 2 diffrent */
                                 /* records are used. One for OSAK and the     */
                                 /* other forstandalone applications.          */
  NDOC2window win;               /* DiskOnChip memory windows                  */

  MTDArgumentPacket args;        /* args passing for MTD private routines */

/*----------------------------------------------------------------------*/
/*                        f l a s h . m a p                             */
/*                                                                      */
/* MTD specific map routine                                             */
/*                                                                      */
/* The default routine maps by socket mapping, and is suitable for all  */
/* NOR Flash.                                                           */
/* NAND or other type Flash should use map-through-copy emulation: Read */
/* a block of Flash to an internal buffer and return a pointer to that  */
/* buffer.                                                              */
/*                                                                      */
/* Parameters:                                                          */
/*      vol                : Pointer identifying drive                  */
/*      address            : Card address to map                        */
/*      length             : Length to map                              */
/*                                                                      */
/* Returns:                                                             */
/*        Pointer to required card address                              */
/*----------------------------------------------------------------------*/
  void FAR0 * (*map)(FLFlash *, CardAddress, int);

/*----------------------------------------------------------------------*/
/*               f l a s h . s e t P o w e r O n C a l l b a c k        */
/*                                                                      */
/* Register power on callback routine. Default: no routine is           */
/* registered.                                                          */
/*                                                                      */
/* Parameters:                                                          */
/*      vol                : Pointer identifying drive                  */
/*                                                                      */
/*----------------------------------------------------------------------*/
  void (*setPowerOnCallback)(FLFlash *);

/*----------------------------------------------------------------------*/
/*                        f l a s h R e a d                             */
/*                                                                      */
/* MTD specific Flash read routine                                      */
/*                                                                      */
/* The default routine reads by copying from a mapped window, and is    */
/* suitable for all NOR Flash.                                          */
/*                                                                      */
/* Parameters:                                                          */
/*      flash                     : Pointer identifying drive           */
/*                                                                      */
/* The routine uses the following inputs:                               */
/*    flash.args.noOfSectors : The number of sectors to read.           */
/*    flash.args.startSector : First sector to start reading from       */
/*    flash.args.readMainBuf : Pointer to user buffer                   */
/*    flash.args.extraBuf    : Pointer to  user buffer                  */
/*    flash.args.opFlags     : Available flags are:                     */
/*        MTD_DATA           : Read from sector info area to mainBuf.   */
/*        MTD_EXTRA          : Read from sector extra area to extraBuf. */
/*        MTD_FAST_MODE      : Save data using fast mode.               */
/*        MTD_REL_MODE       : Save data using reliable mode.           */
/*        MTD_SW_EDC         : Read data while applying the SW EDC.     */
/*        MTD_NO_EDC         : Disable EDC/ECC.                         */
/*                                                                      */
/* The routine sets the following outputs:                              */
/*    flash.args.maxBitsError     :                                     */
/*    flash.args.freeSectorsFound :                                     */
/*    flash.args.noOfSectorsPassed:                                     */
/* The routine return the following error codes:                        */
/*      flOK            : Success                                       */
/*      flDataError     : At least one of the sectors has an            */
/*                        unrecoverable EDC error unless only the extra */
/*                        area was requested and the sector is "free".  */
/*                        A sector is considered "free" if it has 2 or  */
/*                        less '0' bits                                 */
/*      flHWProtection  : A protection violation has occurred while     */
/*                        attempting to reading one of the sectors.     */
/*      flTimedOut      : The maximum delay time has expired, but the   */
/*                        device is not ready yet.                      */
/*                                                                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*flashRead)(FLFlash *);

/*----------------------------------------------------------------------*/
/*                       f l a s h W r i t e                            */
/*                                                                      */
/* MTD specific Flash write routine                                     */
/*                                                                      */
/* The default routine returns a write-protect error.                   */
/*                                                                      */
/* Parameters:                                                          */
/*      flash                     : Pointer identifying drive           */
/*                                                                      */
/* The routine uses the following inputs:                               */
/*    flash.args.noOfSectors : The number of sectors to read.           */
/*    flash.args.startSector : First sector to start reading from       */
/*    flash.args.writeMainBuf: Pointer to user buffer                   */
/*    flash.args.extraBuf    : Pointer to  user buffer                  */
/*    flash.args.verifyWriteMode: An indicator of the read after        */
/*                                     write mode. FL_ON will force a   */
/*                                     read after write operation while */
/*                                     FL_OFF will disable it.          */
/*    flash.args.delayedStatus  : A pointer to where the output of      */
/*                                     the operation will be returned,  */
/*                                     when adding the MTD_DO_NOT_WAIT  */
/*                                     flag.                            */
/*    flash.args.opFlags     : Available flags are:                     */
/*        MTD_DATA           : Write to sector info area to mainBuf.    */
/*        MTD_EXTRA          : Write to sector extra area to extraBuf.  */
/*        MTD_FAST_MODE      : Save data using fast mode.               */
/*        MTD_REL_MODE       : Save data using reliable mode.           */
/*        MTD_SW_EDC         : Write data while applying the SW EDC.    */
/*        MTD_DO_NOT_WAIT    : Forces the MTD to return without polling */
/*                             the last sector write operation.         */
/*                                                                      */
/* The routine sets the following outputs:                              */
/*    flash.args.maxBitsError     : Returns maximum bit error found     */
/*                                       in read after write operation  */
/*    flash.args.noOfSectorsPassed: The number of sectors that were     */
/*                                       successfully written.          */
/*                                                                      */
/* The routine return the following error codes:                        */
/* Returns:                                                             */
/*      flOK            : Success                                       */
/*      flWriteFault    : The write operation failed to one of the      */
/*                        required sectors.                             */
/*      flHWProtection  : A protection violation has occurred while     */
/*                        attempting to write to one of the sectors..   */
/*      flTimedOut      : The maximum delay time has expired, but the   */
/*                        device is not ready yet.                      */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*flashWrite)(FLFlash *);

/*----------------------------------------------------------------------*/
/*                       f l a s h E r a s e                            */
/*                                                                      */
/* Erase one or more contiguous Flash erasable blocks                   */
/*                                                                      */
/* The default routine returns a write-protect error.                   */
/*                                                                      */
/* Parameters:                                                          */
/*      flash                     : Pointer identifying drive           */
/*                                                                      */
/* The routine uses the following inputs:                               */
/*    flash.args.noOfUnits                                              */
/*    flash.args.startUnit                                              */
/*    flash.args.delayedStatus                                          */
/*    flash.args.opFlags     : Available flags are:                     */
/*        MTD_FAST_MODE      : Save data using fast mode.               */
/*        MTD_REL_MODE       : Save data using reliable mode.           */
/*        MTD_DO_NOT_WAIT    : Forces the MTD to return without polling */
/*                             the last erase unit operation.           */
/*        MTD_ADD_ERASE_MARK : The MTD should add the erase mark after  */
/*                             the completion of the erase operation    */
/*                             Disable EDC/ECC.                         */
/*                                                                      */
/* The routine sets the following outputs:                              */
/*    flash.args.noOfUnitsPassed: The number of logical units that      */
/*                                    have been successfully erased.    */
/*                                                                      */
/* The routine return the following error codes:                        */
/*      flOK            : Success                                       */
/*      flWriteFault    : The erase operation failed to one of the      */
/*                        required blocks.                              */
/*      flHWProtection  : protection violation has occurred due to the  */
/*                        erasing of one of the required blocks.        */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*flashErase)(FLFlash *);

/*----------------------------------------------------------------------*/
/*                f l a s h . d e f i n e P a r t i t i o n s           */
/*                                                                      */
/* Instruct the MTD how the TL plans to partition the media. This       */
/* routine is used both to initialize the internal MTD logical to       */
/* physical addressing algorithm and for checking  if the media has     */
/* enough good blocks for the wanted division.                          */
/*                                                                      */
/* Parameters:                                                          */
/*      flash                     : Pointer identifying drive           */
/*                                                                      */
/* The routine uses the following inputs:                               */
/*    flash.args.noOfPartitions  :Indicate no of partition in media     */
/*    flash.args.noOfSkippedUnits:Indicate the no of good logical       */
/*                                     units at the beginning of each   */
/*                                     floor that are saved for TL's    */
/*                                     media header.                    */
/*    flash.args.noOfUnitsArray  :This array indicates the number       */
/*                                     of logical good units found in   */
/*                                     each of the media partitions.    */
/*                                     Only the first noOfPartitions    */
/*                                     cells will be used.              */
/*    flash.args.partitionsFlags :This array indicates the format       */
/*                                     type of the indicated partition. */
/*                                     values are MTD_SM-simple matching*/
/*                                     MTD_AM - advanced matching       */
/*                                                                      */
/* The routine sets the following outputs:                              */
/*    flash.args.unusedBlocks    :The number of logical units that      */
/*                                     are not belong to any partition. */
/*                                     If negative number returned,     */
/*                                     there are units missing.         */
/*    flash.args.noOfFirstUnitsArray:This array indicates the first     */
/*                                     logical unit of each partition.  */
/*                                     Only the first noOfPartitions    */
/*                                     cells will be used.              */
/*                                                                      */
/* The routine return the following error codes:                        */
/*      flOK            : Success                                       */
/*      flBadBBT        : Could not read the BBT.                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*definePartitions)(FLFlash *);  
  
/*----------------------------------------------------------------------*/
/*                        f l a s h . r e a d B B T                     */
/*                                                                      */
/* MTD specific Flash routine returning the media units status          */
/* Note that a unit can contain more then 1 erase block                 */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*      flash                     : Pointer identifying drive           */
/*                                                                      */
/* The routine uses the following inputs:                               */
/*    flash.args.readMainBuf : Pointer to user buffer where  Bad        */
/*                                  blocks table should be written.     */
/*    flash.args.startUnit   : The first logical unit number to         */
/*                                  start the BBT report                */
/*    flash.args.noOfUnits   : How many units to report.                */
/*                                                                      */
/* The routine sets the following outputs:                              */
/*    For each logical unit starting from startUnit and ending at       */
/*    noOfUnits a byte specify one of the following values:             */
/*      BBT_GOOD_UNIT    - Good logical unit.                           */
/*      BBT_BAD_UNIT     - Bad logical unit.                            */
/*      BBT_UNAVAIL_UNIT - unit is used by the controller (i.e IPL, OTP)*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/* The routine return the following error codes:                        */
/*      flOK            : Success                                       */
/*      flBadBBT        : Could not read the BBT.                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*readBBT)(FLFlash *);
  
  FLStatus (*getEraseCount)(FLFlash *);
  FLStatus (*getEraseMark)(FLFlash *);

  FLStatus (*setCallBack)(FLFlash *);
  FLStatus (*dismount)(FLFlash *);

/***********************************************************************
 * Name: M512_PrepareForPowerChanges  
 * Parameters:
 *  flashPtr       - FLFlash struct
 *  bOperationType - FL_PREPARE_FOR_RESUME : Do all needed in order to return 
 *                                           DiskOnChip to the same state it 
 *                                           was before resuming.  
 *                   FL_PREPARE_FOR_SUSPEND: Do all needed in MTD in order to 
 *                                           get ready for power shutdown to DOC  
 * Returns: 
 *    flOK             - No error occurred
 *    flBadParameter   - Bad bOperationType parameter
 *    flSequenceError  - Sequence error was fixed
 *    flHWProtection   - Protection error was fixed 
 ***********************************************************************/
  FLStatus (*prepareForPowerChanges) (FLFlash *,FLByte) ;
 
  FLStatus (*clearCallBack)(FLFlash *);
  FLStatus (*flashCopy)(FLFlash *);


/*----------------------------------------------------------------------*/
/*                    f l a s h . w r i t e I P L                       */
/*                                                                      */
/* MTD specific Flash write IPL area routine                            */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/

#ifndef FL_622_MTD_CODE
  FLStatus (*writeIPL)(FLFlash *flash,const void FAR1* buffer,
                       FLDword dwLength,FLDword dwSectorOffset,FLDword flags);
#else /* FL_622_MTD_CODE */
  FLStatus (*writeIPL)(FLFlash * flash, const void FAR1* buffer, 
                       FLWord length, FLByte offset, unsigned flags);                       
#endif /* FL_622_MTD_CODE */

/*----------------------------------------------------------------------*/
/*                     f l a s h . r e a d I P L                        */
/*                                                                      */
/* MTD specific Flash read area IPL routine                             */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/

#ifndef FL_622_MTD_CODE
  FLStatus (*readIPL)(FLFlash *,void FAR1*,
                      FLDword,FLDword,FLDword *,FLDword *);
#else /* FL_622_MTD_CODE */
  FLStatus (*readIPL)(FLFlash *, void FAR1*, FLWord);                      
#endif /* FL_622_MTD_CODE */
#ifdef HW_OTP

/*----------------------------------------------------------------------*/
/*                        f l a s h . w r i t e O T P                   */
/*                                                                      */
/* MTD specific Flash write and lock OTP area routine                   */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*writeOTP)(FLFlash *, const void FAR1 * buffer,FLWord length);

/*----------------------------------------------------------------------*/
/*                        f l a s h . r e a d O T P                     */
/*                                                                      */
/* MTD specific Flash read OTP area routine                             */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*readOTP)(FLFlash *, FLWord offset, void FAR1 * buffer, FLWord length);

/*----------------------------------------------------------------------*/
/*                        f l a s h . otpSize                           */
/*                                                                      */
/* MTD specific Flash get OTP area size and state                       */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*otpSize)(FLFlash *,  FLDword FAR2* sectionSize,
             FLDword FAR2* usedSize, FLWord FAR2* locked);
#endif /* HW_OTP */

/*----------------------------------------------------------------------*/
/*                  f l a s h . g e t U n i q u e I d                   */
/*                                                                      */
/* MTD specific Flash get the chip unique ID                            */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*getUniqueId)(FLFlash *, void FAR1 * buffer);

#if ((defined(HW_PROTECTION)) || (!defined(NO_IPL_CODE)))  

/*----------------------------------------------------------------------*/
/*        f l a s h . p r o t e c t i o n K e y I n s e r t             */
/*                                                                      */
/* MTD specific Flash insert the protection key routine                 */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Note the key is inserted only to protected areas and to all floors   */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*protectionKeyInsert)(FLFlash *, FLByte areaNo, FLByte FAR1* key);

#endif /* ((defined(HW_PROTECTION)) || (!defined(NO_IPL_CODE))) */

#ifdef  HW_PROTECTION
/*----------------------------------------------------------------------*/
/*        f l a s h . p r o t e c t i o n B o u n d r i e s             */
/*                                                                      */
/* MTD specific Flash get protection boundries  routine                 */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*protectionBoundries)(FLFlash *, FLByte areaNo,
            FLWord* addressLow ,FLWord* addressHigh,FLByte Floor);

/*----------------------------------------------------------------------*/
/*        f l a s h . p r o t e c t i o n K e y R e m o v e             */
/*                                                                      */
/* MTD specific Flash remove the protection key routine                 */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Note the key is removed from all floors.                             */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*protectionKeyRemove)(FLFlash *,FLByte areaNo);

/*----------------------------------------------------------------------*/
/*        f l a s h . a c t i v a t e S t i c k y L o c k               */
/*                                                                      */
/* MTD specific Flash activate the sticky lock routine                  */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*activateStickyLock)(FLFlash *);

/*----------------------------------------------------------------------*/
/*        f l a s h . p r o t e c t i o n T y p e                       */
/*                                                                      */
/* MTD specific Flash get protection type routine                       */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Note the type is the combined attributes of all the floors.          */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*protectionType)(FLFlash *,FLByte areaNo, FLWord* areaType);

/*----------------------------------------------------------------------*/
/*              f l a s h . p r o t e c t i o n S e t                   */
/*                                                                      */
/* MTD specific Flash get protection type routine                       */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*protectionSet )( FLFlash *,FLByte areaNo, FLWord areaType ,
                              FLWord addressLow, FLWord addressHigh,
                              FLByte FAR1* key , FLByte modes,FLByte Floor );

#endif /* HW_PROTECTION */

/*----------------------------------------------------------------------*/
/*      f l a s h . e n t e r D e e p P o w e r D o w n M o d e         */
/*                                                                      */
/* MTD specific Flash enter deep power down mode routine                */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*enterDeepPowerDownMode)(FLFlash *,FLWord state);

/*----------------------------------------------------------------------*/
/*                    f l a s h . d o w n l o a d                       */
/*                                                                      */
/* MTD specific - Reset download mechanism to download IPL and          */
/*                protection attributes.                                */
/*                                                                      */
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*download)(FLFlash *);

/*----------------------------------------------------------------------*/
/*                    f l a s h . specialAPI                            */
/*                                                                      */
/* MTD specific - Change DiskOnChip Mode                                */                                                                     
/* No default routine is implemented for this routine.                  */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*specialAPI)      (FLFlash *,FLWord ,FLWord );

/***********************************************************************
 * Name : M1G_MulticustomerOTP_Read
 *   Read one of Multiple customer OTP pages.
 * 
 ***********************************************************************/
FLStatus (*MultiCOTP_Read) (FLFlash *,void *,FLWord) ;

/***********************************************************************
 * Name : M1G_MulticustomerOTP_Write 
 *   Writes and lock the Multiple customer OTP 
 * 
 ***********************************************************************/
FLStatus (*MultiCOTP_Write) (FLFlash *,const void *,FLWord) ;

/***********************************************************************
 * Name : M1G_MulticustomerOTP_Size
 *   Return MCOTP size & Used size
 * 
 ***********************************************************************/
FLStatus (*MultiCOTP_Size) (FLFlash *,FLDword *,FLDword *,FLWord *) ;

/*----------------------------------------------------------------------*/
/*                    f l a s h . BBManagment                           */
/*                                                                      */
/* MTD specific - Check and mark unmarked bad block                     */                                                                     
/* Default routine is implemented for this routine.                     */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
  FLStatus (*BBManagment)    (FLFlash * flashPtr,
                              FLWord  * dwBlockStatus,
                              FLDword   dwOpMode,                              
                              FLWord    wBlockNo);

/*----------------------------------------------------------------------*/
/* DiskOnChip memory access routines type defintions                    */
/*----------------------------------------------------------------------*/

#ifndef FL_NO_USE_FUNC
  FLMemWindowSize FAR1* memWindowSize; /* Doc memory window size          */
  FLMemRead       FAR1* memRead;       /* Doc memory read routine         */
  FLMemWrite      FAR1* memWrite;      /* Doc memory write routine        */
  FLMemSet        FAR1* memSet;        /* Doc memory set routine          */
  FLMemRead8bit   FAR1* memRead8bit;   /* Doc memory 8 bit read routine   */
  FLMemWrite8bit  FAR1* memWrite8bit;  /* Doc memory 8 bit write routine  */
  FLMemRead16bit  FAR1* memRead16bit;  /* Doc memory 16 bit read routine  */
  FLMemWrite16bit FAR1* memWrite16bit; /* Doc memory 16 bit write routine */
  FLMemSetGetMode FAR1* memSetGetMode; /* Interleave change event -       */
                                       /* call back to plant new routines */
#endif /* FL_NO_USE_FUNC */
};

typedef FLStatus (*MTDidentifyRoutine) (FLFlash *);

extern MTDidentifyRoutine mtdTable[FL_MTDS];
extern void docSocketInit(FLSocket * pVol);

/* Returns specific flash structure of the socket */

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */
extern TFFS_DLL_API FLFlash * NAMING_CONVENTION flFlashOf(FLByte volNo);
#ifdef __cplusplus
}
#endif /* __cplusplus */

/* See interface documentation of functions in flflash.c        */

extern FLStatus flIdentifyFlash(FLSocket *socket, FLFlash *flash);

extern FLStatus flDismountFlash(FLFlash * flash);

#endif /* _FLFLASH_H */

