/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/_fltl.h-arc  $
 * 
 *    Rev 1.0   Mar 27 2005 09:27:28   oris
 * Initial revision.
 * 
 *    Rev 1.5   Mar 27 2005 09:20:44   oris
 * Cosmetics
 * 
 *    Rev 1.4   Mar 09 2005 02:15:28   oris
 * Include flformat.h and not dosformt.h
 * 
 *    Rev 1.3   Jan 05 2005 20:55:54   eugen
 * Commentaries, headers
 * 
 *    Rev 1.2   Dec 27 2004 19:51:36   oris
 * Added FL_STATIC_xxx defintions
 * 
 *    Rev 1.1   Dec 23 2004 19:35:46   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:23:42   oris
 * Initial revision.
 * 
 *    Rev 1.4   Oct 23 2003 13:56:16   oris
 * Changed interface to recommendedClusterInfo routine - removed the used of in type variables
 * 
 *    Rev 1.3   Sep 01 2003 19:09:52   oris
 * - changed TLS definition to FL_TLS 
 * 
 *    Rev 1.2   May 18 2003 13:30:14   OriS
 * Changed handleConversionTable to flHandleConversionTable.
 * 
 * 
 *    Rev 1.1   May 11 2003 08:27:16   OriS
 * Removed FL_LEAVE_BINARY_AREA and FL_LEAVE_SOME_BINARY_ARE. Now using the TL_xxx version.
 * 
 *    Rev 1.0   Apr 09 2003 12:16:02   OriS
 * Initial revision.
 * 
 */

#ifndef _FLTL_H
#define _FLTL_H

typedef struct {
  SectorNo sectorsInVolume;
  FLDword bootAreaSize;
  FLDword eraseCycles;
  FLDword tlUnitBits;
} TLInfo;


/* Defragment modes types */
#define FL_FAST_SPACE			0
#define FL_NORMAL_SPACE			1

/* Defragment length types */
#define FL_MINIMAL_DEFRAGMENTATION	    -1
#define FL_MAXIMUM_DEFRAGMENTATION	    -20
#define FL_STATIC_WEAR_LEVELING_DELAYED -2
#define FL_STATIC_WEAR_LEVELING_NOW     -3
#define FL_STATIC_WEAR_LEVELING_ON      -4
#define FL_STATIC_WEAR_LEVELING_OFF     -5


typedef struct tTLrec TLrec;     /* Defined by translation layer */


struct tTL {
  TLrec       *rec;
  FLByte      partitionNo;
  FLByte      socketNo;

  /* information is necessary for hard-disk TL implementation */
  FLWord cylinders;
  FLWord heads;
  FLWord sectorsPerTrack;
  /* they should not be concealed in TL->rec field: when the media is formatted
  it would be best if geometry written in BPB would match actual disk geometry */

  /* information necessary to support fast+normal geometry */
  FLDword fastAreaSectors;
  FLWord  fastArea_SectorsInUnit_Bits;
  FLWord  normalArea_SectorsInUnit_Bits;
  FLWord  riskZoneSectors;			/* largest of fast and normal risk zones */
  FLWord  recommendedSectorsInCluster;		/* or just call it sectorsInPage (also largest) */
  /* there is no need to hide these vars via "recommendedClusterInfo" :
  they are not used by TL itself. BTW, getTLinfo is not necessary for same reason */

  void FAR1* progressCallback;

#ifndef FL_NEW_MAPSECTOR
  const void FAR0 *(*mapSector)(TLrec *, SectorNo sectorNo, CardAddress *physAddr);
#else
  FLStatus	     (*mapSector)(TLrec *, SectorNo sectorNo, void * *retBuffer);
#endif

  FLStatus       (*writeSector)(TLrec *, SectorNo sectorNo, void FAR1 *fromAddress);

  FLStatus       (*writeMultiSector)(TLrec *, SectorNo sectorNo, void FAR1 *fromAddress,SectorNo sectorCount);
  FLStatus       (*readSectors)(TLrec *, SectorNo sectorNo, void FAR1 *dest,SectorNo sectorCount);

  FLStatus       (*deleteSector)(TLrec *, SectorNo sectorNo, SectorNo noOfSectors);
  FLStatus       (*tlSetBusy)(TLrec *, FLBoolean);
  void           (*dismount)(TLrec *);
  FLStatus       (*defragment)(TLrec *, FLSDword FAR2 *sectorsNeeded, FLDword normalSpace);
  FLStatus       (*checkVolume)(TLrec *);
  SectorNo       (*sectorsInVolume)(TLrec *);
  FLStatus       (*getTLInfo)(TLrec *, TLInfo *tlInfo);
  void           (*recommendedClusterInfo)(TLrec *, FLDword *sectorsPerCluster, FLDword *clusterAlignment);

  FLStatus       (*readBBT)(TLrec *, CardAddress FAR1 * buf, FLSDword FAR2 * mediaSize, unsigned FAR2 * noOfBB);

  FLStatus       (*notifyChange)(TLrec *, void FAR1 * reserved, FLEnvVars variableType , FLDword value);
  FLStatus       (*completeOperation)(TLrec *);

  FLDword	     (* getLastMappedSectorAddress)(TLrec *);	/* use after mapSector */
  FLStatus       (* ensureFreeSectorsInUnit)(TLrec *, FLWord wVirtualUnit, FLWord * wFreeSectorsRequired);
};


#include "flformat.h"

/* The address of this, if returned from map, denotes a data error */
#ifndef FL_NEW_MAPSECTOR
/* extern FLStatus dataErrorObject;
#define dataErrorToken ((void FAR0 *) &dataErrorObject) */

#define dataErrorToken ((void FAR0 *) 0xFFFFFFFF)
/* temporary - should not be necessary with new mapSector
old method (address of dataErrorObject) is not translatable across BD-FS split
or translatable with added code/complexity */
#endif /* FL_NEW_MAPSECTOR */

#ifdef FLFLASH_H
/* Translation layer registration information */

extern int noOfTLs;    /* No. of translation layers actually registered */

typedef struct {
  FLStatus (*mountRoutine)   (unsigned volNo, TL *tl, FLFlash *flash, FLFlash **volForCallback);
  FLStatus (*formatRoutine)  (unsigned volNo, TLFormatParams *deviceFormatParams, FLFlash *flash);
  FLStatus (*preMountRoutine)(FLFunctionNo callType, IOreq FAR2* ioreq ,FLFlash* flash,FLStatus* status);
} TLentry;

extern TLentry tlTable[FL_TLS];
extern FLStatus noFormat (unsigned volNo, TLFormatParams *formatParams, FLFlash *flash);
extern FLStatus flMount(unsigned volNo, unsigned socketNo,TL *, FLBoolean useFilters , FLFlash *flash);
extern FLStatus flPreMount(FLFunctionNo callType, IOreq FAR2* ioreq , FLFlash *flash);
extern unsigned noOfDrives;
extern FLByte   flHandleConversionTable[FL_SOCKETS][FL_MAX_TL_PARTITIONS];

#ifdef FL_FORMAT_VOLUME
extern FLStatus flFormat(unsigned volNo, TLFormatParams *formatParams, FLFlash * flash);
#endif /* FL_FORMAT_VOLUME */

#endif /* FLFLASH_H */

#endif /* FLTL_H */


