/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/blockdev.h-arc  $
 * 
 *    Rev 1.3   Sep 12 2005 13:06:36   roneng
 * Change mediaType definition fo G4 
 * 
 *    Rev 1.2   Aug 14 2005 14:56:00   roneng
 * Add G4 media type number definition
 * 
 *    Rev 1.1   Aug 14 2005 14:54:02   roneng
 * Add G4 mediatype
 * 
 *    Rev 1.0   Mar 27 2005 09:27:22   oris
 * Initial revision.
 * 
 *    Rev 1.2   Mar 27 2005 09:16:20   oris
 * - Changed OREN to H1
 * 
 *    Rev 1.1   Mar 09 2005 02:13:20   oris
 * No longer needs to include dosformt.h
 * 
 *    Rev 1.0   Dec 09 2004 18:24:26   oris
 * Initial revision.
 * 
 *    Rev 1.5   Sep 30 2003 17:56:24   oris
 * Force include of dosformt.h file
 * 
 *    Rev 1.4   Sep 01 2003 19:10:18   oris
 * - Changed comments.
 * 
 *    Rev 1.3   Jun 16 2003 01:17:46   oris
 * Added TFFS_DLL_API prefix.
 * 
 *    Rev 1.2   May 11 2003 08:25:36   OriS
 * Moved format irFlags definition to flformat.h
 * 
 *    Rev 1.1   May 08 2003 23:18:22   OriS
 * Added SUPPORT_RUGGED_FILE_SYSTEM capability.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef BLOCKDEV_H
#define BLOCKDEV_H

#include "flreq.h"
#include "flfuncno.h"
#include "docsys.h"

#ifdef FL_FORMAT_VOLUME
#include "flformat.h"
#endif /* FL_FORMAT_VOLUME */
#ifdef FL_EXTENDED_DISK_INFO
#include "fltl.h"
#endif /* FL_EXTENDED_DISK_INFO */
#ifdef BDK_ACCESS
#include "docbdk.h"
#endif /* BDK_ACCESS */

#include "_blkdev.h"


/*----------------------------------------------------------------------*/
/*                     V o l u m e I n f o R e c o r d                  */
/*                                                                      */
/* A structure that holds general information about the media. The      */
/* information includes Physical Info (see flGetPhysicalInfo), Logical  */
/* partition (number of sectors and CHS), boot area size, S/W versions  */
/* Media life-time etc.                                                 */
/* A pointer to this structure is passed to the function flVolumeInfo   */
/* where it receives the relevant data.                                 */
/*----------------------------------------------------------------------*/

typedef struct {
  FLDword  logicalSectors;    /*  number of logical sectors                  */
  FLDword  bootAreaSize;      /*  boot area size                             */
  FLDword  baseAddress;       /*  physical base address                      */
#ifdef FL_LOW_LEVEL
  FLWord flashType;           /*  JEDEC id of the flash                      */
  FLDword  physicalSize;      /*  physical size of the media                 */
  FLDword  physicalUnitSize;  /*  Erasable block size                        */
  FLByte   DOCType;           /*  DiskOnChip type (MDoc/Doc2000)             */
  FLSByte  lifeTime;          /*  Life time indicator for the media (1-10)   */
                              /*  1 - the media is fresh,                    */
                              /*  10 - the media is close to its end of life */
#endif
  FLSByte  driverVer[10];     /*  driver version (NULL terminated string)    */
  FLSByte  OSAKVer[10];       /*  TrueFFS version that driver is based on    */
                              /*  (NULL terminated string)                   */
#ifdef FL_ABS_READ_WRITE
  FLDword  cylinders;         /*  Media.....                                 */
  FLDword  heads;             /*            geometry......                   */
  FLDword  sectors;           /*                            parameters.      */
#endif
} VolumeInfoRecord;

#ifdef FL_LOW_LEVEL

/*----------------------------------------------------------------------*/
/*                          P h y s i c a l I n f o                     */
/*                                                                      */
/* A structure that holds physical information about the media. The     */
/* information includes JEDEC ID, unit size and media size. Pointer     */
/* to this structure is passed to the function flGetPhysicalInfo where  */
/* it receives the relevant data.                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/

typedef struct {
  FLWord type;                 /* Flash device type (JEDEC id)         */
  FLByte         mediaType;    /* type of media see below              */
  FLDword        unitSize;     /* Smallest physically erasable size    */
                               /* (with interleaving taken in account) */
  FLDword        mediaSize;    /* media size in bytes                  */
  FLDword        chipSize;     /* individual chip size in bytes        */
  FLDword        interleaving; /* device interleaving                  */
} PhysicalInfo;

/* media types */
#define FL_NOT_DOC      0
#define FL_DOC          1
#define FL_MDOC         2
#define FL_DOC2000TSOP  3
#define FL_MDOCP_16     4
#define FL_MDOCP        5
#define FL_DOC_H1       6
#define FL_MDOC512_G3   7
#define FL_MDOC256_P3   8
#define FL_MDOC256_G3   9
#define FL_MDOC128_P3   10
#define FL_AMD_NOR      11
#define FL_SEIJA        12
#define FL_MDOC_G4      13
#define FL_MDOC_P4      14


/* capabilities flags */
typedef enum{
   CAPABILITY_NOT_SUPPORTED           = 0,
   CAPABILITY_SUPPORTED               = 1,
   SUPPORT_UNERASABLE_BBT             = 2,
   SUPPORT_MULTIPLE_BDTL_PARTITIONS   = 3,
   SUPPORT_MULTIPLE_BINARY_PARTITIONS = 4,
   SUPPORT_HW_PROTECTION              = 5,
   SUPPORT_HW_LOCK_KEY                = 6,
   SUPPORT_CUSTOMER_ID                = 7,
   SUPPORT_UNIQUE_ID                  = 8,
   SUPPORT_DEEP_POWER_DOWN_MODE       = 9,
   SUPPORT_OTP_AREA                   = 10,
   SUPPORT_WRITE_IPL_ROUTINE          = 11,
   SUPPORT_OTW_PROTECTION             = 12,
   SUPPORT_STICKY_LOCK_PROTECTION     = 13,
   SUPPORT_RUGGED_FILE_SYSTEM         = 14
}FLCapability;

#endif /* FL_LOW_LEVEL */

#ifdef FL_ENVIRONMENT_VARS

/*----------------------------------------------------------------------*/
/*                   f l S e t E n v V o l u m e                        */
/*                                                                      */
/* Change one of TrueFFS environment variables for a specific partition */
/*                                                                      */
/* Note : This routine is used by all other flSetEnv routines.          */
/*        In order to effect variables that are common to several       */
/*        sockets or volumes use INVALID_VOLUME_NUMBER                  */
/*                                                                      */
/* Parameters:                                                          */
/*      variableType    : variable type to cahnge                       */
/*      socket          : Associated socket                             */
/*      volume          : Associated volume (partition)                 */
/*      value           : varaible value                                */
/*                                                                      */
/* Note: Variables common to all sockets must be addressed using socket */
/*       0 and volume 0.                                                */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus        : 0 on success, otherwise failed                */
/*      prevValue       : The previous value of the variable            */
/*----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */
TFFS_DLL_API FLStatus NAMING_CONVENTION flSetEnvVolume(FLEnvVars variableType ,
                  FLByte  socket,FLByte  volume ,
                  FLDword value, FLDword FAR2 *prevValue);
#ifdef __cplusplus
}
#endif /* __cplusplus */

/*----------------------------------------------------------------------*/
/*                       f l S e t E n v S o c k e t                    */
/*                                                                      */
/* Change one of TrueFFS environment variables for a specific sockets.  */
/*                                                                      */
/* Parameters:                                                          */
/*      variableType    : variable type to cahnge                       */
/*      socket          : socket number                                 */
/*      value           : varaible value                                */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus        : 0 on success, otherwise failed                */
/*      prevValue       : The previous value of the variable            */
/*                        if there are more then 1 partition in that    */
/*                        socket , the first partition value is returned*/
/*----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */
TFFS_DLL_API FLStatus NAMING_CONVENTION flSetEnvSocket(FLEnvVars variableType , FLByte  socket ,
                        FLDword value, FLDword FAR2 *prevValue);
#ifdef __cplusplus
}
#endif /* __cplusplus */

/*----------------------------------------------------------------------*/
/*                       f l S e t E n v All                            */
/*                                                                      */
/* Change one of TrueFFS environment variables for all systems, sockets */
/* and partitions.                                                      */
/*                                                                      */
/* Parameters:                                                          */
/*      variableType    : variable type to cahnge                       */
/*      value           : varaible value                                */
/*                                                                      */
/* Returns:                                                             */
/*      FLStatus        : 0 on success, otherwise failed                */
/*      prevValue       : The previous value of the variable            */
/*----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */
TFFS_DLL_API FLStatus NAMING_CONVENTION flSetEnvAll(FLEnvVars variableType , FLDword value, FLDword FAR2 *prevValue);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* FL_ENVIRONMENT_VARS */

#endif /* BLOCKDEV_H */
