/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/docbdk.h-arc  $
 * 
 *    Rev 1.2   Apr 19 2005 12:15:48   oris
 * Compilation error
 * 
 *    Rev 1.1   Apr 19 2005 02:47:02   oris
 * - Replaced BDK_PAGE_SIZE with BDK_SECTORS_PER_PAGE,
 *   and therefore moved it from docbdk.h
 * - Added hidden customization of the buffer size used for recovery
 *   from flash write failures (BDK_SECTORS_PER_WRITE).
 * 
 *    Rev 1.0   Mar 27 2005 09:27:16   oris
 * Initial revision.
 * 
 *    Rev 1.2   Mar 27 2005 09:16:30   oris
 * - Dynamic bad blocks support.
 * 
 *    Rev 1.1   Jan 05 2005 21:03:08   eugen
 * Commentaries, headers
 * 
 *    Rev 1.0   Dec 09 2004 18:24:26   oris
 * Initial revision.
 * 
 *    Rev 1.2   Oct 17 2004 15:01:54   OriS
 * - Moved customization of the amount of binary partitions to 
 *   flcustom.h (MAX_BINARY_PARTITIONS_PER_DRIVE)
 * 
 *    Rev 1.1   Apr 13 2003 20:29:22   OriS
 * Fixed compilation error for BDK
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

/************************************************************************/
/* Caution: The BDK_ACCESS compilation flag is for M-SYSTEMS internal   */
/*          use ONLY. This flag is used by M-SYSTEMS drivers and        */
/*          therfore it is SHOULD NOT be used by this package           */
/************************************************************************/

/*****************************************************************************
* File Header                                                                *
* -----------                                                                *
* Name : docbdk.h                                                            *
*                                                                            *
* Description : This file contains the binary partition defintions , data    *
*               structures and function prototypes.                          *
*                                                                            *
* Note : The file exports 2 interfaces each under its own compilation flag:  *
*                                                                            *
*        BDK package - Standalone package that exports routines for binary   *
*                      partitions handling(MTD_STANDALONE compilation flag). *
*        OSAK module - Separated module of the OSAK package that exports a   *
*                      common entry point to the same routines. (BDK_ACCESS  *
*                      compilation flag).                                    *
*                                                                            *
* Warning : Do not use this file with the BDK_ACCESS compilation flag unless *
*           you own the full OSAK package.                                   *
*****************************************************************************/

#ifndef DOCBDK_H
#define DOCBDK_H

/*---------------------------------------------------------------------*/
/* Include the proper header files.                                    */
/*---------------------------------------------------------------------*/

#include "nanddefs.h"  /* The MTD for the doc2000 and millennium DiskOnChips */

#ifndef MTD_STANDALONE
#ifdef BDK_ACCESS
#include "flfuncno.h"
#include "flreq.h"
#endif /* BDK_ACCESS */
#endif /* MTD_STANDALONE */

/* Use the define bellow to set the size of the page buffer used by the BDK */
#ifndef BDK_SECTORS_PER_PAGE
#define BDK_SECTORS_PER_PAGE		  (4L /* 2KB PAGE */)
#endif /* BDK_SECTORS_PER_PAGE */

#ifndef BDK_SECTORS_PER_WRITE
#define BDK_SECTORS_PER_WRITE        (4L/*Planes*/*BDK_SECTORS_PER_PAGE/*Sectors per page*/*2/*MLC*/)
#endif /* BDK_SECTORS_PER_WRITE */

#if (BDK_SECTORS_PER_WRITE < (BDK_SECTORS_PER_PAGE))
#error "Error - BDK_SECTORS_PER_WRITE must be at least as large as BDK_PAGE_SIZE.\r\n"
#endif /* (BDK_SECTORS_PER_WRITE < (BDK_PAGE_SIZE >> FL_SECTOR_SIZE_BITS)) */

#if ((BDK_SECTORS_PER_WRITE % (BDK_SECTORS_PER_PAGE)) != 0)
#error "Error - BDK_SECTORS_PER_WRITE must be a multiplication of the number of sectors per BDK_PAGE_SIZE.\r\n"
#endif /* (BDK_SECTORS_PER_WRITE < (BDK_PAGE_SIZE >> FL_SECTOR_SIZE_BITS)) */


/* BDK specific flag area */
#define ERASE_BEFORE_WRITE              8
#define BDK_COMPLETE_IMAGE_UPDATE       16
#define BDK_PARTIAL_IMAGE_UPDATE        0
#define BDK_SIGNATURE_NAME              4
#define BDK_SPARE_BLOCKS_SIGN           "S4BB" /* Must be the same as BINARY_SPARE_BLOCKS_SIGN (FLFORMAT.H) */

#ifdef BDK_ACCESS

/*------------------------------------------------------------------------*/
/* OSAK Routines argument packet                                          */
/*------------------------------------------------------------------------*/

typedef struct {
    FLByte       oldSign[BDK_SIGNATURE_NAME];
    FLByte       newSign[BDK_SIGNATURE_NAME];
    FLByte       signOffset;
    FLDword      startingBlock;
    FLDword      length;
    FLByte       flags;
    FLByte FAR1 *bdkBuffer;
} BDKStruct;

#endif /* BDK_ACCESS */


#include "_docbdk.h"

#endif /* DOCBDK_H */




