/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/dosformt.h-arc  $
 * 
 *    Rev 1.0   Mar 27 2005 09:27:26   oris
 * Initial revision.
 * 
 *    Rev 1.2   Feb 19 2005 12:31:22   eugen
 * now uses FL_PACKED_STRUCTURES_BEFORE and _AFTER
 * 
 *    Rev 1.0   Dec 09 2004 18:23:42   oris
 * Initial revision.
 * 
 *    Rev 1.8   Oct 17 2004 15:04:14   OriS
 * - Renamed reserved field in DirectoryEntry to lowerCaseFlags
 *        ASCII interface 	- FL_LONG_FILENAMES_ASCII_SUPPORT
 *        UTF interface 	- FL_LONG_FILENAMES_UTF8_SUPPORT
 *        User specific 	- FL_TRANSLATE_FROM_UNICODE and 
 *                                   	  FL_TRANSLATE_TO_UNICODE user specific
 *        None for Unicode
 * 
 *    Rev 1.7   Jun 15 2004 08:45:52   eugen
 * renamed reserved field in DirectoryEntry to lowerCaseFlags
 * 
 *    Rev 1.6   Dec 01 2003 16:08:00   oris
 * Added support for windows packed structure pragma using the FL_WINDOWS_PACKING compilation flag.
 * 
 *    Rev 1.5   Nov 16 2003 18:55:12   omerk
 * Remove the Marcos: FL_PACKED_STRUCTURES_BEGIN,  FL_PACKED_STRUCTURES_END .
 * 
 * 
 *    Rev 1.4   Nov 09 2003 17:06:08   OmerK
 * Added default FL_PACKED_STRUCTURES_BEGIN and FL_PACKED_STRUCTURES_END for forced packed structures.
 * 
 *    Rev 1.3   Oct 20 2003 09:01:22   oris
 * Changed word variable to FLWord.
 * 
 *    Rev 1.2   Sep 30 2003 17:57:10   oris
 * Moved none API definitions to _dosformt.h and added FL prefix to most of SureFS definition.
 * Moved SureFS API definitions from _blkdev.h
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.1   Sep 01 2003 19:09:06   oris
 * - Added LfnDirectoryEntry structure.
 * - Added a customizable packed structure prefix macro , to force packing of the FAT structures.
 * - Added SureFS flag definitions. 
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef DOSFORMT_H
#define DOSFORMT_H

#include "flformat.h"
#include "fltl.h"

/* The BIOS parameter block (a part of the boot sector)		    */
/* Note that this is NOT the customary definition of the BPB    */
/* (customary is to start it on 'bytesPerSector'). To avoid the */
/* nuisance of a structure that starts at an odd offset, we add */
/* the first 11 bytes of the boot sector here.			        */

#ifdef FL_PACKED_STRUCTS_SUPPORTED

#ifdef FL_WINDOWS_PACKING
#pragma pack(push, 1)
#endif /* FL_WINDOWS_PACKING */


typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  FLByte	jumpInstruction[3];
  FLSByte	OEMname[8];
  Unaligned	bytesPerSector;
  FLByte	sectorsPerCluster;
  LEushort	reservedSectors;
  FLByte	noOfFATS;
  Unaligned	rootDirectoryEntries;
  Unaligned	totalSectorsInVolumeDOS3;
  FLByte	mediaDescriptor;
  LEushort	sectorsPerFAT;
  LEushort	sectorsPerTrack;
  LEushort	noOfHeads;
  LEulong	noOfHiddenSectors;
  LEulong	totalSectorsInVolume;
} BPB;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_BPB_JUMP_INSTRUCTION_OFFSET		0	/* size 3, array */
#define FL_BPB_OEM_NAME_OFFSET			3	/* size 8, array */
#define FL_BPB_BYTES_PER_SECTOR_OFFSET		11	/* size 2, unaligned */
#define FL_BPB_SECTORS_PER_CLUSTER_OFFSET	13	/* size 1 */
#define FL_BPB_RESERVED_SECTORS_OFFSET		14	/* size 2 */
#define FL_BPB_NO_OF_FATS_OFFSET		16	/* size 1 */
#define FL_BPB_ROOT_DIR_ENTRIES_OFFSET		17	/* size 2, unaligned */
#define FL_BPB_TOTAL_SECTORS_IN_VOL_DOS3_OFFSET	19	/* size 2, unaligned */
#define FL_BPB_MEDIA_DESCRIPTOR_OFFSET		21	/* size 1 */
#define FL_BPB_SECTORS_PER_FAT_OFFSET		22	/* size 2 */
#define FL_BPB_SECTORS_PER_TRACK_OFFSET		24	/* size 2 */
#define FL_BPB_NO_OF_HEADS_OFFSET		26	/* size 2 */
#define FL_BPB_NO_OF_HIDDEN_SECTORS_OFFSET	28	/* size 4 */
#define FL_BPB_TOTAL_SECTORS_IN_VOL_OFFSET	32	/* size 4 */
#define FL_BPB_SIZE				36

typedef FLByte BPB[FL_BPB_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */


#ifdef FL_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  BPB	bpb;
  FLByte	physicalDriveNo;
  FLSByte	reserved0;
  FLSByte	extendedBootSignature;
  FLSByte	volumeId[4];
  FLSByte	volumeLabel[11];
  FLSByte	systemId[8];
  FLSByte	bootstrap[448];
  LEushort	signature;
} DOSBootSector;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_DOSBOOTSECTOR_BPB_OFFSET			0	/* size 36 */
#define FL_DOSBOOTSECTOR_PHYSICAL_DRIVE_NO_OFFSET	36	/* size 1 */
#define FL_DOSBOOTSECTOR_RESERVED0_OFFSET		37	/* size 1 */
#define FL_DOSBOOTSECTOR_EXTENDED_BOOT_SIGNATURE_OFFSET	38	/* size 1 */
#define FL_DOSBOOTSECTOR_VOLUME_ID_OFFSET		39	/* size 4, array */
#define FL_DOSBOOTSECTOR_VOLUME_LABEL_OFFSET		43	/* size 11, array */
#define FL_DOSBOOTSECTOR_SYSTEM_ID_OFFSET		54	/* size 8, array */
#define FL_DOSBOOTSECTOR_BOOTSTRAP_OFFSET		62	/* size 448, array */
#define FL_DOSBOOTSECTOR_SIGNATURE_OFFSET		510	/* size 2 */
#define FL_DOSBOOTSECTOR_SIZE				512

typedef FLByte DOSBootSector[FL_DOSBOOTSECTOR_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */


#ifdef FL_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  FLSByte   name[11];
  FLByte    attributes;		/* mapped below */
  FLByte    lowerCaseFlags;
  FLByte    createTimeMilisec;
  LEushort  createTimeHourMin;
  LEushort  createDate;
  LEushort  lastAccessedDate;
  LEushort  reserved1;
  LEushort	updateTime;
	/* The date field is encoded as follows:     */
	/* 	bit 0-4:	Day of month (1-31)	         */
	/* 	bit 5-8:	Month (1-12)                 */
	/*	bit 9-15:	Year relative to 1980        */
  LEushort	updateDate;
	/* The DOS time field is encoded as follows: */
	/*	bit 0-4:	seconds divided by 2 (0-29)  */
	/*      bit 5-10:	minutes (0-59)           */
	/*	bit 11-15:	hours (0-23)                 */
  LEushort	startingCluster;
  LEulong	fileSize;
} DirectoryEntry;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_DIRECTORY_ENTRY_NAME_OFFSET			0	/* size 11, array */
#define FL_DIRECTORY_ENTRY_ATTRIBUTES_OFFSET		11	/* size 1 */
#define FL_DIRECTORY_ENTRY_LOWERCASE_FLAGS_OFFSET	12	/* size 1 */
#define FL_DIRECTORY_ENTRY_CREATE_TIME_MILLISEC_OFFSET	13	/* size 1 */
#define FL_DIRECTORY_ENTRY_CREATE_TIME_HOURMIN_OFFSET	14	/* size 2 */
#define FL_DIRECTORY_ENTRY_CREATE_DATE_OFFSET		16	/* size 2 */
#define FL_DIRECTORY_ENTRY_LAST_ACCESS_DATE_OFFSET	18	/* size 2 */
#define FL_DIRECTORY_ENTRY_RESERVED1_OFFSET		20	/* size 2 */
#define FL_DIRECTORY_ENTRY_UPDATE_TIME_OFFSET		22	/* size 2 */
#define FL_DIRECTORY_ENTRY_UPDATE_DATE_OFFSET		24	/* size 2 */
#define FL_DIRECTORY_ENTRY_STARTING_CLUSTER_OFFSET	26	/* size 2 */
#define FL_DIRECTORY_ENTRY_FILESIZE_OFFSET		28	/* size 4 */
#define FL_DIRECTORY_ENTRY_SIZE				32

typedef FLByte DirectoryEntry[FL_DIRECTORY_ENTRY_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */


#ifdef FL_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
	FLByte	   orderNum;			/* 6 significant bits of LFN order number */
	Unaligned  name1[5];			/* part 1 of scattered name */
	FLByte	   attributes;			/* file attributes, must be RO, System, Hidden, Volume */
	FLByte	   reserved;			/* must be 0 */
	FLByte	   checksum;			/* checksum created from short form name */
	LEushort   name2[6];			/* part 2 of scattered name */
	LEushort   startingCluster;		/* must be 0 */
	LEushort   name3[2];			/* part 3 of scattered name */
} LfnDirectoryEntry;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_LFN_DIRECTORY_ENTRY_ORDER_NUM_OFFSET			0	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_NAME1_OFFSET			1	/* size 10, unaligned array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_ATTRIBUTES_OFFSET		11	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_RESERVED_OFFSET			12	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_CHECKSUM_OFFSET			13	/* size 1 */
#define FL_LFN_DIRECTORY_ENTRY_NAME2_OFFSET			14	/* size 12, array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_STARTING_CLUSTER_OFFSET		26	/* size 2 */
#define FL_LFN_DIRECTORY_ENTRY_NAME3_OFFSET			28	/* size 4, array of 2 */
#define FL_LFN_DIRECTORY_ENTRY_SIZE				32

typedef FLByte LfnDirectoryEntry[FL_LFN_DIRECTORY_ENTRY_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */

/* Directory entry attribute bits */

#define	FL_ATTR_READ_ONLY   1
#define	FL_ATTR_HIDDEN      2
#define	FL_ATTR_SYSTEM      4
#define	FL_ATTR_VOL_LABEL   8
#define	FL_ATTR_DIRECTORY   0x10
#define	FL_ATTR_ARCHIVE	    0x20


#ifdef FL_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  /* First partition entry starts here. We do not map it as a	*/
  /* separate structure because it is not longword aligned	*/
  FLByte	activeFlag;	/* 80h = bootable */
  FLByte        startingHead;
  LEushort	startingCylinderSector;
  FLSByte       type;
  FLByte	endingHead;
  LEushort	endingCylinderSector;
  Unaligned4	startingSectorOfPartition;
  Unaligned4	sectorsInPartition;
  /* Partition entries 2,3 and 4 are structured as the 1st partition */
} Partition;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_PARTITION_ACTIVE_FLAG_OFFSET				0	/* size 1 */
#define FL_PARTITION_STARTING_HEAD_OFFSET			1	/* size 1 */
#define FL_PARTITION_STARTING_CYLINDER_SECTOR_OFFSET		2	/* size 2 */
#define FL_PARTITION_TYPE_OFFSET				4	/* size 1 */
#define FL_PARTITION_ENDING_HEAD_OFFSET				5	/* size 1 */
#define FL_PARTITION_ENDING_CYLINDER_SECTOR_OFFSET		6	/* size 2 */
#define FL_PARTITION_STARTING_SECTOR_OF_PARTITION_OFFSET	8	/* size 4 */
#define FL_PARTITION_SECTORS_IN_PARTITION_OFFSET		12	/* size 4 */
#define FL_PARTITION_SIZE					16

typedef FLByte Partition[FL_PARTITION_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */


#ifdef FL_PACKED_STRUCTS_SUPPORTED

typedef FL_PACKED_STRUCTURES_BEFORE struct FL_PACKED_STRUCTURES_AFTER {
  FLSByte       reserved1[0x1A6];
  Unaligned4    passwordInfo[3];  /* M-Systems proprietary */
  FLSByte       reserved2[0xC];   /* NT4 or NT5 signature place */

  /* First partition entry starts here. We do not map it as a	*/
  /* separate structure because it is not longword aligned	*/
  Partition     ptEntry[4];
  LEushort      signature;	/* = PARTITION_SIGNATURE */
} PartitionTable;
#else /* FL_PACKED_STRUCTS_SUPPORTED */

#define FL_PARTITION_TABLE_RESERVED1_OFFSET			0	/* size 0x1A6 */
#define FL_PARTITION_TABLE_PASSWORD_INFO_OFFSET			422	/* size 12, unaligned array of 4 */
#define FL_PARTITION_TABLE_RESERVED2_OFFSET			434	/* size 12, array */
#define FL_PARTITION_TABLE_PT_ENTRY_OFFSET			446	/* size 4x16, array of partition entries */
#define FL_PARTITION_TABLE_SIGNATURE_OFFSET			510	/* size 2 */
#define FL_PARTITION_TABLE_SIZE					512

typedef FLByte PartitionTable[FL_PARTITION_TABLE_SIZE];
#endif /* FL_PACKED_STRUCTS_SUPPORTED */

#ifdef FL_PACKED_STRUCTS_SUPPORTED

#ifdef FL_WINDOWS_PACKING
#pragma pack(pop)
#endif /* FL_WINDOWS_PACKING */

#endif /* FL_PACKED_STRUCTS_SUPPORTED */

#define PARTITION_ENTRIES	4
/*----------------------------------------------------------------------*/
/*                          L f n P a r a m                             */
/* Pointer to this structure is passed to those FATLITE functions that  */
/* look for first or next file in directory.                            */
/*----------------------------------------------------------------------*/
typedef struct {
	DirectoryEntry	dirEntry; /* copy of the found directory entry.        */
	void			*name;    /* address of supplied buffer to receive     */
					  /* found file name                           */
	FLWord			bufSize;  /* size of supplied buffer in bytes. The     */
	                          /* buffer size may vary according to         */
				  /* developer will but one must remember      */
				  /* that longest long file name requires      */
				  /* buffer to contain LFN_NAME_MAX_CHARACTERS */
				  /* + 1 characters (ASCII as well as Unicode) */
} LfnParam;


/** Values of irFlags for flOpenFile: */

#define ACCESS_MODE_MASK   3 /* Mask for access mode bits */

/* Individual flags */

#define FL_ACCESS_READ_WRITE  1 /* Allow read and write */
#define FL_ACCESS_CREATE      2 /* Create new file      */

/* Access mode combinations */

#define FL_OPEN_FOR_READ      0 /* open existing file for read-only         */
#define FL_OPEN_FOR_UPDATE    1 /* open existing file for read/write access */
#define FL_OPEN_FOR_WRITE     3 /* create a new file, even if it exists     */

/* Values of irFlags for flSeekFile: */

#define FL_SEEK_START       0 /* offset from start of file    */
#define FL_SEEK_CURR        1 /* offset from current position */
#define FL_SEEK_END         2 /* offset from end of file      */

/** Bit assignment of irFlags for flFindFile: */

#define FL_SET_DATETIME        1 /* Change date/time                        */
#define FL_SET_ATTRIBUTES      2 /* Change attributes                       */
#define FL_FIND_BY_HANDLE      4 /* Find file by handle rather than by name */
#define FL_SET_CREATE_DATETIME 8 /* Change creation date/time               */

/* commands to flGetSetFileInfo (in addition to above) */
#define FL_GET_DATETIME        -1 /* Get date/time                        */
#define FL_GET_ATTRIBUTES      -2 /* Get attributes                       */
#define FL_GET_CREATE_DATETIME -8 /* Get creation date/time               */

typedef struct {
  unsigned        bytesPerSector;
  unsigned        sectorsPerCluster;
  unsigned        totalClusters;
  unsigned        freeClusters;
} DiskInfo;

#include "_dosfrmt.h"


#define FLIsLower(val) (((val) >= 'a') && ((val) <= 'z'))
#define FLIsUpper(val) (((val) >= 'A') && ((val) <= 'Z'))

#define FLToLower(val) (FLIsUpper(val) ? (val-('A'-'a')) : (val))
#define FLToUpper(val) (FLIsLower(val) ? (val+('A'-'a')) : (val))


#endif /* DOSFORMT_H */

