/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/flbase.h-arc  $
 * 
 *    Rev 1.7   Jun 07 2005 06:17:50   OriS
 * - Added new environment variables : FL_HW_STATUS
 * - Esthetics - Moved FL_DEFAULT_BUS_CONFIGURATION from blockdev.h.
 * - Esthetics - Changed the following environment variables default values
 *    to be socket (or socket and partition) specific.
 *   o FL_DEFAULT_BUS_CONFIGURATION
 *   o FL_DEFAULT_BD_DPD_MODE;
 *   o FL_DEFAULT_BD_SUSPEND_MODE;
 *   o FL_DEFAULT_BD_DEBUG_MODE;
 *   o FL_DEFAULT_POLICY;
 *   o FL_DEFAULT_WEARLEVELING_MODE;
 *   o FL_DEFAULT_VERIFY_WRITE_MODE;
 * - Esthetics - Moved debug print zones to flsystyp.h to allow using them without
 *   including flsystem.h (user customization routines for flsystem).
 * - Esthetics - Changed FL_DEFAULT_CACHE_SIZE_FOR_FULL_CACHE 
 *   to FL_MAX_ALLOWED_TL_CACHE_SIZE
 * 
 *    Rev 1.6   Apr 21 2005 15:45:46   oris
 * Split some of the content to flsystyp.h
 * 
 *    Rev 1.5   Apr 21 2005 02:25:32   oris
 * - flsysfunc.h is not a part of the Boot SDK as well
 * 
 *    Rev 1.3   Apr 19 2005 02:54:52   oris
 * - New environment varaibles:
 *   FL_SET_AUTO_DPD_MODE:
 *      FLDword  flBDAutoDPDMode[FL_SOCKETS];
 *   FL_DEBUG_MODES:
 *      FLDword  flTFFSBDDebugMode[FL_SOCKETS][FL_MAX_TL_PARTITIONS];
 * - New environment variables definitions:
 *   FL_QUICK_MOUNT_DO_NOT_READ  - Do not use the quick mount when mounting
 *   FL_QUICK_MOUNT_DO_NOT_WRITE - Do not write the quick mount on dismount
 *   FL_QUICK_MOUNT_DISABLED     - Mask for the above 2 flags
 * - New environment variables defaults:
 *   FL_DEFAULT_BD_DPD_MODE		  FL_OFF
 *   FL_DEFAULT_BD_SUSPEND_MODE		  FL_OFF
 *   FL_DEFAULT_BD_DEBUG_MODE                FL_OFF
 *   FL_DEFAULT_CACHE_SIZE_FOR_MINIMAL_CACHE 0x64
 *   FL_DEFAULT_WEARLEVELING_MODE            FL_OFF
 *   FL_DEFAULT_VERIFY_WRITE_MODE            FL_OFF
 *   FL_DEFAULT_TOOLS_MODE			  FL_OFF
 * - Changed number of FL_DIMAGE_CFG
 * 
 *    Rev 1.2   Apr 05 2005 07:19:28   oris
 * - Fixed more compilation errors for Boot pakcage.
 * - Removed declaration of old memory manipulation routines.
 * 
 *    Rev 1.1   Apr 04 2005 20:17:42   oris
 * - compilation errors for Boot package.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:26   oris
 * Initial revision.
 * 
 *    Rev 1.7   Mar 27 2005 09:20:58   oris
 * - Removed backwards compatibility conversion from FL_TFFS_SLEEP to flsleep 
 * 
 *    Rev 1.6   Mar 09 2005 23:36:48   eugen
 * added extern "C" to LE/UNAL access routines
 * 
 *    Rev 1.5   Mar 07 2005 22:07:28   oris
 * -	Optimized / Simplified big and little endian conversion routines
 * 
 *    Rev 1.4   Feb 08 2005 17:05:30   eugen
 * Replace use of packed structures with byte arrays
 * 
 *    Rev 1.3   Feb 01 2005 09:32:54   eugen
 * FL_GET.._FROM_HANDLE - added () around ioreq
 * DimageCfg support
 * 
 *    Rev 1.2   Jan 05 2005 20:55:54   eugen
 * Commentaries, headers
 * 
 *    Rev 1.1   Dec 23 2004 19:35:46   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:23:42   oris
 * Initial revision.
 * 
 *    Rev 1.18   Oct 21 2004 16:09:26   oris
 * - Added compilation flag to separate between 6.2.2 and down MTD
 * interface and the new 6.3 interfaces (FL_622_MTD_CODE) 
 * Required for FL_TFFS_SLEEP macro
 * 
 *    Rev 1.17   Oct 18 2004 12:32:44   oris
 * - Added compilation flag to separate between 6.2.2 and down 
 *   MTD interface and the new 6.3 interfaces (FL_622_MTD_CODE)
 *   Required for FL_TFFS_SLEEP macro
 * 
 *    Rev 1.16   Oct 17 2004 15:06:46   OriS
 * - Updated FL_TFFS_SLEEP interface for microseconds resolution.
 * - Added FL_PREVENT_IO and FL_USE_SINGLE_DPS definitions.
 * 
 *    Rev 1.13   Jun 13 2004 17:09:26   OriS
 * Added FLZONE_PARA debug print module
 * 
 *    Rev 1.12   Jan 14 2004 22:30:32   oris
 * Removed warnings
 * Replaced FL_SLEEP with FL_TFFS_SLEEP (FL_SLEEP was used by Linux)
 * Added pointer arithmetic's macros in order to remove warnings.
 * 
 *    Rev 1.11   Dec 01 2003 16:07:50   oris
 * Added FLZONE_UTIL for hidden printing in the utilities.
 * 
 *    Rev 1.10   Oct 30 2003 00:52:32   OriS
 * changed flSuspendMode to be socket specific and to accept FL_IGNORE_WRITE and FL_IGNORE_ERASE modes
 * 
 *    Rev 1.9   Oct 23 2003 17:20:02   OriS
 * Bug fix - compilation errors for BDK package
 * 
 *    Rev 1.8   Oct 23 2003 13:56:54   oris
 * Added extern declaration of new environment variable flSkipStaticFilesWearLeveling (FL_NO_STATIC_FILES_WEAR_LEVELING)
 * Added customization check for flsleep and FL_SLEEP macro
 * 
 *    Rev 1.7   Sep 30 2003 17:57:46   oris
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.6   Sep 01 2003 19:08:22   oris
 * - Added new print zone FLZONE_STATISTICS (0x200)
 * - Added new typedef - FLWchar
 * 
 *    Rev 1.5   May 11 2003 14:06:14   OriS
 * Changed MAX_VOLUME_MBYTES to FL_ASSUMED_MAX_VOLUME_MBYTES
 * Changed ASSUMED_NFTL_UNIT_SIZE to FL_ASSUMED_MIN_UNIT_SIZE
 * 
 *    Rev 1.4   May 11 2003 08:25:58   OriS
 * Moved FAR definition to prevent old compiler errors.
 * 
 *    Rev 1.3   May 08 2003 23:24:24   OriS
 * Restored declaration of flmemxxx routines
 * 
 *    Rev 1.2   May 06 2003 11:54:04   OriS
 * Moved backward competability check of memory manipulation routines to flchkdef.h
 * 
 *    Rev 1.1   Apr 13 2003 20:29:26   OriS
 * Fixed compilation error for BDK
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 * 
 */

#ifndef FLBASE_H
#define FLBASE_H

#include "flsystyp.h"
#include "flchkdef.h"

#ifndef MTD_STANDALONE
#include "flsysfun.h"
#endif /* MTD_STANDALONE */

 /* Number of floors supported by the S/W
 *
 * Define Maximum Number of floors in a physical device
 *
 * The actual number of floors depends on the physical device placed 
 * on the socket.
 */

#define FL_MAX_NO_OF_FLOORS  4

/* Sector size
 *
 * Define the log2 of sector size for the FAT & translation layers. Note
 * that the default 512 bytes is the de-facto standard and practically
 * the only one that provides real PC interoperability.
 */

#define FL_SECTOR_SIZE_BITS   9

/* General purpose macros */

#define BYTE_ADD_FAR(x,y) ((FLByte FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))
#define WORD_ADD_FAR(x,y) ((FLWord FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))
#define DWORD_ADD_FAR(x,y) ((FLDword FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))
#define SBYTE_ADD_FAR(x,y) ((FLSByte FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))
#define SWORD_ADD_FAR(x,y) ((FLSWord FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))
#define SDWORD_ADD_FAR(x,y) ((FLSDword FAR1 *)addToFarPointer((FLByte FAR1 *)x,y))

/* Drive handle masks */

#define FL_GET_SOCKET_FROM_HANDLE(ioreq)		(FLByte)((ioreq)->irHandle & 0x0f)
#define FL_GET_FLASH_PARTITION_FROM_HANDLE(ioreq)	(FLByte)(((ioreq)->irHandle & 0xf0) >> 4)
#define FL_GET_LOGICAL_PARTITION_FROM_HANDLE(ioreq) 	(FLByte)(((ioreq)->irHandle & 0xf00) >> 8)
#define FL_GET_DISK_HANDLE_FROM_S_P_LP(s,p, lp) 	((s) | ((p) << 4) | ((FLHandle)(lp) << 8))

#define FL_SOCKET_AND_FLASH_PARTITION_BITS	0xFF

#define FL_MEDIA_WITHOUT_MBR			(1 << 12)	/* used by LogMount & formatFS */
#define FL_MOUNT_ON_GIVEN_RANGE			(1 << 13)	/* used by LogMount & formatFS */
#define FL_MOUNT_FOR_BLOCK_IO			(1 << 14)	/* used by LogMount only */
#define FL_DO_NOT_UPDATE_MBR			(1 << 14)	/* used by formatFS only */

#define INVALID_VOLUME_NUMBER 0xff
#define TL_SIGNATURE          6

#define FL_SECTOR_SIZE        (1 << FL_SECTOR_SIZE_BITS)
#define FL_SECTOR_MASK     (FL_SECTOR_SIZE - 1)
#define BITS_PER_BITE          8

#define FL_LENGTH_IN_BYTES          0
#define FL_LENGTH_IN_SECTORS        1
#define FL_LENGTH_IN_UNITS          2
#define FL_LENGTH_IN_PERCENTS       3
#define FL_LENGTH_IN_CYLINDERS		10

/* General purpose macros adjusted according to the above customization files. */

/* define SectorNo range according to media maximum size */
typedef FLDword SectorNo;
#define    UNASSIGNED_SECTOR 0xffffffffl

/* Call a procedure returning status and fail if it fails. This works only in */
/* routines that return Status: */
#define checkStatus(exp)      {    FLStatus fl__status = (exp);if (fl__status != flOK) return fl__status; }
#define updateStatus(cur__stat, exp)      {    FLStatus fl__status = (exp);if (cur__stat == flOK) cur__stat = fl__status; }

#define TFFSMIN(a,b) (((a)>(b)) ? (b):(a))
#define TFFSMAX(a,b) (((a)<(b)) ? (b):(a))

/***************************************************************************/
/* Big \ Little endien architecture conversion macros.                     */
/***************************************************************************/

#define FL_REF_1(arg, offs)		(((FLByte *)(arg))[offs])
#define FL_REF_2(arg, offs)	 	(*((FLWord *)(((FLByte *)(arg)) + (offs))))
#define FL_REF_4(arg, offs)	 	(*((FLDword *)(((FLByte *)(arg)) + (offs))))

#ifdef FL_PACKED_STRUCTS_SUPPORTED
#define COPY2(to,arg)		*((FLWord *)(&to)) = *((FLWord *)(&arg))
#define COPY4(to,arg)		*((FLDword *)(&to)) = *((FLDword *)(&arg))
#else
#define COPY2(to, offs, arg)		FL_REF_2(to, offs) = FL_REF_2(arg, offs)
#define COPY4(to, offs, arg)		FL_REF_4(to, offs) = FL_REF_4(arg, offs)
#endif /* FL_PACKED_STRUCTS_SUPPORTED */


#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

extern void toUNAL(FLByte FAR0 *unal, FLWord n);
extern FLWord fromUNAL(FLByte const FAR0 *unal);
extern void toUNALLONG(FLByte FAR0 *unal, FLDword n);
extern FLDword fromUNALLONG(FLByte const FAR0 *unal);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#define UNAL2(arg)       fromUNAL(arg)
#define toUNAL2(to,arg)  toUNAL(to,arg)

#define UNAL4(arg)       fromUNALLONG((FLByte const FAR0 *)(arg))
#define toUNAL4(to,arg)  toUNALLONG((FLByte FAR0 *)to,arg)

/* for both LE and BE */

#define FL_GET_UNAL2(arg, offs)		fromUNAL(((FLByte *)(arg)) + (offs))
#define FL_SET_UNAL2(to, offs, arg)	toUNAL(((FLByte *)(to)) + (offs), (arg))
#define FL_GET_UNAL4(arg, offs)		fromUNALLONG(((FLByte *)(arg)) + (offs))
#define FL_SET_UNAL4(to, offs, arg)	toUNALLONG(((FLByte *)(to)) + (offs), (arg))

#ifndef FL_BIG_ENDIAN

typedef FLWord   LEushort;
typedef FLDword  LEulong;

#define LE2(arg)         	arg
#define toLE2(to,arg)    	(to) = (arg)
#define LE4(arg)         	arg
#define toLE4(to,arg)    	(to) = (arg)

typedef FLByte        Unaligned[2];
typedef Unaligned     Unaligned4[2];

#define FL_GET_LE2(arg, offs)	 	FL_REF_2(arg, offs)
#define FL_GET_LE4(arg, offs)	 	FL_REF_4(arg, offs)
#define FL_SET_LE2(to, offs, arg)	FL_REF_2(to, offs) = (arg)
#define FL_SET_LE4(to, offs, arg)	FL_REF_4(to, offs) = (arg)

#else	/* FL_BIG_ENDIAN */

typedef FLByte LEushort[2];
typedef FLByte LEulong[4];

#define Unaligned     LEushort
#define Unaligned4    LEulong

#define LE2(arg)		fromUNAL(arg)
#define toLE2(to,arg)		toUNAL(to, (FLWord)(arg))
#define LE4(arg)		fromUNALLONG((FLByte const FAR0 *)(arg))
#define toLE4(to,arg)		toUNALLONG((FLByte FAR0 *)(to), (arg))

#define FL_GET_LE2(arg, offs)		LE2((((FLByte *)(arg)) + (offs)))
#define FL_GET_LE4(arg, offs)		LE4((((FLByte *)(arg)) + (offs)))

#define FL_SET_LE2(to, offs, arg)	toLE2((((FLByte *)(to)) + (offs)), arg)
#define FL_SET_LE4(to, offs, arg)	toLE4((((FLByte *)(to)) + (offs)), arg)

#endif /* FL_BIG_ENDIAN */

typedef LEulong LEmin;

/*************************************************/
/* Use routines instead of 'c' standard librarys */
/*************************************************/

#ifdef FL_VERIFY_WRITE
extern FLByte  flVerifyWrite[FL_SOCKETS][FL_MAX_TL_PARTITIONS<<1];
/* Verify write */
#define FL_UPS 2
/* 
 * See also FL_ON and FL_OFF
 */
#endif /* FL_VERIFY_WRITE */

/**********************************************/
/* Declare the TrueFFS environment variables  */
/**********************************************/

#ifndef FL_NO_USE_FUNC
extern FLDword  flBusConfig[FL_SOCKETS];
#else
/* The code bellow allows you to customize your own routine for multi bytes */
/*  transfers. Once FL_NO_USE_FUNC is defined, you can define the multi byte */
/* macros to these routines, and code them yourself in flsystem.c */
extern void FAR1 fl16bitDocReadNoShift (volatile  FLByte FAR0 * win, FLWord offset, FLByte FAR1 * dest, FLWord count );
extern void FAR1 fl16bitDocWriteNoShift ( volatile  FLByte FAR0 * win , FLWord offset , FLByte FAR1 * src, FLWord count );
#endif /* FL_NO_USE_FUNC */

#ifdef FL_ENVIRONMENT_VARS

extern void flSetEnvVar(void);

/**********************************************/
/* Declare the TrueFFS environment variables  */
/**********************************************/

extern FLByte   flUse8Bit;
extern FLByte   flDimageCfg;

#ifndef MTD_STANDALONE

extern FLByte   flUseMultiDoc;
extern FLByte   flMTLdefragMode;
extern FLByte   flMaxUnitChain;
#ifdef FL_TFFS_OLD_ENV_VARS 
extern FLByte   flMarkDeleteOnFlash;
extern FLByte   flPolicy[FL_SOCKETS][FL_MAX_TL_PARTITIONS];
#endif /* FL_TFFS_OLD_ENV_VARS */
extern FLWord   flCacheSize[FL_SOCKETS][FL_MAX_TL_PARTITIONS];
extern FLByte   flSkipStaticFilesWearLeveling[FL_SOCKETS][FL_MAX_TL_PARTITIONS];
extern FLByte   flSuspendMode[FL_SOCKETS];
extern FLDword  flTFFSBDDebugMode[FL_SOCKETS][FL_MAX_TL_PARTITIONS];
#ifdef FL_BD_AUTO_DPD_MODE
extern FLDword  flBDAutoDPDMode[FL_SOCKETS];
#endif /* FL_BD_AUTO_DPD_MODE */

#endif /* MTD_STANDALONE */

/* Policies definitions (FL_SET_POLICY) */
#define FL_DEFAULT_POLICY                               0
#define FL_COMPLETE_ASAP                                1
#define FL_SECURED_DELETE                               2
/* Mtl policies definitions (FL_MTL_POLICY) */
#define FL_MTL_DEFRAGMENT_ALL_DEVICES                   0
#define FL_MTL_DEFRAGMENT_SEQUANTIAL                    1
/* Suspend mode other then FL_OFF */
#define FL_SUSPEND_WRITE                                1
#define FL_SUSPEND_IO                                   3
#define FL_IGNORE_WRITE                                 4
#define FL_IGNORE_ERASE                                 8
#define FL_PREVENT_IO     (FL_IGNORE_ERASE | FL_IGNORE_WRITE | FL_SUSPEND_IO | FL_SUSPEND_WRITE)
/* TrueFFS Debug modes */
#define FL_QUICK_MOUNT_DO_NOT_READ    0x01 /* Do not use the quick mount when mounting */
#define FL_QUICK_MOUNT_DO_NOT_WRITE   0x02 /* Do not write the quick mount on dismount */
#define FL_QUICK_MOUNT_DISABLED       0x03 /* Mask for the above 2 flags               */
/* Runtime replacement for compilation flags */
#define FL_DIMAGE				                        0x01
#define FL_PUTIMAGE			      	                    0x02
#define FL_PUTIMAGE_PARALLEL	                        0x04
#define FL_DIMAGE_ALL			(FL_DIMAGE | FL_PUTIMAGE | FL_PUTIMAGE_PARALLEL)
/* Cache full media */
#define FL_MAX_ALLOWED_TL_CACHE_SIZE                    0xFFF


#ifndef FL_DEFAULT_POLICY_MODE
#define FL_DEFAULT_POLICY_MODE(socket,partition)        FL_OFF
#endif /* FL_DEFAULT_POLICY_MODE */
#ifndef FL_DEFAULT_BD_DPD_MODE
#define FL_DEFAULT_BD_DPD_MODE(socket)			        FL_OFF
#endif /* FL_DEFAULT_BD_DPD_MODE */
#ifndef FL_DEFAULT_BD_SUSPEND_MODE
#define FL_DEFAULT_BD_SUSPEND_MODE(socket)     			FL_OFF
#endif /* FL_DEFAULT_BD_SUSPEND_MODE */
#ifndef FL_DEFAULT_BD_DEBUG_MODE
#define FL_DEFAULT_BD_DEBUG_MODE(socket,partition)      FL_OFF
#endif /* FL_DEFAULT_BD_DEBUG_MODE */
/* Either per partition or for all (backward compatible) */
#ifndef FL_DEFAULT_CACHE_SIZE_FOR_MINIMAL_CACHE
#define FL_DEFAULT_CACHE_SIZE_FOR_MINIMAL_CACHE 0x40
#endif  /* FL_DEFAULT_CACHE_SIZE_FOR_MINIMAL_CACHE */
#ifndef FL_DEFAULT_TL_CACHE_SIZE
#define FL_DEFAULT_TL_CACHE_SIZE(socket,partition)      FL_DEFAULT_CACHE_SIZE_FOR_MINIMAL_CACHE
#endif  /* FL_DEFAULT_TL_CACHE_SIZE */
#ifndef FL_DEFAULT_WEARLEVELING_MODE
#define FL_DEFAULT_WEARLEVELING_MODE(socket,partition)  FL_OFF
#endif /* FL_DEFAULT_WEARLEVELING_MODE */
#ifndef FL_DEFAULT_VERIFY_WRITE_MODE
#define FL_DEFAULT_VERIFY_WRITE_MODE(socket,partition)  FL_OFF
#endif /* FL_DEFAULT_VERIFY_WRITE_MODE */
#ifndef FL_DEFAULT_TOOLS_MODE
#define FL_DEFAULT_TOOLS_MODE					        FL_OFF
#endif /* FL_DEFAULT_TOOLS_MODE */

#endif /* FL_ENVIRONMENT_VARS */

/* Set default to bus configuration (unless specified differently in flsystem.h */
#ifndef FL_DEFAULT_BUS_CONFIGURATION
#if (!defined(CHARSIZE) || (CHARSIZE != 16))
#define FL_DEFAULT_BUS_CONFIGURATION (FL_NO_ADDR_SHIFT | FL_BUS_HAS_8BIT_ACCESS  | \
                                      FL_BUS_HAS_16BIT_ACCESS | FL_BUS_HAS_32BIT_ACCESS)
#else
#define FL_DEFAULT_BUS_CONFIGURATION (FL_NO_ADDR_SHIFT | FL_BUS_HAS_16BIT_ACCESS)
#endif /* CHARSIZE */
#endif /* FL_DEFAULT_BUS_CONFIGURATION */

#ifndef FL_DEFAULT_BUS_MODE
#define FL_DEFAULT_BUS_MODE(socket) FL_DEFAULT_BUS_CONFIGURATION
#endif /* FL_DEFAULT_BUS_MODE */


typedef enum {        /* Variable type code for flSetEnv routine */
      FL_ENV_VARS_PER_SYSTEM           = 0,
      FL_IS_RAM_CHECK_ENABLED          = 1, /* No longer supported from 6.1 */
      FL_TL_CACHE_ENABLED              = 2, /* No longer supported from 6.1 */
      FL_DOC_8BIT_ACCESS               = 3,
      FL_MULTI_DOC_ENABLED             = 4, /* No longer supported from 6.1 */
      FL_SET_MAX_CHAIN                 = 5, /* For internal use only        */
      FL_MARK_DELETE_ON_FLASH          = 6, /* No longer supported from 6.1 */
      FL_MTL_POLICY                    = 7, /* No longer supported from 6.1 */
      FL_SECTORS_VERIFIED_PER_FOLDING  = 8, /* No longer supported from 6.1 */

      FL_ENV_VARS_PER_SOCKET           = 100,

      FL_VERIFY_WRITE_OTHER            = 101,
      FL_MTD_BUS_ACCESS_TYPE           = 102,
      FL_SUSPEND_MODE                  = 103,
	  FL_SET_AUTO_DPD_MODE             = 104, /* Added in TrueFFS 6.3.0     */

      FL_ENV_VARS_PER_VOLUME           = 200,

      FL_SET_POLICY                    = 201, /* No longer supported from 6.1 */
      FL_VERIFY_WRITE_BDTL             = 202,
      FL_VERIFY_WRITE_BINARY           = 203,
      FL_MAX_CACHE_SIZE                = 204,
      FL_DO_NOT_WAIT_FOR_READY         = 205, /* No longer supported from 6.1 */
      FL_NO_STATIC_FILES_WEAR_LEVELING = 206,
	  FL_DEBUG_MODES                   = 207, /* Added in TrueFFS 6.3 alpha 1 */
	  FL_HW_STATUS                     = 208, /* Added in TrueFFS 6.3 alpha 2 */
      
	  FL_ENV_VARS_PER_COMPILATION_FLAG = 300, /* Compilation flag replacement */

	  FL_DIMAGE_CFG                    = 301 
} FLEnvVars;
/* this enum is always defined, because TL structure has routine with such parameter;
because for binary compatibility structures should have all fields */

#endif /* FLBASE_H */


