
/*********************************************************************************** 
 *                                                                                 * 
 *                        M-Systems Confidential                                   * 
 *           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            * 
 *                         All Rights Reserved                                     * 
 *                                                                                 * 
 *********************************************************************************** 
 *                                                                                 * 
 *                            NOTICE OF M-SYSTEMS OEM                              * 
 *                           SOFTWARE LICENSE AGREEMENT                            * 
 *                                                                                 * 
 *      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 * 
 *      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           * 
 *      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              * 
 *      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               * 
 *      E-MAIL = info@m-sys.com                                                    * 
 *                                                                                 * 
 ***********************************************************************************/



/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/version 6.x/TrueFFS/examples/drivers/VxWorks/custom/flcustom.h-arc  $
 * 
 *    Rev 1.1   Aug 03 2004 20:22:54   andrayk
 * TrueFFS-6.2.2/vxWorks
 * 
 *    Rev 1.0   12 Aug 2003 01:00:54   andrayk
 * Initial revision.
 */



#ifndef FLCUSTOM_H
#define FLCUSTOM_H



/* 
 * Driver & TrueFFS version strings 
 */

#define driverVersion   "6.3.2.0"
#define OSAKVersion     "6.3.2.0"



/* Number of sockets
 *
 * Defines the maximum number of physical drives supported.
 *
 * The actual number of sockets depends on which socket controllers are
 * actually registered and the number of sockets in the system.
 */

#define FL_SOCKETS 4



/* Number of volumes
 *
 * Defines the maximum number of logical drives supported.
 *
 * The actual number of drives depends on which socket controllers are
 * actually registered , the amount of devices in the system and the 
 * TL format of each device.
 */

#define FL_VOLUMES  (6 * FL_SOCKETS)



/* Number of Binary partitions on the DiskOnChip.
 *
 * Defines Maximum Number of Binary partitions on the DiskOnChip.
 *
 * The actual number of partitions depends on the format placed
 * on each device.
 */

#define FL_BINARY_PARTITIONS (3 * FL_SOCKETS)



/* Maximum MTD's and Translation Layers
 *
 * Define here the maximum number of Memory Technology Drivers (MTD) and
 * Translation Layers (TL) that may be installed. Note that the actual
 * number installed is determined by which components are installed in
 * 'flRegisterComponents' (flcustom.c).
 */

#define FL_MTDS    5       /* MDOC512, H1T, H1H, G4, one spare */
#define FL_TLS     2       /* SAFTL, one spare */

/* Disk partition cache
 *
 * Enable Block Device Translation Layer cache.
 *
 * Turning on this option improves performance but requires additional
 * RAM resources.
 *
 * The NAND Flash Translation Layer (NFTL , INFTL and SAFTL) are specifications
 * for storing data on the DiskOnChip in a way that enables accessing the
 * DiskOnChip as a Virtual Block Device. If this option is on, then the BDTL 
 * keeps in RAM a table that saves some of the flash accesses.
 * Whenever it is needed to change table entry, the BDTL updates it in the
 * RAM table and on the DiskOnChip. If the TL has to read table entry, then you
 * can save time on reading sector from DiskOnChip.
 *
 * You can specify the number of units you would like to cache. The RAM required
 * for this operation is basically 2 bytes x the number of sectors per flash
 * unit. This size may change from one flash device to another.
 *
 * You can specify any number ranging from 1 to 0xFFF.
 */

#define FL_DEFAULT_TL_CACHE_SIZE(socket,partition) 64


/* TrueFFS Heap size.
 *
 * TrueFFS uses the FL_MALLOC and FL_FREE routines to allocate the memory it
 * requires. However if your native system does not have standard libraries,
 * for dynamic memory allocation you can use the TrueFFS implementation for
 * those routines called flMalloc and flFree.
 *
 * In such case you will need to define at compile time the heap size for
 * those routines using the following macro.
 */

/* #define  FL_TRUEFFS_HEAP_SIZE 200000 */ /* not used */ 


/* Flash Formatting (Block device format)
 *
 * Uncomment the following line if you need to prepare a virging DiskOnChip
 * for work or in case you want to re-partiton your existing device. 
 *
 * This routine compiles the following 3 API routines:
 *    flFlashFormat(), flEraseBD() and flUnformat()
 */

#define FL_FORMAT_VOLUME


/* Absolute read & write
 *
 * Uncomment the following line if you want to be able to read & write
 * sectors by absolute sector number (i.e. without regard to files and
 * directories).
 */

#define FL_ABS_READ_WRITE



/* Environment Variables
 *
 * Enable environment variables control of the TrueFFS features.
 *
 */

#define FL_ENVIRONMENT_VARS


/* IO Controll Interface
 *
 * Support standard IOCTL interface. Used only by some block device drivers.
 *
 */

#define FL_IOCTL_INTERFACE 



/* Binary Partition
 *
 * Enables access to the Binary partition.
 *
 */

#define BDK_ACCESS


/*
 * This compilation flag has diffrent meanings in TrueFFS SDK and Boot SDK.
 *
 * TrueFFS SDK - Removes the flReadBBT routine. 
 *
 * Boot SDK    - Removes the support for none SPL partition handling.
 */

#define NO_READ_BBT_CODE



/* H/W OTP
 *
 * Enable H/W One Time Programing capability including the unique ID feature.
 *
 */

#define HW_OTP



/* H/W Protection
 *
 * Enable H/W protection of the device.
 *
 */

#define HW_PROTECTION




/* Application exit
 *
 * If the FLite application ever exits, it needs to call flEXit before
 * exiting. Uncomment the following line to enable this.
 */

#define FL_EXIT


/* Protection agains power failures / Read after write
 *
 * Compiles the code for dealing with partially written pages caused by power
 * failures. The actual mode of operation is controlled by an environment
 * variable FL_VERIFY_WRITE_BDTL. When ENVIRONMENT_VARS is undefined this 
 * compilation flag will result in read after every write.
 */

#define FL_VERIFY_WRITE



/* Verify entire volume
 *
 * Compile the flVerifyVolume routine - this routine scans the entire disk
 * partition for power failures symptoms and correct them.
 *
 */

#define VERIFY_VOLUME



/* Compile the extended Disk information routine
 *
 * TrueFFS 6.0 and up provide an extended information routine for internal
 * M-Systems utilities named flGetExtendedDiskInfo(). 
 * To compile its code define
 */

#define FL_EXTENDED_DISK_INFO



/* Fixed or removable media
 *
 * If your Flash media is fixed, uncomment the following line.
 */

#define FL_FIXED_MEDIA


/* Compile the auto DPD code
 *
 * TrueFFS supports a mode, where TrueFFS automatically enters and exit DPD
 * after every operation. 
 *
 * The compilation below compiles that code. Note that it's operation is 
 * also under the affect of the dedicated environment variable.
 */

#define FL_BD_AUTO_DPD_MODE

#define FS_CREATE_LOGICAL_PARTITIONS
#define FS_FORMAT_VOLUME


#endif /* FLCUSTOM_H */


