/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
* $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/flioctl.h-arc  $
 * 
 *    Rev 1.4   Sep 12 2005 13:11:36   roneng
 * Add MCOTP IOCTL structs
 * 
 *    Rev 1.3   Jun 07 2005 06:19:40   OriS
 * - Esthetics - Removed call to _fsioctl.h as it no longer exists.
 * 
 *    Rev 1.2   Apr 19 2005 02:57:00   oris
 * - Changed ioctl numbers that are not longer supported to: 
 *   FL_IOCTL_NO_LONGER_SUPPORTED_x
 * - Added:
 *   FL_IOCTL_FLASH_FORMAT   	- Replacing FL_IOCTL_FORMAT_PHYSICAL_DRIVE
 *   FL_IOCTL_ERASE_BD       	- Added in TrueFFS 6.3.0
 *   FL_IOCTL_FLASH_UNFORMAT 	- Added in TrueFFS 6.3.0
 * - And their structures:
 *   flFormatFlashInput
 *   flEraseBDInput
 *   flFlashUnformatInput
 * - Changed   FL_IOCTL_WRITE_IPL enum number and its input record from
 *   flIplInput -> flIPLInput
 * - Added PROTECTION_APPLY_STICKY flag to FL_IOCTL_BINARY_HW_PROTECTION ioctl
* 
*    Rev 1.1   Apr 04 2005 20:18:10   oris
* - Compilation errors with different compilation flags:
*   Binary partition support and file system ioctls.
* 
*    Rev 1.0   Mar 27 2005 09:27:28   oris
* Initial revision.
* 
*    Rev 1.2   Mar 27 2005 09:21:36   oris
* Cosmetics
* 
*    Rev 1.1   Dec 23 2004 19:35:46   oris
* Drop 2.1
* 
*    Rev 1.0   Dec 09 2004 18:23:44   oris
* Initial revision.
* 
*    Rev 1.3   Dec 09 2003 18:44:40   oris
* Fixed ARM Compiler error.  
* 
*    Rev 1.2   May 11 2003 08:27:48   OriS
* Added support for the 3 quick mount routine (get, discard and write).
* 			 FL_IOCTL_IS_QUICK_MOUNT_VALID
* 			 FL_IOCTL_WRITE_QUICK_MOUNT_INFO
* 			 FL_IOCTL_CLEAR_QUICK_MOUNT_INFO
* 
*    Rev 1.1   May 08 2003 23:16:06   OriS
* Added FL_IOCTL_IS_VOLUME_RUGGEDIZED ioctl.
* 
*    Rev 1.0   Apr 09 2003 12:16:00   OriS
* Initial revision.
* 
*/

#ifndef FLIOCTL_H
#define FLIOCTL_H

#include "flbase.h"
#include "dosformt.h"
#ifdef BDK_ACCESS
#include "docbdk.h"
#endif /* BDK_ACCESS */

#include "_flioctl.h"

#if defined(FL_IOCTL_INTERFACE) || defined(FS_IOCTL_INTERFACE)

/* In every call to flIOctl function, the irFlags field in the structure
IOreq should hold one of the following: */
typedef enum{FL_IOCTL_GET_INFO = FL_IOCTL_START,
  FL_IOCTL_DEFRAGMENT,
  FL_IOCTL_NO_LONGER_SUPPORTED_0,	/* FL_IOCTL_WRITE_PROTECT             */
  FL_IOCTL_MOUNT_VOLUME,
  FL_IOCTL_NO_LONGER_SUPPORTED_1,	/* FL_IOCTL_FORMAT_VOLUME             */
  FL_IOCTL_BDK_OPERATION,
  FL_IOCTL_DELETE_SECTORS,
  FL_IOCTL_READ_SECTORS,
  FL_IOCTL_WRITE_SECTORS,
  FL_IOCTL_NO_LONGER_SUPPORTED_2,	/* FL_IOCTL_FORMAT_PHYSICAL_DRIVE     */
  FL_IOCTL_NO_LONGER_SUPPORTED_3,	/* FL_IOCTL_FORMAT_LOGICAL_DRIVE      */
  FL_IOCTL_BDTL_HW_PROTECTION,
  FL_IOCTL_BINARY_HW_PROTECTION,
  FL_IOCTL_OTP,
  FL_IOCTL_CUSTOMER_ID,
  FL_IOCTL_UNIQUE_ID,
  FL_IOCTL_NUMBER_OF_PARTITIONS,
  FL_IOCTL_NO_LONGER_SUPPORTED_4,	/* FL_IOCTL_INQUIRE_CAPABILITIES      */ 
  FL_IOCTL_NO_LONGER_SUPPORTED_5,	/* FL_IOCTL_SET_ENVIRONMENT_VARIABLES */
  FL_IOCTL_NO_LONGER_SUPPORTED_6,	/* FL_IOCTL_PLACE_EXB_BY_BUFFER       */
  FL_IOCTL_NO_LONGER_SUPPORTED_7,	/* FL_IOCTL_WRITE_IPL                 */
  FL_IOCTL_DEEP_POWER_DOWN_MODE,
  FL_IOCTL_EXTENDED_ENVIRONMENT_VARIABLES, 
  FL_IOCTL_VERIFY_VOLUME,	 
  FL_IOCTL_SET_ACCESS_ROUTINE,
  FL_IOCTL_GET_ACCESS_ROUTINE,
  FL_IOCTL_NO_LONGER_SUPPORTED_8,	/* FL_IOCTL_EXTENDED_WRITE_IPL        */  
  FL_IOCTL_NO_LONGER_SUPPORTED_9,	/* FL_IOCTL_OPEN_TRANSACTION          */
  FL_IOCTL_NO_LONGER_SUPPORTED_10,	/* FL_IOCTL_COMMIT_TRANSACTION        */
  FL_IOCTL_NO_LONGER_SUPPORTED_11,	/* FL_IOCTL_IS_VOLUME_RUGGEDIZED      */
  FL_IOCTL_IS_QUICK_MOUNT_VALID,      
  FL_IOCTL_WRITE_QUICK_MOUNT_INFO,
  FL_IOCTL_CLEAR_QUICK_MOUNT_INFO,
  FL_IOCTL_NO_LONGER_SUPPORTED_12,	/* FL_IOCTL_COMPLETE_OPERATION        */
  FL_IOCTL_FLASH_FORMAT, /* Replacing FL_IOCTL_FORMAT_PHYSICAL_DRIVE       */
  FL_IOCTL_ERASE_BD,                  /* Added in TrueFFS 6.3.0            */
  FL_IOCTL_FLASH_UNFORMAT,            /* Added in TrueFFS 6.3.0            */
  FL_IOCTL_WRITE_IPL,                 /* Changed in TrueFFS 6.3.0          */
  FL_IOCTL_MCOTP
} flIOctlFunctionNo;


/* In every call to flIOctl function, the irData field in the structure
IOreq should point to the structure defined below. The fields
inputRecord and outputRecord should point to structures which are
specific to each IOctl function as defined in this file. */
typedef struct {
	void FAR1 *inputRecord;
	void FAR1 *outputRecord;
} flIOctlRecord;


/* General output record that returns only status. */
typedef struct {
	FLStatus status;
} flOutputStatusRecord;



/* Input and output records for the different IOCTL functions: */
/* =========================================================== */

#ifdef BLOCKDEV_H

/* Get disk information (FL_IOCTL_GET_INFO) */
/* Input record: NULL */
/* Output record: */
typedef struct {
	VolumeInfoRecord info;  /* VolumeInfoRecord is defined in blockdev.h */
	FLStatus status;
} flDiskInfoOutput;
/* Output record: flOutputStatusRecord */
#endif
/*************************************************************************/
/* Defragment volume (FL_IOCTL_DEFRAGMENT) */
/* Input record: */
typedef struct {
	FLSDword requiredNoOfSectors;   /* Minimum number of sectors to make */
	/* available.                        */ 
} flDefragInput;
/* Outout record: */
typedef struct {
	FLSDword actualNoOfSectors;     /* Actual number of sectors available */
	FLStatus status;
} flDefragOutput;

/*************************************************************************/
/* Mount volume (FL_IOCTL_MOUNT_VOLUME) */
/* Input record: */
typedef struct {
	FLByte  type;        /*  type of operation: FL_MOUNT\FL_DISMOUNT */
} flMountInput;
#define FL_MOUNT          0
#define FL_DISMOUNT        1
/* Output record: flOutputStatusRecord */
/*************************************************************************/
#ifdef FL_FORMAT_VOLUME
/* Format flash device (FL_IOCTL_FLASH_FORMAT) */
/* Input record: */
typedef struct {
	FLDword dwFormatFlags;  /* type of format as defined in blockdev.h             */
	FLSDword sdwNoOfKeeps;
	FormatParams3 * fpPtr;     /* Format parameters structure (defined in flformat.h) */
} flFlashFormatInput;
/* Output record: flOutputStatusRecord */
/*************************************************************************/
/* Format flash device (FL_IOCTL_ERASE_BD) */
/* Input record: */
typedef struct {
	FLDword dwFlags;  /* Reserved - Must be set to 0 */
} flEraseBDInput;
/* Output record: flOutputStatusRecord */
/*************************************************************************/
/* Format flash device (FL_IOCTL_FLASH_UNFORMAT) */
/* Input record: */
typedef struct {
	FLDword dwFlags;  /* Reserved - Must be set to 0 */
} flFlashUnformatInput;
/* Output record: flOutputStatusRecord */
#endif /* FL_FORMAT_VOLUME */
/*************************************************************************/
#ifdef BDK_ACCESS
#include "docbdk.h"
/* BDK operations read\write\erase\create (FL_IOCTL_BDK_OPERATION) */
/* Input record: */
typedef struct {
	FLByte  type;  /* type of operation: BDK_INIT_READ\BDK_READ\BDK_INIT_WRITE\ */
	/* BDK_WRITE\BDK_ERASE\BDK_CREATE\BDK_GET_INFO               */
	BDKStruct bdkStruct; /* parameters for BDK operations (defined in docbdk.h)       */
} flBDKOperationInput;
#define BDK_INIT_READ   0
#define BDK_READ        1
#define BDK_INIT_WRITE  2
#define BDK_WRITE       3
#define BDK_ERASE       4
#define BDK_CREATE      5
#define BDK_GET_INFO    6
/* Output record: flOutputStatusRecord */
#endif                                  /* BDK_ACCESS  */
/*************************************************************************/
#ifdef HW_PROTECTION
/* BDK and BDTL protection operations: (FL_IOCTL_BINARY_HW_PROTECTION) */
/*                                     (FL_IOCTL_BDTL_HW_PROTECTION)   */
/*   insert key \ remove key \ identify \ change key \                 */
/*   change protection type \ change lock status                       */
/* Input record: */
typedef struct {
	FLByte  protectionType;    /* see flflash.h for the protection attributes */
	FLByte  key[8];            /* The new key to the change Key call          */
	FLByte  type;              /* Operation type see list bellow              */
} flProtectionInput;
#define PROTECTION_INSERT_KEY   0
#define PROTECTION_REMOVE_KEY   1
#define PROTECTION_GET_TYPE     2
#define PROTECTION_DISABLE_LOCK 3
#define PROTECTION_ENABLE_LOCK  4
#define PROTECTION_CHANGE_KEY   5
#define PROTECTION_CHANGE_TYPE  6
#define PROTECTION_APPLY_STICKY 7
/* Output record: */
typedef struct {
	FLByte  protectionType;    /* see flflash.h for the protection attributes */
	FLStatus status;
} flProtectionOutput;
#endif /* HW_PROTECTION */
/*************************************************************************/
#ifdef HW_OTP
/* One Time Programing operations: (FL_IOCTL_OTP */
/*   OTP size \ OTP read \ OTP write and lock    */
/* Input record: */
typedef struct {
	FLDword       length;           /* Length to read\write\size                  */
	FLDword       usedSize;         /* The written size of the area \ Area offset */
	FLByte         lockedFlag;       /* The area condition LOCKED_OTP (flflash.h)  */
	FLByte         FAR1* buffer;     /* pointer to user buffer                     */
	FLWord        type;             /* defined bellow                             */
} flOtpInput;
typedef struct {
	FLWord        wPageNo ;         /* Page to write                              */
	FLDword       length;           /* Length to read\write\size                  */
	FLDword       usedSize;         /* The written size of the area \ Area offset */
	FLByte        lockedFlag;       /* The area condition LOCKED_OTP (flflash.h)  */
	FLByte        FAR1* buffer;     /* pointer to user buffer                     */
	FLWord        type;             /* defined bellow                             */
} flMCOtpInput;


#define OTP_SIZE        1
#define OTP_READ        2
#define OTP_WRITE_LOCK        3

#define MCOTP_SIZE        1
#define MCOTP_READ        2
#define MCOTP_WRITE_LOCK        3
/* Output record: flOutputStatusRecord */
/*************************************************************************/
/* Read customer ID (FL_IOCTL_CUSTOMER_ID) */
/* Input record: NULL */
/* Output record: */
typedef struct {
	FLByte  id[4];
	FLStatus status;
} flCustomerIdOutput;
/*************************************************************************/
/* Read unique ID (FL_IOCTL_UNIQUE_ID) */
/* Input record: NULL */
/* Output record: */
typedef struct {
	FLByte  id[16];
	FLStatus status;
} flUniqueIdOutput;
#endif /* HW_OTP */
/*************************************************************************/
/* Read unique ID (FL_IOCTL_NUMBER_OF_PARTITIONS) */
/* Input record: NULL */
/* Output record: */
typedef struct {
	FLByte  noOfPartitions;
	FLStatus status;
} flCountPartitionsOutput;
/*************************************************************************/
#ifdef FL_LOW_LEVEL
/* Output record: flOutputStatusRecord */
/*************************************************************************/
/* Place the device into and out of the power down mode (FL_IOCTL_DEEP_POWER_DOWN_MODE) */
typedef struct {
	FLByte  state; /* DEEP_POWER_DOWN - low power consumption      */
	/* otherwise       - regular power consumption  */
} flPowerDownInput;
/* DEEP_POWER_DOWN is defined in flflash.h */
#endif /* FL_LOW_LEVEL */
/* Output record: flOutputStatusRecord */
/*************************************************************************/
#if defined(FL_ABS_READ_WRITE) || defined(FS_ABS_READ_WRITE)
/* Delete logical sectors (FL_IOCTL_DELETE_SECTORS) */
/* Input record: */
typedef struct {
	FLSDword firstSector;                /* First logical sector to delete */
	FLSDword numberOfSectors;                /* Number of sectors to delete */
} flDeleteSectorsInput;
/* Output record: flOutputStatusRecord */
/*************************************************************************/
/* read & write logical sectors (FL_IOCTL_READ_SECTORS & FL_IOCTL_WRITE_SECTORS) */
/* Input record: */
typedef struct {
	FLSDword firstSector;                  /* first logical sector */
	FLSDword numberOfSectors;         /* Number of sectors to read\write */
	FLByte  FAR1 *buf;               /* Data to read\write */
} flReadWriteInput;
/* Output record: */
typedef struct {
	FLSDword numberOfSectors;         /* Actual Number of sectors read\written */
	FLStatus status;
} flReadWriteOutput;
#endif /* FL_ABS_READ_WRITE */
/******************************************************************************/
/* Set environment variables values (FL_IOCTL_EXTENDED_ENVIRONMENT_VARIABLES) */
#ifdef FL_ENVIRONMENT_VARS
/* Input record:  */
typedef struct {
	FLEnvVars      varName;  /* Enum describing the variable */
	FLDword          varValue; /* New variable value           */
	FLDword          flags;    /* FL_APPLY_TO_ALL    - All socket and partitions */
	/* FL_APPLY_TO_SOCKET - All socket and partitions */
	/* FL_APPLY_TO_VOLUME - All socket and partitions */
} flExtendedEnvVarsInput;
#define FL_APPLY_TO_ALL    1
#define FL_APPLY_TO_SOCKET 2
#define FL_APPLY_TO_VOLUME 3
/* Output record: */
typedef struct {
	FLDword    prevValue;   /* The previous value of the variable */
	FLStatus status;
} flExtendedEnvVarsOutput;
#endif /* FL_ENVIRONMENT_VARS */
/******************************************************************************/
/* Check partition for power failures symptoms (FL_IOCTL_VERIFY_VOLUME) */
#ifdef VERIFY_VOLUME
/* Input record: NULL */
typedef struct {
	FLDword flags;   /* Must be set to 0 */    
} flVerifyVolumeInput;
/* Output record: */
typedef struct {
	void FAR1* callBack;   /* Must be set to null */  
	FLStatus status;
} flVerifyVolumeOutput;
#endif /* VERIFY_VOLUME */
/******************************************************************************/
#ifdef BLOCKDEV_H

#ifndef FL_NO_USE_FUNC
/* Plant user defined memory access routines (FL_IOCTL_SET_ACCESS_ROUTINE) */
/* Input record:  */
typedef struct {
	FLAccessStruct FAR1 * structPtr; /* Pointer memory access routine struct */
} flSetMemoryAccessInput;
/* Output record: flOutputStatusRecord */
#endif /* FL_NO_USE_FUNC */
/******************************************************************************/
/* Plant user defined memory access routines (FL_IOCTL_GET_ACCESS_ROUTINE) */
/* Input record: NULL */
#ifndef FL_NO_USE_FUNC
typedef struct {
	FLAccessStruct FAR1 * structPtr; /* Pointer memory access routine struct */
	FLStatus              status;
} flGetMemoryAccessOutput;
#endif /* FL_NO_USE_FUNC */

#endif
/******************************************************************************/
#ifndef NO_IPL_CODE
/* Write IPL area for docPlus family (FL_IOCTL_WRITE_IPL) */
/* Input record:  */
typedef struct {
	FLByte   FAR1* bBufPtr; /* IPL data buffer           */
	FLDword  dwFlags;       /* IPL flags (see flflash.h) */
	FLSDword sdwLength;     /* IPL length                */
	FLSDword sdwOffset;     /* IPL offset                */
} flIPLInput;
/* Output record: flOutputStatusRecord */
#endif /* NO_IPL_CODE */
/******************************************************************************/
/* Check quick mount validity (FL_IOCTL_IS_QUICK_MOUNT_VALID) */
#ifndef FL_NO_QUICK_MOUNT_FEATURE
/* Input record: NULL */
typedef struct {
	FLByte                mode; /* Either FL_ON for valid and FL_OFF for not */
	FLStatus              status;
} flGetQuickMountOutput;
/******************************************************************************/
/* Write quick mount data (FL_IOCTL_WRITE_QUICK_MOUNT_INFO) */
/* Input record: NULL */
/* Output record: flOutputStatusRecord */
/******************************************************************************/
/* Clear quick mount data (FL_IOCTL_CLEAR_QUICK_MOUNT_INFO) */
/* Input record: NULL */
/* Output record: flOutputStatusRecord */
#endif /* FL_NO_QUICK_MOUNT_FEATURE */
#endif /* IOCTL_INTERFACE */
#endif /* FLIOCTL_H */
