/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/version 6.3-up/TrueFFS/src/flstdcmp.h-arc  $
 * 
 *    Rev 1.8   Sep 12 2005 13:13:50   roneng
 * Change G4 registration routines name
 * 
 *    Rev 1.7   Aug 14 2005 14:58:44   roneng
 * Add G4 registration prototypes
 * 
 *    Rev 1.6   Jul 26 2005 20:19:10   roneng
 * Casting bug fix
 * 
 *    Rev 1.5   Jul 19 2005 18:57:26   roneng
 * Support hooking to H1H and H1T MTD 
 * 
 *    Rev 1.4   Jul 18 2005 19:15:46   oris
 * - Preparation for additional H1 DiskOnChip devices.
 * 
 *    Rev 1.3   Jul 06 2005 23:39:44   OriS
 * Prepare for H1 Second source support.
 * 
 *    Rev 1.2   Jun 09 2005 13:39:58   oris
 * Added H1S (H1 - second source) socket registration routine. Please note that while both MTD's needs to be registered, only one socket routine should be called.
 * 
 *    Rev 1.1   Jun 07 2005 06:20:56   OriS
 * - Added flRegisterDOCH1S(void) registration routine for H1 second source.
 * 
 *    Rev 1.0   Apr 20 2005 16:20:30   oris
 * Initial revision.
 */

#ifndef FLSTDCOMP_H
#define FLSTDCOMP_H

#include "flbase.h"

/************************************************************************/
/* Registration routines for MTDs supplied with TrueFFS			*/
/************************************************************************/

/* Currently supported     */
FLStatus    flRegisterDOCH1T(void);                   /* see H1_LGC.C   */
FLStatus    flRegisterDOCH1H(void);                   /* see H1H_LGC.C  */
FLStatus    flRegisterDOCM512(void);                  /* see N512_LGC.C */
FLStatus    flRegisterDOCG4(void);                   /* see M1G_LGC.C */
FLStatus    flRegisterDOCRAM( void );                 /* see RAMMTD.C   */

/* Not currently supported */
FLStatus    flRegisterI28F008(void);                  /* see I28F008.C  */
FLStatus    flRegisterI28F016(void);                  /* see I28F016.C  */
FLStatus    flRegisterAMDMTD(void);                   /* see AMDMTD.C   */
FLStatus    flRegisterWAMDMTD(void);                  /* see WAMDMTD.C  */
FLStatus    flRegisterCDSN(void);                     /* see NFDC2048.C */
FLStatus    flRegisterCFISCS(void);                   /* see CFISCS.C   */
FLStatus    flRegisterDOC2000(void); 	              /* see DISKONC.C  */
FLStatus    flRegisterDOCPLUS(void); 	              /* see MDOCPLUS.C */
FLStatus    flRegisterDOCH1S(void);					  /* see H1S_LGC.c  */


#if ((!defined(FL_NO_H1T_SUPPORT)) && (!defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1() ((FLStatus)(flRegisterDOCH1T() | flRegisterDOCH1H()))
#elif ((!defined(FL_NO_H1T_SUPPORT)) && (defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1() (flRegisterDOCH1T()) 
#elif ((defined(FL_NO_H1T_SUPPORT)) && (!defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1() (flRegisterDOCH1H()) 
#endif
/************************************************************************/
/* Registration routines for socket I/F supplied with TrueFFS		*/
/************************************************************************/

/* Currently supported     */
#if ((!defined(FL_NO_H1T_SUPPORT)) && (!defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1SOC  flRegisterDOCH1TSOC 
#elif ((!defined(FL_NO_H1T_SUPPORT)) && (defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1SOC flRegisterDOCH1TSOC 
#elif ((defined(FL_NO_H1T_SUPPORT)) && (!defined(FL_NO_H1H_SUPPORT)))
#define flRegisterDOCH1SOC flRegisterDOCH1HSOC
#endif

FLStatus    flRegisterDOCH1TSOC(FLDword dwLowAddress, 
							   FLDword dwHighAddress);   /* see H1_MTD.C   */
FLStatus    flRegisterDOCH1HSOC(FLDword dwLowAddress, 
							   FLDword dwHighAddress);   /* see H1H_MTD.C   */
FLStatus    flRegisterDOCH1SSOC(FLDword dwLowAddress,    
							   FLDword dwHighAddress);   /* see H1S_MTD.C  */
FLStatus    flRegisterDOCM512SOC(FLDword dwLowAddress, 
								 FLDword dwHighAddress); /* see M512_MTD.C */
FLStatus    flRegisterDOCG4SOC(FLDword dwLowAddress, 
								 FLDword dwHighAddress); /* see M1G_MTD.C */
FLStatus    flRegisterDOCRamSOC(void);                   /* see RAMSOC.C   */

/* Not currently supported */

FLStatus    flRegisterPCIC(unsigned int,                 /* see PCIC.C     */
						   unsigned int, FLByte);						      
FLStatus    flRegisterElanPCIC(unsigned int,             /* see PCICELAN.C */
							   unsigned int, FLByte);						      
FLStatus    flRegisterLFDC(FLBoolean);                   /* see LFDC.C     */

FLStatus    flRegisterElanRFASocket(int,                 /* see ELRFASOC.C */ 
									int);           
FLStatus    flRegisterElanDocSocket(FLSDword,            /* see ELDOCSOC.C */ 
									FLSDword, int); 
FLStatus    flRegisterVME177rfaSocket(FLDword,           /* see FLVME177.C */
									  FLDword); 
FLStatus    flRegisterCobuxSocket(void);                 /* see COBUXSOC.C */
FLStatus    flRegisterCEDOCSOC(void);                    /* see CEDOCSOC.C */
FLStatus    flRegisterCS(void);                          /* see CSwinCE.C */
FLStatus    flRegisterDOCSOC(FLDword, FLDword);          /* see DOCSOC.C */
FLStatus    flRegisterDOCPLUSSOC(FLDword, FLDword);      /* see DOCSOC.C */

/************************************************************************/
/* Registration routines for translation layers supplied with TrueFFS	*/
/************************************************************************/

/* Currently supported     */

FLStatus    flRegisterSAFTL(void);                    /* see SAFTL.C    */

/* Not currently supported */

FLStatus    flRegisterFTL(void);                      /* see FTLLITE.C  */
FLStatus    flRegisterNFTL(void);                     /* see NFTLLITE.C */
FLStatus    flRegisterINFTL(void);                    /* see INFTL.C    */
FLStatus    flRegisterSSFDC(void);                    /* see SSFDC.C    */
FLStatus    flRegisterATAtl(void);                    /* see ATAL.C     */
FLStatus    flRegisterZIP(void);		              /* see ZIP.C	    */
FLStatus    flRegisterCACHETL(void);                  /* see CACHETL.C  */

/************************************************************************/
/* Multi-TL also known as Multi-DOC: Combine different devices into a   */
/* single big device allowing each of the devices to be formatted with  */
/* any kind of the TL mentioned above                                   */
/************************************************************************/

FLStatus    flRegisterMTL(void);                      /* see FLMTL.C    */

/************************************************************************/
/* Component registration routine in FLCUSTOM.C                         */
/************************************************************************/

FLStatus    flRegisterComponents(void);

#endif /* FLSTDCOMP_H */
