/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/flsysfun.h-arc  $
 * 
 *    Rev 1.3   Apr 21 2005 15:22:18   oris
 * Removed flmalloc.h
 * 
 *    Rev 1.2   Apr 19 2005 02:57:44   oris
 * - Removed flInportb, flOutportb and flSwapBytes routines.
 * - Include flmalloc.h instead of flbase.h
 * 
 *    Rev 1.1   Apr 04 2005 20:18:20   oris
 * - Removed flDelayMsecs routine prototype. It was replaced with FL_DELAY_MSEC
 * 
 *    Rev 1.0   Mar 27 2005 09:27:28   oris
 * Initial revision.
 * 
 *    Rev 1.0   Dec 09 2004 18:23:44   oris
 * Initial revision.
 * 
 *    Rev 1.4   Oct 23 2003 13:57:14   oris
 * Moved flsleep prototype to flbase.h to allow the use of a new FL_SLEEP macro.
 * 
 *    Rev 1.3   May 18 2003 15:32:18   OriS
 * Fixed spaces
 * 
 *    Rev 1.2   May 16 2003 00:59:56   OriS
 * Changed spaces
 * 
 *    Rev 1.1   May 11 2003 08:27:58   OriS
 * Removed flAddLongToFarPointer prototype.
 * 
 *    Rev 1.0   Apr 09 2003 12:16:00   OriS
 * Initial revision.
 * 
 */

#ifndef FLSYSFUN_H
#define FLSYSFUN_H


/*----------------------------------------------------------------------*/
/*                       f l S y s f u n I n i t                        */
/*                                                                      */
/* Do any necessary initialization for routines in this module.         */
/*                                                                      */
/* Called from fatlite.c (flInit)                                       */
/*                                                                      */
/* Parameters:                                                          */
/*      None                                                            */
/*                                                                      */
/*----------------------------------------------------------------------*/

extern void flSysfunInit(void);


/*----------------------------------------------------------------------*/
/*                         f l R a n d B y t e                          */
/*                                                                      */
/* Returns a random number between 0 and 255                            */
/*                                                                      */
/* Called from FTLLITE.C                                                */
/*                                                                      */
/* Parameters:                                                          */
/*      None                                                            */
/*                                                                      */
/* Returns:                                                             */
/*        A random number between 0 and 255                             */
/*----------------------------------------------------------------------*/

extern unsigned flRandByte(void);


/*----------------------------------------------------------------------*/
/*                      f l I n s t a l l T i m e r                     */
/*                                                                      */
/* Installs an interval timer.                                          */
/* The implementation of this routine usually means hooking a clock     */
/* interrupt. The polling interval is specified as a parameter. If the  */
/* clock frequency is faster, the interval timer should count several   */
/* clock ticks before calling the interval routine.                     */
/*                                                                      */
/* This routine is necessary if POLLING_INTERVAL (custom.h) is greater  */
/* than 0. In this case this routine will be called from socket.c       */
/* (init). It will be called to install 'socketIntervalRoutine' with a  */
/* period specified by POLLING_INTERVAL.                                */
/*                                                                      */
/* Parameters:                                                          */
/*      routine                : Routine to call at each interval       */
/*        interval        : Milliseconds per polling interval           */
/*                                                                      */
/* Returns:                                                             */
/*         FLStatus        : 0 on success, otherwise failure            */
/*----------------------------------------------------------------------*/

extern FLStatus flInstallTimer(void (*routine)(void), unsigned interval);


#ifdef FL_EXIT

/*----------------------------------------------------------------------*/
/*                      f l R e m o v e T i m e r                       */
/*                                                                      */
/* Removes the active interval timer.                                   */
/* This routine removes the active interval timer set by 'removeTimer'. */
/*                                                                      */
/* Parameters:                                                          */
/*      None                                                            */
/*                                                                      */
/*----------------------------------------------------------------------*/

extern void flRemoveTimer(void);

#endif


/*----------------------------------------------------------------------*/
/*                       f l C u r r e n t D a t e                      */
/*                                                                      */
/* Returns the current DOS-format date                                  */
/*                                                                      */
/* The DOS date format is documented in dosformt.h.                     */
/*                                                                      */
/* If a TOD clock is not available, return the value of 1/1/80.         */
/*                                                                      */
/* Parameters:                                                          */
/*      None                                                            */
/*                                                                      */
/* Returns:                                                             */
/*        Current date                                                  */
/*----------------------------------------------------------------------*/

extern unsigned flCurrentDate(void);


/*----------------------------------------------------------------------*/
/*                      f l C u r r e n t T i m e                       */
/*                                                                      */
/* Returns the current DOS-format time                                  */
/*                                                                      */
/* The DOS time format is documented in dosformt.h.                     */
/*                                                                      */
/* If a TOD clock is not available, return the value of 0:00 AM.        */
/*                                                                      */
/* Parameters:                                                          */
/*      None                                                            */
/*                                                                      */
/* Returns:                                                             */
/*        Current time                                                  */
/*----------------------------------------------------------------------*/

extern unsigned flCurrentTime(void);


/*----------------------------------------------------------------------*/
/*                      f l C r e a t e M u t e x                       */
/*                                                                      */
/* Creates and initializes a mutex object                               */
/*                                                                      */
/* The mutex is initializes as not owned by anyone.                     */
/*                                                                      */
/* Parameters:                                                          */
/*      mutex                : Pointer to mutex object                  */
/*                                                                      */
/* Returns:                                                             */
/*         FLStatus        : 0 on success, otherwise failure            */
/*----------------------------------------------------------------------*/

extern FLStatus flCreateMutex(FLMutex *mutex);


/*----------------------------------------------------------------------*/
/*                      f l D e l e t e M u t e x                       */
/*                                                                      */
/* Destroys a mutex object                                              */
/*                                                                      */
/* This function frees any resources taken by flCreateMutex.            */
/*                                                                      */
/* Parameters:                                                          */
/*      mutex                : Pointer to mutex object                  */
/*                                                                      */
/* Returns:                                                             */
/*         None                                                         */
/*----------------------------------------------------------------------*/

extern void flDeleteMutex(FLMutex *mutex);


/*----------------------------------------------------------------------*/
/*                        f l T a k e M u t e x                         */
/*                                                                      */
/* Attempts to take ownership of a mutex. If the mutex is currently not */
/* owned, TRUE is returned and the mutex becomes owned. If the mutex is */
/* currently owned, FALSE is returned and ownership is not taken.       */
/*                                                                      */
/* Parameters:                                                          */
/*      mutex                : Pointer to mutex object                  */
/*                                                                      */
/* Returns:                                                             */
/*         FLBoolean        : TRUE if ownership taken, FALSE otherwise  */
/*----------------------------------------------------------------------*/

extern FLBoolean flTakeMutex(FLMutex *mutex);


/*----------------------------------------------------------------------*/
/*                        f l F r e e M u t e x                         */
/*                                                                      */
/* Frees ownership of a mutex                                           */
/*                                                                      */
/* Parameters:                                                          */
/*      mutex                : Pointer to mutex object                  */
/*                                                                      */
/* Returns:                                                             */
/*         None                                                         */
/*----------------------------------------------------------------------*/

extern void flFreeMutex(FLMutex *mutex);


#endif /* FLSYSFUN_H */
