/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/common/fltl.h-arc  $
 * 
 *    Rev 1.2   Jun 07 2005 06:21:58   OriS
 * - Added the following fields to extended info routine:
 *   o dwType             - Flash type
 *   o dwSubType       - Flash Sub type
 *   o bMediaType       - Controller family
 * 
 * 
 *    Rev 1.1   Apr 19 2005 02:58:08   oris
 * - Cosmetics.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:26   oris
 * Initial revision.
 * 
 */

#ifndef FLTL_H
#define FLTL_H

#include "flfuncno.h"
#include "flreq.h"

#define FL_BDK_PARTITION  1
#define FL_DISK_PARTITION 0
#define FL_ALL_FLOORS     0xff

/*----------------------------------------------------------------------*/
/*                  f l E x t e n d e d D i s k I n f o                 */
/*                                                                      */
/* This structure supplies very low-level information on the device     */
/*                                                                      */
/* Note: This structure is used for M-Systems low level utilities. It   */
/*       be subject to chages from one TrueFFS version to the other.    */
/*                                                                      */
/*----------------------------------------------------------------------*/

typedef struct {

  /******************************************/
  /* Fields that belong to the entire media */
  /******************************************/
  FLDword  dwStructSize;          /* INPUT - Size of the given structure      */
  FLDword  dwTLType;              /* See definitions in flflash.h             */
  FLDword  dwFormatFlags;         /* Format attributes for the entire media   */
  FLDword  dwTrueFFSVersion;      /* Version of OSAK used to format the media */
  FLDword  dwFlashTechnology;     /* Flash technology supported properties    */  

  FLDword  dwType;                /* Flash type                               */
  FLDword  dwSubType;             /* Sub Flash type                           */
  FLByte   bMediaType;            /* Controller family                        */
  FLByte   bNoOfFloors;           /* Number of floors in the device           */
  FLByte   bNoOfPlanes;           /* Number of flash planes                   */
  FLByte   bNoOfBanks;            /* Number of flash banks                    */
  FLByte   bDataBusWidth;         /* Value of the IF_CFG (either 8 or 16 bits)*/
  FLByte   bSectorsPerPage;       /* Number of sectors in a flash page        */  
  FLByte   bSharedSectors;        /* The minimal number of logical sectors    */
                                  /* that might be affected by one another    */
                                  /* write operation.                         */
  FLByte   bFastAreaSharedSectors; /* Minimal number of sectors that can affect */
                                   /* one another when written in fast mode.    */  
  FLByte   bMaxRelatedSectors;    /* Max number of sectors that might share   */
                                  /* the same write operation or might be     */
                                  /* affected by each other write operation.  */ 
  FLDword  dwIPLSize;             /* Number of bytes of SRAM in the device    */
  FLByte   bChangeableProtectedAreas; /* Number of protection areas capable   */
                                      /* of changing.                         */  
  FLByte   btotalProtectedAreas;  /* Total number of protection areas         */
  FLDword  dwUnitsInFirstFloor;   /* Number of units in the first floor       */
  FLDword  dwUnitSize;            /* Size of logical erasable unit            */
  FLDword  dwMaxBadPercentage; /* Flash guaranteed  worst bad blocks percentage */ 
  FLByte   bNoOfBinaryPartitions; /* Number of binary partition on the media  */
  FLByte   bNoOfDiskPartitions;   /* Number of disk partition on the media    */
  FLByte   bBlockMultiplierBits;  /* no of bits used to represent the time a  */ 
                                  /* flash logical unit fits inside in a TL   */
                                  /* erasable unit.                           */
  FLByte   dwPercentUsed;         /* Percent of the media to be exported      */
  FLByte   bHeaderUnits;          /* Number of units reserved in the begining */
                                  /* of each floor for the media header       */
  FLWord   wHeaderLocation;       /* Unit number where the header of the      */
                                  /* current floor resides. In case all floors*/
                                  /* are requested, the value should indicate */
                                  /* the location of the header in the first  */
                                  /* floor.                                   */

  FLByte   bProgrammerNamePtr[4];    /* Programmer name                       */
  FLByte   bProgrammerVersionPtr[4]; /* Programmer version                    */

  /************************************************/
  /* Fields that belong to the specific partition */
  /************************************************/

  FLDword    dwVirtualUnits;  /* Virtual size exported by the translation layer */
                              /* Note some units are smaller then others        */
  FLDword    dwFastUnits;     /* Number of virtual units that are fast units    */ 
  FLDword    dwFirstQuickMountUnit; /* First unit used for the quick mount data */
  FLDword    dwFirstUnit;     /* First unit of the partition                    */
  FLDword    dwLastUnit;      /* Last unit of the partition                     */
  FLDword    dwSpareUnits;    /* Number of spare units requested by the user    */
  FLDword    dwTransferUnits; /* Number of transfer units used for chaining     */
  FLDword    dwPartitionFlags;/* See list above "Partition flags"               */
  FLDword    dwNormalAreaUnitSize;         /* Logical size of a TL normal unit */
  FLDword    dwFastAreaLogicalUnitSize;    /* Logical size of a TL fast   unit */
  FLDword    dwFastAreaVirtualFactor;      /* Virtual unit factor between fast */
                                           /* and normal area (power of 2)     */

  FLByte     bMinSectorsForFolding;        /* Number of sectors to be copied   */
                                           /* together in a folding operation  */
  FLByte     bMinSectorsForNextWrite;      /* For performance reasons the TL   */
                                           /* decide to reduce the above value */
  FLByte     bfastAreaMinSectorsForNextWrite; /* For performance reasons the TL   */
                                              /* decide to reduce the above value */

  /******************************************/
  /* Fields that are valid only after mount */
  /******************************************/

  FLDword    dwUsedUnits;                  /* Number of used virtual chains    */
  FLDword    dwFreeUnits;                  /* Number of currently free units   */
  FLByte     bNeededFreeUnits;             /* Minimal number of free units     */
                                           /* before folding                   */
  FLDword    dwUMDBBTSector;               /* Location of latest UMDBBT sector */
} FLExtendedDiskInfo;

typedef struct tTL TL;

#include "_fltl.h"

#endif /* FLTL_H */


