/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/surefs/src/fsdb.h-arc  $
 * 
 *    Rev 1.3   Jun 07 2005 06:23:30   OriS
 * - Esthetics - Made sure DB entries are not dependant on compilation flags:
 *   o FL_LIMIT_UNINTERRUPTED_FILE_ACCESS
 *   o FS_SUPPORT_FILE_CACHE
 * 
 *    Rev 1.2   Apr 25 2005 02:04:08   oris
 * Cosmetics
 * 
 *    Rev 1.1   Mar 28 2005 01:52:22   eugen
 * added extern "C" to declarations
 * 
 *    Rev 1.0   Mar 27 2005 09:26:52   oris
 * Initial revision.
 * 
 *    Rev 1.5   Mar 07 2005 22:15:26   oris
 * Comments
 * 
 *    Rev 1.4   Feb 19 2005 12:24:46   eugen
 * experimental option FL_PREPARE_FOR_WRITE_NO_ENSURE
 * 
 *    Rev 1.3   Feb 01 2005 10:13:28   eugen
 * make FSO fields compilation-time conditional on relevant options
 * cache granularity -> cache size in sectors
 * 
 *    Rev 1.2   Jan 05 2005 21:09:32   eugen
 * Commentaries, headers
 * 
 *    Rev 1.1   Dec 23 2004 19:35:54   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:22:18   oris
 * Initial revision.
 * 
 * 
 */

#ifndef FSDB_H
#define FSDB_H


#ifdef FS_MAX_FSOS

typedef struct {
FLWord * directoryPath;		/* path to match, can be empty, can not have wildcards, "\" at the end means exact match */
FLWord * filename;		/* name to match, can be empty, can have wildcards */
FLDword expectedFileSize;	/* will be reserved on creation (reduce FS defragmentation) */
FLDword flags;
FLSDword defragmentCloseVal;
FLSDword defragmentDeleteVal;
FLDword unintrIOrequestLimit;	/* will cut very long requests of this field is non-0 */
FLWord FSO_ID;
FLWord diskHandle;		/* combination of Socket, BDTL partition and LogicalPartition */
FLWord cacheSizeInSectors;	/* recommended value is the cluster size */
FLByte cacheBuffers;		/* number of cache buffers - 1 */
FLByte cachePriority;		/* when cache is scarce, allocate less or free others */
} FSO;

/* ---------------------------------------------------------------------------- */
/* user interface: */
#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

TFFS_DLL_API FLStatus NAMING_CONVENTION flRegisterFSObject(FLHandle diskHandle,
		FLWord * dirPath, FLWord * filename, FLWord FSO_ID, FLBoolean insertAtEnd);

TFFS_DLL_API FLStatus NAMING_CONVENTION flUnRegisterFSObject(FLWord FSO_ID);

TFFS_DLL_API FLStatus NAMING_CONVENTION flConfigureFSObject(
		FLWord FSO_ID, unsigned configOption, FLDword configParameter);

extern FLWord fsDBanyFile[];
extern FLWord fsDBRootDir[];

#ifdef __cplusplus
}
#endif /* __cplusplus */

/* ---------------------------------------------------------------------------- */
/* what configuration options are possible: */
#define FL_CONFIG_FSO_EXPECTED_FILE_SIZE	  0
#define FL_CONFIG_FSO_FLAGS			          1
#define FL_CONFIG_FSO_CACHE_SIZE_IN_SECTORS	  2
#define FL_CONFIG_FSO_CACHE_PRIORITY		  3
#define FL_CONFIG_FSO_CACHE_BUFFERS		      4
#define FL_CONFIG_UNINTERRUPTIBLE_IOREQ_LIMIT 5
#define FL_CONFIG_DEFRAGMENT_CLOSE_VALUE	  6
#define FL_CONFIG_DEFRAGMENT_DELETE_VALUE	  7

/* ---------------------------------------------------------------------------- */
/* definitions for static initialization */
#define FSDB_ANY_DIR		NULL
#define FSDB_NO_FILE		NULL

#define FSDB_ANY_FILE	fsDBanyFile
#define FSDB_ROOT_DIR	fsDBRootDir

/* ---------------------------------------------------------------------------- */
#define fastFSOinitTrail	0, 0, 0, 4, 0, 0
#define defaultFSOinitTrail	16384, 0, 0, 0, 0, 0

#define DEFINE_FSO(dirName, fileName, expectedSize, flags, defragCloseVal, defragDeleteVal,	\
	unintrIOLimit, FSO_ID, diskHandle, cacheSizeInSectors, cachePriority, cacheBuffers)	\
	{dirName, fileName, expectedSize, flags, defragCloseVal, defragDeleteVal,		\
	unintrIOLimit, FSO_ID, diskHandle, cacheSizeInSectors, cachePriority, cacheBuffers}
/* ---------------------------------------------------------------------------- */
#define FAST_STREAM(dir, file, size)		{(FLWord *)dir, (FLWord *)file, size, 	\
						FL_PREPARE_FOR_WRITE,			\
						0, FL_MINIMAL_DEFRAGMENTATION,		\
						fastFSOinitTrail }

#define FAST_DIR(dir)				{(FLWord *)dir, FSDB_NO_FILE, 0, 	\
						FL_USE_FAST_AREA,			\
						0, FL_MINIMAL_DEFRAGMENTATION,		\
						0, 0}

#define CRITICAL_STREAM(dir, file, size)	{(FLWord *)dir, (FLWord *)file, size, 	\
						FL_USE_FAST_AREA |			\
						FL_PREPARE_FOR_WRITE |			\
						FL_NO_WEAR_LEVELING_WHILE_OPEN,		\
						0, FL_MAXIMUM_DEFRAGMENTATION,		\
						fastFSOinitTrail }

/* Note!! While these definitions may fit some systems, they are here mostly as example
different systems may have different requirements for "fast" and "critical",
in any case the above macros are good for logical partition 0 on flash partition 0 on socket 0 */

#define DEFAULT_FILE(size, flags)		{FSDB_ROOT_DIR, FSDB_NO_FILE, size, flags,	\
						FL_MINIMAL_DEFRAGMENTATION,		\
						FL_MINIMAL_DEFRAGMENTATION,		\
						defaultFSOinitTrail }

#endif	/* FS_MAX_FSOS */

#ifdef FS_MM_PERFORMANCE_TUNEUPS
/* ---------------------------------------------------------------------------- */
/* FSO flags. Since they can also be combined with regular File->flags, should be above 1st byte */
#define FL_USE_NORMAL_AREA				0x00000000
#define FL_USE_FAST_AREA				0x00000100
#define FL_NO_WEAR_LEVELING_WHILE_OPEN			0x00000200
	/* note: this one will affect only files opened for write */
#define FL_PREPARE_FOR_WRITE				0x00000400
/* #define FL_PREPARE_FOR_WRITE_NO_ENSURE			0x00000800 */
/* #define FL_USE_WRITEBACK_CACHE				0x00000800
  no need - it is true if if cacheGranularity != 0 */

/* since irFlags is defined as unsigned, these better fit in 16-bits */

/* ---------------------------------------------------------------------------- */
#endif	/* FS_MM_PERFORMANCE_TUNEUPS */

#endif	/* FSDB_H */
