/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2006            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Version 6.3.2/TrueFFS FS/src/g4_exp.h-arc  $
 * 
 *    Rev 1.4   Feb 14 2006 11:09:42   DoronC
 * Update copyright note.
 * 
 *    Rev 1.3   Dec 06 2005 07:55:10   DoronC
 * Add OTP feature for G4.
 * 
 *    Rev 1.2   Dec 04 2005 13:15:06   DoronC
 * Finalizing packages for trueFFS 7.0.0.
 * 
 *    Rev 1.1.1.0   Nov 14 2005 09:58:42   RonenG
 * FAST Bit in FLASH_CONTROL_REG should be set always (Reduced FRE# timing) 
 * 
 *    Rev 1.1   Oct 30 2005 20:41:08   roneng
 * Bug in support of G4DD in big endian platform
 * 
 *    Rev 1.0   Sep 12 2005 13:20:36   roneng
 * Initial revision.
 * 
 *    Rev 1.5   Jan 27 2004 08:14:02   oris
 * Changed comments
 * 
 *    Rev 1.4   Sep 30 2003 17:54:14   oris
 * changed comments
 * 
 *    Rev 1.3   Jun 02 2003 08:53:30   roneng
 * Change printings
 * 
 *    Rev 1.2   May 16 2003 00:57:40   OriS
 * Added G4_protectionKeyInsert prototype.
 * 
 *    Rev 1.1   Apr 13 2003 20:29:38   OriS
 * Added fast read routine.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:54   OriS
 * Initial revision.
 * 
 */

/************************************************************************
 * Name    : G4_exp.h                                                  *
 *                                                                      *
 * Written : Ronen Golan                                                *
 *                                                                      *
 * Updated  : Tal Heller                                                *
 *                                                                      *
 * Abstract: This file contains the exported from the physical MTD      *
 ************************************************************************/




#ifndef _G4_EXP_H
#define _G4_EXP_H


/****************************** Macros *********************************/
/***********************************************************************
 * Name : G4_DPS0_LowProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define G4_DPS0_LowProtectedUnit(volume) \
        (G4_Read16BitReg(volume,G4_DPS0_ADDRESS_LOW_REG))  

/***********************************************************************
 * Name : G4_DPS0_HighProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define G4_DPS0_HighProtectedUnit(volume) \
        (G4_Read16BitReg(volume,G4_DPS0_ADDRESS_HIGH_REG))

/***********************************************************************
 * Name : G4_DPS1_LowProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define G4_DPS1_LowProtectedUnit(volume) \
        (G4_Read16BitReg(volume,G4_DPS1_ADDRESS_LOW_REG))

/***********************************************************************
 * Name : G4_DPS1_HighProtectedUnit                                                  
 *   returns DPS0 low Protected unit  
 *
 ***********************************************************************/
#define G4_DPS1_HighProtectedUnit(volume) \
        (G4_Read16BitReg(volume,G4_DPS1_ADDRESS_HIGH_REG))
  
/***********************************************************************
 * Name : G4_DPS0_UnitLocation                                                  
 *   returns DPS0 unit location in the Flash 
 *
 ***********************************************************************/
#define G4_DPS0_UnitLocation(volume)               1

/***********************************************************************
 * Name : G4_DPS1_UnitLocation                                                  
 *   returns DPS1 unit location in the Flash 
 *
 ***********************************************************************/
#define G4_DPS1_UnitLocation(volume)               3
                                     
/***********************************************************************
 * Name : G4_IsDPS0_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is write protected          
 *   0       - DPS_0 is NOT write protected 
 ***********************************************************************/
#define G4_IsDPS0_WP(volume) \
        (G4_Read8BitReg(volume,G4_DPS0_STATUS_REG) & G4_WRITE_PROTECTED)
           
/***********************************************************************
 * Name : G4_IsDPS1_WP                                                 
 *   Checks if dps is write protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is write protected          
 *   0       - DPS_1 is NOT write protected 
 ***********************************************************************/
#define G4_IsDPS1_WP(volume) \
        (G4_Read8BitReg(volume,G4_DPS1_STATUS_REG) & G4_WRITE_PROTECTED)

/***********************************************************************
 * Name : G4_IsDPS0_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_0 is read protected
 *   0       - DPS_0 is NOT read protected
 ***********************************************************************/
#define G4_IsDPS0_RP(volume) \
        (G4_Read8BitReg(volume,G4_DPS0_STATUS_REG) & G4_READ_PROTECTED)

/***********************************************************************
 * Name : G4_IsDPS1_RP                                                 
 *   Checks if DPS is read protected 
 *
 * Returns:        
 *   None 0  - DPS_1 is read protected
 *   0       - DPS_1 is NOT read protected
 ***********************************************************************/
#define G4_IsDPS1_RP(volume) \
        (G4_Read8BitReg(volume,G4_DPS1_STATUS_REG) & G4_READ_PROTECTED)

/***********************************************************************
 * Name : G4_IsDPS0_LOCK_Enabled                                                  
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0  - DPS_0 HW locked          
 *   0       - DPS_0 is not HW locked
 ***********************************************************************/
#define G4_IsDPS0_LOCK_Enabled(volume) \
        (G4_Read8BitReg(volume,G4_DPS0_STATUS_REG) & G4_HW_LOCK_ENABLED)

/***********************************************************************
 * Name : G4_IsDPS1_LOCK_Enabled                                                
 *   Checks if DPS is HW_LOCK protected 
 *
 * Returns:        
 *   None 0   - DPS_1 HW locked          
 *   0        - DPS_1 is not HW locked
 ***********************************************************************/
#define G4_IsDPS1_LOCK_Enabled(volume) \
        (G4_Read8BitReg(volume,G4_DPS1_STATUS_REG) & G4_HW_LOCK_ENABLED)
 
/***********************************************************************
 * Name : G4_Is_LOCK_Negated                                                 
 *   Checks if the LOCK# H/W signal is assterted 
 *
 * Returns:        
 *   None 0  - H/W LOCK# signal is assterted         
 *   0       - H/W LOCK# signal is asserted
 ***********************************************************************/
#define G4_Is_LOCK_Negated(volume) \
        (G4_Read8BitReg(volume,G4_PROTECTION_STATUS_REG) & G4_LOCK_INPUT_NEGATED)

/***********************************************************************
 * Name : G4_Is_StickyLOCK_Asserted                                                 
 *   Checks if the sticky lock signal is activated (asserted)
 *
 * Returns:        
 *   None 0  - Sticky lock activated (asserted)         
 *   0       - Sticky lock not activated (not asserted)   
 ***********************************************************************/
#define G4_Is_StickyLOCK_Asserted(volume) \
        (G4_Read8BitReg(volume,G4_PROTECTION_STATUS_REG) & G4_STICKY_LOCK_ACTIVATED)

/***********************************************************************
 * Name : G4_IsDPS0_KeyInserted                                                 
 *   Checks if the key is correct for DPS_0
 *
 * Returns:        
 *   None 0  - Key is correct The protected area is not locked          
 *   0       - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define G4_IsDPS0_KeyInserted(volume) \
        (G4_Read8BitReg(volume,G4_DPS0_STATUS_REG) & G4_KEY_OK)

/***********************************************************************
 * Name : G4_IsDPS1_KeyInserted                                                 
 *   Checks if the key is correct for DPS_1
 *
 * Returns:        
 *   None 0  - Key is correct The protected area is not locked          
 *   0       - Key is not the right one - DPS is protected 
 ***********************************************************************/
#define G4_IsDPS1_KeyInserted(volume) \
        (G4_Read8BitReg(volume,G4_DPS1_STATUS_REG) & G4_KEY_OK)

/***********************************************************************
 * Name : G4_IsDPS0_OTP_Protected
 *   Checks if DPS_0 is OTP protected 
 *
 * Returns:        
 *   0           - DPS is not OTP protected
 *   None 0      - DPS is OTP protected 
 ***********************************************************************/
#define G4_IsDPS0_OTP_Protected(volume) \
        (G4_Read8BitReg(volume,G4_DPS0_STATUS_REG) & G4_OTP_PROTECTED)

/***********************************************************************
 * Name : G4_IsDPS1_OTP_Protected
 *   Checks if DPS_0 is OTP protected 
 *
 * Returns:        
 *   0           - DPS is not OTP protected
 *   None 0      - DPS is OTP protected 
 ***********************************************************************/
#define G4_IsDPS1_OTP_Protected(volume) \
        (G4_Read8BitReg(volume,G4_DPS1_STATUS_REG) & G4_OTP_PROTECTED)


/********************************************************************************
 *  Name : G4_CheckandRecoverResetMode 
 *  - Check for RESET_MODE in entry & exit points. 
 *  - If In entry point RESET_MODE detected 
 *    - Report an ERROR 
 *    - If DO_NOT_RECOVER compilation flag is set return flSuspendModeDetected else continue operation.
 *  - If In exit point RESET_MODE detected 
 *    - Report an error
 *    - return flSuspendModeDetected. 
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bMode      - G4_ENTRY_POINT : Check for RESET_MODE at entry point 
 *  bMode      - G4_EXIT_POINT  : Check for RESET_MODE at entry point 
 * Status: 
 * Assumptions :   
 *******************************************************************************/
FLStatus G4_CheckandRecoverResetMode(FLFlash *flashPtr,FLByte bMode) ;

/***********************************************************************
 * Name:  G4_AsicSetFloor                                                  
 *                                                                      
 * Write to DeviceSelector register to set the desired floor .                                    
 *                                                                      
 * Parameters:                                                          
 *      flashPtr     : Pointer identifying drive                             
 *      bFloor     : The selected Floor (0,1,2,3)
 ***********************************************************************/

#ifndef FL_BIG_ENDIAN
#define G4_AsicSetFloor(flashPtr,bFloor) \
	G4_DirectWrite16BitReg(flashPtr,G4_DEVICE_ID_SELECT_REG,bFloor) ; \
    G4_Write8BitReg (flashPtr,G4_FLASH_CONTROL_REG,G4_FCE_SIGNAL_ACTIVE|G4_FLASH_FAST_FRE) ; 
#else
#define G4_AsicSetFloor(flashPtr,bFloor) \
	G4_DirectWrite16BitReg(flashPtr,G4_DEVICE_ID_SELECT_REG,((FLByte)(bFloor>>8)|((FLByte)(bFloor)<<8))) ; \
    G4_Write8BitReg (flashPtr,G4_FLASH_CONTROL_REG,G4_FCE_SIGNAL_ACTIVE|G4_FLASH_FAST_FRE) ; 
#endif

/************************************************************************
 **********************  Exported functions   ***************************
 ************************************************************************/

/***********************************************************************
 * Name : G4_isGoodBlock
 *   Determines if a block is a good block or a bad block 
 *
 * Parameters:                                                            
 *   volume             : Pointer identifying drive    
 *   wUnit              : The Block that we wants to check
 *   bOpFlags           : USE_BBT_CACHE - Checks if the page is already in the 
 *                        MTD_VARS_BUFFER (because it was read on the last call)
 *                        and if it does => Read the buffer witout accessing the flash
 *                        ALWAYS_READ_FROM_FLASH - When you are not sure that the 
 *                        MTD_VARS_BUFFER holds the bbt information           
 *                        use this option which forces the routine to read the bbt page   
 *  Note : When u want to use the isGoodBlock in a routine 
 *         It is the safest to call the first time with ALWAYS_READ_FROM_FLASH 
 *         and in the next calls to use USE_BBT_CACHE  
 *  Returns: 
 *    fIsGoodBlock :
 *           TRUE  => Block is a good block. 
 *           FALSE => Block is a bad block.
 *   flOK on success, any other value on failure.
 ************************************************************************/
FLStatus  G4_isGoodBlock          (FLFlash *volume,FLDword dwUnit,FLByte bOpFlags,FLBoolean *fIsGoodBlock) ;

/************************************************************************
 * Name:  G4_asicSetMode                                              *
 *                                                                      *
 * Set the controller (ASIC) operation mode.                            *
 *                                                                      *
 * Parameters:                                                          *
 *      volume     : Pointer identifying drive                          *
 *      mode    : One of the modes below:                               *
 *                RESET_MODE      = 0                                   *
 *                NORMAL_MODE     = 1                                   *    
 *                POWER_DOWN_MODE = 2                                   *
 *                                                                      *
 * Note: The mode is common to all cascaded floors.                     *
 ************************************************************************/
void  G4_asicSetMode          (FLFlash *volume, FL_AsicModeE AsicMode) ; 

/***********************************************************************
 * Name : G4_erase_Seq 
 *   Executes the erase sequence 
 * 
 * Parameters:                                                          
 *       flashPtr           : Pointer identifying drive           
 *       wUnitNo            : Unit to erase 
 *       dwOpFlags          : MTD_DO_NOT_WAIT  - Do not poll waiting for R/B# to be Ready      
 *                            MTD_REL_MODE     - Erase in Reliable mode.
 *                            MTD_FAST_MODE    - Erase in Fast mode. 
 *       bFloor             : The desired floor 
 *  Returns: 
 *  ErrorStatus:
 *      flOK on success, any other value on failure
 ***********************************************************************/
FLStatus G4_erase_Seq (FLFlash *flashPtr,PhyUnitType wUnitNo,
                        FLDword dwOpFlags,FLByte bFloor) ;

/***********************************************************************
 *  Name : G4_read_Seq  
 *   Executes the FLASH Read sequence command for reading one sector  
 *   
 * Parameters:     
 *       volume                : Pointer identifying drive           
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       dwOpFlags          : MTD_DATA      - Read from  the main  area of the physical sector 
 *                              MTD_EXTRA     - Read from the extra area of the physical sector
 *                            MTD_RAW_MODE  - Read with fixed offset and with no EDC/ECC correction
 * readArgs_OUTPUTS:
 *       MaxBitErrors       : The Maximum of bit errors found (0..5)
 *       FreeSectorsFound   : Number of free sectors found (0 or 1) 
 *       NoOfSectorsPassed  : Number of Sectors read successfully (0 or 1)  
 *  ErrorStatus:
 *  NOTE : It assumkes that page size is 512 => PAGE_SIZE=FL_SECTOR_SIZE   
 ***********************************************************************/
FLStatus     G4_read_Seq             (FLFlash *volume,G4_OperationArgsStructPtr READ_ARGS);

/***********************************************************************
 *  Name : G4_readData_Seq  
 *   Executes the FLASH Read sequence command for reading sectors      
 *
 * Parameters:                                                          
 *    flashPtr              : Pointer identifying drive
 * readArgs inputs :
 *       PhyUnit            : An pointer to the physical unit that the sector needed to be read is in 
 *       phyPage            : The page that would be read  
 *       mainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       extraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       dwOpFlags          : MTD_DATA        - Read from  the main  area of the physical sector 
 *                            MTD_EXTRA       - Read from the extra area of the physical sector                                                  
 *                            MTD_REL_MODE    - Force the reliable mode. 
 *                            MTD_FAST_MODE   - Forces the fast mode. 
 *       sectorOffset       : The sector offset in the page 0 / 1
 *             
 * readArgs_OUTPUTS:
 *       MaxBitErrors       : The Maximum of bit errors found (0..5)
 *       FreeSectorsFound   : Number of free sectors found (0 or 1) 
 *       NoOfSectorsPassed  : Number of Sectors read successfully (0 or 1)  
 * +--------------------------------------------------------------------------------+ *
 * |   Supported OpFlags |   MainBuffer   |  ExtraBuffer  |    Operation            | *
 * |   ----------------- |   ----------   |  -----------  |    ---------            | *
 * | MTD_DATA            |     Valid      |    Valid      | Read DATA + ECC         | *
 * | MTD_DATA|MTD_EXTRA  |     Valid      |    Valid      | Read DATA + Extra + ECC | *
 * +--------------------------------------------------------------------------------+ *
 **************************************************************************************/
FLStatus G4_readData_Seq   (FLFlash *flashPtr,G4_OperationArgsStructPtr readArgs) ;

/***********************************************************************
 * Name : G4_pageProgram_Seq 
 *   Executes the Page program sequence command                                                                     
 *   This routine can write in parallel to 1..N pages (N = Number of planes)
 * 
 * Parameters:                                                          
 *       volume             : Pointer identifying drive           
 *       PhyUnits           : An array that holds the number of the 1..N 
 *                            physical units that needs to be written in parallel
 *       phyPage            : The page that would be programmed in all units in parallel
 *       noOfPhySectors     : Number of physical sectors that needs to be programmeed                              
 *                              (This is the size of the PhySectors array)
 *       MainBuffer         : The Pointer to user buffer containing the data
 *                            that should be written to the sector main area.
 *       ExtraBuffer        : Pointer to user buffer containing the data
 *                              that should be written to the sector info area. 
 *       VerifyWriteMode    : An indicator of the read after write mode.
 *                              FL_ON will force a read after write operation
 *                            while FL_OFF will disable it !!!. 
 *       dwOpFlags            : MTD_DATA        - Write to the main  area of the physical sector 
 *                              MTD_EXTRA       - Write to the extra area of the physical sector 
 *                            MTD_DO_NOT_WAIT - Return without waiting for the R/B# to be ready
 *                                              in the last parallel PageProgram command   
 *                                        
 *          
 *  Returns: 
 *  ErrorStatus:
 *     
 ***********************************************************************/
FLStatus     G4_pageProgram_Seq      (FLFlash *volume,G4_OperationArgsStructPtr WRITE_ARGS);

/***********************************************************************
 * Name : G4_phyWriteEraseMark 
 *   Writes the erase mark to the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     volume     : The flash struct
 *     phyUnit    : The physical unit to check erase mark 
 *    dwEraseMark : The erase Mark that would be written 
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus    G4_phyWriteEraseMark    (FLFlash *volume,PhyUnitType phyUnit,FLByte bEraseMark)  ; 


/***********************************************************************
 * Name : G4_phyGetEraseCount 
 *   Gets the Number of times a logical erase unit has been erased 
 * 
 * Parameters:    
 *   Inputs:
 *     volume    : The flash struct
 *     StartUnit : The physical unit to check erase count 
 *     bFloor    : The floor  
 *   Outputs:
 *     EraseCount: The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus G4_phyGetEraseCount (FLFlash *flashPtr,PhyUnitType *PhyUnitArray,FLDword *dwEraseCount,FLByte bFloor) ;


/***********************************************************************
 * Name : G4_phyGetEraseMark 
 *   Reads the erase mark from the physical unit  
 * 
 * Parameters:    
 *   Inputs:
 *     volume     : The flash struct
 *     phyUnit    : A pointer to the physical unit array  
 *     bEraseMark : The erase Mark that was read 
 *     bFloor     : The floor
 *   Outputs:
 *     EraseCount : The Erase count of the physical erase unit 
 *     
 ***********************************************************************/
FLStatus    G4_phyGetEraseMark      (FLFlash *volume,PhyUnitType *phyUnit,FLByte *bEraseMark,FLByte bFloor) ;

/***********************************************************************
 * Name: G4_DPS_Read                                                        
 *   Read DPS data structure from DPS unit of the current floor         
 *                                                                        
 * Parameters:                                                          
 *    volume     : Pointer identifying drive                                 
 *  bDPS_No  : The dps number                                            
 *  bFlags   : G4_COPY_0-Read the 1st copy                                   
 *             G4_COPY_1-Read the 2nd copy                                     
 *             G4_COPY_2-Read the 3rd copy                                    
 *             G4_COPY_3-Read the 4th copy                                    
 *             G4_FIRST_GOOD_COPY- Read the first good copy                 
 *             find the first one that is good.                            
 * Returns:                                                              
 *    FLStatus    : 0 on success, otherwise failed.                        
 ***********************************************************************/
FLStatus G4_DPS_Read(FLFlash *flashPtr,FLByte *DPS_Ptr,FL_DPS_IndexE DPS_No,
					  FLByte bFlags,FLByte bFloor);

/***********************************************************************
 * Name: G4_DPS_Write                                                         
 *   Writes a new DPS to the flash                                                    
 *   It erases the main and redundency units , writes                    
 *    a new DPS struct                                                    
 * Parameters:                                                          
 *    volume         : Pointer identifying drive                            
 *  DPS_No   : The dps number                                            
 *  DPS_PTR  : The DPS struct                                            
 *    bFloor   : The Asic FLOOR                                            
 * Returns:                                                              
 *    FLStatus    : 0 on success, otherwise failed.                        
 * Assumption   : If SYS_INTLV=2 than the interleave would be written   
 *                as 0                                                    
 *                INTLV_2_CASCADED always                                 
 ***********************************************************************/
FLStatus G4_DPS_Write (FLFlash *flashPtr,FLByte *DPS,FL_DPS_IndexE DPS_No,FLByte bFloor) ;

/************************************************************************
 * Name: G4_tryKey
 *   Try to write the KEY to the DPS_0/DPS_ 1 KEY Register
 *
 * Parameters:
 *  volume        : Pointer identifying drive
 *  bDPS_No       : The  DPS number    0/1
 *  bKey          : The 8 bytes key
 *  bFloor        : The floor in which the DPS is in
 ************************************************************************/
FLStatus      G4_tryKey               (FLFlash *volume,FLByte bDPS_No,FLByte FAR1* bKey,
                                         FLByte bFloor);

/***********************************************************************
 * Name: G4_protectionKeyInsert 
 *   Sends protection key only to protected areas.
 *
 * Parameters:
 *  flashPtr   - FLFlash struct
 *  bDPS_No : indicated which protection area to work on. 0 or 1.
 *  Key     : An 8 FLByte long array containing the protection password.          
 *  Notes
 *   1.    The key should be sent to all of the device's floors even if 
 *      one of the floors indicated that the key did not fit.
 *      The key should not be sent to a floor that indicates that 
 *      it's key was already inserted. 
 *      This way allows us to open partition that might have different
 *      keys for different floors as a result of a bug or a power failure.
 *   2.    Before sending the given key the MTD will try and send "00000000" 
 *      (Ascii) key.
 ***********************************************************************/
FLStatus G4_protectionKeyInsert(FLFlash *flashPtr,FLByte bDPS_No,FLByte FAR1* Key);

/********************************************************************************
 *  Name : G4_phyDoc2Identify 
 *    This routine does the following 
 *    1. Initialization of volume->mtdVars pointer
 *    2. Find the interleave & if_cfg
 *    2. Hooking the correct access routines according to the flash interleave & IF_CFG
 *    3. DOC init - Insert DOC into NORMAL mode.
 *    4  Find board & bank properties                            
 *           - Find the system interleave 1/2                    
 *           - Find how many floors exist                    
 *           - Find how many banks are in each floor            
 *           - Find how many banks are in the last floor        
 *           - Find floor size                                
 *    5. Find Logical Block properties                            
 *           - Find LogicalUnitsInBank                              
 *           - Find LogicalUnitsInFloor                            
 *    6. Find Logical Sector Properties                          
 *          - Find SectorsInPage                                
 *          - Find SectorsInUnit                            
 *          - Find SectorsInLogicalUnit                        
 *            - Find SectorsInChip                            
 *          - Find SectorsInBank                            
 *            - Find SectorsInFloor                                
 *    7. Update FLFLASH struct                                   
 *    8. Update pointers to MTD exported functions 
 *  
 *      volume: The flash struct.
 * Status:
 * Assumptions : It assumes that all flashes are of the same kind (MAKER & CODE ID)       
 *******************************************************************************/
FLStatus      G4_phyDoc2Identify      (FLFlash *volume) ; 

/***********************************************************************
 * Name : G4_Read8BitReg
 *  Read 8 bit register in two stages
 *  1. Write the address to the G4_ADDRESS_REG
 *  2. Read from the specified address (only important for debugging)
 *
 * Parameters:
 *    flashPtr:   The flash struct.
 *    wOffset:    The offset of the FLASH_DATA_REGISTER
 *
 * Returns: The value of this register 
 ***********************************************************************/
FLByte G4_Read8BitReg(FLFlash *flashPtr,FLWord wOffset) ;

/***********************************************************************
 * Name : G4_Read16BitReg
 *  Read 16 bit register in two stages
 *  1. Write the address to the G4_ADDRESS_REG
 *  2. Read from the specified address (only important for debugging)
 *
 * Parameters:
 *    flashPtr:   The flash struct.
 *    wOffset:    The offset of the FLASH_DATA_REGISTER
 *
 * Returns: The value of this register 
 ***********************************************************************/
FLWord G4_Read16BitReg(FLFlash *flashPtr,FLWord wOffset) ;

#endif /* _G4_EXP_H */

/* end of G4_exp.h */

