/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/h1_mtd.h-arc  $
 * 
 *    Rev 1.9   Sep 19 2005 15:36:04   roneng
 * Add definition for sequence error recovery
 * 
 *    Rev 1.8   Jul 17 2005 19:32:42   oris
 * - Reduced max busy delay to 400,000 cycles (20ms/50ns)
 * 
 *    Rev 1.7   Jul 14 2005 06:37:14   oris
 * Bug fix: FL_BYTE_SWAP did not work on big endian platforms.
 * 
 *    Rev 1.6   Jul 07 2005 00:03:10   OriS
 * - Bug fix: Virgin DiskOnChip reported bad download, and did not complete the flash identification process.
 * - Removed unused download busy loop definition.
 * 
 *    Rev 1.4   Apr 21 2005 02:28:28   oris
 * - Changed H1_FL_IRQ_RB_INIT(socketNo) to FL_IRQ_RB_INIT(socketNo)
 * - Changed H1_FL_DPS_2_COPIES_INIT(socketNo) to FL_DPS_2_COPIES_INIT(socketNo)
 * - Added single compile time access layer macros, for easier customization.
 * 
 *    Rev 1.3   Apr 19 2005 02:49:24   oris
 * - Added defintion for maximum download time - 
 *   H1_MAX_DOWNLOAD_TIME_MSEC
 * 
 *    Rev 1.2   Apr 04 2005 21:08:38   oris
 * - Support FL_SWAP_BYTE (for wrong customization in big endian platforms)
 * 
 *    Rev 1.1   Apr 04 2005 20:16:06   oris
 * - Added defaults for :
 *   H1_FL_IRQ_RB_INIT and H1_FL_DPS_2_COPIES_INIT.
 * 
 *    Rev 1.0   Mar 27 2005 09:27:20   oris
 * Initial revision.
 * 
 *    Rev 1.5   11 Sep 2003 16:02:48   oris
 * Added backward compatability flag FL_XSCALE_BOOT_MODE
 * 
 *    Rev 1.4   Sep 01 2003 19:04:02   oris
 * - Added 2K page new sequences offset in sequence cache RAM
 * - Added 2K page flash commands definitions
 * - Added sectorOffset to H1_OperationsArgsStruct
 * 
 *    Rev 1.3   Aug 10 2003 14:40:54   roneng
 * - Added 2K page new sequences offset in sequence cache RAM
 * - Added 2K page flash commands definitions
 * - Added sectorOffset to H1_OperationsArgsStruct
 * 
 *    Rev 1.2   May 12 2003 20:46:48   roneng
 * - Change the COTP struct
 * 
 *    Rev 1.1   May 06 2003 11:55:24   OriS
 * Removed warnings
 * 
 *    Rev 1.0   Apr 09 2003 12:15:56   OriS
 * Initial revision.
 * 
 */


/************************************************************************
 * Name    : mtd_oren.h                                                 *
 *                                                                      *
 * Written : Ronen Golan                                                *
 *                                                                      *
 * Abstract: This file contains the prototypes of the H1 MTD            *
 ************************************************************************/

#ifndef MTD_H1_H
#define MTD_H1_H
/***************************  Definitions    ****************************/
#define MTD_VARS         ((MTDVarsStruct *) flashPtr->mtdVars)
#define MTD_VARS_BUFFER  (MTD_VARS->buffer->flData)        

/****************************  Typedefs      ****************************/
/************************* Run Time Definitions *************************/
#define MAX_BANKS                4
#define FOUNDRY_DATA_SIZE        0x200  
#define FOUNDRY_RESERVED_SIZE    8

#ifndef FL_IRQ_RB_INIT    
#define FL_IRQ_RB_INIT(wSockeNo)                     FL_INT_RB_DISABLED
#endif

#ifndef FL_DPS_2_COPIES_INIT
#define FL_DPS_2_COPIES_INIT(wSockeNo)               FL_DISABLED 
#endif

/************************ Access layer routines *************************/
#ifndef H1_Write8BitReg
#define H1_Write8BitReg(flash,offset,val)                 flDirectWrite8BitReg(flash,offset,val)
#endif
#ifndef H1_DirectWrite16BitReg
#define H1_DirectWrite16BitReg(flash,offset,val)          flDirectWrite16BitReg(flash,offset,val)
#endif
#ifndef H1_Read8BitReg
#define H1_Read8BitReg(flash,offset)                      flDirectRead8BitReg(flash,offset)
#endif
#ifndef H1_DirectRead16BitReg 
#define H1_DirectRead16BitReg(flash,offset)               flDirectRead16BitReg(flash,offset)
#endif
#ifndef H1_ReadEvenNumberOfBytes
#define H1_ReadEvenNumberOfBytes(flash,offset,dest,count) flReadEvenNumberOfBytes(flash,offset,dest,count)
#endif
#ifndef H1_WriteEvenNumberOfBytes
#define H1_WriteEvenNumberOfBytes(flash,offset,src,count) flWriteEvenNumberOfBytes(flash,offset,src,count)
#endif
#ifndef H1_Read512Bytes
#define H1_Read512Bytes(flash,offset,dest)                flRead512Bytes(flash,offset,dest)
#endif
#ifndef H1_Write512Bytes
#define H1_Write512Bytes(flash,offset,src)                flWrite512Bytes(flash,offset,src)
#endif
#ifndef H1_DocWindow
#define H1_DocWindow(flash)                               flDocWindow(flash)
#endif

/*************************    Include files      *********************/

/***************************     MEMORY MAP       **********************/
#define H1_IPL_AREA_0                 0x0000
#define H1_IPL_AREA_1                 0x0400
#define H1_IPL_AREA_2                 0x1800
#define H1_IPL_AREA_3                 0x1c00

#ifndef FL_XSCALE_BOOT_MODE
#define H1_FLASH_DATA_REG_ALIASED     0x0800 
#else /* Do not read from 4 bytes aligned address */
#define H1_FLASH_DATA_REG_ALIASED     0x0802  /* Prevent access to 32bit alligned address */ 
#endif /* FL_XSCALE_BOOT_MODE */


#define H1_MEMORY_MAP_SIZE            0x2000   
#define H1_IPL_CODE_SIZE              0x400L

/************************** Register Address Map ************************/
/**************************   DOC CONTROL REGS   ************************/
#define H1_CHIP_ID_REG                0x1000  /* READ       */ 
#define H1_CHIP_ID_CONFIRM_REG        0x1074  /* READ+WRITE */
#define H1_NOP_REG                    0x1002  /* READ+WRITE */ 
#define H1_ALIAS_RESOLUTION_REG       0x1004  /* READ+WRITE */ 
#define H1_BUS_LOCK_REGISTER          0x1006  /* READ+WRITE */ 
#define H1_BUS_SWAP_REG               0x1008  /* READ+WRITE */ 
#define H1_DEVICE_ID_SELECT_REG       0x100A  /* READ+WRITE */ 
#define H1_DOC_CONTROL_REG            0x100C  /* READ+WRITE */ 
#define H1_DOC_CONTROL_CONFIRM_REG    0x1072  /* READ+WRITE */ 
#define H1_CONFIGURATION_REG          0x100E  /* READ+WRITE */ 
#define H1_INTERRUPT_CONTROL_REG      0x1010  /* READ+WRITE */ 
#define H1_FOUNDRY_TEST_REG           0x1076  /* READ+WRITE */ 
#define H1_HW_PROTECTION_CONTROL_REG  0x1012  /* READ+WRITE */ 
#define H1_FOUNDRY_KEY_REG            0x1078  /* READ+WRITE */ 
#define H1_OUTPUT_CONTROL_REG         0x1014  /* READ+WRITE */ 
/**************************   ICMU REGISTERS   **************************/
#define H1_ICMU_CORRECT_FACTOR_REG    0x1016  /* READ+WRITE */ 
#define H1_ICMU_CONTROL_REG           0x1018  /* READ+WRITE */ 
#define H1_CLOCK_COUNTER_REG          0x101A  /* READ+WRITE */ 
/**************************    LPC REGISTERS   **************************/
#define H1_LPC_CONFIGURATION_REG      0x1020  /* READ+WRITE */ 
#define H1_LPC_STRAPPING_REG          0X1022  /* READ       */   
#define H1_LPC_DMA_REG                0x1024  /* READ+WRITE */ 
#define H1_LPC_INTERRUPT_REG          0x1026  /* READ+WRITE */ 
#define H1_LPC_IO_INDEX_REG           0x0007  /* READ+WRITE */ 
/**************************   FLASH REGISTERS  **************************/ 
#define H1_FLASH_CONTROL_REG          0x1038  /* READ+WRITE */ 
#define H1_FLASH_STATUS_REG           0x103A  /* READ       */ 
#define H1_FLASH_ACCESS_CONFIG_REG    0x103E  /* READ+WRITE */ 
#define H1_FLASH_SEQUENCE_REG         0x1032  /* READ+WRITE */ 
#define H1_FLASH_COMMAND_REG          0x1034  /* READ+WRITE */ 
#define H1_FLASH_ADDRESS_REG          0x1036  /* READ+WRITE */ 
#define H1_FLASH_DATA_REG             0x1030  /* READ+WRITE */ 
#define H1_FLASH_DATA_REG_LB_BYTE     0x103C  /* READ+WRITE */
/**************************    ECC REGISTERS   **************************/
#define H1_FLASH_BCH_PARITY_REG       0x1048  /* READ+WRITE */ 
#define H1_HAMMING_PARITY_REG         0x1046  /* READ       */ 
#define H1_ECC_PRESET_REG             0x1044  /* READ+WRITE */ 
#define H1_ECC_CONTROL_REG_0          0x1040  /* READ+WRITE */ 
#define H1_ECC_CONTROL_REG_1          0x1042  /* READ+WRITE */ 
/************************** PROTECTION REGISTERS **************************/
#define H1_COTP_POINTER_REG           0x1050  /* READ       */ 
#define H1_COTP_LOCK_POINTER_REG      0x1052  /* READ       */ 
#define H1_BBT_POINTER_REG            0x1054  /* READ       */ 
#define H1_PROTECTION_STATUS_REG      0x1056  /* READ       */ 
#define H1_DPS0_ADDRESS_LOW_REG       0x1060  /* READ       */ 
#define H1_DPS0_ADDRESS_HIGH_REG      0x1062  /* READ       */ 
#define H1_DPS1_ADDRESS_LOW_REG       0x1064  /* READ       */ 
#define H1_DPS1_ADDRESS_HIGH_REG      0x1066  /* READ       */ 
#define H1_DPS0_POINTER_REG           0x1068  /* READ       */ 
#define H1_DPS1_POINTER_REG           0x106A  /* READ       */ 
#define H1_DPS0_STATUS_REG            0x106c  /* READ       */ 
#define H1_DPS1_STATUS_REG            0x106e  /* READ       */  
#define H1_DPS_0_KEY_REG              0x105c  /* WRITE       */ 
#define H1_DPS_1_KEY_REG              0x105e  /* WRITE      */
#define H1_FLASH_GEOMETRY_REG_0       0x1058  /* READ       */ 
#define H1_FLASH_GEOMETRY_REG_1       0x105a  /* READ       */ 
/**************************  DOWNLOAD REGISTERS  **************************/
#define H1_DOWNLOAD_STATUS_REGISTER0  0x1070  /* READ+WRITE */ 
#define H1_SEQUENCE_CASH_RAM_START    0x1100  /* READ+WRITE */ 
/**************************************************************************/

/**********************************************************************************
 *****************************   Register Description   ***************************
 **********************************************************************************/

/*******************************   BUS_SWAP_REG       *****************************/
#define H1_SWAP_L                      0x0001
#define H1_SWAP_H                      0x0100 
#define H1_SWAP_DATA_BYTES             (H1_SWAP_L|H1_SWAP_H)	 

/*******************************   DOC_CONTROL_REG    *****************************/
#define H1_MODE_BITS                   0x03
#define H1_MODE_WRITE_ENABLE           0x04
#define H1_BDETCT_RESET                0x08  /* Set whenever the controller entered a reset mode as a result of Boot detector trigger. Cleared by writing 1        */
#define H1_RSTIN_RESET                 0x10  /* Set whenever the controller entered a reset mode as a result of RSTIN# input signal asserted .Cleared by writing 1 */
#define H1_CLOCK_STOP                  0x20

/*******************************  CONFIGURATION_REG  *****************************/
#define H1_BUS_CONFIGURATION_BITS      0x03
#define H1_RAM_WRITE_ENABLE            0x04                
#define H1_RAM_READ_ENABLE             0x08
#define H1_MAX_ID_BITS                 0x30
#define H1_BOOT_DETECTOR_INHIBIT       0x40
#define H1_IF_CFG_STATUS               0x80

/*******************************  INTERRUPT_CONTROL_REG  *************************/
#define H1_IRQ_ENABLED                 0x80
#define H1_IRQ_FIFO_PENDING            0x40
#define H1_IRQ_PROTECTION_PENDING      0x20
#define H1_IRQ_FREADY_PENDING          0x10
#define H1_IRQ_EDGE_SENSITIVE          0x08
#define H1_IRQ_ENABLE_FIFO_INT         0x04
#define H1_IRQ_ENABLE_PROTECTION_INT   0x02
#define H1_IRQ_ENABLE_FREADY_INT       0x01

/*******************************     FOUNDRY_TEST_REG      ************************/
#define H1_SOFT_RESET                  0x80
#define H1_ICMU_CLOCK_ON_IRQ_PIN       0x08   /* When set ICMU will be output on external pin */ 
#define H1_BDETECT_TEST_MODE           0x04   /* In this mode the boot detector will be incremented by 11H instead of 01H */
#define H1_DOWNLOAD_TEST_MODE          0x02   /* Download test mode - data is downloaded from D[7:0] instead of the flash */
#define H1_FLASH_TEST_MODE             0x01   /* When set provides all the pin signals on the I/O pins */

/******************************* HW_PROTECTION_CONTROL_REG *************************/
#define H1_PROTECTION_DNLD_ENABLED     0x01
#define H1_FLASH_EMPTY                 0x02
#define H1_NO_OF_FLASH_RD_SEQ_BITS     0x0C
#define H1_FOUNDRY_KEY_ENABLE          0x80  

/*******************************    OUTPUT_CONTROL_REG     **************************/
#define H1_ASSERT_BUSY_DURING_DNLD     0x01
#define H1_DISABLE_PULL_UP_BITS        0x06
#define H1_RAM_ADDRESS_LE              0x08 
#define H1_SOFTWRAE_LOCK_ENABLED       0x10 

/*******************************     ICMU_CONTROL_REG      *************************/
#define H1_EXTERNAL_CLOCK_ENABLE       0x01   /* Disable the ICMU operation and move to external clock operation */
#define H1_CLOCK_AVAILABLE             0x02   /* Clock available for the DOC Asic */
#define H1_CORRECTION_FACTOR_WRITE_EN  0x04
 
/*******************************     CLOCK_COUNTER_REG     *************************/
#define H1_CLOCK_COUNTER_RESET         0x8000 /* The counter starts to count from zero */
#define H1_CLOCK_COUNTER_STOP          0x4000 /* The counter is stopped until CLOCK_COUNTER_RESET is set */  
#define H1_CLOCK_COUNTER_BITS          0x3FFF   
 
/*******************************    LPC_CONFIGURATION_REG  *************************/
#define H1_LPC_IO_WINDOW               0x8000 /* Refer to bit in the high byte */
#define H1_LPC_PROGRAMMABLE_DECODE_EN  0x4000   
#define H1_LPC_STRAPPING_DECODE_EN     0x2000 
#define H1_LPC_PROGRAMMABLE_ADDR_BITS  0x1FFF

/*******************************      LPC_IO_INDEX_REG     **************************/ 
#define H1_LPC_IO_INDEX_REG_BITS       0x3F

/*******************************        LPC_DMA_REG        **************************/ 
#define H1_DMA_CHANNEL_BITS            0x07   /* The DOC will request to DMA in this Channel */
#define H1_DMA_INITIATE_DMA_REQUEST    0x08
#define H1_DMA_LDRQ_SIGNAL_EN          0x10
#define H1_DMA_CYCLE_SIZE_BITS         0x60

/*******************************      LPC_INTERRUPT_REG    **************************/
#define H1_LPC_INT_NUMBER_BITS         0x1F   /* Interrupt will be requested by DOC on SERIRQ */
#define H1_LPC_EDGE_INT                0x20
#define H1_LPC_ENABLE_DMA_INT          0x40
#define H1_LPC_DMA_INT_PENDING         0x80

/*******************************     FLASH_CONTROL_REG     **************************/
#define H1_FSE_SIGNAL_ACTIVE           0x40   /* Controls the assertion of the FSE# bit */
#define H1_FWP_SIGNAL_ACTIVE           0x20   /* After protocol violation detected we must clear this bit */
#define H1_FCE_SIGNAL_ACTIVE           0x10   /* Controls the assertion of the CE# to the BANK defined by the BANK bits */ 
#define H1_BANK_BITS                   0x0c   /* Control with which bank we are working */
#define H1_BANK_BITS_SHIFT             0x2    /* Shift of the bank bits in the register */
#define H1_HIGH_BYTE_LANE              0x02   /* When FLASH_ACCESS is 16 bit and SYSTEM_INTERLEAVE is 1 -> controls to which lane we would write to */
#define H1_ACTUAL_INTERLEAVE           0x01   /* 1 - Access to the flash is Performed in 8 bits , 1- Access is Performed in 16 bits (or to two 8 bits devices */

/*******************************    FLASH_STATUS_REG        *************************/
#define H1_FLASH_READY                 0x01   /* FLASH READY/BUSY# output signal */
#define H1_DATA_FIFO_FULL              0x02   /* Data FIFO is full  */  
#define H1_DATA_FIFO_EMPTY             0x04   /* Data FIFO is empty */  
#define H1_DATA_FIFO_FLOW              0x08   /* Data O.F/U.F detected & will be cleared after the current access is finished  */  
#define H1_SEQUENCE_ERROR_DETECTED     0x10   /* Sequence Error detected & will be cleared only by FLASH reset command */
#define H1_PROTECTION_ERROR_DETECTED   0x20   /* Protection Error & will be cleared after FLASH reset command */
#define H1_SYSTEM_INTERLEAVE_2         0x40   /* Reflection of system interleave */
#define H1_SINGLE_16BIT_FLASH_CONNECT  0x80   /* Indicates that a 16 bit single flash is connected to the DOC */

/*******************************  FLASH_ACCESS_CONFIG_REG    ************************/
#define H1_FRE_FWE_ACCESS_CYCLES_BITS  0x07   /* Duration of the FRE/FWE access cycle (access duration = FRE_FWE_ACCESS_CYCLES_BITS + 1) */
#define H1_RISE_EDGE_DELAY_EN          0x08   /* Enable of delay of rising edge of half a clock */        

/*******************************      ECC_CONTROL_REG_0      ************************/
#define H1_DATA_COUNTER_BITS           0x07FF    
#define H1_BCH_EN                      0x0800     
#define H1_HAMMING_EN                  0x1000
#define H1_ECC_IN_SW                   0x2000  
#define H1_AUTO_ECC_READ_WRITE_EN      0x4000     
#define H1_FLASH_READ_MODE             0x8000 /* Host is doing a read operation */

#define H1_READ_DATA_EXTRA             0x9A0F 
#define H1_READ_SW_EDC                 0x9A10 
#define H1_READ_EXTRA                  0x9008 

/*******************************      ECC_CONTROL_REG_1      ************************/
#define H1_HM_DATA_NUM_BITS            0x0F
#define H1_ECC_BUSY                    0x10
#define H1_PAGE_READ_IS_USED           0x20
#define H1_IGNORE_DATA                 0x40
#define H1_BCH_SYNDROM_ERROR           0x80                 

/*******************************    PROTECTION_STATUS_REG    *************************/
#define H1_FOUNDRY_OTP_LOCKED          0x01
#define H1_CUSTOMER_OTP_LOCKED         0x02
#define H1_PROTECTION_ERROR            0x08
#define H1_LOCK_INPUT_ASSERTED         0x10
#define H1_PROTECTION_ENABLED          0x20
#define H1_FOUNDRY_OTP_REVISION_BITS   0xC0

/*******************************         DPS0_STATUS_REG     *************************/
#define H1_MTD_READ_PROTECTED          0x01 
#define H1_MTD_WRITE_PROTECTED         0x02
#define H1_HW_LOCK_ENABLED             0x04
#define H1_LOW_BANK_BITS               0x18
#define H1_HIGH_BANK_BITS              0x60
#define H1_KEY_OK                      0x80

/*******************************  DOWNLOAD_CONTROL_REGISTER0 *************************/
#define H1_SEQ_NUM_BITS                0x1F 

/*******************************  DOWNLOAD_CONTROL_REGISTER1 *************************/
#define H1_SEQ_CASH_BITS               0x0F

/*******************************    DOWNLOAD_STATUS_REG_1    *************************/
#define H1_DOWNLOAD_IN_PROGRESS        0x40                 
#define H1_SEQ_DNLD_ERROR_DEMAND       0x80  /* Sequence error occured when downloading the page sequences page */

/*******************************    DOWNLOAD_STATUS_REG_0    *************************/
#define H1_DPS0_DOWNLOAD_ERROR_BITS    0x03
#define H1_DPS0_DOWNLOAD_REDUNDANT     0x01
#define H1_DPS1_DOWNLOAD_ERROR_BITS    0x0C
#define H1_DPS1_DOWNLOAD_REDUNDANT     0x04
#define H1_IPL_DOWNLOAD_ERROR_BITS     0x30
#define H1_IPL_DOWNLOAD_REDUNDANT      0x10
#define H1_SEQUENCE_DOWNLOAD_ERROR     0x40
#define H1_OTP_DOWNLOAD_ERROR          0x80 

/* Allow loading IPL/DPS from redundant copy */
#define H1_CLEAN_DOWNLOAD              0xCA

/*******************************     FLASH_GEOMETRY_REG_0    **************************/
#define H1_ROW_ADDR_BYTE_NUM           0x01
#define H1_PAGE_ADDR_BIT_NUM_BITS      0x0E  
#define H1_UNIT_ADDR_BIT_NUM_BITS      0xF0

/*******************************     FLASH_GEOMETRY_REG_0    **************************/
#define H1_PAGE_OFS_ADDR_BIT_NUM_BITS  0x0F
#define H1_ADDR_BYTE_NUM_BITS          0x70  
#define H1_PAGE_MAPPING_BIT            0x80


/******************************* Flash Parameters  *****************************/

#define H1_SECTOR_BCH_PROTECTED  SECTOR_DATA_SIZE+SECTOR_PAGEINFO_SIZE+SECTOR_HM_SIZE
#define H1_SECTOR_HM_PROTECTED   SECTOR_PAGEINFO_SIZE    


/******************************* Flash Bank interface  *****************************/
#define H1_ONE_FLASH_8_BITS      0x00      /* Only one 8 bit flash is connected  */
#define H1_ONE_FLASH_16_BITS     0x01   /* only one 16 bit flash is connected */
#define H1_TWO_FLASHES_CASCADED  0x02   /* Two 8 bit flashes cascaded         */
#define H1_TWO_FLASHES_SEPERATED 0x03   /* Two 8 bit flashes connected to the */
                                          /* same bank but the distinction is   */
                                          /* by the BL bit                      */

/*******************************    Flash Status Reg   *****************************/
#define H1_OPERATION_OK          0x00
#define H1_OPERATION_FAILED      0x01   
#define H1_PLANE_0_FAIL          0x02
#define H1_PLANE_1_FAIL          0x04
#define H1_PLANE_2_FAIL          0x08
#define H1_PLANE_3_FAIL          0x10

/*******************************     IRQ FLAGS        *****************************/
#define H1_NO_IRQ_TRIGGER             0x0
#define H1_IRQ_ENABLE                 0x1
#define H1_IRQ_FIFO_ERROR             0x2
#define H1_IRQ_PROTECTION_ERROR       0x4
#define H1_IRQ_FREADY_TRIGGER         0x8
#define H1_IRQ_CLEAR_IRQ_FREADY       0x10
#define H1_IRQ_CLEAR_IRQ_PROTECTION   0x20
#define H1_IRQ_CLEAR_IRQ_FIFO         0x40
     
/*******************************     ECC FLAGS         *****************************/
#define H1_NO_ECC                     0x0
#define H1_BCH_ENABLE                 0x2
#define H1_HM_ENABLE                  0x4
#define H1_AUTO_PARITY                0x8

/*******************************     INTLV_FLAG       *****************************/
#define H1_INTLV_1_8_BIT              0x01
#define H1_INTLV_1_16_BIT             0x02    /* Currently not supported */
#define H1_INTLV_2_CASCADED           0x04    /* Two 8 bits flash are cascaded to create one virtual 16 bit flash */     
#define H1_INTLV_2_SEPERATED          0x08    /* Two seperate 8 bit flash exists in each bank */

/*******************************     DOWNLOAD FLAGS   *****************************/
#define H1_COPY_0                     0x0
#define H1_COPY_1                     0x1
#define H1_COPY_2                     0x2
#define H1_COPY_3                     0x3
#define H1_FIRST_GOOD_COPY            0x10 
#define H1_ALL_COPIES                 0x20
/*******************************       LPC            *****************************/
#define H1_LPC_FLASH_DATA_INDEX       0x40

/*******************************    LOCK FLAG         *****************************/
#define H1_LOCKED                        1
#define H1_NOT_LOCKED                 0

/*******************************  PROTECTION FLAG     *****************************/
#define H1_PROTECTED                  1
#define H1_NOT_PROTECTED              0

/*******************************  CORRUPTION          *****************************/ 
#define H1_COPY_GOOD                  0x0
#define H1_COPY_CORRUPTED             0x1

/*******************************  PROTECTION FLAGS    *****************************/ 
#define H1_NO_PROTECTION              0x0 
#define H1_WP                         0x1
#define H1_RP                         0x2
#define H1_HW_LOCK                    0x4
/*******************************    DOWNLOAD FLAGS    *****************************/
#define H1_DOWNLOAD_OK                0x0
#define H1_DOWNLOAD_ERROR             0x1  
#define H1_DOWNLOAD_MAIN_UNIT_ERROR   0x2

/*******************************   ERASE_FLAGS        *****************************/
#define H1_WAIT_FOR_READY             0x1

/*******************************   FLASH_TYPE      *****************************/
#define H1_FAST_TYPE                  0x1
#define H1_REG_TYPE                   0x2

/**************************** wFlags definition ***********************************/
#define FL_DPS_2_COPIES_BIT              0x1
#define FL_PAGE_MODE_FLAG_BIT            0x2
#define FL_ALT_MAP                       0x4
#define HW_DMA_ENABLED                   0x8
#define FL_IDENTIFICATION_DONE           0x10

/*******************************   Micellencious      *****************************/
#define H1_CHIP_ID_OREN               0xaa   
#define H1_CHIP_ID_COMPLEMENT         0x55
#define H1_KEY_LENGTH                 8
#define H1_BUSY_DELAY                 400000L
#define H1_FIFO_DELAY                 30000L
#define H1_ICMU_DELAY                 30000L  
#define H1_RECOVER_DELAY              100    
#define H1_ALIAS_RESOLUTION           17
#define H1_UNIQUE_ID_LENGTH           16                     
#define H1_DEFAULT_START_ADR          0xC8000L
#define H1_DEFAULT_STOP_ADR           0xE0000L
#define H1_OTP_LOCK_MARK              0
#define H1_ERASE_MARK                 0x55
#define H1_CUSTOMER_OTP_SIZE          6L*1024L
#define H1_BITS_IN_BYTE               8
#define H1_CHANGEABLE_PROTECTED_AREA  1
#define H1_TOTAL_PROTECTED_AREA       1
#define H1_BBT_BLOCKS_PERPAGE_SHIFT   12
#define H1_BBT_BLOCKS_PERPAGE_MASK    0xfff
#define OP_COMPLETED                  0x0 
#define H1_REG_BAD_BLOCK_PERCENTAGE   5
#define H1_FAST_BAD_BLOCK_PERCENTAGE   2
#define H1_MAX_NUMBER_OF_PLANES       4

/**************************** Sequences offsets ************************************/
/************** Rom sequences **************/ 
#define  RESET_SEQ                      0x07   
#define  READ_ID_SEQ_ROM                0x09       
#define  STATUS_READ_SEQ                0x0D    
/************* Common sequences ************/
#define  READ_ID_SEQ_RAM                0x80 
#define  STATUS_READ_MULTI_PLANE        0x84 
#define  AREA_A_POINT_SEQ               0x87     
#define  AREA_C_POINT_SEQ               0x89  
#define  PAGE_PROG_3_BYTES_AUTO         0x8B 
#define  PAGE_PROG_4_5_BYTES_AUTO       0x90  
#define  PAGE_PROG_4_5_BYTES_DUMMY      0x95  
#define  READ_3_BYTES                   0x9A  
#define  READ_4_BYTES                   0x9D 
#define  BLOCK_ERASE_2_BYTES            0xA0 
#define  BLOCK_ERASE_3_BYTES_1_PLANE    0xA4 
#define  READ_ID_2                      0xA8    
#define  AREA_B_POINT_SEQ               0xAC  
#define  BLOCK_ERASE_3_BYTES_4_PLANES   0xAE 
#define  BLOCK_ERASE_3_BYTES_2_PLANES   0xB8 
/************* 2K page sequences ***********/

#define  PAGE_PROG_1_SECTORS_2K         0x95        
#define  PAGE_PROG_2_SECTORS_2K         0x9B            
#define  PAGE_PROG_3_SECTORS_2K         0xA8        
#define  PAGE_PROG_4_SECTORS_2K         0xB2         
#define  CACHE_PAGE_PROG_4_SECTOR_2K    0xBE        
#define  PAGE_PROG_NEXT_PLANES          0xCA         

#define  COLUMN_ADDRESS_CHANGE          0x87   

#define  READ_SECTOR_SETUP              0x8C     
#define  READ_SECTOR_DATA               0x82     
#define  READ_SECTOR_CACHE              0x80   
#define  READ_SECTOR_CACHE_LAST         0xDC     

#define  BLOCK_ERASE_2K                 0xA4
#define  BLOCK_ERASE_2K_PLANES          0xD6

#define  H1_MAX_DOWNLOAD_TIME_MSEC      20
#define  DPS_H1_SIZE                    17  
#define  H1_NOPS_TO_EXIT_DPD            5
/**********************************************************************************/

/*******************************        ENUMERATIONS         **************************/
typedef enum FL_DPS_IndexEnum
{
  DPS_0 = 0,
  DPS_1 = 1
} FL_DPS_IndexE ;

typedef enum FL_AsicModeEnum 
{
  RESET_MODE      = 0,
  NORMAL_MODE     = 1,
  POWER_DOWN_MODE = 2 
} FL_AsicModeE;

typedef enum FL_EccModeEnum
{
  ECC_WRITE_MODE = 0,
  ECC_READ_MODE  = 1      
}FL_EccModeE;

typedef enum FL_IrqTriggerEnum
{
  IRQ_PROTECTION ,
  IRQ_FREADY     ,
  IRQ_FIFO_ERROR ,
  IRQ_NOT_PENDING 
} FL_IrqTriggerE;

typedef enum FL_IrqTypeEnum
{
  LEVEL_TYPE_INT = 0,
  EDGE_TYPE_INT  = 1
} FL_IrqTypeE;


typedef enum FL_FlashCommandsEnum
{
  READ_AREA_A_CMD        = 0x00 ,
  READ_AREA_B_CMD        = 0x01 ,
  READ_AREA_C_CMD        = 0x50 , 
  READ_ID_CMD            = 0x90 ,
  READ_ID_2_CMD          = 0x91 ,
  PAGE_PROG_CYCLE_1_CMD  = 0x80 , 
  PAGE_PROG_CYCLE_2_CMD  = 0x10 ,
  PAGE_PROG_DUMMY_CMD    = 0x11 ,
  PAGE_PROG_CACHE_CMD    = 0x15,  
  PAGE_ERASE_CYCLE_1_CMD = 0x60 ,
  PAGE_ERASE_CYCLE_2_CMD = 0xD0 ,
  READ_STATUS_CMD        = 0x70 ,   
  READ_STATUS_MULTI_CMD  = 0x71 ,
  RESET_COMMAND          = 0xFF,
  READ_2_CMD             = 0x30,
  READ_DATA_OUTPUT_1     = 0x05,
  READ_DATA_OUTPUT_2     = 0xE0,
  WRITE_DATA_OUTPUT      = 0x85,
  READ_CACHE_NOT_LAST    = 0x31,
  READ_CACHE_LAST        = 0x3F,
  COLUMN_CHANGE_CMD_1    = 0x05,
  COLUMN_CHANGE_CMD_2    = 0xE0
}FL_FlashCommandsE ;

typedef enum FL_ResetTriggerEnum
{
  BOOT_DETECTOR_TRIGGER,
  RSTIN_TRIGGER, 
  NOT_KNOWN_TRIGGER
} FL_ResetTriggerE ; 

typedef enum FL_Reg8_Enum
{ 
  H1_OUTPUT_CONTROL_REG_E      = 0,
  H1_INTERRUPT_CONTROL_REG_E   = 1,
  H1_FLASH_ACCESS_CONFIG_REG_E = 2,
  H1_ECC_CONTROL_REG_1_E       = 3,
  H1_CONFIGURATION_REG_E       = 4
} FL_Reg8_E ;

typedef enum FL_Reg16_Enum
{ 
  H1_BUS_SWAP_REG_E            = 0
} FL_Reg16_E ;


 
/*******************************      STRUCTS         ******************************/
typedef struct 
{   
    FLByte banks                 ; /* 2 bits for low bank            */
                                   /* 2 bits for high bank           */ 
                                   /* 4 bits reserved as 0           */
    FLByte rfu_0_b_1             ; /* 8 bits reserved as 0           */
    LEushort lowUnit             ; /* low unit physical unit number  */
    LEushort highUnit            ; /* high unit physical unit number */
    FLWord rfu_0_w_1             ; /* 16 bits reserved as 0          */                     
    FLByte protectionKey[8]      ; /* 64 bits protection key         */
    FLByte protectionType        ; /* 1 bits reserved as 0           */
                                   /* 1 bits read protection         */ 
                                   /* 1 bits write protection        */
                                   /* 1 bits lock set                */
                                   /* 4 bits revision number         */
    FLByte rfu_0_b_2             ; /* 8 bits reserved as 0           */
} DPS_Struct ,*DPS_StructPtr ;

typedef enum FL_DPS_OFFSET_E
{ 
  DPS_H1_BANKS           = 0x00 ,
  DPS_H1_RFU_0           = 0x01 ,
  DPS_H1_LOW_UNIT_LSB    = 0x02 ,
  DPS_H1_LOW_UNIT_MSB    = 0x03 ,
  DPS_H1_HIGH_UNIT_LSB   = 0x04 ,
  DPS_H1_HIGH_UNIT_MSB   = 0x05 ,
  DPS_H1_RFU_1           = 0x06 ,
  DPS_H1_RFU_2           = 0x07 ,
  DPS_H1_PROTECTION_KEY  = 0x08 ,
  DPS_H1_PROTECTION_TYPE = 0x10  	  
} FL_H1_DPS_E ;


typedef struct  
{
  /*************************    Input parameters    ********************************/
  PhyUnitType  *phyUnits ;        /*** The actual physical units                          ***/
  FLWord        phyPage ;         /*** The page that the units reside in                  ***/
  FLByte        floor ;           /*** The floor                                          ***/
  FLByte        bank ;            /*** The bank                                           ***/
  FLWord        noOfPhySectors ;  /*** Number of physical units to access in  parallel    ***/  
  FLByte        sectorOffset;     /*** Offset of sector in page                           ***/    
  FLByte FAR1  *mainBuffer ;      /*** Pointer to the main buffer                         ***/
  FLByte FAR1  *extraBuffer;      /*** Pointer to the extra buffer                        ***/
  FLDword       opFlags ;         /*** Operation flags                                    ***/  
  FLByte        verifyWrite ;     /*** Verify that write succeeded                        ***/
  /*************************    RawMode parameters   *******************************/ 
  FLWord        offset;           /*** Offset when working in RAW mode                    ***/    
  FLWord        length;           /*** Length when working in raw mode                    ***/ 
  /**************************    Output parameters   ********************************/
  FLSByte       maxBitErrors;      /*** In verify write option it returns the Max errors in sector read ***/ /* cahr is used because of the EDC code */
  FLDword      noOfSectorsPassed; /*** Number of sectors that were successfully passed   ***/                                      
  FLDword      freeSectorsFound;  /*** Number of  free sectors found                        ***/   
  /**********************************************************************************/
} H1_OperationArgsStruct ,*H1_OperationArgsStructPtr ;

typedef struct 
{
  FLDword  LockMark ;
  LEulong  COTP_Size ;
} customerLockStruct,*customerLockStructPtr ;

typedef struct 
{
  FLWord wH1_BUS_SWAP_REG_Val ;
  FLByte bH1_CONFIGURATION_REG_Val ; 
  FLByte bH1_INTERRUPT_CONTROL_REG_Val ;
  FLByte bH1_OUTPUT_CONTROL_REG_Val ;
  FLByte bH1_FLASH_ACCESS_CONFIG_REG_Val ;
  FLByte bH1_ECC_CONTROL_REG_1_Val ;
} H1_RegSavedVal_S,*H1_RegSavedVal_P ;

#endif /* MTD_H1_H */

