/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/logicmtd.h-arc  $
 * 
 *    Rev 1.2   Sep 12 2005 13:18:18   roneng
 * Add MTD buffer
 * 
 *    Rev 1.1   Mar 29 2005 17:00:48   oris
 * Added FL_MATCHING_FROM_HALF flag for H1 support
 * 
 *    Rev 1.0   Mar 27 2005 09:27:24   oris
 * Initial revision.
 * 
 *    Rev 1.3   Mar 27 2005 09:17:16   oris
 * - Added H1 MTD support
 * 
 *    Rev 1.2   Mar 07 2005 22:05:04   oris
 * -	Renamed MTD private method from (*asicSetInNormalMode) to (*CheckandRecoverResetMode)
 * -	Added some constants from MTD use (used in resume code) ENTRY_POINT / EXIT_POINT
 * 
 *    Rev 1.1   Feb 01 2005 09:20:00   eugen
 * new MTD parameters, RegSavedVal_S and ReadCacheParams_S structures
 * 
 *    Rev 1.0   Dec 09 2004 18:24:30   oris
 * Initial revision.
 * 
 *    Rev 1.6   Sep 22 2004 13:01:48   roneng
 * MTD for TFFS6.3 - Basic functionality
 * 
 *    Rev 1.5   Aug 10 2003 13:50:34   roneng
 * - Add bUnitsInChipBits field in MTD_VARS struct
 * 
 *    Rev 1.4   May 28 2003 18:07:50   roneng
 * Remove MTD_VARS->bNumOfUnitBits
 * 
 *    Rev 1.3   May 27 2003 20:10:46   roneng
 * - Add bTimesRecovering to avoid recovering from seq error in endless loop
 * 
 *    Rev 1.2   May 16 2003 00:57:18   OriS
 * Added wChipId field.
 * 
 *    Rev 1.1   May 06 2003 11:56:04   OriS
 * Added support for complete operation mode.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:54   OriS
 * Initial revision.
 * 
 */

/************************************************************************
 * Name     : logicalmtd.h                                              *
 *                                                                      *
 * Written  : Ronen Golan                                               *
 *                                                                      *
 * Abstract : This file contains the routines of the Logical OREN MTD   *
 ************************************************************************/
#ifndef MTD_LOGICAL_H
#define MTD_LOGICAL_H

#include "matchalg.h"

/*******************************   #define      *****************************/
#define USE_BBT_CACHE               0x1
#define ALWAYS_READ_FROM_FLASH      0x2
#define MAX_BYTES_NEEDED_FOR_REG_SAVING 20
#define MAX_16_BITS_REGS            5 
#define MAX_8_BITS_REGS             15 
#define ENTRY_POINT                 1
#define EXIT_POINT                  2

#define FL_MATCHING_FROM_HALF       0x8000 

typedef struct 
{
  FLByte       bFloor ;
  FLByte       bBank ;
  FLByte       bNoOfPhyUnits ;        /* Erase only								  */
  FLWord       wNoOfPhySectors  ;     /* Write only								  */
  FLWord       FirstPhyUnitInPlane ;  /* Erase only (where to put the erase Mark) */
} completeOpVarsStruct,*completeOpVarsStructPtr ;

typedef struct 
{
  FLWord wordReg[MAX_16_BITS_REGS] ;
  FLByte byteReg[MAX_8_BITS_REGS] ;
} RegSavedVal_S ;
typedef struct 
{
  RegSavedVal_S RegSavedVal ; 
  FLByte DPS_0_Key[PROTECTION_KEY_LENGTH] ;
  FLByte DPS_1_Key[PROTECTION_KEY_LENGTH] ;
} SavedDOC_State_S,*SavedDOC_State_P ;

typedef struct 
{
  FLDword   dwCacheStartSector ;
  FLDword   dwCacheFlashMode ;
  FLBoolean fDataCacheValid ;
  FLBoolean fLastOpReadCache ;
} ReadCacheParams_S ;

typedef struct MTDVars_T 
{
  /*******************************************************************************/
  /****************************      Buffers       *******************************/
  /*******************************************************************************/    
  FLBuffer*               buffer; /* buffer for map through buffer   */
#ifdef FL_VERIFY_WRITE
  FLByte FAR1 *           readBackBuffer; /* buffer for read back    */
#endif /* FL_VERIFY_WRITE */
  /*******************************************************************************/
  /*************************    Flash properties   *******************************/
  /*******************************************************************************/  
  FLWord              wMakerCode  ;       /* Maker  code that the flash returned   */
  FLWord              wDeviceCode ;       /* Device code that the flash returned   */
  FLByte              bNoOfPlanesBits ;   /* How many planes exist in chip         */   
  FLDword             dwChipSize;         /* chip size                             */
  FLByte              bAddressBytes;      /* Address bytes for READ/WRITE operation*/
  FLByte              bFlashType;         /* SLC_TYPE / MLC_TYPE                   */
  FLByte              bPlaneArraysInChip; /* No of plane arrays in chip            */
  /*******************************************************************************/
  /*************************    Page properties   ********************************/
  /*******************************************************************************/
  FLWord              wPageSize;          /* size of flash page in bytes          */
  FLByte              bPageSizeBits;      /* How many bits the page size contains */
  FLWord              wPageAreaSize;      /* Size of AREA_A/AREA_B                */
  FLWord              wPageTailSize;      /* Size of AREA_C                       */
  FLWord              wPagesPerBlockBits; /* How many bits used for page in block */ 
  /*******************************************************************************/
  /*************************    Unit properties   ********************************/
  /*******************************************************************************/
  FLDword             dwUnitSize;         /* Smallest physically erasable size    */
  FLWord              wUnitsInChip;       /* total erasable blocks in flash device*/
  FLByte              bUnitsInChipBits;   /* No Of Bits representing unit in addr */
  /*******************************************************************************/
  /*************************      Bank properties     ****************************/
  /*******************************************************************************/
  FLWord              wBanksInFloor;      /* no of chips in array                */
  FLWord              wBanksInLastFloor;  /* How many banks exists in last floor */
  /*** INTERLEAVE_FLAGS:                                                       ***/      
  /*** INTLV_1_8_BIT    - 8  bit flash is connected to each bank            ***/
  /*** INTLV_1_16_BIT   - 16 bit flash is connected to each bank            ***/
  /*** INTLV_2_CASCADED - 2 8 bit flashes is connected to each bank         ***/ 
  FLByte              bInterleaveFlag;    
  /*******************************************************************************/
  /************************   Logical Block properties  **************************/
  /*******************************************************************************/
  FLDword            dwLogicalUnitsInBank  ;
  FLDword            dwLogicalUnitsInFloor ;
  FLDword            dwLogicalUnitsInPlatform ;
  /*******************************************************************************/
  /**********************  Logical Sector Properties    **************************/
  /*******************************************************************************/
  FLByte             bSectorsInPage; 
  FLDword            dwSectorsInUnit;
  FLDword            dwSectorsInLogicalUnit;
  FLDword            dwSectorsInChip ;
  FLDword            dwSectorsInBank;
  FLDword            dwSectorsInFloor;
  FLWord             wSectorsInLogicalPage ;
  FLByte             bSectorsInFloorBits ; 
  FLByte             bSectorsInBankBits ; 
  FLByte             bSectorsInLogicalUnitBits ; 
  FLByte             bSectorsInLogicalPageBits ;
  /*******************************************************************************/
  /**********************  Matching Algorithm Vars   *****************************/
  /*******************************************************************************/
  void              *MatchingAlgVars ;     
  /*******************************************************************************/
  /**********************  Exported routines from MatchingAlg Struct *************/
  /*******************************************************************************/
  FLStatus         (*isGoodBlock)                       (FLFlash *,FLDword,FLByte,FLBoolean *);
  FLStatus         (*CheckandRecoverResetMode)          (FLFlash *,FLByte);
  void             (*MATCH_init)                        (FLFlash *) ; 
  FLStatus         (*MATCH_unitLogicalToPhysical)       (FLFlash *,LogicUnitType,PhyUnitType *) ; 
  FLBoolean        (*MATCH_searchPhyUnitInLOOKUP_Table) (FLFlash *,PhyUnitType,FLByte,FLByte,LogicUnitType *) ;
  /*******************************************************************************/
  /**********************  LastOperation  Properties    **************************/
  /*******************************************************************************/
  
  /*******************************************************************************/
  /**********************            Misc.                ************************/
  /*******************************************************************************/
  FLWord                  wFlags; /* bitwise: BIG_PAGE, SLOW_IO etc. */
  FLByte                  FirstUsableBlock[FL_MAX_NO_OF_FLOORS] ;
  FLDword                 LastOperation ; 
  completeOpVarsStruct    completeOpVars ; 
  LastOperationResults*   LastOperationResultsPtr ; 
  FLByte                  bLastFloor;
  FLByte                  bLastBBT_Page ;
  MTDCallBackRoutine      WriteCallBackRoutinePtr ;
  void                    *WriteCallBackArgument ;
  MTDCallBackRoutine      EraseCallBackRoutinePtr ;
  void                    *EraseCallBackArgument ; 
  SavedDOC_State_P        SavedDOC_State_Ptr ; 
  FLWord                  wChipId;
  FLByte                  bTimesRecovering ;
  FLByte                  bCurrentFloor ;
  FLByte                  bCurrentArea[FL_MAX_NO_OF_FLOORS] ;
  FLByte                  bCurrentFlashMode[FL_MAX_NO_OF_FLOORS] ;
  FLBoolean               fSeqReadCurrentlyEn[FL_MAX_NO_OF_FLOORS] ;
  FLWord                  wBurstWord ;
  FLByte                  bDiskOnChipMode ; 
  FLWord                  wIRQ_Word ;
  FLWord                  wDMA_Word ;
  FLWord                  wDMA_Control_0_Word ;
  FLWord                  wDMA_Control_1_Word ;
  DMA_Params_S            DMA_Params ; 
  ReadCacheParams_S       ReadCacheParams ;
  FLByte                  bMTDBuf[FLASH_SECTOR_DATA_SIZE<<2];
  /*******************************************************************************/
} MTDVarsStruct ;

#define   MTD_VARS  ((MTDVarsStruct *) flashPtr->mtdVars)

#endif /* MTD_LOGICAL_H */

