/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/m512_mtd.h-arc  $
 * 
 *    Rev 1.6   Jul 17 2005 19:34:02   oris
 * - Reduced max busy delay to 400,000 cycles (20ms/50ns)
 * 
 *    Rev 1.5   Jul 14 2005 06:37:20   oris
 * Bug fix: FL_BYTE_SWAP did not work on big endian platforms.
 * 
 *    Rev 1.4   Jul 06 2005 23:48:52   OriS
 * Bug fix: Wait for ready timeout counter was too short. This is a theoretical bug, as it was never found on a real device.
 * 
 *    Rev 1.3   Apr 21 2005 02:31:40   oris
 * - Removed the M512_ prefix from:
 *   FL_IRQ_RB_INIT, FL_DPD_INIT, FL_PAGE_MODE_INIT,
 *   FL_TURBO_MODE_INIT, FL_DPS_2_COPIES_INIT, FL_DMA_INIT
 * - Added FL_ prefix to FL_INT_RB_DISABLED and FL_DMA_HW_DISABLED
 * - Added single compile time access layer macros, for easier customization.
 * 
 *    Rev 1.2   Apr 19 2005 02:51:00   oris
 * TrueFFS 6.3.0 - EQA9
 * 
 *    Rev 1.1   Apr 04 2005 20:16:56   oris
 * - Added defaults for:
 *   M512_FL_IRQ_RB_INIT, M512_FL_DPD_INIT, M512_FL_PAGE_MODE_INIT
 *   M512_FL_TURBO_MODE_INIT, M512_FL_DPS_2_COPIES_INIT, M512_FL_DMA_INIT
 * 
 *    Rev 1.0   Mar 27 2005 09:27:24   oris
 * Initial revision.
 * 
 *    Rev 1.4   Mar 27 2005 09:19:36   oris
 * - Added definition for the number of required NOP for exiting burst mode
 * 
 *    Rev 1.3   Mar 09 2005 02:14:32   oris
 * Changed comments and spelling
 * 
 *    Rev 1.2   Mar 07 2005 22:45:00   oris
 * -	Added support for packed structures even when compiler does not support this mode.
 * -	Added definition for DMA alignment (M512_DOC_DMA_BUS_MASK) currently set to 3 
 * 
 *    Rev 1.1   Feb 01 2005 09:21:12   eugen
 * FL_Reg8_E and FL_Reg16_E enums,
 * different M512_RegSavedVal structure,
 * M512_NEXT_PAGE_MASK_BITS 0xF instead of 0x4
 * 
 *    Rev 1.0   Dec 09 2004 18:24:30   oris
 * Initial revision.
 * 
 *    Rev 1.6   Jun 09 2004 14:41:28   roneng
 * Change M512_FLASH_DATA_REG to 0x103C
 * 
 *    Rev 1.5   Jan 27 2004 08:14:00   oris
 * Changed comments
 * 
 *    Rev 1.4   Sep 21 2003 09:34:14   OriS
 * Added backward compatability flag FL_XSCALE_BOOT_MODE
 * 
 *    Rev 1.3   May 18 2003 15:32:14   OriS
 * Fixed spaces
 * 
 *    Rev 1.2   May 12 2003 20:38:20   roneng
 * - Change the COTP struct
 * 
 *    Rev 1.1   May 06 2003 11:55:58   OriS
 * Wrong address for NOP register
 * 
 *    Rev 1.0   Apr 09 2003 12:15:54   OriS
 * Initial revision.
 * 
 */

/************************************************************************
 * Name    : m512_mtd.h													*
 *																		*
 * Written : Ronen Golan												*
 *																		*
 * Abstract: This file contains the prototypes of the M512 MTD			*
 ************************************************************************/

#ifndef MTD_M512_H
#define MTD_M512_H


/***************** Intenral unused flags - DO NOT USE **********************/
/* #define FL_SET_CALLBACK_FEATURE   */ /* Compile call back rouins in R/B   */
/* #define FL_MTD_EXTENDED_AREA_INFO */ /* Compile code to read from the     */
                                        /* flash hidden (extra) area data    */
/* #define FL_PLANT_BBT_FOR_TESTING  */ /* Allow user application to "tell"  */
                                        /* the MTD about bad blocks. This    */
                                        /* is used for testing of BB support */
/* #define FL_VRAM_FAKE_SUPPORT      */ /* Internal mode for supporting      */
                                        /* virtual RAM on platforms that do  */
                                        /* not support CPU hold signal       */
/* #define DEBUG_MTD                 */ /* DO NOT USE - MTD debugging        */


/***************************  Definitions    ****************************/
#define MTD_VARS         ((MTDVarsStruct *) flashPtr->mtdVars)
#define MTD_VARS_BUFFER  (MTD_VARS->buffer->flData)        

/****************************  Typedefs      ****************************/
#ifndef FL_IRQ_RB_INIT    
#define FL_IRQ_RB_INIT(wSockeNo)            FL_INT_RB_DISABLED
#endif

#ifndef FL_DPD_INIT
#define FL_DPD_INIT(wSockeNo)               FL_DPD_DISABLED
#endif

#ifndef FL_PAGE_MODE_INIT
#define FL_PAGE_MODE_INIT(wSockeNo)         FL_DISABLED
#endif

#ifndef FL_TURBO_MODE_INIT
#define FL_TURBO_MODE_INIT(wSockeNo)        FL_NONTURBO_MODE
#endif

#ifndef FL_DPS_2_COPIES_INIT
#define FL_DPS_2_COPIES_INIT(wSockeNo)      FL_DISABLED 
#endif

#ifndef FL_DMA_INIT
#define FL_DMA_INIT(wSockeNo)               FL_DMA_HW_DISABLED
#endif

/************************* Run Time Definitions *************************/
#define MAX_BANKS              4
#define PAGE_INFO_LENGTH       7
#define BCH_ECC_LENGTH         7 
#define HM_ECC_LENGTH          1 
#define FOUNDRY_DATA_SIZE      0x200  
#define FOUNDRY_RESERVED_SIZE  8

/************************ Access layer routines *************************/
#ifndef M512_Write8BitReg
#define M512_Write8BitReg(flash,offset,val)                 flDirectWrite8BitReg(flash,offset,val)
#endif
#ifndef M512_DirectWrite16BitReg
#define M512_DirectWrite16BitReg(flash,offset,val)          flDirectWrite16BitReg(flash,offset,val)
#endif
#ifndef M512_DirectRead8BitReg
#define M512_DirectRead8BitReg(flash,offset)                flDirectRead8BitReg(flash,offset)
#endif
#ifndef M512_DirectRead16BitReg 
#define M512_DirectRead16BitReg(flash,offset)               flDirectRead16BitReg(flash,offset)
#endif
#ifndef M512_ReadEvenNumberOfBytes
#define M512_ReadEvenNumberOfBytes(flash,offset,dest,count) flReadEvenNumberOfBytes(flash,offset,dest,count)
#endif
#ifndef M512_WriteEvenNumberOfBytes
#define M512_WriteEvenNumberOfBytes(flash,offset,src,count) flWriteEvenNumberOfBytes(flash,offset,src,count)
#endif
#ifndef M512_Read512Bytes
#define M512_Read512Bytes(flash,offset,dest)                flRead512Bytes(flash,offset,dest)
#endif
#ifndef M512_Write512Bytes
#define M512_Write512Bytes(flash,offset,src)                flWrite512Bytes(flash,offset,src)
#endif
#ifndef M512_DocWindow
#define M512_DocWindow(flash)                               flDocWindow(flash)
#endif
/*************************    Include files      *********************/

/*************************    Alignment      *********************/

#ifndef M512_DOC_DMA_BUS_MASK
#define M512_DOC_DMA_BUS_MASK        3 /* Allign to 32 bits */
#endif

/***************************     MEMORY MAP       **********************/
#define M512_IPL_AREA_0              0x0000
#define M512_IPL_AREA_1              0x1800
#define M512_IPL_AREA_0_PAGE_1       0x0400

#ifndef FL_XSCALE_BOOT_MODE
#define M512_FLASH_DATA_REG_ALIASED  0x0800 
#else /* Do not read from 4 bytes aligned address */
#define M512_FLASH_DATA_REG_ALIASED  0x0802 /* Prevent access to 32bit alligned address */ 
#endif /* FL_XSCALE_BOOT_MODE */

#define M512_MEMORY_MAP_SIZE         0x2000    /* Size of memory window is - 8K */
#define M512_IPL_CODE_SIZE           0x800     /* IPL size is 2KByte            */

/************************** Register Address Map ************************/
/**************************   DOC CONTROL REGS   ************************/
#define M512_PAGE_COMMAND_REG           0x0030  /* WRITE      */
#define M512_PAGE_SELECT_REG            0x0070  /* WRITE      */
#define M512_CHIP_ID_REG                0x1000  /* READ       */ 
#define M512_CHIP_ID_CONFIRM_REG        0x1074  /* READ+WRITE */
#define M512_NOP_REG                    0x103E  /* READ+WRITE */ 
#define M512_ALIAS_RESOLUTION_REG       0x1004  /* READ+WRITE */ 
#define M512_BUS_LOCK_REGISTER          0x1006  /* READ+WRITE */ 
#define M512_BUS_SWAP_REG               0x1008  /* READ+WRITE */ 
#define M512_DEVICE_ID_SELECT_REG       0x100A  /* READ+WRITE */ 
#define M512_DOC_CONTROL_REG            0x100C  /* READ+WRITE */ 
#define M512_DOC_CONTROL_CONFIRM_REG    0x1072  /* READ+WRITE */ 
#define M512_CONFIGURATION_REG          0x100E  /* READ+WRITE */ 
#define M512_INTERRUPT_CONTROL_REG      0x1010  /* READ+WRITE */ 
#define M512_FOUNDRY_TEST_REG           0x1076  /* READ+WRITE */ 
#define M512_OUTPUT_CONTROL_REG         0x1014  /* READ+WRITE */ 
#define M512_DPD_CONTROL_REG            0x107C  /* READ+WRITE */  
#define M512_RTC_ENABLE_REG             0x1016  /* READ+WRITE */  
#define M512_RTC_REG                    0x1018  /* READ+WRITE */
#define M512_READ_ADDRESS_REG           0x101A  /* READ+WRITE */
#define M512_BURST_CONTROL_REG          0x101C  /* READ+WRITE */
#define M512_INTERRUPT_STATUS_REG       0x1020  /* READ+WRITE */
#define M512_FIRST_GOOD_UNIT_REG        0x1022  /* READ+WRITE */
#define M512_DMA_CONTROL_REG_0          0x1078  /* READ+WRITE */
#define M512_DMA_CONTROL_REG_1          0x107A  /* READ+WRITE */    
#define M512_LAST_DOWNLOAD_RAM_PAGE     0x1028  /* READ       */   
#define M512_VIRTUAL_RAM_STATUS_PAGE    0x1024  /* READ       */   
/**************************   FLASH REGISTERS  **************************/ 
#define M512_FLASH_CONTROL_REG          0x1038  /* READ+WRITE */ 
#define M512_FLASH_SEQUENCE_REG         0x1032  /* READ+WRITE */ 
#define M512_FLASH_COMMAND_REG          0x1034  /* READ+WRITE */ 
#define M512_FLASH_ADDRESS_REG          0x1036  /* READ+WRITE */ 
#define M512_FLASH_DATA_REG             0x103C  /* READ+WRITE */ 
#define M512_FLASH_END_OF_DATA_REG      0x101E  /* READ+WRITE */
/**************************    ECC REGISTERS   **************************/
#define M512_FLASH_BCH_PARITY_REG       0x1048  /* READ+WRITE */ 
#define M512_HAMMING_PARITY_REG         0x1046  /* READ       */ 
#define M512_ECC_PRESET_REG             0x1044  /* READ+WRITE */ 
#define M512_ECC_CONTROL_REG_0          0x1040  /* READ+WRITE */ 
#define M512_ECC_CONTROL_REG_1          0x1042  /* READ+WRITE */ 
/************************** PROTECTION REGISTERS **************************/
#define M512_PROTECTION_STATUS_REG      0x1056  /* READ       */ 
#define M512_DPS0_ADDRESS_LOW_REG       0x1060  /* READ       */ 
#define M512_DPS0_ADDRESS_HIGH_REG      0x1062  /* READ       */ 
#define M512_DPS1_ADDRESS_LOW_REG       0x1064  /* READ       */ 
#define M512_DPS1_ADDRESS_HIGH_REG      0x1066  /* READ       */ 
#define M512_DPS0_STATUS_REG            0x106c  /* READ       */ 
#define M512_DPS1_STATUS_REG            0x106e  /* READ       */  
#define M512_DPS_0_KEY_REG              0x105c  /* WRITE 	  */ 
#define M512_DPS_1_KEY_REG              0x105e  /* WRITE      */
/**************************  DOWNLOAD REGISTERS  **************************/
#define M512_DOWNLOAD_STATUS_REG        0x1070  /* READ+WRITE */ 
/**************************************************************************/

/**********************************************************************************
 *****************************   Register Description   ***************************
 **********************************************************************************/

/*****************************   M512_BUS_SWAP_REG       **************************/
#define M512_SWAP_L                      0x0001
#define M512_SWAP_H                      0x0100 
#define M512_SWAP_DATA_BYTES             (M512_SWAP_H | M512_SWAP_L) 

/*****************************   M512_DOC_CONTROL_REG    **************************/
#define M512_MODE_BITS                   0x03
#define M512_MODE_WRITE_ENABLE           0x04
#define M512_BDETCT_RESET                0x08  
#define M512_RSTIN_RESET                 0x10
#define M512_RAM_WE                      0x20

/*****************************  M512_CONFIGURATION_REG   **************************/
#define M512_VDET_OUTPUT                 0x01
#define M512_BOOT_DETECTOR_INHIBIT       0x08
#define M512_MAX_ID_BITS                 0x30
#define M512_IF_CFG_STATUS               0x80

/***************************** M512_INTERRUPT_CONTROL_REG *************************/
#define M512_IRQ_ENABLED                 0x8000
#define M512_IRQ_EDGE_SENSITIVE          0x4000
#define M512_IRQ_ENABLE_DMA_INT          0x2000
#define M512_IRQ_ENABLE_RTC_INT          0x1000
#define M512_IRQ_ENABLE_BCH_ECC_INT      0x0800
#define M512_IRQ_ENABLE_DATA_OVF_INT     0x0400
#define M512_IRQ_ENABLE_PROTECTION_INT   0x0200
#define M512_IRQ_ENABLE_FREADY_INT       0x0100
#define M512_IRQ_MASK_DMA_INT            0x0020
#define M512_IRQ_MASK_RTC_INT            0x0010
#define M512_IRQ_MASK_BCH_ECC_INT        0x0008
#define M512_IRQ_MASK_DATA_OVF_INT       0x0004
#define M512_IRQ_MASK_PROTECTION_INT     0x0002
#define M512_IRQ_MASK_FREADY_INT         0x0001

/*****************************  M512_INTERRUPT_STATUS_REG  *************************/
#define M512_IRQ_DMA_PENDING_INT          0x20
#define M512_IRQ_RTC_PENDING_INT          0x10
#define M512_IRQ_BCH_PENDING_ECC_INT      0x08
#define M512_IRQ_DATA_OVF_PENDING_INT     0x04
#define M512_IRQ_PROTECTION_PENDING_INT   0x02
#define M512_IRQ_FREADY_PENDING_INT       0x01
  
/*****************************    M512_FOUNDRY_TEST_REG    *************************/
#define M512_SOFT_RESET                  0x80
#define M512_WR_ALL_PIN                  0x08   
#define M512_TEST_A_MODE                 0x04   /* In this mode the boot detector will be incremented by 11H instead of 01H */
#define M512_DOWNLOAD_TEST_MODE          0x02   /* Download test mode - data is downloaded from D[7:0] instead of the flash */
#define M512_FLASH_TEST_MODE             0x01   /* When set provides all the pin signals on the I/O pins */

/*****************************    M512_OUTPUT_CONTROL_REG  *************************/
#define M512_ASSERT_BUSY_DURING_DNLD     0x01
#define M512_DISABLE_PULL_UP_BITS        0x02
#define M512_TURBO_MODE                  0x04
#define M512_PAGE_READ_MODE              0x10 

/*******************************  M512_DPD_CONTROL_REG     *************************/
#define M512_POWER_DOWN_OK               0x80
#define M512_POWER_DOWN_MODE_BITS        0x0F

/*******************************    M512_DMA_CONTROL_REG_0     *********************/
#define M512_DMA_ENABLE                  0x8000
#define M512_DMA_PAUSE                   0x4000L
#define M512_DMA_EDGE_MODE               0x2000
#define M512_DMA_DMARQ_POLARITY          0x1000
#define M512_SECTOR_COUNT_BITS           0x007F
  
/*******************************    M512_DMA_CONTROL_REG_1     *********************/
#define M512_NEGATE_COUNT_BITS           0x03FF

/*******************************    M512_DMA_CONTROL_REG_1     *********************/
#define M512_NEGATE_COUNT_BITS           0x03FF

/******************************* M512_BURST_MODE_CONTROL_REG   *********************/
#define M512_BURST_ENABLE                0x0001
#define M512_BURST_CLOCK_INVERT          0x0002
#define M512_BURST_HOLD                  0x0004
#define M512_BURST_LATENCY_BITS          0x0F00
#define M512_BURST_LENGTH_BITS           0xF000

/*******************************  M512_CLOCK_COUNTER_REG       *********************/
#define M512_CLOCK_COUNTER_RESET         0x8000 /* The counter starts to count from zero */
#define M512_CLOCK_COUNTER_STOP          0x4000 /* The counter is stopped until CLOCK_COUNTER_RESET is set */  
#define M512_CLOCK_COUNTER_BITS          0x3FFF   
 
/*******************************   M512_READ_ADDRESS_REG   *************************/ 
#define M512_ADDR_INCREMENTED            0x8000
#define M512_ADDR_ACCESS_ONE_BYTE        0x4000
#define M512_ADDR_ADDRESS_BITS           0x1FFF

/*******************************   M512_REAL_ADDRESS_REG   *************************/ 
#define M512_ADDR_INCREMENTED            0x8000
#define M512_ADDR_READ_ONE_BYTE          0x4000
#define M512_ADDR_ADDRESS_BITS           0x1FFF

/*******************************   M512_RTC_ENABLE_REG     *************************/
#define M512_ENABLE_RTC                  0x0001 

/*******************************   M512_FLASH_CONTROL_REG  **************************/
#define M512_FWP_SIGNAL_ACTIVE           0x20   /* After protocol violation detected we must clear this bit */
#define M512_FCE_SIGNAL_ACTIVE           0x10   /* Controls the assertion of the CE# to the BANK defined by the BANK bits */ 
#define M512_SEQ_STATE_MACHINE_READY     0x08
#define M512_PROTECTION_ERROR_DETECTED   0x04   /* Protection Error & will be cleared after FLASH reset command */
#define M512_SEQUENCE_ERROR_DETECTED     0x02   /* Sequence Error detected & will be cleared only by FLASH reset command */
#define M512_FLASH_READY                 0x01   /* FLASH READY/BUSY# output signal */

/*******************************   M512_ECC_CONTROL_REG_0  ************************/
#define M512_DATA_COUNTER_BITS           0x07FF    
#define M512_BCH_EN                      0x0800     
#define M512_HAMMING_EN                  0x1000
#define M512_AUTO_ECC_READ_WRITE_EN      0x4000     
#define M512_FLASH_READ_MODE             0x8000 /* Host is doing a read operation */
#define M512_ECC_0_READ_DATA_EXTRA       0x9A0F

#define M512_ECC_0_READ_DATA_EXTRA_NOECC 0x820F
#define M512_ECC_0_READ_DATA_EXTRA       0x9A0F

/*******************************   M512_ECC_CONTROL_REG_1  ************************/
#define M512_HM_DATA_NUM_BITS            0x0F
#define M512_PAGE_READ_IS_USED           0x20
#define M512_IGNORE_DATA                 0x40
#define M512_BCH_SYNDROM_ERROR           0x80                 

/****************************** M512_PROTECTION_STATUS_REG *************************/
#define M512_FOUNDRY_OTP_LOCKED          0x01
#define M512_CUSTOMER_OTP_LOCKED         0x02
#define M512_LOCK_INPUT_NEGATED          0x04
#define M512_STICKY_LOCK_ACTIVATED       0x08
#define M512_PROTECTION_ENABLED          0x10
#define M512_DONT_DOWNLOAD_IPL           0x20
#define M512_PROTECTION_ERROR            0x80

/*******************************         DPS0_STATUS_REG     *************************/
#define M512_OTP_PROTECTED               0x01
#define M512_READ_PROTECTED              0x02 
#define M512_WRITE_PROTECTED             0x04
#define M512_HW_LOCK_ENABLED             0x08
#define M512_KEY_OK                      0x80

/*******************************  M512_FIRST_GOOD_UNIT_REG   *************************/
#define M512_GOOD_UNIT_WRITE_EN          0x80
#define M512_GOOD_UNIT_NUM_BITS          0x7F

/*******************************   FLASH_TYPE      *****************************/
#define M512_FAST_TYPE                   0x1
#define M512_REG_TYPE                    0x2
#define M512_ERASE_MARK                  0xAA

/*******************************  M512_DOWNLOAD_STATUS_REG   *************************/
#define M512_DPS0_DOWNLOAD_ERROR_BITS    0x03
#define M512_DPS0_DOWNLOAD_REDUNDANT     0x01
#define M512_DPS1_DOWNLOAD_ERROR_BITS    0x0C
#define M512_DPS1_DOWNLOAD_REDUNDANT     0x04
#define M512_IPL_DOWNLOAD_ERROR_BITS     0x30
#define M512_IPL_DOWNLOAD_REDUNDANT      0x10
#define M512_OTP_DOWNLOAD_ERROR          0x40 
#define M512_DOWNLOAD_ERROR_BITS         0x4A
#define M512_DOWNLOAD_IN_PROGRESS        0x80

/****************************** M512_VIRTUAL_RAM_STATUS_PAGE  ************************/
#define M512_VIRTUAL_RAM_SUPPORTED       0x80
#define M512_VIRTUAL_RAM_ENABLE          0x01 
#define M512_ALTERNATE_MAP               0x02 
#define M512_PAGED_RAM_FIXED_1K_LOW      0x00
#define M512_PAGED_RAM_FIXED_1K_HIGH     0x01
#define M512_VIRTUAL_RAM_LOADED_BOTTOM   0x02
#define M512_VIRTUAL_RAM_LOADED_TOP      0x03
#define M512_IPL_MODE_BITS               0x03
   
/****************************** M512_PAGE_SELECT_REG           ***********************/
#define M512_PAGE_SELECT_SEQ             0x80
#define M512_PAGE_SELECT_DIRECT          0x00
#define M512_PAGE_SELECT_BITS            0x3F

/****************************** M512_LAST_DOWNLOAD_RAM_PAGE    ***********************/
#define M512_NEXT_UNIT_MASK_BITS         0x3FF0
#define M512_NEXT_PAGE_MASK_BITS         0x000F
#define M512_LAST_DOWNLOAD_MASK_BITS     0x3FFF

/****************************** M512_PAGE_COMMAND_REG          ***********************/
#define M512_PAGE_CMD_READ_NEXT          0x71

/****************************** M512_PAGE_COMMAND_REG          ***********************/
#define M512         0x0004

#define M512_DPS_REVISION_NUMBER         0x10 
/*******************************        ENUMERATIONS         **************************/

/******************************* Flash Parameters  *****************************/
#define M512_SECTOR_BCH_PROTECTED  SECTOR_DATA_SIZE+SECTOR_PAGEINFO_SIZE+SECTOR_HM_SIZE
#define M512_SECTOR_HM_PROTECTED   SECTOR_PAGEINFO_SIZE    

/******************************* Flash Bank interface  *****************************/
#define M512_ONE_FLASH_8_BITS      0x00	  /* Only one 8 bit flash is connected  */
#define M512_ONE_FLASH_16_BITS     0x01   /* only one 16 bit flash is connected */
#define M512_TWO_FLASHES_CASCADED  0x02   /* Two 8 bit flashes cascaded         */
#define M512_TWO_FLASHES_SEPERATED 0x03   /* Two 8 bit flashes connected to the */
                                          /* same bank but the distinction is   */
                                          /* by the BL bit                      */

/*******************************    Flash Status Reg   *****************************/
#define M512_OPERATION_OK          0x00
#define M512_OPERATION_FAILED      0x01   
#define M512_PLANE_0_FAIL          0x02
#define M512_PLANE_1_FAIL          0x04
#define M512_PLANE_2_FAIL          0x08
#define M512_PLANE_3_FAIL          0x10

/*******************************     IRQ FLAGS        *****************************/
#define M512_NO_IRQ_TRIGGER             0x0
#define M512_IRQ_ENABLE                 0x1
#define M512_IRQ_FIFO_ERROR             0x2
#define M512_IRQ_PROTECTION_ERROR       0x4
#define M512_IRQ_FREADY_TRIGGER         0x8
#define M512_IRQ_CLEAR_IRQ_FREADY       0x10
#define M512_IRQ_CLEAR_IRQ_PROTECTION   0x20
#define M512_IRQ_CLEAR_IRQ_FIFO         0x40
     
/*******************************     ECC FLAGS         *****************************/
#define M512_NO_ECC                     0x0
#define M512_BCH_ENABLE                 0x2
#define M512_HM_ENABLE                  0x4
#define M512_AUTO_PARITY                0x8

/*******************************     INTLV_FLAG       *****************************/
#define M512_INTLV_1_8_BIT              0x01
#define M512_INTLV_1_16_BIT             0x02    /* Currently not supported */
#define M512_INTLV_2_CASCADED           0x04    /* Two 8 bits flash are cascaded to create one virtual 16 bit flash */     
#define M512_INTLV_2_SEPERATED          0x08    /* Two seperate 8 bit flash exists in each bank */

/*******************************     DOWNLOAD FLAGS   *****************************/
#define M512_COPY_0                     0x0
#define M512_COPY_1                     0x1
#define M512_COPY_2                     0x2
#define M512_COPY_3                     0x3
#define M512_FIRST_GOOD_COPY            0x10 
#define M512_ALL_COPIES                 0x20

/*******************************    LOCK FLAG         *****************************/
#define M512_LOCKED	                    1
#define M512_NOT_LOCKED                 0

/*******************************  PROTECTION FLAG     *****************************/
#define M512_PROTECTED                  1
#define M512_NOT_PROTECTED              0

/*******************************  CORRUPTION          *****************************/ 
#define M512_COPY_GOOD                  0x0
#define M512_COPY_CORRUPTED             0x1

/*******************************  PROTECTION FLAGS    *****************************/ 
#define M512_NO_PROTECTION              0x0 
#define M512_WP                         0x1
#define M512_RP                         0x2
#define M512_HW_LOCK                    0x4
/*******************************    DOWNLOAD FLAGS    *****************************/
#define M512_DOWNLOAD_OK                0x0
#define M512_DOWNLOAD_ERROR             0x1  
#define M512_DOWNLOAD_MAIN_UNIT_ERROR   0x2

/*******************************   ERASE_FLAGS        *****************************/
#define M512_WAIT_FOR_READY             0x1

/**************************** wFlags definition ***********************************/
#define FL_DPS_2_COPIES_BIT              0x1
#define FL_PAGE_MODE_FLAG_BIT            0x2
#define FL_ALT_MAP                       0x4
#define HW_DMA_ENABLED                   0x8
#define FL_IDENTIFICATION_DONE           0x10

/**************************** M512_BURST_CONTROL_REG ***********************************/
#define M512_BURST_DISABLE              0x0 
/*******************************   Micellencious      *****************************/
#define M512_CHIP_ID                    0x0200   
#define M512_CHIP_ID_COMPLEMENT         0xFDFF
#define M256_CHIP_ID                    0x0100   
#define M256_CHIP_ID_COMPLEMENT         0xFEFF
#define M1G_CHIP_ID                     0x0400   
#define M1G_CHIP_ID_COMPLEMENT          0xFBFF

#define M512_KEY_LENGTH                 8
#define M512_BUSY_DELAY                 400000L
#define M512_PD_DELAY                   30000L
#define M512_DOWNLOAD_DELAY             400000L  
#define M512_FIFO_DELAY                 30000L
#define M512_ICMU_DELAY                 30000L
#define PAGED_RAM_TIMEOUT               300000L
#define M512_ALIAS_RESOLUTION           17
#define M512_UNIQUE_ID_LENGTH           16      
#define M512_FOUNDRY_OTP_SIZE           16               
#define M512_DEFAULT_START_ADR          0xC8000L
#define M512_DEFAULT_STOP_ADR           0xE0000L
#define M512_OTP_LOCK_MARK              0
#define M512_OTP_START_PAGE             8
#define M512_OTP_LOCK_PAGE              0x3E
#define M512_CUSTOMER_OTP_SIZE          6L*1024L
#define M512_BITS_IN_BYTE               8
#define M512_CHANGEABLE_PROTECTED_AREA  2
#define M512_TOTAL_PROTECTED_AREA       2
#define M512_BBT_BLOCKS_PERPAGE_SHIFT   12
#define M512_BBT_BLOCKS_PERPAGE_MASK    0xfff
#define M512_MLC_BB_PERCENTAGE          5
#define M512_NUM_OF_PLANES              2 
#define M512_BBT_PAGE                   4
#define M512_DPS_0_MAIN_UNIT            1
#define M512_DPS_0_REDUNDANT_UNIT       2
#define M512_DPS_1_MAIN_UNIT            3
#define M512_DPS_1_REDUNDANT_UNIT       4
#define M512_RAM_READS_TO_EXIT_DPD      14 /*** Fits also turbo mode ***/
#define OP_COMPLETED			        0x0 
#define M512_MLC_BAD_BLOCK_PERCENTAGE   5
#define M512_FOTP_SIZE                  0x20
#define M512_PRODUCT_TYPE_LOCATION      2 
#define M512_MAX_NUMBER_OF_PLANES       2
#define M512_IPL_NONLP_SIZE             2048


#define FL_AREA_A						0
#define FL_AREA_C						0x50

#define FL_REG_MODE                     0x1
#define FL_REL_MODE                     0x2
#define FL_FAST_MODE                    0x4



#define M512_IRQ_LEVEL_TYPE               0x0 
#define M512_IRQ_EDGE_TYPE                0x4000 
#define M512_IRQ_GLOBAL_INT_ENABLE        0x8000

#define M512_IRQ_RB_EN                    0x0001
#define M512_IRQ_PROTECTION_ERR_EN        0x0002
#define M512_IRQ_DCNT_OVF_ERR_EN          0x0004
#define M512_IRQ_BCH_ERR_EN               0x0008
#define M512_IRQ_RTC_EN                   0x0010
#define M512_IRQ_DMA_COMPLETE_EN          0x0020

#define M512_IRQ_RB_UNMASK                0x0100
#define M512_IRQ_PROTECTION_ERR_UNMASK    0x0200
#define M512_IRQ_DCNT_OVF_ERR_UNMASK      0x0400
#define M512_IRQ_BCH_ERR_UNMASK           0x0800
#define M512_IRQ_RTC_UNMASK               0x0100
#define M512_IRQ_DMA_COMPLETE_UNMASK      0x0200

/*** MDOC512-G3 read access time          ***/
#define M512_READ_ACCESS_TIME_SRAM        90
#define M512_READ_ACCESS_TIME_NONTURBO    56
#define M512_READ_ACCESS_TIME_TURBO       35

#define M512LP_READ_ACCESS_TIME_SRAM      105
#define M512LP_READ_ACCESS_TIME_NORMAL    56
#define M512LP_READ_ACCESS_TIME_PAGED     78 

#define M512_SUBTYPE_G3                  0x00
#define M512_SUBTYPE_G3_LP               0x01

#define M512_VIRTUAL_RAM_MAX_SIZE        8192   /* 8KB  */
#define M512_PAGED_RAM_MAX_SIZE          15360  /* 15KB */

#define M512_READ_PAGED_RAM_DELAY        20
#define M512_VIRTUAL_RAM_NOPS            1000

#define M512_ALL_FLOORS                  5  
#define M512_MAX_DOWNLOAD_TIME_MSEC      20      
#define M512_DPS_SIZE                    16
#define M512_NOPS_EXIT_BURST             7	
/**********************************************************************************/

/***************************       Enumarations     ********************************/    
typedef enum FL_DPS_IndexEnum
{ 
  DPS_0 = 0,
  DPS_1 = 1
} FL_DPS_IndexE ;

typedef enum FL_AsicModeEnum 
{
  RESET_MODE      = 0,
  NORMAL_MODE     = 1,
  POWER_DOWN_MODE = 2 
} FL_AsicModeE;

typedef enum FL_EccModeEnum
{
  ECC_WRITE_MODE = 0,
  ECC_READ_MODE  = 1	  
}FL_EccModeE;

typedef enum FL_IrqTriggerEnum
{
  IRQ_PROTECTION ,
  IRQ_FREADY     ,
  IRQ_FIFO_ERROR ,
  IRQ_NOT_PENDING 
} FL_IrqTriggerE;

typedef enum FL_IrqTypeEnum
{
  LEVEL_TYPE_INT = 0,
  EDGE_TYPE_INT  = 1
} FL_IrqTypeE;

typedef enum FL_BusConfigEnum
{            
  SRAM_MODE = 0 ,
  MUX_MODE  = 1 ,
  LPC_MODE  = 2 ,
  ICP_MODE  = 3 
} FL_BusConfigE;

typedef enum FL_FlashCommandsEnum
{
  /***   Read commands     ***/
  READ_AREA_A_CMD          = 0x00,
  READ_AREA_C_CMD          = 0x50, 
  MULTI_PLANE_READ         = 0x30, 
  /***   Output commands   ***/
  READ_STATUS_CMD          = 0x70,   
  READ_STATUS_MULTI_CMD    = 0x71,
  READ_ID_CMD              = 0x90,
  ADDR_INPUT_FOR_REG_READ  = 0x05,
  REG_READ                 = 0xE0,      
  /***   Program commands  ***/
  PAGE_PROG_CYCLE_1_CMD    = 0x80, 
  PAGE_PROG_CYCLE_2_CMD    = 0x10,
  PAGE_PROG_CACHE_CMD      = 0x15,
  PAGE_PROG_DUMMY_CMD      = 0x11,
  /***  Erase commands     ***/
  BLOCK_ADDRESS_INPUT      = 0x60,
  PAGE_ERASE_CYCLE_2_CMD   = 0xD0,
  /*** Other commands      ***/
  RESET_COMMAND            = 0xFF,
  /*** Mode commands       ***/
  SET_PAGE_SIZE_532_BYTES  = 0x3C,
  SET_FAST_MODE            = 0xA2,
  SET_RELIABLE_MODE        = 0x22,
  SET_AUTO_PAGE_INC_ENABLE = 0xB3
}FL_FlashCommandsE ;

typedef enum FL_ResetTriggerEnum
{
  BOOT_DETECTOR_TRIGGER,
  RSTIN_TRIGGER, 
  NOT_KNOWN_TRIGGER
} FL_ResetTriggerE ; 

typedef enum FL_SequenceID_Enum
{
  RESET_SEQ                 = 0 , 
  PAGE_SIZE_532_BYTES_SEQ   = 3 ,       
  SET_FAST_MODE_SEQ         = 5 ,  
  SET_VPGM_LOW_SEQ          = 7 ,
  SET_RELIABLE_MODE_SEQ     = 9 ,
  ENABLE_AUTO_PAGE_INC_SEQ  = 12,
  AREA_A_POINT_SEQ          = 14,
  AREA_C_POINT_SEQ          = 16,
  READ_SEQ                  = 18,
  PAGE_PROGRAM_SEQ          = 29,
  ERASE_SEQ                 = 39,
  STATUS_READ_SEQ           = 46,
  STATUS_READ_MULTI_PLANE   = 49,
  READ_ID_SEQ               = 52,
  PAGE_BUFFER_LOOP_BACK     = 68,
  CACHE_PROGRAM_SEQ         = 82
} FL_SequenceID_E ; 

typedef enum FL_Reg8_Enum
{ 
  M512_OUTPUT_CONTROL_REG_E       = 0,
  M512_INTERRUPT_CONTROL_REG_E    = 1,
  M512_DPD_CONTROL_REG_E          = 2,
  M512_DMA_CONTROL_REG_1_E        = 3,
  M512_BURST_CONTROL_REG_E        = 4,
  M512_ECC_CONTROL_REG_1_E        = 5,
  M512_CONFIGURATION_REG_E        = 6
} FL_Reg8_E ;

typedef enum FL_Reg16_Enum
{ 
  M512_BUS_SWAP_REG_E             = 0,
  M512_DMA_CONTROL_REG_0_E        = 1
} FL_Reg16_E ;

/*******************************      STRUCTS         ******************************/

typedef struct  
{
  /*************************    Input parameters    ********************************/
  PhyUnitType   *phyUnits ;        /*** The actual physical units					    ***/
  FLByte         phyPage ;         /*** The page that the units reside in			    ***/
  FLWord         sectorOffset;     /*** Offset when working in RAW mode					***/    
  FLDword        opFlags ;         /*** Operation flags								    ***/  
  FLWord         noOfPhySectors ;  /*** Number of physical units to access in  parallel ***/  
  FLByte         floor ;           /*** The floor                                       ***/       
  FLByte FAR1   *mainBuffer ;      /*** Pointer to the main buffer						***/
  FLByte FAR1   *extraBuffer;      /*** Pointer to the extra buffer					    ***/
  FLByte         verifyWrite ;     /*** Verify that write succeeded						***/
  /*************************    RawMode parameters   *******************************/ 
  FLWord         offset;           /*** Offset when working in RAW mode					***/    
  FLWord         length;           /*** Length when working in raw mode					***/ 
  /**************************    Output parameters   ********************************/
  FLSByte        maxBitErrors;     /*** In verify write option it returns the Max errors in sector read ***/ /* cahr is used because of the EDC code */
  FLDword        noOfSectorsPassed;/*** Number of sectors that were successfully passed ***/  							        
  FLDword        freeSectorsFound; /*** Number of  free sectors found				    ***/   
  /**********************************************************************************/
} M512_OperationArgsStruct ,*M512_OperationArgsStructPtr ;

typedef enum FL_DPS_OFFSET_E
{ 
  DPS_LOW_UNIT_LSB       = 0x00 ,
  DPS_LOW_UNIT_MSB       = 0x01 ,
  DPS_HIGH_UNIT_LSB      = 0x02 ,
  DPS_HIGH_UNIT_MSB      = 0x03 ,
  DPS_PROTECTION_KEY     = 0x04 ,
  DPS_PROTECTION_TYPE    = 0x0C ,
  DPS_VRAM_MODE          = 0x0D ,
  DPS_FILL_ZERO          = 0x0E 	  
} FL_DPS_E ;

typedef struct 
{
  LEulong  LockMark ;
  LEulong  COTP_Size ;
} customerLockStruct,*customerLockStructPtr ;

typedef struct 
{
  FLWord wM512_BUS_SWAP_REG_Val ;
  FLWord wM512_INTERRUPT_CONTROL_REG_Val ;
  FLWord wM512_DMA_CONTROL_REG_0 ; 
  FLWord wM512_BURST_CONTROL_REG ;
  FLByte bM512_CONFIGURATION_REG_Val ; 
  FLByte bM512_OUTPUT_CONTROL_REG_Val ;
  FLByte bM512_ECC_CONTROL_REG_1_Val ;
  FLByte bM512_DPD_CONTROL_REG ; 
  FLByte bM512_DMA_CONTROL_REG_1 ;
} M512_RegSavedVal_S,*M512_RegSavedVal_P ;

#endif /* MTD_M512_H */


