/* mqueueCommon.h - POSIX usr/kernel shared message queue header file */

/* Copyright 2003 Wind River Systems, Inc. */

/*
modification history
--------------------
01a,14nov03,m_s  code-inspection changes
01a,31oct03,m_s  Written
*/

#ifndef __INCmqueueCommonh
#define __INCmqueueCommonh


#ifdef __cplusplus
extern "C" {
#endif

/* includes */

#include "fcntl.h"
#include "sigevent.h"

/* defines */

#define MQ_PRIO_MAX		32	/* max priority levels */

#define MQ_NUM_MSGS_DEFAULT	16	/* default messages per queue */
#define MQ_MSG_SIZE_DEFAULT	16	/* default message size in bytes */

/* typedefs */

/* message queue attributes */

struct mq_attr
    {
    size_t	mq_maxmsg;
    size_t	mq_msgsize;
    unsigned	mq_flags;
    size_t	mq_curmsgs;
    };


/* message queue descriptor for applications */
struct mq_des;
typedef struct mq_des * mqd_t;

/* function declarations */

extern mqd_t mq_open    (const char *, int, ...);
extern int   mq_close   (mqd_t);
extern int   mq_unlink  (const char *);
extern int   mq_send    (mqd_t, const void *, size_t, int);
extern int   mq_receive (mqd_t, void *, size_t, int *);
extern int   mq_notify  (mqd_t, const struct sigevent *);
extern int   mq_setattr (mqd_t, const struct mq_attr *, struct mq_attr *);
extern int   mq_getattr (mqd_t, struct mq_attr *);

#ifdef __cplusplus
}
#endif

#endif	/* INCmqueueCommonh */
