/**
   \file        niMemoryFileLib.h
   \author      Charles Crain
   \date        08/16/2005

   COPYRIGHT 2005. NATIONAL INSTRUMENTS. ALL RIGHTS RESERVED.
*/

#ifndef ___niMemoryFileLib_h___
#define ___niMemoryFileLib_h___

#ifdef __cplusplus
extern "C"
{
#endif

   /**
      Given a file descriptor, read that file into memory
      and create a "memory file" around it.

      This function will read gzipped files transparently!

      Once you create a memory file, you can get the fd to it
      by calling niGetMemoryFileFd().  If this function succeeds,
      you must eventually destroy the handle it creates by calling
      niDestroyMemoryFile().

      NOTE!  This function is not strictly reentrant.  Specifically
      you cannot call it with the same file name more than once
      simultaneously.

      \return a handle to a newly created memory file, or NULL upon
      failure.
    */
   void* niCreateMemoryFile(int infd);

   /** Destroy a memory file from niCreateMemoryFile(). */
   void niDestroyMemoryFile(void* memoryFileHdl);

   /**
      Get the file descriptor associated with a memory file.

      You can then use this just as it was any other file descriptor
      only it will be super fast!
    */
   int niGetMemoryFileFd(void* memoryFileHdl);

#ifdef __cplusplus
}
#endif

#endif /* ___niMemoryFileLib_h___ */
