/* poolLibP.h - private header file for Wind Memory Pools */

/* Copyright 1984-2004 Wind River Systems, Inc. */

/*
modification history
--------------------
01c,18feb04,dat  POOL structure is private, not public
01b,20aug03,zl   added item alignment support.
01a,22feb98,dat, original version.  TOR 1.0.1/VXW 5.3 compatible.
	    mas,tm
*/

#ifndef INCpoolLibPh
#define INCpoolLibPh

#ifdef __cplusplus
extern "C" {
#endif

/* Includes */

#include "semLib.h"
#include "poolLib.h"
#include "lstLib.h"


/* Private macros */

#define POOL_SEM_CREATE_ATTRIB \
    (SEM_Q_PRIORITY | SEM_INVERSION_SAFE | SEM_DELETE_SAFE)

#define POOL_SEM_TAKE_ATTRIB	WAIT_FOREVER
#define CREATE_SEM_TAKE_ATTRIB	WAIT_FOREVER

/* indicator of a block that is provided by user */

#define BLOCK_DO_NOT_FREE	0x00000001

/* Pool Block records */

typedef struct pool	/* Pool structure */
    {
    NODE		poolNode;	/* pool list for pond */
    const char *	poolName;	/* pool name */
    SEM_ID		mutex;		/* Mutex to use for thread-safe pools */
    ULONG		sizItem;	/* individual item size in bytes */
    ULONG		sizItmReal;	/* actual item size, due to alignment */
    ULONG		alignment;	/* individual item alignment in bytes */
    ULONG		numIncr;	/* nr. of incrementally allocated itm */
    ULONG		numTotl;	/* total number of items created */
    PART_ID		partId;		/* memory partition ID */
    ULONG		options;	/* pool options word */
    LIST		blockList;	/* allocated block list */
    LIST		freeItems;	/* free item list */
    } POOL;

typedef struct pool_block
    {
    NODE	blockNode;	/* block node for pool */
    void *	pItmBlk;	/* ptr to start of item pool block */
    ULONG	itemCnt;	/* count of items in this block */
    } POOL_BLOCK;


#ifdef __cplusplus
}
#endif

#endif  /* INCpoolLibPh */
