/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    This header file defines the VxWorks specific API function interface
    to Reliance.

    This header file, along with any sub-headers it may include, defines
    the entire set of structures, macros, and function prototypes necessary
    to write a program that interfaces with Reliance.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: reliance_vxworks.h $
    Revision 1.3  2006/01/26 04:23:22  brandont
    Merged with Reliance 2.0 code base.
    Revision 1.5  2006/01/10 23:57:43Z  pauli
    Added prototype for VxWorks 5.4 specific function to initialize the 
    FFX discard wrapper interface.
    Revision 1.4  2006/01/06 01:43:18Z  brandont
    Updated prototype for usrFileDiskInit.
    Revision 1.3  2005/12/17 02:04:39Z  brandont
    Removed references to the obsolete discard and rfsb interfaces.
    Updated prototype for relFsDevCreate and usrRelFsInit. Added
    prototypes for relFsProbeInstall, usrFileDiskInit, and relFsPortTestInit.
    Revision 1.2  2006/01/24 02:14:00Z  brandont
    Removed DCL includes
    Revision 1.1  2005/12/02 18:06:50Z  Pauli
    Initial revision
    Revision 1.3  2005/12/02 18:06:50Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.2  2005/08/31 20:45:31Z  pauli
    Added prototype for getting transaction mode.
    Revision 1.1  2005/04/21 03:41:58Z  GaryP
    Initial revision
---------------------------------------------------------------------------*/

#ifndef RELIANCE_VXWORKS_H_INCLUDED
#define RELIANCE_VXWORKS_H_INCLUDED


/*  There is no need to propagate DCL types to the application level
    interface.
*/
#if 0

#ifndef DCLFUNC
  #define DCLFUNC(fn)                 (Rel##fn)
#endif

#ifndef DCL_H_INCLUDED
  #ifndef DCL_VXWORKS_H_INCLUDED
    /*  If neither dcl.h nor dcl_vxworks.h have already been included
        by higher level headers, we must be here because we're being
        included by externally built software which will not have
        the DCL include path.  Therefore use a path relative to the
        current directory.
    */
    #include "..\dcl\include\dcl_vxworks.h"
  #endif
#endif

#endif


/*  VxWorks includes
*/
#include <iosLib.h>

/*  Reliance includes
*/
#include <rloilcmn.h>


/*-------------------------------------------------------------------
    Reliance Error Codes

    Note:
       The codes which are defined with ERRNO_PX_FLAG can be mapped
       to POSIX error codes:

          if(errno & ERRNO_PX_FLAG) errno &= 0x7FFF;
-------------------------------------------------------------------*/
#define M_relFsLib                              (212 << 16)
#define S_relFsLib_DISK_FULL                    (M_relFsLib | 1)
#define S_relFsLib_FD_OBSOLETE                  (M_relFsLib | 2)
#define S_relFsLib_DELETED                      (M_relFsLib | 3)
#define S_relFsLib_DENIED                       (M_relFsLib | 4)
#define S_relFsLib_FILE_NOT_FOUND               (M_relFsLib | 5)
#define S_relFsLib_NO_FREE_FILE_DESCRIPTORS     (M_relFsLib | 6)
#define S_relFsLib_NOT_DIRECTORY                (M_relFsLib | 7)
#define S_relFsLib_NOT_FILE                     (M_relFsLib | 8)
#define S_relFsLib_UNKNOWN_VOLUME_FORMAT        (M_relFsLib | 9)
#define S_relFsLib_READ_ONLY                    (M_relFsLib | 10)
#define S_relFsLib_FILE_EXISTS                  (M_relFsLib | 11)
#define S_relFsLib_INVALID_PARAMETER            (M_relFsLib | 12)
#define S_relFsLib_DIR_NOT_EMPTY                (M_relFsLib | 13)
#define S_relFsLib_NOT_SAME_VOLUME              (M_relFsLib | 14)
#define S_relFsLib_WRITE_ONLY                   (M_relFsLib | 15)
#define S_relFsLib_ACCESS_BEYOND_EOF            (M_relFsLib | 16)
#define S_relFsLib_INVALID_NUMBER_OF_BYTES      (M_relFsLib | 17)


/*-------------------------------------------------------------------
    Function Prototypes

    See the Reliance documentation for information about using
    these functions.
-------------------------------------------------------------------*/

/* General Functions
*/
STATUS relFsLibInit( void );
STATUS relFsCommitAll(          void *pDevName);
STATUS relFsCreateDiskImage(    char *szDeviceName, char *szImageName);
STATUS relFsDevCreate(          char *pDevName, void *pDev, u_int maxFiles );
STATUS relFsGetTimedThreshold(  char *pDevName, unsigned long *pulThreshold);
STATUS relFsSetTimedThreshold(  char *pDevName, unsigned long ulThreshold);
STATUS relFsGetTransactionMode( char *pDevName, unsigned *puTransactionMode);
STATUS relFsSetTransactionMode( char *pDevName, unsigned uTransactionMode);
STATUS relFsShow(               void *pDevName);
STATUS relFsSys(                char *szCommandLine);
int    relFsTest(               char *szCommandLine);
STATUS relFsTestLibInit(        void);
STATUS relFsToolLibInit(        void);
STATUS relFsTransactionPoint(   void *pDevName);
STATUS relFsVolMount(           char *pDevName);
STATUS relFsVolUnmount(         char *pDevName);

/*  Miscellaneous Functions
*/
STATUS RelRegInit(void);
STATUS RelSysInit(void);
STATUS RelFmtInit(void);
STATUS usrRelFsInit( unsigned uConfigOptions, unsigned uMaxFiles );
STATUS relFsProbeInstall( void );


/*-------------------------------------------------------------------
    IOCTL Interface

    See the Reliance documentation for information about using
    these functions.
-------------------------------------------------------------------*/
#define REL_IOCTL_BASE                  0x7700

/*  IOCTL functions specific to Reliance (relFs)
*/
#define REL_IOCTL_GET_TRANSACT_MODE     ( REL_IOCTL_BASE + 0 )
#define REL_IOCTL_SET_TRANSACT_MODE     ( REL_IOCTL_BASE + 1 )
#define REL_IOCTL_PERFORM_TRANSACTION   ( REL_IOCTL_BASE + 2 )
#define REL_IOCTL_SUSPEND_TRANSACTION   ( REL_IOCTL_BASE + 3 )
#define REL_IOCTL_RESUME_TRANSACTION    ( REL_IOCTL_BASE + 4 )
#define REL_IOCTL_TFS_PRIVATE           ( REL_IOCTL_BASE + 5 )

/*  IOCTLS not supported in VxWorks 5.4.
    These are supported by relFs.
*/
#ifndef _WRS_VXWORKS_5_X
#define FIOMOVE                         ( REL_IOCTL_BASE + 6 )
#define FIOCHKDSK                       ( REL_IOCTL_BASE + 7 )
#define FIOTRUNC64                      ( REL_IOCTL_BASE + 8 )
#define FIONREAD64                      ( REL_IOCTL_BASE + 9 )
#define FIOSEEK64                       ( REL_IOCTL_BASE + 10 )
#define FIOWHERE64                      ( REL_IOCTL_BASE + 11 )
#define FIONFREE64                      ( REL_IOCTL_BASE + 12 )
#endif


/*-------------------------------------------------------------------
    Formatter settings and function prototypes
-------------------------------------------------------------------*/
#define REL_FMT_OPT_QUIET        0x00
#define REL_FMT_OPT_SUMMARY      0x01
#define REL_FMT_OPT_PROGRESS     0x02
#define REL_FMT_OPT_DO_SCAN      0x04

/*  Default block size for relFsVolFormat()
*/
#define REL_FMT_DEFAULT_BLOCK_SIZE  512

STATUS relFsFmtLibInit(         void);
STATUS relFsVolFormat(          void *pDevName, unsigned long ulBlockSize, unsigned uOptions);

/*  Generic command line version of the format utility.

    Example for formatting the /rel1 volume with a 1024
    byte block size:

        relfmt "/rel1 -b1024"

    Example for showing the usage screen:

        relfmt ""
*/
STATUS relfmt( char * szCommandLine );


/*-------------------------------------------------------------------
    Checker settings and function prototypes
-------------------------------------------------------------------*/
#define REL_CHK_OPT_VERBOSE         0x01
#define REL_CHK_OPT_NOPROMPT        0x02
#define REL_CHK_OPT_DUMP_INDEX      0x04
#define REL_CHK_OPT_QUIET           0x08
#define REL_CHK_OPT_SHOW_ERRORS     0x10

#define REL_CHK_OPT_DEFAULT         (   \
            REL_CHK_OPT_NOPROMPT    |   \
            REL_CHK_OPT_SHOW_ERRORS )

STATUS relFsChkLibInit(void);
STATUS relFsChk(char *pDevName, unsigned uOptions);


/*-------------------------------------------------------------------
    Partitioning Interface

    Note that this is only available on VxWorks 5.5
-------------------------------------------------------------------*/
#ifdef _WRS_VXWORKS_5_X

#include <cbioLib.h>
#include <dpartCbio.h>

/*  These are the flags for relFsPartCreate.  Normally
    only the REL_PART_DEFAULT setting is used.
*/
#define REL_PART_DEFAULT                0x00
#define REL_PART_DONT_WIPE_BOOT_SECTOR  0x01

STATUS relFsPartLibInit(void);
STATUS relFsPartCreate(CBIO_DEV_ID CBIODeviceId, unsigned int *puPercentageArray, unsigned uFlags);
STATUS relFsPartRead(  CBIO_DEV_ID CBIODeviceId, PART_TABLE_ENTRY *pPartTableEntry, int iNumberOfPartitions);

#endif /* #ifdef _WRS_VXWORKS_5_X */



STATUS usrFileDiskInit(
    unsigned uConfigOptions,
    char * szBasePath,
    unsigned uSectorSize,
    unsigned long ulNumberOfSectors,
    char * szDeviceName,
    unsigned eFileSystem,
    unsigned eFormatState,
    unsigned uLogicalBlockSize );


void relFsPortTestInit(void);
STATUS discardWrapperFFXInit(void);

#endif /* #ifndef RELIANCE_VXWORKS_H_INCLUDED */


