/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlcmdln.h $
    Revision 1.1  2005/04/01 03:54:30  Pauli
    Initial revision
    Revision 1.1  2005/04/01 03:54:30Z  Garyp
    Initial revision
    Revision 1.1  2005/04/01 02:54:30Z  pauli
    Initial revision
    Revision 1.5  2005/04/01 01:54:29Z  brandont
    Added a new option to indicate that a command line option should only
    be used once.
    Revision 1.4  2005/03/25 23:18:21Z  brandont
    Added an internal command line parser flag for marking an argument
    as used.
    Revision 1.3  2005/03/23 00:58:05Z  PaulI
    Updated data type for consistency with usage.
    Revision 1.2  2005/03/18 00:14:14Z  brandont
    Added a new command line type for offset and value pair.
    Revision 1.1  2005/01/29 01:01:16Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef __rlcmdln_h
#define __rlcmdln_h


#define TFS_CMDLN_INT32     1   /* 32-bit integer argument */
#define TFS_CMDLN_STR       2   /* string argument */
#define TFS_CMDLN_BOOL      3   /* boolean option */
#define TFS_CMDLN_GEOMETRY  4   /* for block/blocksize pairs */
#define TFS_CMDLN_TEST      5   /* test function with parameters */
#define TFS_CMDLN_HELP      6   /* help for the entire application */
#define TFS_CMDLN_HEX32     7   /* 32-bit hex integer argument */
#define TFS_CMDLN_UINT64    8   /* 64-bit unsigned integer argument */
#define TFS_CMDLN_HEX64     9   /* 64-bit hex unsigned integer argument */
#define TFS_CMDLN_OFFSET    10  /* tfsOffset argument */
#define TFS_CMDLN_VALOFF    11  /* sTfsValueOffset argument */

/*  command flags
*/
#define TFS_CMDFLAG_REQUIRED        0x01
#define TFS_CMDFLAG_POWER2          0x02
#define TFS_CMDFLAG_RANGE_CHECK     0x04
#define TFS_CMDFLAG_CASE_SENSITIVE  0x08
#define TFS_CMDFLAG_OPTIONAL        0x10
#define TFS_CMDFLAG_ARG_USED        0x20    /* parser internal do not use */
#define TFS_CMDFLAG_ARG_USE_ONCE    0x40    /* Allow only one use of this argument */


typedef tfsBool(*TfsCmdLineFn)(void *pValue);


/*  describes command line options and arguments
*/
struct TfsTagCmdLine
{
    int             iType;      /* TFS_CMDLN_XXX */
    int             iFlags;     /* different meaning depending on type */
    tfsChar         cOptChar;   /* option character */
    void           *pValue;     /* points to the value */
    void           *pCheck;     /* points to a validity check structure */
};
typedef struct TfsTagCmdLine sTfsCmdLine;


/*  a commandline line argument numeric range checked
*/
struct TfsTagCmdLineRange
{
    tfsUint32       iLow;
    tfsUint32       iHigh;
};
typedef struct TfsTagCmdLineRange sTfsCmdLineRange;


/*  commands to a subtest or category
*/
#define TFS_CMD_REQUEST          0x01
#define TFS_CMD_NODEFAULT        0x02


/*  a subtest
*/
struct TfsTagCmdLineSubTest
{
    tfsChar        *szName;
    tfsBool        *pfValue;
    sTfsCmdLine    *pParms;
    int             fCmd;
};
typedef struct TfsTagCmdLineSubTest sTfsCmdLineSubTest;


/*  A offset and value pair.  Size of value is also defined.
*/
typedef struct {
    tfsUint32 ulOffset;
    tfsUint32 ulValue;
    tfsUint32 ulSizeOfValue;
} sTfsCmdLineValueOffset;


/*  a whole category of tests
*/
struct TfsTagCmdLineCategory
{
    tfsChar        *szName;
    tfsBool        *pfValue;
    sTfsCmdLineSubTest *pSubTests;
    tfsChar        *szHelp;
    int             fCmd;
};
typedef struct TfsTagCmdLineCategory sTfsCmdLineCategory;


/*  command line functions
*/
int             TfsCmdLineProcessCmdLine(
    sTfsCmdLine * pOrgOptions,
    int argc,
    tfsChar * argv[],
    tfsChar ** pBadArg);
int             TfsCmdLineGetNextArg(
    tfsChar * szLine,
    unsigned *piIndex,
    int argc,
    tfsChar * argv[],
    int iMaxLen);
unsigned        TfsCmdLineCountArgs(
    int argc,
    tfsChar * argv[]);



#endif /* #ifndef __rlcmdln_h */
