/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    Include this header file in any C program that needs to

---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlioctl.h $
    Revision 1.1  2005/06/25 01:31:52  Pauli
    Initial revision
    Revision 1.2  2005/06/25 02:31:51Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.9  2005/06/25 02:31:51Z  brandont
    Removed the IOCTL function for setting the number of reserved blocks.
    Revision 1.8  2005/05/03 18:12:05Z  PaulI
    Added an ioctl for retrieving information about the discard extent table.
    Revision 1.7  2005/05/03 02:18:42Z  brandont
    Added a field to the IOCTL for cache info to return the memory pool size.
    Also corrected the type used to return block size for the same IOCTL.
    Revision 1.6  2005/05/02 22:15:39Z  PaulI
    Added a command for getting the max keep count.
    Revision 1.5  2005/03/21 23:09:50Z  brandont
    Added IOCTL functions for retrieving the index for a file, creating a
    contiguous file, converting an index to a block number, and performing
    I/O read and writes.
    Revision 1.4  2005/02/19 02:24:26Z  brandont
    Moved the core bucket defines and macros to the rlbucket.h file.
    Removed the unused core transaction flags.
    Revision 1.3  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.2  2004/12/07 21:47:09Z  brandont
    Changed the members of the installation check ioctl structure.
    Revision 1.1  2004/09/23 00:49:54Z  brandont
    Initial revision
    Revision 1.7  2004/08/27 00:36:52  brandont
    - IR ? *.* Changed header on all the source modules to the FlashFx style header
---------------------------------------------------------------------------*/

#ifndef TFS_IOCTL_H
#define TFS_IOCTL_H     1

#include <rlcore.h>


/*  TFS IOCTL calls
*/
#define TFS_IOCTL_INSTALLED                 0x30  /* is TFS installed? */
#define TFS_IOCTL_UNMOUNT                   0x31  /* unmount TFS */
#define TFS_IOCTL_REMOUNT                   0x32  /* remount TFS */
#define TFS_IOCTL_TRANSACTION               0x33  /* perform a transaction point */
#define TFS_IOCTL_SETRDONLY                 0x34  /* set TFS to be read only */
#define TFS_IOCTL_SET_TRANMODE              0x35  /* set transaction mode */
#define TFS_IOCTL_UNUSED_1                  0x36  /* unused */
#define TFS_IOCTL_GET_TRANSACTION_POINTS    0x37  /* get # of transactions */
#define TFS_IOCTL_SET_DEBUG                 0x38  /* set debug on or off */
#define TFS_IOCTL_GET_BUCKETS               0x39  /* get bucket counts */
#define TFS_IOCTL_GET_CACHEINFO             0x3A  /* get cache block size, # blocks */
#define TFS_IOCTL_KILL                      0x3B  /* set kill points for testing */
#define TFS_IOCTL_TRANBITS                  0x3C  /* get/set transaction bits */
#define TFS_IOCTL_INTERVAL                  0x3D  /* get/set tick interval bits */
#define TFS_IOCTL_GET_DISK_INFO             0x3E  /* get volume block info */
#define TFS_IOCTL_GET_INDEX                 0x3f  /* given a path, return the index # */
#define TFS_IOCTL_CREATE_CONTIGUOUS         0x40  /* Create a contiguous file */
#define TFS_IOCTL_IO_READ                   0x41  /* Read from the I/O device */
#define TFS_IOCTL_IO_WRITE                  0x42  /* Write to the I/O device */
#define TFS_IOCTL_INDEX_TO_BLOCK            0x43  /* Dereference the index to the block number */
#define TFS_IOCTL_GET_CACHE_KEEP_MAX        0x44  /* get the max cache keep count */
#define TFS_IOCTL_GET_EXTENTINFO            0x45  /* get extent table information */


/*  ioctl call
*/
int             TfsCLibIoctl(
    int iIoctlCmd,
    void *pBuffer);

/*  for use with TFS_IOCTL_INSTALLED
*/
struct TfsTagIoctlInstalled
{
    tfsUint32       ulVersion;  /*  = VERSIONVAL */
    tfsUint32       ulBuildNumber;      /*  = RELBUILDNUM */
    tfsUint32       iTfsLayoutVersion;  /*  = TFS_LAYOUT_VERSION */
    tfsUint32       ulDebug;    /*  = TFS_DEBUG */
    tfsChar         szVersionStr[100];  /*  = VERSIONNUM */
};
typedef struct TfsTagIoctlInstalled sTfsIoctlInstalled;

/*  for use with TFS_IOCTL_GET_CACHEINFO
*/
struct TfsTagIoctlCacheInfo
{
    tfsBlock        iBufferSize;
    tfsUint32       iNumBuffers;
    tfsUint32       ulGatherSizeInKb;
};
typedef struct TfsTagIoctlCacheInfo sTfsIoctlCacheInfo;

/*  for use with TFS_IOCTL_GET_EXTENTINFO
*/
struct TfsTagIoctlExtentInfo
{
    tfsUint32       iInitialSize;
    tfsUint32       iCurrentSize;
    tfsBool         fGrowthEnabled;
};
typedef struct TfsTagIoctlExtentInfo sTfsIoctlExtentInfo;

/*  for use with TFS_IOCTL_SET_TRANMODE
*/
struct TfsTagIoctlTranMode
{
    tfsUint32       iMode;
    tfsUint32       iMilliseconds;      /* for timed mode */
};
typedef struct TfsTagIoctlTranMode sTfsIoctlTranMode;

/*  kill ioctl call
*/
struct TfsTagIoctlKill
{
    tfsUint32       iBits;      /* kill point bit mask */
    tfsUint32       iCount;     /* # of writes ops before killing */
    tfsBool         fFlush;     /* flush buffers before killing? */
};
typedef struct TfsTagIoctlKill sTfsIoctlKill;


/*  Get an index # from a full path.
*/
struct TfsTagIoctlGetIndex
{
    tfsIndex iIndex;
    tfsChar * szPath;
};
typedef struct TfsTagIoctlGetIndex sTfsIoctlGetIndex;


/*  Create a contiguous file
*/
typedef struct {
    tfsHandle ulHandle;
    tfsOffset ulFileSize;
    tfsBlock ulDataStartBlock;
    tfsBlock ulDataNumBlocks;
    tfsUint32 ulINodeSignature;
} TfsIoctlContig;


/*  I/O read/write
*/
typedef struct {
    tfsBlock ulStartBlock;
    tfsBlock ulNumBlocks;
    tfsBuffer * pBuffer;
} TfsIoctlReadWrite;


/*  Get the block number that corresponds to this index
*/
typedef struct {
    tfsIndex ulIndex;
    tfsBlock ulBlockNumber;
} TfsIoctlIndexToBlock;


#endif /* TFS_IOCTL_H */
