/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description

    Header for the Reliance File System Library. Used by the utilities in
    conjunction with the OIL.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rllib.h $
    Revision 1.1  2005/05/13 19:54:12  Pauli
    Initial revision
    Revision 1.1  2005/05/13 20:54:12Z  Garyp
    Initial revision
    Revision 1.1  2005/05/13 20:54:12Z  pauli
    Initial revision
    Revision 1.8  2005/05/13 20:54:12Z  garyp
    Minor formatting changes -- no functional changes.
    Revision 1.7  2005/03/18 02:44:04Z  brandont
    Added TfsLibVersionCheck prototype.
    Revision 1.6  2005/02/23 02:40:12Z  brandont
    Moved the command line module defines to the rlcmdln.h file.
    Removed the core function prototypes.
    Revision 1.5  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.4  2004/12/07 23:40:59Z  brandont
    Updated with new common signon function prototypes.
    Revision 1.3  2004/12/02 21:58:24Z  PaulI
    Updated lib uninitialize API to take the address of a log handle pointer.
    Revision 1.2  2004/11/22 23:19:11Z  brandont
    Added log handle argument to the prototypes for TfsLibDisplayVersion(),
    TfsLibSignon(), TfsLibDisplayVolumeInfo(), TfsLibInitialize(), and
    TfsLibUninitialized().  Removed prototype for TfsPause.  Removed global
    for log max size.
    Revision 1.1  2004/09/23 00:49:46Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef TFS_LIB_H
#define TFS_LIB_H 1

#define TFS_MAX_32DIGITS         13     /* G,MMM,KKK,BBB */
#define TFS_MAX_GEOMETRY_DIGITS     17  /* T,GGG,MMM,KKK,BBB */


/*  sign-on, cmdline functions
*/
TFSERR    TfsLibVersionCheck(tfsChar * szDrive );
void      TfsLibSignon(LogHandle * pLogHandle, tfsChar * szTitle);
void      TfsLibSignonCopyright(LogHandle * pLogHandle);
void      TfsLibSignonCoreVersion(LogHandle * pLogHandle);
void      TfsLibDisplayVolumeInfo(LogHandle * pLogHandle, sTfsVolumeInfo * pVolumeInfo);

/*  connect/disconnect from a specific TFS drive
*/
int       TfsLibInitialize(tfsChar * szLogName, LogHandle ** ppLogHandle);
int       TfsLibUninitialize(LogHandle ** pLogHandle);
int       TfoLibConnect(sTfsVolume ** ppVolume, tfsChar * szVolumeName, sTfsGeometry * pGeometry, void *pOilData);
int       TfoLibDisconnect(sTfsVolume * pVolume);

/*  character functions
*/
void      TfsLibGeometryToNumber(tfsChar * szNumber, sTfsGeometry * pGeometry);
void      TfsLibGetCommaNumber(    tfsChar * szNumber, tfsUint32 iNumber);
void      TfsLibNumberToShortForm(tfsChar * szNumber, tfsUint32 iNumber);
void      TfsLibGeometryToShortForm(tfsChar * szNumber, sTfsGeometry * pGeometry);
tfsChar * TfsLibGetBigNumber(tfsUint32 iValue1, tfsUint32 iValue2);
tfsUint32 TfsLibShortFormToNumber(tfsChar * szNumber);

/*  BT 04-26-04
    IR 4437

    While adding support for larger file sizes it was necessary to create
    functions to perform 64-bit translations.
*/
#if TFS_ENABLE_64BIT_TYPES
void      TfsLibShortFormToNumber64(tfsChar * szNumber, tfsUint64 * pResult);
#endif
tfsBool   TfsIsWhite(tfsChar c);
tfsUint32 TfsAToX(tfsChar * szHex);

/*  BT 04-26-04
    IR 4437

    While adding support for larger file sizes it was necessary to create
    functions to perform 64-bit translations.
*/
#if TFS_ENABLE_64BIT_TYPES
void      TfsAToX64(tfsChar * szHex, tfsUint64 * pResult);
#endif
tfsBool   TfsIsXDigit(tfsChar c);



extern tfsUint32 giTfsRandomSeed;       /* random seed (defaults to clock) */



#endif /* TFS_LIB_H */
