/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rllog.h $
    Revision 1.3  2006/03/02 00:16:06  joshuab
    Changed test logging functions to use char instead of tfsChar.
    Revision 1.2  2006/02/11 00:09:01Z  brandont
    Updated prototypes for RelDebugLogWrite() and RelLogFileWrite().
    Revision 1.1  2005/08/09 19:43:06Z  Pauli
    Initial revision
    Revision 1.2  2005/08/09 20:43:06Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.10  2005/08/09 20:43:06Z  brandont
    Added prototype for TfsTestLogSkipped.
    Revision 1.9  2005/04/15 23:28:32Z  brandont
    Added stdarg.h include back into this header file.
    Revision 1.8  2005/04/15 20:44:03Z  brandont
    Removed the include for stdarg.h.
    Revision 1.7  2005/03/25 23:07:04Z  Pauli
    Added stdarg include for variable arg list.
    Revision 1.6  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.5  2004/12/09 23:18:58Z  PaulI
    Added prototype for TfsTestLogUsage().
    Revision 1.4  2004/12/04 04:03:31Z  brandont
    Renamed TfsOsAlloc and TfsOsFree to RelMalloc and RelFree respectively.
    Revision 1.3  2004/12/02 22:10:09Z  PaulI
    Updated log uninitialize API to take the address of a log handle pointer.
    Revision 1.2  2004/11/29 21:04:29Z  brandont
    Added the new debug logging function prototypes.
    Revision 1.1  2004/11/20 01:33:02Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef RLLOG_H
#define RLLOG_H   1


#include <stdarg.h>


/*  TFS logging functions for the test suite
*/
void            TfsTestLogBeginSection(
    char * szTitle);
void            TfsTestLogEndSection(
    void);
void            TfsTestLogDiffTime(
    tfsClock iClock);
void TFS_CDECL TfsTestLogSkipped(
    char * szFmt,
    ...);
void TFS_CDECL  TfsTestLogError(
    char * szFmt,
    ...);
int TFS_CDECL   TfsTestLogPrintf(
    char * szFmt,
    ...);
int TFS_CDECL   TfsTestLogPrintfV(
    char * szFmt,
    ...);
void TFS_CDECL  TfsTestLogUsage(
    tfsChar * szUsage);
void            TfsTestLogEnable(
    void);
void            TfsTestLogDisable(
    void);
int             TfsTestLogInitialize(
    tfsChar * szLogName);
void            TfsTestLogUninitialize(
    void);


/*  Core debug logging functions
*/
void            RelDebugLogInitialize(
    tfsChar * szFullPathName);
void            RelDebugLogUninitialize(
    void);
void            RelDebugLogDisable(
    void);
void            RelDebugLogEnable(
    void);
void            RelDebugLogWrite(
    const char * pBuffer,
    tfsUint32 ulLength);


/*  LogHandle

    Handle for logging to a file.
*/
typedef struct sLogHandle LogHandle;


/*  Generic logging functions
*/
LogHandle      *TfsLogInitialize(
    tfsChar * szLogName);
void            TfsLogUninitialize(
    LogHandle ** ppLogHandle);
void            TfsLogEnable(
    LogHandle * pLogHandle);
void            TfsLogDisable(
    LogHandle * pLogHandle);
int             TfsLogVPrintf(
    LogHandle * pLogHandle,
    char * szFmt,
    va_list arglist);
int TFS_CDECL   TfsLogPrintf(
    LogHandle * pLogHandle,
    char * szFmt,
    ...);


/*  LogFileHandle

    OIL specific handle for logging to a file.
*/
typedef struct sLogFileHandle LogFileHandle;


/*  OIL implemented low-level logging functions
*/
LogFileHandle  *RelLogFileOpen(
    tfsChar * szFullPathName);
void            RelLogFileWrite(
    LogFileHandle * pLogFileHandle,
    const void *pBuffer,
    tfsUint32 ulLength);
void            RelLogFileClose(
    LogFileHandle ** pLogFileHandle);


#endif /* #ifndef RLLOG_H */
