/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

                  Copyright (c) 2003 - 2005 Datalight, Inc.
                    All Rights Reserved.  Patent Pending.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rlmacro.h $
    Revision 1.3  2006/01/09 17:43:36  Pauli
    Replaced TFS endian and ptr math macros with DCL.
    Revision 1.2  2006/01/03 23:37:25Z  Pauli
    Mapped macros to DCL equivalents for testing.
    Revision 1.1  2005/12/02 18:11:58Z  Pauli
    Initial revision
    Revision 1.2  2005/12/02 18:11:57Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.12  2005/09/28 22:21:47Z  brandont
    Added the TfsPtrMask macro.
    Revision 1.11  2005/06/24 19:42:15Z  brandont
    Added the non port specific path name separator check macro.
    Revision 1.10  2005/06/20 19:45:20Z  PaulI
    Updated to use DCL 64-bit support settings.
    Revision 1.9  2005/06/16 21:48:16Z  PaulI
    Removed obsolete TFS assert macros.
    Revision 1.8  2005/06/15 19:27:46Z  Pauli
    Removed obsolete TfsAssertModuleName().
    Revision 1.7  2005/06/14 19:40:53Z  PaulI
    Updated assert macros to use DCL implementations.
    Revision 1.6  2005/03/18 02:37:37Z  brandont
    Allow the low level Endian swap functions to exist regardless of the
    current Endian mode.
    Revision 1.5  2005/02/04 21:42:28Z  brandont
    Changed TfsIsPower2 to a macro instead of a function. Added IMap
    size calculation macro.
    Revision 1.4  2004/12/31 01:17:52Z  GaryP
    Updated to use indents of 4, hard tabs of 8 (if used), and a standard
    comment style.
    Revision 1.3  2004/12/01 21:12:52Z  brandont
    Removed prototype for TfsOsHalt.  Added defines for initializing memory
    pools in debug mode.
    Revision 1.2  2004/11/18 23:30:26Z  brandont
    Moved macros for TfsToUpper, TfsIsDigit, TfsIsPrint to rlctypes.h and
    changed their Tfs prefix to Rel.
    Revision 1.1  2004/09/29 19:45:02Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef TFS_MACRO_H
#define TFS_MACRO_H     1


/*  -------------------------------------------------------------------------

    temporary placement
*/

#define TfsIsPathSeparator(c) \
        ((c) == TFS_PATH_SEPARATOR || (c) == TFS_PATH_SEPARATOR2)


/*  DOS specific keyword
*/
#define TFS_CDECL


/*  -------------------------------------------------------------------------
    debugging help for any subsystems that use memory blocks
*/
#define TfsIsPower2(_value) (((_value) & ((_value)-1)) == 0)

/*  -------------------------------------------------------------------------
    Core calculations
*/


/*  Determine the number of IMap blocks
*/
#define TfsIMapDetermineSize(_iMapEntries,_iNumBlocks)  \
        (((_iNumBlocks) + (_iMapEntries) - 1) / (_iMapEntries))


/*  -------------------------------------------------------------------------
    The following macros handle unicode/ascii text strings
*/
#ifndef TFS_UNICODE
#error  TFS_UNICODE not defined before including tfsmacro.h
#endif

#if TFS_UNICODE

/*  unicode characters
*/
#define TfsStr(sz)   _T(sz)

#else

/*  ascii characters
*/
#define TfsStr(sz)   sz

#endif


/*  -------------------------------------------------------------------------
    The following macros handle ascii character type checks and conversions
*/
#define RelIsDigitComma(c) ((c) == TfsStr(',') || ((c) >= TfsStr('0') && (c) <= TfsStr('9')))


/*  -------------------------------------------------------------------------
    The following macro determines the number of elements in an array.
*/
#if 0
#define TfsArraySize(_a_)  (sizeof(_a_) / sizeof(*(_a_)))
#else
#define TfsArraySize(_a_)   DCLDIMENSIONOF(_a_)
#endif


/*  -------------------------------------------------------------------------
    The following macro determines if a value is within a certain range.
*/
#define TfsInRange(val, start, num) ((val) >= (start) && (val) < (start) + (num))


/*  -------------------------------------------------------------------------
    The following macros are used to set common data format (CDF) field and retrieve CDF fields.
*/


/*  COPYFROMCDF

    This macro is used to copy a value from a CDF buffer.  The buffer pointer (__CDFsrc)
    will be updated for convience of any subsequent calls.  The size of the
    field is automatically determined by the sizeof __dst argument.

    Typical use: Copying from a buffer to an in-memory structure.
*/
#define COPYFROMCDF( __CDFsrc, __dst ) \
   DclCopyLittleToNativeEndian( __dst, __CDFsrc, sizeof(*(__dst)) );  \
   __CDFsrc = DclPtrAddByte( __CDFsrc, sizeof(*(__dst)) )


#if TFS_ENABLE_64BIT_TYPES
#if DCL_NATIVE_64BIT_SUPPORT

/*  COPYFROMCDF64

    This macro is used to copy a 64-bit value from a CDF buffer.  The buffer
    pointer (__CDFsrc) will be updated for convience of any subsequent calls.

    This macro uses DCL_NATIVE_64BIT_SUPPORT to determine how to copy 64-bit
    values around.

    Typical use: Copying from a buffer to an in-memory structure.
*/
#define COPYFROMCDF64( __CDFsrc, __dst )  \
   DclCopyLittleToNativeEndian64( __dst, __CDFsrc );   \
   __CDFsrc = DclPtrAddByte( __CDFsrc, 8 )
#else

/*  COPYFROMCDF64

    This macro is used to copy a 64-bit value from a CDF buffer.  The buffer
    pointer (__CDFsrc) will be updated for convience of any subsequent calls.

    This macro uses DCL_NATIVE_64BIT_SUPPORT to determine how to copy 64-bit
    values around.

    Typical use: Copying from a buffer to an in-memory structure.
*/
#define COPYFROMCDF64( __CDFsrc, __dst )  \
   DclCopyLittleToNativeEndian( &(__dst)->ulLowDword, __CDFsrc, sizeof((__dst)->ulLowDword) );   \
   __CDFsrc = DclPtrAddByte( __CDFsrc, sizeof((__dst)->ulLowDword) );   \
   DclCopyLittleToNativeEndian( &(__dst)->ulHighDword, __CDFsrc, sizeof((__dst)->ulHighDword) ); \
   __CDFsrc = DclPtrAddByte( __CDFsrc, sizeof((__dst)->ulHighDword) )
#endif
#endif


/*  COPYTOCDF

    This macro is used to copy a value to a CDF buffer.  The buffer pointer    (__CDFdst)
    will be updated for convience of any subsequent calls.  The size of the
    field is automatically determined by the sizeof __src argument.

    Typical use: Copying an in-memory structure to a buffer in preparation for
       writing the buffer to disk.
*/
#define COPYTOCDF( __CDFdst, __src )   \
   DclCopyNativeToLittleEndian( __CDFdst, __src, sizeof(*(__src)) );  \
   __CDFdst = DclPtrAddByte( __CDFdst, sizeof(*(__src)) )


#if TFS_ENABLE_64BIT_TYPES
#if DCL_NATIVE_64BIT_SUPPORT

/*  COPYTOCDF64

    This macro is used to copy a 64-bit value to a CDF buffer.  The buffer
    pointer (__CDFdst) will be updated for convience of any subsequent calls.

    This macro uses DCL_NATIVE_64BIT_SUPPORT to determine how to copy 64-bit
    values around.

    Typical use: Copying an in-memory structure to a buffer in preparation for
       writing the buffer to disk.
*/
#define COPYTOCDF64( __CDFdst, __src ) \
   DclCopyNativeToLittleEndian64( __CDFdst, __src );   \
   __CDFdst = DclPtrAddByte( __CDFdst, 8 )
#else

/*  COPYTOCDF64

    This macro is used to copy a 64-bit value to a CDF buffer.  The buffer
    pointer (__CDFdst) will be updated for convience of any subsequent calls.

    This macro uses DCL_NATIVE_64BIT_SUPPORT to determine how to copy 64-bit
    values around.

    Typical use: Copying an in-memory structure to a buffer in preparation for
       writing the buffer to disk.
*/
#define COPYTOCDF64( __CDFdst, __src ) \
   DclCopyNativeToLittleEndian( __CDFdst, &(__src)->ulLowDword, sizeof((__src)->ulLowDword) );   \
   __CDFdst = DclPtrAddByte( __CDFdst, sizeof((__src)->ulLowDword) );   \
   DclCopyNativeToLittleEndian( __CDFdst, &(__src)->ulHighDword, sizeof((__src)->ulHighDword) ); \
   __CDFdst = DclPtrAddByte( __CDFdst, sizeof((__src)->ulHighDword) )
#endif
#endif


/*  COPYFROMCDFTOTEMP

    This macro is used to copy a value from a a CDF buffer.  Unlike the
    COPYFROMCDF macro, this macro takes a third argument which it uses to
    compute the size of the field.  Debug asserts are included in the macro
    for validating the size of the destination against the sizeof the third
    argument.  The buffer pointer (__CDFsrc) will be updated for convience of
    any subsequent calls.

    Typical use: Retrieving the value from within a structure in a buffer.
*/
#define COPYFROMCDFTOTEMP( __CDFsrc, __dst, __sizeofvar )   \
   DclAssert( sizeof(*(__dst)) == (__sizeofvar) ); \
   DclCopyLittleToNativeEndian( __dst, __CDFsrc, (__sizeofvar) );  \
   __CDFsrc = DclPtrAddByte( __CDFsrc, (__sizeofvar) )


#endif /* TFS_MACRO_H */
