/*---------------------------------------------------------------------------
               ----> DO NOT REMOVE THE FOLLOWING NOTICE <----

      Copyright (c) 2003 - 2004 Datalight, Inc.  All Rights Reserved.

  Datalight, Incorporated is a Washington State corporation located at:

        21520 30th Dr SE, Suite 110,      Tel:  425-951-8086
        Bothell, WA  98021                Fax:  425-951-8094
        USA                               Http://www.datalight.com

  This software, including without limitation, all source code and documen-
  tation, is the confidential, trade secret property of Datalight, Inc., and
  is protected under the copyright laws of the United States and other juris-
  dictions.

  ---> Portions of the software are patent pending <---

  In addition to civil penalties for infringement of copyright under applic-
  able U.S. law, 17 U.S.C. 1204 provides criminal penalties for violation of
  (a) the restrictions on circumvention of copyright protection systems found
  in 17 U.S.C. 1201 and (b) the protections for the integrity of copyright
  management information found in 17 U.S.C. 1202.

  U.S. Government Restricted Rights:  Use, duplication, reproduction, or
  transfer of this commercial product and accompanying documentation is
  restricted in accordance with FAR 12.212 and DFARS 227.7202 and by a
  License Agreement.

  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS PROTECTED UNDER A
  SOURCE CODE AGREEMENT, NON-DISCLOSURE AGREEMENT (NDA), OR SIMILAR BINDING
  CONTRACT BETWEEN DATALIGHT, INC. AND THE LICENSEE ("BINDING AGREEMENT").
  YOUR RIGHT, IF ANY, TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,
  IS SUBJECT TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT.  BY USING
  THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE TO BE BOUND BY
  THE TERMS OF THE BINDING AGREEMENT.  CONTACT DATALIGHT, INC. AT THE ADDRESS
  SET FORTH ABOVE IF YOU OBTAINED THIS SOFTWARE IN ERROR.
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Description
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
                              Revision History
    $Log: rltypes.h $
    Revision 1.1  2005/09/27 23:58:38  Pauli
    Initial revision
    Revision 1.2  2005/09/28 00:58:37Z  Pauli
    Merge with 2.0 product line, build 458.
    Revision 1.5  2005/09/28 00:58:37Z  brandont
    Changed the tfsBool type from int to tfsInt32 so that its size was fixed.
    Revision 1.4  2005/03/25 23:30:35Z  Pauli
    Added define for 64-bit file sizes.
    Revision 1.3  2005/03/21 22:43:27Z  brandont
    Added defines for the CLIB and NLIB function error code types.  Added
    the typedef for sTfsVolume but not the structure definition.
    Revision 1.2  2005/03/09 20:24:45Z  brandont
    Removed the pointer from sOilBlkDevInfo typedef. Added the new
    reentrant progress function typedef.
    Revision 1.1  2005/02/28 21:08:46Z  brandont
    Initial revision
---------------------------------------------------------------------------*/

#ifndef TFS_TYPES_H
#define TFS_TYPES_H     1


/*  TFSERR

    Reliance (TFS) error code type.  Reliance error codes all begin with the
    prefix TFSERR_xxx where xxx is an identifier for the type of error.
*/
typedef int TFSERR;


/*  CLIBERR

    Reliance TfsCLibXXX functions error code type.
*/
typedef int CLIBERR;


/*  NLIBERR

    Reliance TfsNLibXXX functions error code type.
*/
typedef int NLIBERR;


/*  true/false
*/
#define TFS_FALSE       0
#define TFS_TRUE        (!TFS_FALSE)
typedef tfsInt32 tfsBool;


/*  tfsBlock

    A block number or a total number of blocks
*/
typedef tfsUint32 tfsBlock;


/*  tfsBlockSize

    A size of a logical block in bytes.  128 to 65536.
*/
typedef tfsBlock tfsBlockSize;


/*  tfsIndex

    An index to a block number (indirection)
*/
typedef tfsUint32 tfsIndex;


/*  tfsHandle

    A handle that referes to a file or directory is really just the
    index for the inode.
*/
typedef tfsIndex tfsHandle;


/*  tfsOffset

    The offset within a file.
*/
#define TFS_USING_64BIT_FILESIZES            0  /* IR ? */

#if TFS_USING_64BIT_FILESIZES
#error 64-bit offsets are not implemented in the Reliance core
typedef tfsUint64 tfsOffset;
#else
typedef tfsUint32 tfsOffset;
#endif


/*  sTfsGeometry

    The volume geometry is a combination of a logical block size and a total
    number of blocks.
*/
typedef struct {
    tfsBlockSize iBlockSize;
    tfsBlock iNumBlocks;
} sTfsGeometry;


/*  sRelOilData

    The structures contents are private to the OIL layer.
*/
typedef struct sRelPrivateOilData sRelOilData;


/*  sRelIoData

    An IO Layer handle.  All the Reliance I/O layer functions use this
    handle.  The handle was returned by a call to TfsIoInitialize.

    The structures contents are private to the IO layer.
*/
typedef struct sRelPrivateIoData sRelIoData;


/*  sOilBlkDevInfo

    An OS specific name or handle that the I/O layer uses to communcate
    with the OS block device.
*/
typedef struct sRelOilBlkDevInfo sOilBlkDevInfo;


/*  sTfsVolume

    For those header files that have to know about the typedef but don't need
    to dereference the structure.
*/
typedef struct TfsTagVolume sTfsVolume;


/*  sTfsExtent
    sTfsExtentTable

    Extent definitions used by some functions.  For complete definitions see
    the rlextent.h file.
*/
typedef struct TfsTagExtent sTfsExtent;
typedef struct TfsTagExtentTable sTfsExtentTable;


/*  for use with progress bars
*/
typedef void(*fnRelProgress)(tfsBlock ulValue,tfsBlock ulLimit,unsigned *puPercent);


#endif /* TFS_TYPES_H */

