/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/bddoc/src/saftl.h-arc  $
 * 
 *    Rev 1.1   Apr 19 2005 02:54:28   oris
 * - Removed the affect of FL_TL_CACHE. The cache size is now
 *   configurable either through the environment variables or though
 *    the FL_DEFAULT_TL_CACHE_SIZE(socket,partition) macro.
 * - You can not set the number of re-tries in the TL:
 *   SAFTL_MAX_HW_RECOVERY_RETRIES
 *   SAFTL_MAX_PARTITIONING_TRIES
 *   MAX_WRITE_FAULT_TRIES
 * 
 * 
 *    Rev 1.0   Mar 27 2005 09:27:18   oris
 * Initial revision.
 * 
 *    Rev 1.5   Mar 27 2005 09:20:20   oris
 * - Added definitions.
 * 
 *    Rev 1.4   Mar 09 2005 02:15:16   oris
 * Renamed noOfFastUnits -> noOfVirFastUnits
 * 
 *    Rev 1.3   Mar 07 2005 22:06:04   oris
 * -	Added fields to Cnand record to support DBB support (failedWriteUnitLog, failedWriteUnitVir, wSectorsPtrArr)
 * -	Changed SAFTL_OLDER_UNIT_LIMIT from 0xEFFF  to  0xF7FF
 * 
 *    Rev 1.2   Feb 01 2005 09:22:06   eugen
 * TL *tl in tTLrec
 * 
 *    Rev 1.1   Dec 27 2004 19:54:08   oris
 * Added static files wear leveling enable field in Cnand record
 * 
 *    Rev 1.0   Dec 09 2004 18:24:32   oris
 * Initial revision.
 * 
 *    Rev 1.12   Oct 17 2004 15:21:52   OriS
 * - Basic support for DSP (CPU without an 8 bit variabes)
 * - New definitions required for dynamic bad blocks table management
 * 
 *    Rev 1.11   Jan 27 2004 08:14:00   oris
 * Changed comments
 * 
 *    Rev 1.10   Sep 30 2003 17:53:36   oris
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.9   Sep 21 2003 10:18:46   OriS
 * Added noOfQuickMountUnits to TL private record.
 * 
 *    Rev 1.8   Sep 01 2003 19:03:26   oris
 * - Changed SAFTL_UNMOUNTED_UNIT from 0xf805 to 0xF810 - Unit not mounted yet
 * - Changed SAFTL_TO_BE_ERASED   from 0xf806 to 0xF811 - Unit should be erased
 * - Added SAFTL_FREE_NOT_ERASED_UNIT 0xf805 - Unused unit that wasa not erased yet
 * - Added 2 new fields to the media header: 
 *      ProgrammerName    - Programmer name
 *      ProgrammerVersion - Programmer version
 * 
 *    Rev 1.7   Jun 19 2003 13:35:52   OriS
 * Removed warnings
 * 
 *    Rev 1.6   Jun 16 2003 01:18:50   oris
 * Added GetSSC and GetVSO macros.
 * 
 *    Rev 1.5   May 16 2003 00:58:46   OriS
 * Bug fix - leave binary partition when disk partition is protected might cause media header to become corrupted.
 * 
 *    Rev 1.4   May 12 2003 22:53:50   oris
 * Added minimal TrueFFS version field in the media header to support future competability.
 * 
 *    Rev 1.3   May 11 2003 08:28:42   OriS
 * Changed static folding buffer size to 2KB instead of 4KB.
 * Added SAFTL_ALGORITHM_SWITCH_PEERCENTAGE definition for setting the percentage of the media to switch folding algorithms.
 * Improved cache size default algorithm.
 * 
 *    Rev 1.2   May 06 2003 11:55:46   OriS
 * Changed SAFTL_K_NUMBER_OF_CACHED_UNITS to FL_NUMBER_OF_CACHED_UNITS
 * 
 *    Rev 1.1   Apr 27 2003 09:04:18   Oris
 * changed static files wear leveling counter to 0x1000
 * Added MAX_BAD_UNIT_IN_FORMAT  which controllers the number of allowed dynamic bad blocks.
 * Added unused fields to the media header.
 * 
 *    Rev 1.0   Apr 09 2003 12:15:54   OriS
 * Initial revision.
 * 
 */


/*******************************************/
/* T r u e F F S   S o u r c e   F i l e s */
/* --------------------------------------- */
/*******************************************/

/*****************************************************************************
* File Header                                                                *
* -----------                                                                *
* Name : saftl.h                                                             *
*                                                                            *
* Description : Data strucute and genreal defintions for SAFTL flash         *
*               translation layer.                                           *
*                                                                            *
*****************************************************************************/


#ifndef SAFTL_H
#define SAFTL_H

#include "flbuffer.h"
#include "flflash.h"
#include "fltl.h"

/*********************************/
/* Private macro's Used by SAFTL */
/*********************************/
         
/* Check if the 2 ANAC are consecutive */
#define ConsecutiveANAC(A,B) \
        (((A+1) & SAFTL_CHAIN_LENGTH_MASK) == (B & SAFTL_CHAIN_LENGTH_MASK))
/* Get ANAC from Chain length table */
#define GetANAC(volPrmPtr,wLogicalUnit) \
        (((volPrmPtr)->CLT[wLogicalUnit] & SAFTL_ANAC_MASK) >> SAFTL_ANAC_SHIFT)
#define SetANAC(volPrmPtr,wLogicalUnit,val) \
        (volPrmPtr)->CLT[wLogicalUnit] &= ~SAFTL_ANAC_MASK; \
        (volPrmPtr)->CLT[wLogicalUnit] |= ((val) << SAFTL_ANAC_SHIFT) & SAFTL_ANAC_MASK
/* Get Chains length from Chain length table */
#define GetChainLength(volPrmPtr,wVirtualUnit) \
        ((volPrmPtr)->CLT[wVirtualUnit] & SAFTL_CHAIN_LENGTH_MASK)
#define SetChainsLength(volPrmPtr,wVirtualUnit,val)               \
        (volPrmPtr)->CLT[wVirtualUnit] &= ~SAFTL_CHAIN_LENGTH_MASK; \
        (volPrmPtr)->CLT[wVirtualUnit] |= (val) & SAFTL_CHAIN_LENGTH_MASK
/* Check if virtual unit is cached */
#define IsUnitCached(volPrmPtr,virtualUnitNo)                    \
        (((volPrmPtr)->VUT[virtualUnitNo] >= SAFTL_CACHED_UNIT) && \
         ((volPrmPtr)->VUT[virtualUnitNo] != SAFTL_UNUSED_UNIT))  
/* Check if virtual unit is in use */
#define IsVirtualUnitFreeOrCached(volPrmPtr,wVirtualUnit) \
        (volPrmPtr->VUT[wVirtualUnit] >= SAFTL_CACHED_UNIT)
/* Check if a logical unit is in use */
#define IsLogicalUnitInUse(volPrmPtr,wLogicalUnit) \
        (volPrmPtr->LUT[wLogicalUnit] > SAFTL_UNUSED_UNIT_BOUNDARY)
/* Get virtual unit cache entry index */
#define GetCacheIndex(volPrmPtr,wVirtualUnit) \
        (((volPrmPtr)->VUT[wVirtualUnit]) & (~SAFTL_CACHED_UNIT))
/* Get pointer to unit cache entry using its index */
#define GetCachePtr(volPrmPtr,wCacheIndex)  \
        ((volPrmPtr)->CT + (wCacheIndex))
/* Round upwards to full sectors */
#define RoundToSectors(size) \
        ((((size) + FL_SECTOR_MASK)>>FL_SECTOR_SIZE_BITS)<<FL_SECTOR_SIZE_BITS)
/* Get VSO value */
#define GetVSO(VSO) \
        (LE2((VSO)) & SAFTL_VSO_SECTOR_OFFSET_MASK) 
/* Get SSC value */
#define GetSSC(offset,SSC) \
	    (((offset) == 0) ? 0 : LE2((SSC)) & SAFTL_VSO_SECTOR_OFFSET_MASK)
/* Check if discard quick mount is required */
#define ConsiderDiscardingQuickMount(volPrmPtr) \
       if(((volPrmPtr)->fIsQuickMountValid == TRUE) && \
          (((volPrmPtr)->volumeFlags & SAFTL_QUICK_MOUNT) != 0)) \
          checkStatus(discardQuickMountInfo(volPrmPtr))


/* Get sector cache entry using the unit cache entry */
#define SectorCacheEntry(volPrmPtr,cachePtr)  \
          ((FLWord FAR1 *)((volPrmPtr)->sectors + \
           (((cachePtr) - (volPrmPtr)->CT) << (volPrmPtr)->sectorsInUnitBits)))

/* Change sector number to the begining of the next page */
#define SetToNextPageStart(volPrmPtr,target) \
        (((target) < (volPrmPtr)->fastAreaVirSectorLimit) ? \
         (((target) & (volPrmPtr)->fastAreaPageMask) + (volPrmPtr)->fastAreaMinSectorsForNextWrite) : /* Fast area */ \
         (((target) & (volPrmPtr)->pageMask) + (volPrmPtr)->minSectorsForNextWrite)) /* Normal area */

#define UnitAndOffset(volPrmPtr,unitNo,sectorNo)        \
   (((unitNo) < (volPrmPtr)->noOfVirFastUnits) ?           \
    (((FLDword)((unitNo)+(volPrmPtr)->firstUnit) <<     \
      (volPrmPtr)->sectorsInFastAreaUnitBits) + (sectorNo)) : \
    ((volPrmPtr)->fastAreaVirSectorLimit + (sectorNo) +      \
      ((FLDword)((unitNo)+(volPrmPtr)->firstUnit-(volPrmPtr)->noOfVirFastUnits) \
       <<volPrmPtr->sectorsInUnitBits)))

/* Find virtual unit based on virtual sector */
#define ToVirtualUnit(volPrmPtr,sectorNo) \
 ((FLWord)(((sectorNo) < (volPrmPtr)->fastAreaVirSectorLimit) ?   \
           ((sectorNo) >> (volPrmPtr)->sectorsInFastAreaUnitBits) : \
           ((volPrmPtr)->noOfVirFastUnits +                 \
            (((sectorNo) - (volPrmPtr)->fastAreaVirSectorLimit)   \
            >> (volPrmPtr)->sectorsInUnitBits)))) 

#define VirtualUnit2MTDFlags(volPrmPtr,virtualUnit) \
        (((virtualUnit) < (volPrmPtr)->noOfVirFastUnits) ? (volPrmPtr)->MTDFlags : 0);
        
#define ToNumOfVirSectors(volPrmPtr,virtualUnit)  \
   ((FLWord)(((virtualUnit) < (volPrmPtr)->noOfVirFastUnits) ?  \
             (volPrmPtr)->virSectorsInFastAreaUnit :         \
             (volPrmPtr)->sectorsInUnit))

/* Get logical address offset in the unit */
#define ToUnitOffset(volPrmPtr,logicalSector,wVirtualUnit)  \
   (FLWord)((logicalSector) & ((wVirtualUnit >= volPrmPtr->noOfVirFastUnits) ?  \
    volPrmPtr->sectorsInUnitMask : volPrmPtr->sectorsInFastAreaUnitMask))
             
/* Check if unit size needs to be halfed */
#define MediaSupportsFastMode(Technology) \
        (((Technology)   & FL_SUPPORT_MLC         ) && \
         (((Technology) & FL_SUPPORT_RELIABLE) ||      \
          ((Technology) & FL_SUPPORT_FAST    )    )    )

#define UnitShouldBeHalfed(MTDFlags,Technology) \
        (((Technology)   & FL_SUPPORT_MLC          ) && \
          ((MTDFlags)    & MTD_REL_MODE            ) && \
          (((Technology) & FL_SUPPORT_RELIABLE) ||      \
           ((Technology) & FL_SUPPORT_FAST    )    )    )
          

/* TL/Socket buffer */
#define saftlBuffer         volPrmPtr->buffer

/*************************/
/* Customization Section */
/*************************/

/* TL limits */
#ifdef FL_ENVIRONMENT_VARS /* Set at run time */
#define FL_NUMBER_OF_CACHED_UNITS(socket,partition)	 flCacheSize[socket][partition]
#else /* Set at compile time */
#define FL_NUMBER_OF_CACHED_UNITS(socket,partition)	 FL_DEFAULT_TL_CACHE_SIZE(socket,partition)
#endif /* FL_ENVIRONMENT_VARS */

#ifndef SAFTL_WEAR_LEVELING_LIMIT
#define SAFTL_WEAR_LEVELING_LIMIT           0x1000L /* Limit for wear leveling */
#endif /* SAFTL_WEAR_LEVELING_LIMIT */

#ifndef SAFTL_MAX_FORMAT_TRYS
#define SAFTL_MAX_FORMAT_TRYS               255 /* (no more then 255) */
#endif /* SAFTL_MAX_FORMAT_TRYS */

#ifndef SAFTL_MAX_HW_RECOVERY_RETRIES
#define SAFTL_MAX_HW_RECOVERY_RETRIES       20
#endif /* SAFTL_MAX_HW_RECOVERY_RETRIES */

#ifndef SAFTL_MAX_PARTITIONING_TRIES
#define SAFTL_MAX_PARTITIONING_TRIES        5
#endif /* SAFTL_MAX_PARTITIONING_TRIES */

#ifndef MAX_WRITE_FAULT_TRIES
#define MAX_WRITE_FAULT_TRIES               10
#endif /* MAX_WRITE_FAULT_TRIES */


#ifndef MAX_UNIT_CHAIN
#ifdef FL_ENVIRONMENT_VARS
#define MAX_UNIT_CHAIN                      flMaxUnitChain
#else /* FL_ENVIRONMENT_VARS not defined */
#define MAX_UNIT_CHAIN                      7
#endif /* FL_ENVIRONMENT_VARS */
#endif /* MAX_UNIT_CHAIN */

#ifndef SAFTL_ALGORITHM_SWITCH_PEERCENTAGE
#define SAFTL_ALGORITHM_SWITCH_PEERCENTAGE  65
#endif /* SAFTL_ALGORITHM_SWITCH_PEERCENTAGE */


/***********************/
/* General Definitions */
/***********************/

/* Media flags / formatFlags */
#define SAFTL_NO_FLAGS              0x00000000L
#define SAFTL_DBBT_IN_MEDIA_HEADER  0x00000001L

/* Partition flags */
#define SAFTL_ADVANCED_MATCHING     0x00100000L
#define SAFTL_REL_PARTITION         0x00200000L
#define SAFTL_FAST_PARTITION        0x00400000L
#define SAFTL_SPACED_UNITS          0x20000000L
#define SAFTL_NO_MATCHING           0x00800000L
#define SAFTL_RUGGEDIZED            0x01000000L
#define SAFTL_BINARY                0x02000000L
#define SAFTL_BDTL                  0x04000000L
#define SAFTL_LAST                  0x08000000L
#define SAFTL_QUICK_MOUNT           0x10000000L

/* PROTECTABLE            defined in flflash.h as 1   */
/* READ_PROTECTED         defined in flflash.h as 2   */
/* WRITE_PROTECTED        defined in flflash.h as 4   */
/* LOCK_ENABLED           defined in flflash.h as 8   */
/* LOCK_ASSERTED          defined in flflash.h as 16  */
/* KEY_INSERTED           defined in flflash.h as 32  */
/* CHANGEABLE_PROTECTION  defined in flflash.h as 64  */
/* STICKY_LOCK_ASSERTED   defined in flflash.h as 128 */

/* VUT values */
#define SAFTL_CACHED_UNIT          0xF000 /* Indicates a unit is cached     */

/* Additional LUT values valid only during the mount proccess */
#define SAFTL_UNMOUNTED_UNIT       0xF810 /* Unit not mounted yet           */
#define SAFTL_TO_BE_ERASED         0xF811 /* Unit should be erased          */ 

/* LUT values */
#define SAFTL_UNINITIALIZED_OLDEST 0xF800 /* Oldest in chain unit with      */
                                          /* uninitialized last used sector */
#define SAFTL_FREE_UNIT            0xF801 /* Unused logical unit            */
#define SAFTL_BAD_UNIT             0xF802 /* Unsable unit marked by the MTD */
#define SAFTL_UNAVAILABLE_UNIT     0xF803 /* Unsable unit marked by the TL  */
#define SAFTL_FAILED_TO_ERASE      0xF804 /* Failed erasing this unit, try  */
#define SAFTL_FREE_NOT_ERASED_UNIT 0xF805 /* Unused unit that was not       */
                                          /* verified as an erased unit     */
#define SAFTL_FAILED_TO_WRITE      0xF806 /* Failed writing to this unit    */
#define SAFTL_FAILED_TO_READ       0xF807 /* Failed reading from this unit  */
#define SAFTL_UNUSED_UNIT_BOUNDARY 0xF800 /* Higher LUT value indicate the  */
                                          /* unit is not in use.            */
#define SAFTL_OLDER_UNIT_LIMIT     0xF7FF /* Largest LUT value that points  */
                                          /* to older logical unit          */
                                          /* not to use it                  */

/* Both LUT and VUT values */
#define SAFTL_UNUSED_UNIT          0xFFFF


/* Sector values */
#define SAFTL_INVALID_SECTOR_NO        0xFFFFFFFF /* ANAND_UNASSIGNED_ADDRESS */

/* Sector cache entrie values */
#define SAFTL_SECTOR_NOT_VALID         0x8001 /* Uninitialized            */
#define SAFTL_SECTOR_NOT_FOUND         0x8000 /* Deleted or never written */
#define SAFTL_NONE_USED_SECTOR_MASK    0x8000 /* Either of the above      */

/* Sector flash flags */
#define SAFTL_VSO_SECTOR_FREE_FLAG     0xC000 /* Sector is free : 14-15 LSB */
#define SAFTL_VSO_SECTOR_USED_FLAG     0x8000 /* Sector is used : 14-15 LSB */
#define SAFTL_VSO_SECTOR_TRANSACTION   0x3800 /* 11-13 LSB - transaction    */
#define SAFTL_SSC_SECTOR_RESERVED      0xC000 /* 14-15 LSB - SSC reserved   */
#define SAFTL_ANAC_SECTOR_RESERVED     0x70   /*  4-6  LSB - ANAC reserved  */

/* Sector flash masks */
#define SAFTL_VSO_SECTOR_OFFSET_MASK       0x7FF  /*  0-10 LSB - sector offset      */
#define SAFTL_VSO_SECTOR_TRANSACTION_MASK  0x3800 /* 11-13 LSB - sector transcation */
#define SAFTL_VSO_SECTOR_FLAGS_MASK        0xC000 /* 14-15 LSB - sector flags       */
#define SAFTL_SSC_RESERVED_MASK            0xC000 /* 14-15 LSB - SSC reserved       */
#define SAFTL_ANAC_INFO_MASK               0x0F00 /*  9-12 LSB - ANAC bits          */
#define SAFTL_ANAC_RESERVED_MASK           0x7000 /* 12-14 LSB - ANAC reserved      */

#define SAFTL_UNIT_OFFSET_MASK             0x7800 /* 11-14 LSB - unit position in chain */
#define SAFTL_UNIT_OFFSET_SHIFT            11     /* unit position shift       */

/* CLT masks */
#define SAFTL_CHAIN_LENGTH_MASK            0x0F
#define SAFTL_ANAC_MASK                    0xF0
#define SAFTL_ANAC_SHIFT                   4

/* NORMAL/FAST flash block mark */
#define SAFTL_NORMAL_BLOCK_MARK            0x0
#define SAFTL_FAST_BLOCK_MARK              0xFF


#ifndef SAFTL_HEADER_SEARCH_LIMIT
#define SAFTL_HEADER_SEARCH_LIMIT          16
#endif /* SAFTL_HEADER_SEARCH_LIMIT */

#define MAX_BAD_UNIT_IN_FORMAT             10
#define DOUBLE_CHAIN_LENGTH                (MAX_UNIT_CHAIN << 1) 
#define MAX_CACHED_UNITS                   0x1000 /* TL limitataion, can not be changed */
#define HEADERS_SPACING                    8UL
#define NO_OF_MEDIA_HEADERS                2
#define MAX_PROTECTION_AREAS               2

#define SAFTL_PAGE_EXTRA_SIZE              8
#define SAFTL_ALL_FLOORS                   0xff
#define ILLEGAL_DPS_NO                     0xff
#define SAFTL_FORCE_POLLING                2
#define SAFTL_ENDIAN_VALIDATION_MARK       0xaa551100L
#define SAFTL_QUICK_MOUNT_VERSION          0L
#define SAFTL_DBBT_MH_MARK                 "UMDBBT"


/*******************/
/* Data structures */
/*******************/

/* Unit cache entry record */
typedef struct {
   FLWord        VUN;         /* Virtual unit this entry belongs to         */
   FLWord        LUN;         /* Newest logical unit of the chain           */
   FLWord        olderUnit;   /* Cache entry less recently used             */
   FLWord        newerUnit;   /* Cache entry more recently used             */
   FLWord        oldGrade;    /* Grade of folding from the oldest unit      */
   FLWord        newGrade;    /* Grade of folding from the newest unit      */
} CNANDCache;

/* Unit control information of the flash - first sector only */
typedef struct {
   LEushort VSO;     
   LEushort VUN;
   LEushort PUN;
   FLByte ANAC;
   FLByte unused;
}CNANDUnitHeader;

/* Sector control information of the flash - none first sectors */
typedef struct {
  LEushort VSO;
  LEushort SSC;
  LEushort reserved1;
  LEushort reserved2;
}CNANDSectorHeader;

/* Wear leveling data structure */
typedef struct {
  FLWord alarm;    /* Counter to next initiated wear leveling     */
  FLWord currUnit; /* Roving unit to perform static wear leveling */
} WLdata;

/* Media Header Global Record */
typedef struct {
  LEulong    bootRecordId[2];      /* "CNAND" */
  LEulong    noOfBootImageBlocks;  /* no of good blocks in the boot image area */
  LEulong    noOfBinaryPartitions; /* no of binary partitions                  */
  LEulong    noOfBDTLPartitions;   /* no of BDTL partitions                    */
  LEulong    blockMultiplierBits;  /* no of bits used to represent the         */
                                   /* times a flash erasable block fits inside */
                                   /* an INFTL erasable unit.                  */
  LEulong    formatFlags;          /* Currently only QUICK_MOUNT               */
  LEulong    osakVersion;          /* version of TFFS used to format the media */
  LEulong    percentUsed;          /* Percent of the media to be exported      */
  LEulong    headerUnits;          /* Number of units reserved in the begining */
                                   /* of each floor for the media header       */
  LEulong    floorNo;              /* Number of current floor                  */
  LEulong    minOsakVersion;       /* Smallest TrueFFS version supported       */
  LEulong    programmerName;       /* Programmer name                          */
  LEulong    programmerVersion;    /* Programmer version                       */
  LEulong    unused1;
} CNANDBootRecord;

/* Media Header Volume record */
typedef struct {
  LEulong  totalGoodUnits;  /* Total good unit (sum of the next 4 fields)     */
  LEulong  virtualUnits;    /* Virtual unit size exported by the TL           */
  LEulong  transferUnits;   /* Number of good units found in BB percentage    */
  LEulong  spareUnits;      /* Number of spare units requested by the user    */
  LEulong  quickMountUnits; /* Number of good units used by the quick mount   */  
  LEulong  firstUnit;       /* First unit of the partition                    */
  LEulong  lastUnit;        /* Last unit of the partition                     */
  LEulong  firstQuickMountUnit; /* First unit used for the quick mount data   */
  LEulong  flags;           /* See list above "Partition flags"               */
  LEulong  protectionArea;  /* Number of the partitions protection area       */
  LEulong  fastAreaUnits;   /* Number of units using FAST mode                */
  LEulong  fastAndNormalFactor; /* Bits 0-7 are normal Area spacing           */
				/* Bits 8-15 are fast Area spacing            */
  LEulong  unused3;
  LEulong  unused4;
} CNANDVolumeHeaderRecord;

/* Quick mount flash structure */
typedef struct {
   FLDword       quickMountVersion;   /* Version of the quick mount data      */
   FLDword       endianValidation;    /* This string is read differently      */
                                      /* pending the endian architecture      */ 
   FLDword       noOfCachedUnits;     /* The number of cached units           */
   FLDword       neededFreeUnits;     /* Minimal number of free units that    */
                                      /* require folding                      */
   FLDword       freeUnits;           /* Free units on media                  */
   FLDword       roverUnit;           /* Starting point for allocation search */
   FLDword       bootUnits;           /* No of boot units before partition 0  */
   FLDword       multiUnitListHead;   /* Head of multi unit chains cache      */
   FLDword       multiUnitListTail;   /* Tail of multi unit chains cache      */
   FLDword       singleUnitListHead;  /* Head of single unit chains cache     */
   FLDword       singleUnitListTail;  /* Tail of single unit chains cache     */
   FLDword       freeCacheUnitList;   /* free cache entries queue             */
                                      /* Accumulated statistics.              */
   FLDword       sectorsRead,         /* Virtual sectors read                 */
                 sectorsWritten,      /* Virtual sectors written              */
                 sectorsDeleted,      /* Virtual sectors deleted              */
                 parasiteWritesFromOldest, /* Sectors written while folding   */
                 parasiteWritesFromNewest, /* Sectors written while folding   */ 
                 unitsFoldedFromOldest,    /* Units folded from oldest        */
                 unitsFoldedFromNewest;    /* Units folded from newest        */ 
   FLDword       alarm;       /* Counter to next initiated wear leveling      */
   FLDword       currUnit;    /* Unit to perform static wear leveling         */
   FLDword       eraseSum;    /* Accumulator of the erase operations          */
   FLDword       noOfUsedUnits;  /* No of used virtual units                  */
   FLDword       dwUMDBBTSector; /* Location of latest UMDBBT data            */                 
} CNANDQuickMountInfo;

/* Format private record */
typedef enum {
   binaryPartition, 
   BDTLPartition,
   leaveBinary
} PartitionType;

typedef struct {
   FLByte     bSignPtr[BINARY_SIGNATURE_LEN]; /* Binary signature for binary partition */
   FLDword    dwMTDFlags;              /* MTD flags of the parititon                */
   FLDword    dwMediaFlags;            /* Volume header flags for the partition     */
   FLWord     wLength;                 /* Number of units in the partition          */
   FLWord     wLengthSoFar;            /* Number of units used up to this partition */
   FLWord     wPreviousLengthSoFar;    /* Number of units used up to this floor     */
   FLWord     wLengthInFloor;          /* Number of units used in this floor        */
   FLWord     wSpare;                  /* Number of spare units for this partition  */ 
   FLWord     wFastAreaUnits;
   FLWord     wVirtualUnits;     
   FLByte     bFastFactor;
   FLByte     bNormalFactor;   
   FLWord     wQuickMountUnits;        /* Number of quick mount units               */
   FLWord     wDynamicBadBlocks;       /* Number of dynamic bad blocks              */
   FLSWord    swTransferUnits;         /* Number of transfer units so far           */
   FLSWord    swPreviousTransferUnits; /* Number of transfer units in previous floors */
   PartitionType   type;               /* Type - binary / leave binary / bdtl       */   
   FLByte     bMatchingType;           /* Type of matching SM/AM                    */  
   FLByte     bProtectionType;         /* Protection attributes                     */
#ifdef HW_PROTECTION
   FLByte     protectionKey[8];        /* Partition protection key                  */
   FLByte     bFloorsOccupied[FL_MAX_NO_OF_FLOORS]; /* On which floors is it on     */
#endif /*HW_PROTECTION*/
   FLByte     bProtectionArea;         /* DPS number used for the partition         */
}SaftlFormatPartition;

/* Translation Layer context record */
struct tTLrec{
  CNANDSectorHeader FAR1 * myHeaders;  /* Pointer to array of sector headers  */
  FLWord            FAR1 * VUT;        /* Pointer to Virtual Unit Table.      */
                                       /* Indexed by virtual unit no'.        */
                                       /* Stores the logical unit no' of the  */
                                       /* newest logical unit of the virtual  */
                                       /* chain. In case unit is cached, it   */
                                       /* will indicate the cache entry no'   */
                                       /* used for this virtual unit data.    */
  FLWord            FAR1 * LUT;        /* Pointer to the Logical Unit Table.  */
                                       /* Indexed by logical unit no'.        */
                                       /* Stores the no' of the older logical */
                                       /* unit in the virtual chain.          */
                                       /* A value of 0xFXXX indicates the     */
                                       /* does not have an older unit. Such a */
                                       /* unit is either the oldest unit      */
                                       /* marked with 0x1111,0XXX,XXXX,XXXX.  */
                                       /* where the 11 LSB represent a        */
                                       /* zero-based index of the last used   */
                                       /* sector in the newest unit of the    */
                                       /* chain, or any of the following:     */
                                       /* F800H - Unit is in use, but its     */
                                       /*         entry is not valid.         */
                                       /* F801H - Unit is free.               */ 
                                       /* F802H - Unit is bad.                */
                                       /* F803H - Unit is unavailable.        */
   FLByte           FAR1 * CLT;        /* Chains Length Table.                */
                                       /* Indexed by both logical unit and    */
                                       /* virtual unit.                       */
                                       /* Stores the length of virtual chain  */
                                       /* Stores the ANAC of the logical unit */
   FLByte     FAR1 * foldingBuffer;    /* Internal buffer for folding.        */
   CNANDCache FAR1 * CT;               /* Pointer to the SAFTL cache table.   */                 
   FLWord     FAR1 * sectors;          /* Pointer to sector cache table.      */
   FLByte     FAR1 * buffer;           /* Pointer to TL buffer                */   
   FLFlash         * flash;            /* Pointer to MTD record               */
   TL              * tl;               /* Pointer to TL                       */
   FLWord FAR1    ** wSectorsPtrArr;
   CNANDUnitHeader   unitHeader;       /* Unit header - used for mount        */   
   FLBoolean       fIsQuickMountValid; /* Indicator for writing the quick     */
                                       /* mount                               */   
   FLBoolean       staticFilesWearLeveling;   
   FLDword         dwUMDBBTSector;     /* UMDBB sector offset                 */
   FLDword      mediaFlags;          /* media specific flags                 */
   FLDword      volumeFlags;         /* Volume specific flags                */
   FLDword      MTDFlags;            /* MTD modes - common dwOpFlags fields  */
   FLByte       socketNo;            /* Socket number of the volume          */
   FLByte       partitionNo;         /* Partition number of the volume       */                                  
   FLByte       neededFreeUnits;     /* Minimal number of free units that    */
                                     /* require folding                      */
   FLDword      pageMask;            /* Risk zone mask                       */
   FLDword      fastAreaPageMask;    /* Risk zone mask in a fast unit        */
   FLWord       foldingBufferSize;   /* Actual size of the folding buffer    */   
   FLByte       minSectorsForFolding;   /* Minimum sectors that can be written together */
   FLByte       minSectorsForNextWrite; /* Minimum number of sectors alignment for write operation */
   FLByte       fastAreaMinSectorsForNextWrite;
   FLByte       maxRelatedSectors;   /* Max number of sectors that might be written together */
   FLByte       fastAreaMaxRelatedSectors;
   FLWord       failedWriteUnitLog;  /* Logical unit to be marked as DBB     */
   FLWord       failedWriteUnitVir;  /* Virtual unit to be marked as DBB     */
   FLWord       noOfUnits;           /* No of logical units in the partition */
   FLWord       firstQuickMountUnit; /* The quick mount first unit           */
   FLWord       firstUnit;           /* First unit number of the volume      */
   FLWord       freeUnits;           /* Free units on media                  */
   FLWord       noOfVirtualUnits;    /* No of units exported by the TL       */
   FLWord       noOfTransferUnits;   /* No of spare units for chaining       */
   FLWord       noOfQuickMountUnits; /* No of quick mount units              */   
   FLWord       noOfUsedUnits;       /* No of used virtual units             */
   FLWord       noOfCachedUnits;     /* No' of cached units                  */      
   FLWord       noOfVirFastUnits;       /* No of fast units exported by the TL  */
   FLDword      fastAreaVirSectorLimit; /* No of sectors using the fast mode    */   
   FLWord       fastAreaVirtualFactor;  /* Factor of combined spaced and fast   */
   FLWord       normalAreaFactor;    /* Factor of area B                     */  
   FLWord       roverUnit;           /* Starting point for allocation search */
   FLWord       bootUnits;           /* No of boot units before partition 0  */
   FLWord       doubleHeadChain[4];  /* The 2 heads of the same virtual unit */
   
   FLWord       multiUnitListHead;   /* Head of multi unit chains cache      */
   FLWord       multiUnitListTail;   /* Tail of multi unit chains cache      */
   FLWord       singleUnitListHead;  /* Head of single unit chains cache     */
   FLWord       singleUnitListTail;  /* Tail of single unit chains cache     */
   FLWord       freeCacheUnitList;   /* Free cache entries queue             */
   
   FLByte       sectorsInUnitBits;   /* No' of bits used to represent the    */
                                     /* no' of sectors in a logical unit     */
   FLWord       sectorsInUnitMask;   /* Mask of number of sectors in a unit  */
   FLWord       sectorsInUnit;       /* No of sectors in a logical unit      */
   
   FLByte       sectorsInFastAreaUnitBits; /* No' of bits used to represent the    */
                                           /* no' of sectors in a logical unit     */     
   FLWord       sectorsInFastAreaUnitMask; /* Mask of number of sectors in a unit  */
   FLWord       sectorsInFastAreaUnit;     /* No of sectors in a logical unit      */   

   FLByte       virSectorsInFastAreaUnitBits; /* No' of bits used to represent the    */
                                              /* no' of sectors in a logical unit     */     
   FLWord       virSectorsInFastAreaUnitMask; /* Mask of number of sectors in a unit  */
   FLWord       virSectorsInFastAreaUnit;     /* No of sectors in a logical unit      */   
   FLWord          wHeaderLocation[FL_MAX_NO_OF_FLOORS];   
   const void FAR0 *mappedSector;
   CardAddress      mappedSectorAddress;
   SectorNo         mappedSectorNo; /* Currently mapped sector            */
#ifndef TL_NO_STATISTICS
   /* Accumulated statistics. */
   FLDword          sectorsRead,    /* Virtual sectors read               */
                    sectorsWritten, /* Virtual sectors written            */
                    sectorsDeleted, /* Virtual sectors deleted            */
                    parasiteWritesFromNewest, /* Logical sectors written in folding */
                    parasiteWritesFromOldest, /* Logical sectors written in folding */
                    unitsFoldedFromNewest,    /* Units erase as part of folding     */
                    unitsFoldedFromOldest;    /* Units erase as part of folding     */
#endif /* TL_NO_STATISTICS */
   WLdata           wearLevel;           /* Wear leveling data                   */
   FLDword          eraseSum;            /* Accumulator of the erase operations  */
   SectorNo         virtualSectors;      /* No of sectors exported by the TL     */
};

typedef TLrec Cnand;

#endif /* SAFTL_H */
