/***********************************************************************************/
/*                        M-Systems Confidential                                   */
/*           Copyright (C) M-Systems Flash Disk Pioneers Ltd. 1995-2003            */
/*                         All Rights Reserved                                     */
/***********************************************************************************/
/*                            NOTICE OF M-SYSTEMS OEM                              */
/*                           SOFTWARE LICENSE AGREEMENT                            */
/*                                                                                 */
/*      THE USE OF THIS SOFTWARE IS GOVERNED BY A SEPARATE LICENSE                 */
/*      AGREEMENT BETWEEN THE OEM AND M-SYSTEMS. REFER TO THAT AGREEMENT           */
/*      FOR THE SPECIFIC TERMS AND CONDITIONS OF USE,                              */
/*      OR CONTACT M-SYSTEMS FOR LICENSE ASSISTANCE:                               */
/*      E-MAIL = info@m-sys.com                                                    */
/***********************************************************************************/

/*
 * $Log:   V:/PVCSDB/DiskOnChip/archives/Testing/TrueFFS 6.3/Drop 2.5/3/surefs/src/sscache.h-arc  $
 * 
 *    Rev 1.3   Jul 11 2005 19:06:04   Polina.Marimont
 * FL_INIT_SECTOR_CACHE when FS_FILES not defined
 * 
 *    Rev 1.2   Jul 11 2005 13:24:02   oris
 * Removed warnings and required for init cache bug fix.
 * 
 *    Rev 1.1   Jun 07 2005 06:27:30   OriS
 * -	Bug fix - Failing to allocate the file system cache might will not exit properly (release mutexes , )
 * 
 *    Rev 1.0   Mar 27 2005 09:26:52   oris
 * Initial revision.
 * 
 *    Rev 1.5   Mar 09 2005 23:48:12   eugen
 * different macros to support new read with sector cache
 * 
 *    Rev 1.4   Mar 07 2005 22:17:16   oris
 * -	Comments
 * 
 *    Rev 1.3   Feb 19 2005 12:25:44   eugen
 * _HI/_LO 13/3 instead of 12/4
 * 
 *    Rev 1.2   Feb 01 2005 10:14:16   eugen
 * file cache support - allocation for both FAT/dirs and files
 * 
 *    Rev 1.1   Dec 23 2004 19:35:54   oris
 * Drop 2.1
 * 
 *    Rev 1.0   Dec 09 2004 18:22:20   oris
 * Initial revision.
 * 
 *    Rev 1.1   Sep 30 2003 17:57:58   oris
 * Added FL prefix to SECTOR_SIZE and SECTOR_SIZE_BITS
 * 
 *    Rev 1.0   Apr 09 2003 12:15:58   OriS
 * Initial revision.
 *
 */

#ifndef SSCACHE_H
#define SSCACHE_H

#include "flbase.h"


#ifdef FL_EXTENDED_SECTOR_CACHE

#ifdef FS_SUPPORT_FILE_CACHE
#define FL_SEARCH_ELEMENTS_LO		3
#define FL_SEARCH_ELEMENTS_HI		13

#else

#define FL_SEARCH_ELEMENTS_LO		8	/* does not matter for !SUPPORT_FILE_CACHE */
#define FL_SEARCH_ELEMENTS_HI		8	/* only the sum is used */
#endif

/* #define FL_SEARCH_ELEMENTS		16
	For slower platforms 4 and 8 might be better.
	In any case should not be less than 3
	because dirty and previous elements are skipped */



#if (FL_SEARCH_ELEMENTS_LO + FL_SEARCH_ELEMENTS_HI) < 3
#error FL_SEARCH_ELEMENTS is too small
#endif

typedef struct {
  SectorNo	sectorNo;            	/* current sector in buffer */
} FLCacheSectorInfo;


typedef struct {
  FLByte FAR0 *	flCacheData;		/* all sector memory of this volume... */
  FLCacheSectorInfo * sectorBuffers;	/* indexing overhead */

#ifdef FS_SUPPORT_FILE_CACHE
  unsigned * inUseByFiles;		/* bitmaps */
  unsigned * inUseByFATnDIRs;		/* ... and by files without cache too */
#endif

  unsigned	sectorsNumber;
  SectorNo	dirtySectorNo;
  SectorNo	prevSectorNo;
  FLBoolean	checkPoint;          /* dirty sector must be flushed */
} FLSectorCache;


#define cacheSectorData(cacheInVol, bufIdx)	addToFarPointer(cacheInVol->flCacheData,(FLDword)bufIdx<<FL_SECTOR_SIZE_BITS)

#else	/* FL_EXTENDED_SECTOR_CACHE */

#include "flbuffer.h"
#endif	/* FL_EXTENDED_SECTOR_CACHE */


typedef struct FSVolume FSVolume;

FLStatus flushBuffer(FSVolume * pVol);
FLStatus updateSector(FSVolume * pVol, SectorNo sectorNo, FLBoolean read, void * * bufPtr);
FLStatus findSector(FSVolume * pVol, SectorNo sectorNo, void FAR0 ** retBuffer);

#ifdef FS_SUPPORT_FILE_CACHE
FLByte * flAllocateCacheSectors(FSVolume * pVol, unsigned needSectors /* , unsigned cachePriority */);
void flFreeCacheSectors(FSVolume * pVol, FLByte * cacheBuffer, unsigned sectorNum);
#endif
/* ------------------------------------------------------------------------------ */
#ifdef FS_FILES

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_INVALIDATE_SECTOR_BUFFER(pVol, currSector)			\
        if ((currSector == pVol->volBuffer.sectorNo) && (((void *)pVol) == pVol->volBuffer.owner)) {	\
          pVol->volBuffer.sectorNo = UNASSIGNED_SECTOR;			\
          pVol->volBuffer.dirty = pVol->volBuffer.checkPoint = FALSE;	\
          }
#else
void flInvalidateSector(FSVolume * pVol, SectorNo sectorNo);

#define FL_INVALIDATE_SECTOR_BUFFER(pVol, secNo)	flInvalidateSector(pVol, secNo)
#endif	/* FL_EXTENDED_SECTOR_CACHE */

#else
#define FL_INVALIDATE_SECTOR_BUFFER(pVol, secNo)
#endif	/* no FS_FILES > 0 */
/* ------------------------------------------------------------------------------ */
#ifdef FS_FILES

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_EXIT_SECTOR_CACHE(pVol)  {					\
	pVol->volBuffer.sectorNo = UNASSIGNED_SECTOR;			\
	pVol->volBuffer.dirty = pVol->volBuffer.checkPoint = FALSE;	\
	}

#else

void flExitCache(FSVolume * pVol);

#define FL_EXIT_SECTOR_CACHE(pVol) flExitCache(pVol)

#endif	/* FL_EXTENDED_SECTOR_CACHE */

#else
#define FL_EXIT_SECTOR_CACHE(pVol)
#endif	/* no FS_FILES > 0 */
/* ------------------------------------------------------------------------------ */
#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_CLEAN_SECTOR_CACHE(pVol, volNum)  {			\
	pVol->volBuffer.dirty = FALSE;				\
	pVol->volBuffer.owner = NULL;				\
	pVol->volBuffer.sectorNo = UNASSIGNED_SECTOR;		\
	pVol->volBuffer.checkPoint = FALSE;			\
	}
#else

#ifndef FS_SUPPORT_FILE_CACHE

#define FL_CLEAN_SECTOR_CACHE(pVol, volNum)  {			\
	pVol->volBuffer.dirtySectorNo = UNASSIGNED_SECTOR;	\
	pVol->volBuffer.flCacheData = NULL;			\
	pVol->volBuffer.sectorBuffers = NULL;			\
	pVol->volBuffer.sectorsNumber = 0;			\
	pVol->volBuffer.checkPoint = FALSE;			\
	}
#else /* FS_SUPPORT_FILE_CACHE */

#define FL_CLEAN_SECTOR_CACHE(pVol, volNum)  {			\
	pVol->volBuffer.dirtySectorNo = UNASSIGNED_SECTOR;	\
	pVol->volBuffer.flCacheData = NULL;			\
	pVol->volBuffer.sectorBuffers = NULL;			\
	pVol->volBuffer.inUseByFiles = NULL;			\
	pVol->volBuffer.inUseByFATnDIRs = NULL;			\
	pVol->volBuffer.sectorsNumber = 0;			\
	pVol->volBuffer.checkPoint = FALSE;			\
	}
#endif /* FS_SUPPORT_FILE_CACHE */

#endif

/* this one is called from FATLITE - so no need for definition if no FILES */
/* ------------------------------------------------------------------------------ */
#if (defined(FS_FILES) && (defined(FL_EXTENDED_SECTOR_CACHE)))

FLStatus flInitCache(FSVolume * pVol, int socket, int partition, int logPartition);

#else
#define FL_INIT_SECTOR_CACHE(pVol, socket, partition, logPartition)
#endif	/* no FS_FILES > 0 */

/* ------------------------------------------------------------------------------ */
#ifdef FS_FILES

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_DIRTY_SECTOR_IN_RANGE(pVol, sectorToRead, sectorCount)			\
		(pVol->volBuffer.owner == pVol) && (sectorToRead <= pVol->volBuffer.sectorNo) &&	\
		((sectorToRead+sectorCount) > pVol->volBuffer.sectorNo)

#define FL_DIRTY_SECTOR(pVol)	pVol->volBuffer.sectorNo

#define FL_COPY_DIRTY_SECTOR(pVol, userData, sectorNumInData)				\
	  tffscpy(&userData[sectorNumInData<<FL_SECTOR_SIZE_BITS], pVol->volBuffer.flData, FL_SECTOR_SIZE);

#else
unsigned flFindExistingSector(FLSectorCache * cacheInVol, SectorNo sectorNo);

#define FL_DIRTY_SECTOR_IN_RANGE(pVol, sectorToRead, sectorCount)			\
		(sectorToRead <= pVol->volBuffer.dirtySectorNo) &&			\
		((sectorToRead+sectorCount) > pVol->volBuffer.dirtySectorNo)

#define FL_DIRTY_SECTOR(pVol)	pVol->volBuffer.dirtySectorNo

#define FL_COPY_DIRTY_SECTOR(pVol, userData, sectorNumInData)  {				\
	  unsigned buffIndex = flFindExistingSector(&pVol->volBuffer, pVol->volBuffer.dirtySectorNo);	\
	  tffscpy(&userData[sectorNumInData<<FL_SECTOR_SIZE_BITS], cacheSectorData((&pVol->volBuffer), buffIndex), FL_SECTOR_SIZE);	\
	  }

#endif	/* FL_EXTENDED_SECTOR_CACHE */

#else
#define FL_DIRTY_SECTOR_IN_RANGE(pVol, sectorToRead, sectorCount)	FALSE
#define FL_DIRTY_SECTOR(pVol)						                FALSE
#define FL_COPY_DIRTY_SECTOR(pVol, userData, sectorNumInData)
#endif	/* no FS_FILES > 0 */
/* ------------------------------------------------------------------------------ */
#ifdef FS_FILES

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_INVALIDATE_SECTOR_RANGE(pVol, sectorToWrite, sectorCount)		\
      if (((sectorToWrite+sectorCount > pVol->volBuffer.sectorNo) &&		\
	(sectorToWrite <= pVol->volBuffer.sectorNo)) && ((void *)pVol == pVol->volBuffer.owner)) {	\
        pVol->volBuffer.sectorNo = UNASSIGNED_SECTOR;				\
        pVol->volBuffer.dirty = pVol->volBuffer.checkPoint = FALSE;			\
        }

#else
#define FL_INVALIDATE_SECTOR_RANGE(pVol, sectorToWrite, sectorCount)  {			\
	SectorNo sIdx;									\
	for (sIdx = sectorToWrite; sIdx < (sectorToWrite + sectorCount); sIdx++)	\
  	  flInvalidateSector(pVol, sIdx);						\
  	}

#endif	/* FL_EXTENDED_SECTOR_CACHE */

#else
#define FL_INVALIDATE_SECTOR_RANGE(pVol, sectorToWrite, sectorCount)
#endif	/* no FS_FILES > 0 */
/* ------------------------------------------------------------------------------ */

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_CACHE_IS_DIRTY(pVol)	pVol->volBuffer.dirty

#else

#define FL_CACHE_IS_DIRTY(pVol)	(pVol->volBuffer.dirtySectorNo != UNASSIGNED_SECTOR)

#endif

/* ------------------------------------------------------------------------------ */

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_SET_CACHE_DIRTY_FALSE(pVol)	pVol->volBuffer.dirty = FALSE

#else

#define FL_SET_CACHE_DIRTY_FALSE(pVol)	pVol->volBuffer.dirtySectorNo = UNASSIGNED_SECTOR

#endif

/* ------------------------------------------------------------------------------ */

#ifndef FL_EXTENDED_SECTOR_CACHE

#define FL_RENAME_DIRTY_SECTOR(pVol, newSectorNum)	pVol->volBuffer.sectorNo = newSectorNum

#else

#define FL_RENAME_DIRTY_SECTOR(pVol, newSectorNum)	{			\
	  unsigned buffIndex = flFindExistingSector(&pVol->volBuffer, pVol->volBuffer.dirtySectorNo);	\
  	  pVol->volBuffer.sectorBuffers[buffIndex].sectorNo = newSectorNum;	\
  	  pVol->volBuffer.dirtySectorNo = newSectorNum;				\
	  }

#endif
/* ------------------------------------------------------------------------------ */

#endif
