/* timeCommon.h - POSIX timer header file shared btw. user/kernel */

/* Copyright 2003-2004 Wind River Systems, Inc. */

/*
modification history
--------------------
01h,10aug04,lei  added declarations of timer_close() & timer_unlink() 
                 (SPR# 99106).
01g,22apr04,dcc  added 'context' parameter to timer_open().
01f,02apr04,mcm  Fix for SPR 95564 - clockid_t redefined on user side.
01e,26feb04,yvp  Corrected typo on line 67 (_WRS_KERNEL).
01d,09feb04,mcm  Header file cleanup.
01c,04dec03,pad  Added declarations of clock_gettime(), clock_settime() and
		 clock_getres(). Removed the non-ANSI prototypes. Removed
		 prototypes for reentrant POSIX APIs.
01b,02dec03,ans  code inspection comments.
01a,02oct03,ans  written
*/

#ifndef __INCtimecommonh
#define __INCtimecommonh

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __RTP__
#include <time.h>
#endif /* __RTP__ */

#include "types/vxANSI.h"
#include "sigevent.h"
#include "objLib.h"

#ifndef NULL
#if defined __GNUG__
#define NULL (__null)
#else
#if !defined(__cplusplus) && 0
#define NULL ((void*)0)
#else
#define NULL (0)
#endif
#endif
#endif

#ifdef  _TYPE_timer_t
_TYPE_timer_t;
#undef  _TYPE_timer_t
#endif

#ifdef _WRS_KERNEL
#ifdef _TYPE_clock_t
_TYPE_clock_t;
#undef _TYPE_clock_t
#endif
#endif /* _WRS_KERNEL */


#ifdef _WRS_KERNEL
#ifdef _TYPE_time_t
_TYPE_time_t;
#undef _TYPE_time_t
#endif

typedef int clockid_t;
#endif /* _WRS_KERNEL */

#define CLOCK_REALTIME	0x0	/* system wide realtime clock */
#define TIMER_ABSTIME	0x1	/* absolute time */
#define TIMER_RELTIME   (~TIMER_ABSTIME)        /* relative time */

#ifdef _WRS_KERNEL
struct timespec
    {
    					/* interval = tv_sec*10**9 + tv_nsec */
    time_t tv_sec;			/* seconds */
    long tv_nsec;			/* nanoseconds (0 - 1,000,000,000) */
    };
#endif /* _WRS_KERNEL */

struct itimerspec
    {
    struct timespec it_interval;	/* timer period (reload value) */
    struct timespec it_value;		/* timer expiration */
    };

/* function declarations */

extern int 	timer_create (clockid_t clock_id, struct sigevent *evp,
			      timer_t *ptimer);
extern int 	timer_delete (timer_t timerid);
extern int 	timer_gettime (timer_t timerid, struct itimerspec *value);
extern int 	timer_settime (timer_t timerid, int flags,
		               const struct itimerspec *value,
			       struct itimerspec *ovalue);
extern int 	timer_getoverrun (timer_t timerid);

extern int      timer_connect (timer_t timerid, VOIDFUNCPTR routine, int arg);
extern int      timer_cancel (timer_t timerid);
extern int      nanosleep (const struct timespec *rqtp, struct timespec *rmtp);
extern timer_t  timer_open (const char * name, int mode, clockid_t clockId, 
                            struct sigevent * evp, void * context);
extern STATUS   timer_close (timer_t timerId);
extern STATUS   timer_unlink (const char * name);

extern int	clock_gettime (clockid_t clock_id, struct timespec *tp);
extern int	clock_settime (clockid_t clock_id, const struct timespec *tp);
extern int	clock_getres (clockid_t clock_id, struct timespec *res);

#ifdef __cplusplus
}
#endif

#endif /* __INCtimeh */




