/* toolMacros.h - compile time macros for Diab C compiler */

/* Copyright 2001-2004 Wind River Systems, Alameda, CA */

/*
modification history
--------------------
01j,17mar04,dat  remove leading underscore from _absSymbols_xxx (95052)
01i,05mar04,dat  temp fix for c++ and deprecated
01h,05feb04,dat  add macros for _WRS_ABSOLUTE
01g,26nov03,dat  Adding INLINE and DEPRECATED
01f,12oct03,dat  new macros for V7 of portable C guide.
01e,23sep03,zl   added _WRS_CONSTRUCTOR definition
01d,26nov01,dat  New macros for alignment checking and unaligned copies
01c,14nov01,dat  Adding underscores to macro names
01b,18oct01,jab  fixed WRS_ASM macro
01a,04sep01,dat  written (assumes Diab 4.4b or later)
*/

#ifndef __INCtoolMacrosh
#define __INCtoolMacrosh

#define _WRS_PACK_ALIGN(x) __attribute__((packed, aligned(x)))

#define _WRS_ASM(x) __asm volatile (x)

#define _WRS_DATA_ALIGN_BYTES(x) __attribute__((aligned(x)))

/* This tool uses the ANSI variadic macro style */

#undef  _WRS_GNU_VAR_MACROS	/* Diab uses ANSI syntax */

/* New macros for alignment issues */

#define _WRS_ALIGNOF(x) 	sizeof(x,1)

#define _WRS_UNALIGNED_COPY(pSrc, pDest, size) \
	(memcpy ((pDest), (pSrc), (size)))

#define _WRS_ALIGN_CHECK(ptr, type) \
	(((int)(ptr) & ( _WRS_ALIGNOF(type) - 1)) == 0 ? TRUE : FALSE)

/*
 * macro for static initializer (constructor) definitions; assumes
 * Diab 5.2 or later with constructor support.
 */

#define _WRS_CONSTRUCTOR(rtn,lvl) \
	__attribute__((constructor(lvl))) void _STI__##lvl##__##rtn (void)

/*
 * New Basix macros, (Portable Coding Guide, v7)
 *
 * HAS_GCC_ASM_SYNTAX if Gnu inline assembly syntax is supported.
 * HAS_DCC_ASM_SYNTAX if Diab inline assembly function syntax is supported.
 * INLINE for static inline C functions, (prefix)
 * DEPRECATED for obsolete API warnings. (postfix)
 */


#undef  _WRS_HAS_GCC_ASM_SYNTAX
#define _WRS_HAS_DCC_ASM_SYNTAX
#define _WRS_INLINE		static __inline__

#ifdef __cplusplus
#define _WRS_DEPRECATED(x)	/* temp workaround for c++ compiler failure */
#else
#define _WRS_DEPRECATED(x) 	__attribute__((deprecated(x)))
#endif

/* (New V7) For absolute symbol support. (use with semicolon) */

#define _WRS_ABSOLUTE(name,value) \
	const int name __attribute__((absolute)) = value

/* prefix and suffix to surround absolute symbol declarations (nops for diab)*/

#define _WRS_ABSOLUTE_BEGIN(x) STATUS absSymbols_##x (void) {return OK;}
#define _WRS_ABSOLUTE_END 


/*
 * For backward compatibility with v5 of portable C coding guide.
 *
 * These are now obsolete, please don't use them.
 */

#define WRS_PACK_ALIGN(x)	_WRS_PACK_ALIGN(x)
#define WRS_ASM(x) 		_WRS_ASM(x)
#define WRS_DATA_ALIGN_BYTES(x)	_WRS_DATA_ALIGN_BYTES(x)
#undef WRS_GNU_VAR_MACROS

#endif /* __INCtoolMacrosh */
