/* transCbio.h - Transaction Block I/O library */

/* Copyright 2004 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
01a,23sep04,rfr  Added include guards
    17mai04,alr  created
*/

/*
DESCRIPTION

This library provides...

See description of cbioLib.c for details on Block devices and Cbio.


SEE ALSO
.I "VxWorks Programmers Guide: I/O System"

INCLUDE FILES
transCbio.h

INTERNAL
*/

#ifndef __INCtransCbioh
#define __INCtransCbioh

#ifdef __cplusplus
extern "C" {
#endif

/* error codes */
#define ERROR_LOCK			0	/* error aquiring the lock (semTake() error */
#define ERROR_TRANS_WRITE	1	/* error writing transaction */
#define ERROR_CONSISTENCY	3	/* internal consistency check error */
#define ERROR_IO_OPERATION	4	/* general IO error (underlying layer should not return error!) */
#define ERROR_OUTOFMEMORY	5	/* out of heap memory */
#define ERROR_UNCATCHED		6	/* system was unable to recover from warning */
#define ERROR_OUT_OF_UNITS	7	/* out of physical units */
#define ERROR_DEFRAG		8	/* error while defragmenting */

/* warning codes */
#define WARN_OUT_OF_UNITS	10	/* out of physical units */
#define WARN_TRANS_CRC_MIS	11	/* transaction CRC mismatch */

/* START - CBIO private header */
#define	CBIO_DEV_EXTRA	struct transCtrl
#include "private/cbioLibP.h"
/* END - CBIO private header */

/* Implementation dependent data structures */
typedef struct transCtrl
    {
	/* physical disk geometry */
	block_t		physStartOffset;	/* spare offset to leave on disk (e.g. TFFS=1) */
    block_t		physTrans1Offset;	/* physical block offset translation table 1 */
    block_t		physTrans2Offset;	/* physical block offset translation table 2 */
    block_t		physTransSize;		/* # of physical data blocks for a translation table */
    block_t		physDataOffset;		/* physical block offset for data */
    block_t		physDataSize;		/* # of physical data blocks */
    
	/* translation information */
	int			blkshift;			/* blk size shifter (used to increase the logical block sizes) */
	int			overhead;			/* overhead size */
	
    /* physical disk geometry seen in units (blocks including blkshift) */
    block_t		physDataUnits;		/* # of physical data units */
	block_t		physFreeUnits;		/* # of free physical data units */
	block_t		physReplacedUnits;	/* # of replaced physical data units */
    char *		physUnitFree;		/* physical units free table */
    char *		physUnitReplaced;	/* physical units replaced table */
	block_t		physLastFreeUnit;	/* last found free physical data units */
	
	/* logical disk geometry */
    block_t *	virtToPhys;			/* virtual # to physical # mapping table */
    char *		virtToPhysDirty1;	/* virtToPhys dirty table */
    char *		virtToPhysDirty2;	/* virtToPhys dirty table */
    
    /* transaction information */
    u_long		transSerial;		/* current transaction serial number */
    u_char		transNumber;		/* current transaction image */
    
	/* disk identification */
	u_long		diskSerial1;		/* disk serial number 1 (random) */
	u_long		diskSerial2;		/* disk serial number 2 (random hash) */
    
    /* transaction layer options */
    u_long		options;			/* options for transaction layer */
    } TRANS_CTRL;

/* custom use of CBIO fields */
#define	cbioDirtyMask	cbioPriv1
#define	cbioBlkNo		cbioPriv2

/* Function Pointers */
extern STATUS (*transpanic)(CBIO_DEV_ID dev, int code, char * message);
extern STATUS (*transwarning)(CBIO_DEV_ID dev, int code, char * message);

/* format structure + types */
#define FORMAT_REGULAR	0 	/* Regular type. It includes a TMR on the beginning and end of the media
                               increase reliability. It should be used for all regular devices, which
                               are neither TFFS, nor require DosFS compatiblity (will never be accessed
                               in a pure FAT system). */
#define FORMAT_TFFS		1	/* TFFS type. It leaves the first Sector empty (for TFFS), and only uses
                               a TMR on the end of the media. */
#define FORMAT_DOS		2	/* DOS FAT compatiblity type. It immediatelly starts the media, moving all
                               transaction data and TMR to the end of the media. A media prepared with
                               this (and defragmented) will be read and accessible in a Dos FAT system */

typedef struct transFmt
    {
	/* translation information */
	int			blkshift;			/* blk size shifter (used to increase the logical block sizes) */
	int			overhead;			/* overhead size */
	int			type;				/* format type */
    } TRANS_FORMAT;
    
CBIO_DEV_ID transDevCreate(CBIO_DEV_ID subDev);
STATUS transShow(CBIO_DEV_ID dev, int verb);

#ifdef __cplusplus
}
#endif

#endif
