/** 
 *  Darin Gillis   5-21-04
 *  This file sums up VxWorks support of wide chars...
 */

#ifdef _DIAB_TOOL
#include "../../host/diab/include/diab/wchar.h"
#elif defined(__GNUC__)

#ifndef ___wchar_h___
#define ___wchar_h___

#include <ctype.h>
#include <stdlib.h> // malloc needed for wcsdup
#include <wctype.h> // wint_t, etc. (for user convenience, not used below)

#ifdef __cplusplus
extern "C" {
#endif

extern inline wchar_t *wcscpy(wchar_t *ws1, const wchar_t *ws2) {
	wchar_t *p;
	p = ws1;

	while (*ws2)
		*p++ = *ws2++;
	*p = '\0';
	return ws1;
}

extern inline wchar_t *wcsncpy(wchar_t *ws1, const wchar_t *ws2, size_t n) {
	size_t i;
	wchar_t *p;
	p = ws1;

	for (i = 0; i < n ; i++) 
		*p++ = (*ws2) ? *ws2++ : '\0';
	
	return ws1;
}

extern inline size_t wcslen(const wchar_t *ws) {
	size_t i=0;
	while (*ws++) i++;
	return i;
}

extern inline wchar_t *wcschr(const wchar_t *ws, wchar_t wc) {
	while (*ws) {
		if (*ws == wc)
			return (wchar_t*)ws;	
		ws++;
	}
	return (wchar_t *)0;
}

extern inline int wcscmp(const wchar_t *ws1, const wchar_t *ws2) {
	// in all these functions, caller ensures ws1 and ws2 != NULL
	while (*ws1 && *ws2 && (*ws1 == *ws2)) {
		ws1++; ws2++;
	}
	return (*ws1 - *ws2);
}

extern inline int wcsicmp(const wchar_t *ws1, const wchar_t *ws2) {
	// "tolower" on a wchar_t is sketchy, but should work for our use
	while (*ws1 && *ws2 && (tolower(*ws1) == tolower(*ws2))) {
		ws1++; ws2++;
	}
	return (tolower(*ws1) - tolower(*ws2));
}

extern inline int wcsncmp(const wchar_t *ws1, const wchar_t *ws2, size_t n) {
	if (n==0) return 0;
	while (n && *ws1 && *ws2 && (*ws1 == *ws2)) {
		n--;
		if (n==0) return 0;
		ws1++; ws2++;
	}
	return (*ws1 - *ws2);
}

extern inline int wcsnicmp(const wchar_t *ws1, const wchar_t *ws2, size_t n) {
	if (n==0) return 0;
	while (n && *ws1 && *ws2 && (tolower(*ws1) == tolower(*ws2))) {
		n--;
		if (n==0) return 0;
		ws1++; ws2++;
	}
	return (tolower(*ws1) - tolower(*ws2));
}

extern inline wchar_t *wcsstr(const wchar_t *ws1, const wchar_t *ws2) {
	while (*ws1) {
		if (wcscmp(ws1, ws2) == 0)
			return (wchar_t*)ws1;
		ws1++;
	}	
	return (wchar_t *)0;
}

extern inline wchar_t *wcscat(wchar_t *destination, const wchar_t *append) {
	// NOTE: unsafe (destination buffer overflow possible)
	wchar_t *pos;
	if (!destination || !append)
		return destination;
	pos = destination + wcslen(destination);
	do *pos++ = *append++;
	while (*append); 
	*pos = L'\0'; // replace terminator
	return destination;
}

extern inline wchar_t *wcsdup(const wchar_t *in) {
	wchar_t *dup = (wchar_t*)malloc((wcslen(in)+1) * sizeof(wchar_t));
	if (dup == NULL)
		return NULL;
	wcscpy(dup, in);
	return dup;
}

#ifdef __cplusplus
}
#endif

#endif /* ___wchar_h___ */

#else
#warning Including VxWorks-only wchar header file.
#endif
