/* dot1xSuppLib.h - 802.1X supplicant non-EAP specific library header file */

/* Copyright 2002-2004 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
02f,02jun05,eja  Add statistics for physical port state and mac address
02e,30may05,eja  change default log size to 10 entries from 200
02d,12may05,eja  Add members for wep key stats
02c,10may05,eja  Add API dot1xSuppHandlerCacheInstall()
02b,09may05,eja  Change dot1xSuppShutdownAll() API
02a,04may05,eja  Add member to handler type to enable/disable eap type.
01z,20apr05,eja  Remove extra includes for Base6.1 RTP compliance
01y,11apr05,eja  Add new structure member
01x,24mar05,eja  Add debug event counters
01w,23mar05,eja  Add stats for debugging to PAE
01v,22mar05,eja  Add reset timer member to PAE
01u,22mar05,eja  Add reset flag member to PAE
01t,18mar05,eja  Moved EAP types to separate file and added logging
01s,01mar05,eja  Code cleanup
01r,18jan05,eja  Add 802.1X 2001 rev MIB statistics
01q,05jan05,eja  Add authWith element to DOT1X_SUPP_PAE
01p,21dec04,eja  Add public API prototype and make DOT1X_MODE type mirror BOOL.
01o,08dec04,eja  Fix trace bug B001
01n,07dec04,eja  Updated support
01m,19oct04,eja  Added support for multiple supplicants
01l,03dec03,dxb  Updated from code review
01k,21aug03,ggg  Add separate sign key in addition to session key
01j,05may03,ggg  Undefine debug macro by default
01i,30apr03,ss   added version info
01h,21apr03,ggg  File name, location change
01g,15apr03,ggg  Update copyright date
01f,15apr03,ggg  Further preparation for additional EAP types support
01e,08jul02,ggg  Undef debug by default
01d,08jul02,ggg  Update debug macro
01c,14jun02,ggg  Diab sensitivities
01b,11jun02,ggg  Add code review comments
01a,26mar02,ggg  written.
*/

#ifndef __INCdot1xSuppLibh
#define __INCdot1xSuppLibh

#ifdef __cplusplus
extern "C" {
#endif

#include <vxWorks.h>
#include <msgQLib.h>

/* supplicant version info */
#define DOT1X_SUPP_VERSION "2.0"
IMPORT char *dot1xSuppVersion;

/* Flag to indicate special conditions between "base" supplicant layer 
and the authentication protocol implementing a specific EAP method 
above it. */
#define DOT1X_SUPP_AUTH_STARTUP ((UINT8*)-1)/* Allocate and init */
#define DOT1X_SUPP_AUTH_SHUTDOWN ((UINT8*)-2) /* Shutdown and free */
#define DOT1X_SUPP_AUTH_RESET ((UINT8*)-3) /* Re-initialize */
#define DOT1X_SUPP_AUTH_ERROR ((UINT8*)-4) /* Internal Error */

/* method specific to base supplicant communication */
#define DOT1X_SUPP_AUTH_DOUBLY ((UINT8*)-1)

/* Maximum sizes for username and password.  Must fit in a RADIUS 
attribute */
#define DOT1X_SUPP_MAX_USERNAME_LENGTH 256
#define DOT1X_SUPP_MAX_PASSWORD_LENGTH 256

/* Maxium sizes for session key.  Must fit in a RADIUS attribute */
#define DOT1X_SUPP_MAX_SESSION_KEY_LENGTH 256

/* Maximum sizes for name.  Must fit in a RADIUS attribute */
#define DOT1X_MAX_NAME_SIZE DOT1X_SUPP_MAX_USERNAME_LENGTH

/* Maximum amount of data an EAP_REQRSP_PACKET will hold */
typedef STATUS (*AUTH_HDLR)(VOID *sessionId);
typedef VOID (*AUTH_DESTROY)(VOID *sessionId);
typedef STATUS (*AUTH_SHOW)(VOID);

typedef struct supp_env_s
    {
    VOID *ingress; /* Incoming Packet */
    UINT32 ingressLen; /* Incoming Packet length */
    VOID *egress; /* Outgoing Packet */
    UINT32 egressLen; /* Outgoing Packet length */
    UINT8 *uid; /* User name */
    UINT8 *pwd; /* Password */
    STATUS status; /* Authentication status */
    INT32 authType; /* Authentication type */
    INT32 innerAuthType; /* Inner Authentication type */
    VOID *tunnel; /* ingress tunnel data */
    UINT32 tunnelLen; /* ingress tunnel data size */
    VOID *private; /* Private data */
    INT32 eapType;
    } AUTH_ENV_OBJ;

typedef struct auth_session_s
    {
    AUTH_HDLR handler; /* Authentication handler */
    AUTH_DESTROY destroy; /* Used to tear down session - EJA */
    AUTH_SHOW show; /* Authenticator show routine */
    VOID *session; /* Authentication session object */   
    UINT8 *label; /* Session Key label used by PRF */
    } AUTH_SESSION_OBJ;

/* Log defines */
#define DOT1X_LOG_FATAL "(f) " __FILE__ " (s) FATAL (r) "
#define DOT1X_LOG_ERROR "(f) " __FILE__ " (s) ERROR (r) "
#define DOT1X_LOG_INFO  "(f) " __FILE__ " (s) INFO  (r) "
#define DOT1X_LOG_DEBUG "(f) " __FILE__ " (s) DEBUG (r) "
#define DOT1X_LOG_ENTRY_SZ (100)
#define DOT1X_LOG_ENTRYS (10)

/* Supplicant State Machine states, IEEE 802.1X, 8.5.10 */
typedef enum
    {
    SUPP_LOGOFF = 0,
    SUPP_DISCONNECTED,
    SUPP_CONNECTING,
    SUPP_ACQUIRED,
    SUPP_AUTHENTICATING,
    SUPP_HELD,
    SUPP_AUTHENTICATED,
    SUPP_DOUBLY_AUTHENTICATED       
    } SUPP_STATE;

/* Handler for a specific EAP type */
typedef STATUS (*HANDLER_SHOW)(VOID);

typedef struct
    {
    char *(*routine)(); /* handler routine for this EAP type */
    UINT32 type; /* identifier assigned to this EAP type */
    HANDLER_SHOW show;
    BOOL enabled;
    } DOT1X_SUPP_AUTH_HANDLER;

/* Enumeration to define mode of operation */
typedef enum 
    {
    DOT1X_MODE_WIRELINE = 0,/* Defines mode for wireline operation */
    DOT1X_MODE_WIRELESS /* Defines mode for wireless operation */
    } DOT1X_SUPP_MODE;

/* EAP handler and type node for singly linked list of them */
typedef struct eap_handler_s
    {
    DOT1X_SUPP_AUTH_HANDLER *handler; /* EAP type handler */
    struct eap_handler_s *next; /* Node for list processing */
    } DOT1X_EAP_HANDLER;

/* 802.1X supplicant statistics */
typedef struct dot1x_supp_stats_s
    {
    UINT32 portNumber; /* Supplicants port number */
    UINT32 eapolFramesRxd; /* EAPOL frames received */
    UINT32 eapolFramesTxd; /* EAPOL frames transmitted */
    UINT32 eapolStartFramesTxd; /* EAPOL start frames transmitted */
    UINT32 eapolLogoffFramesTxd; /* EAPOL logoff frames transmitted */
    UINT32 eapolRespIdFramesTxd; /* EAPOL RespId frames transmitted */
    UINT32 eapolRespFramesTxd; /* EAPOL Response frames transmitted */
    UINT32 eapolReqIdFramesRxd; /* EAPOL ReqId frames received */
    UINT32 eapolReqFramesRxd; /* EAPOL Request frames received */
    UINT32 eapolInvalidFramesRxd; /* EAPOL invalid frames received */
    UINT32 eapolLenErrFramesRxd; /* EAPOL length error frames received */
    UINT32 eapolLastFrameVerRxd; /* EAPOL last frame version received */
    UINT8 eapolLastFrameSrcRxd[6]; /* EAPOL last frame source received */
    } DOT1X_SUPP_STATS;

/* PAE for supplicant */
typedef struct supplicant_s
    {
    SUPP_STATE suppState; /* State of the Machine */

    /* Supplicant Variables, IEEE 802.1X, 8.5.10.1.1 */
    BOOL userLogoff; /* Is user logged off? */
    BOOL logoffSent; /* Has logoff message been sent? */
    BOOL reqId; /* Got EAP-Request/Identity? */
    BOOL reqAuth; /* Got EAP-Request not Identity? */
    BOOL eapSuccess; /* Got EAP-Success? */
    BOOL eapFail; /* Got EAP-Failure? */
    INT32 startCount; /* How many EAPOL-Starts sent */
    INT32 previousId; /* Last request ID responded to */

    /* Timer Variables, IEEE 802.1X, 8.5.2.1 */
    INT32 authWhile; /* When to time out on Auth. resp. */
    INT32 heldWhile; /* How long to wait before acquiring */
    INT32 startWhen; /* When to send EAPOL-Start */

    /* "Global" (used by more than one SM) Variables, IEEE 802.1X, 8.5.2.2 */
    BOOL initialize; /* Forces all state machines to init */
    BOOL portEnabled; /* Is the MAC for the port up? */
    BOOL receivedId; /* Request ID most recently received */
    BOOL status; /* Authorized or Unauthorized */

    /* Supplicant Constants, IEEE 802.1X, 8.5.10.1.2 */
    INT32 authPeriod; /* in secs, authWhile timer init value */
    INT32 heldPeriod; /* in secs, heldWhile timer init value */
    INT32 startPeriod; /* in secs, startWhen timer init value */
    INT32 maxStart; /* max # of EAPOL-Start messages */

    /* Other things needed */
    UINT8 authMacAddr[6]; /* MAC address of the authenticator */
    UINT8 userName[DOT1X_SUPP_MAX_USERNAME_LENGTH]; /* username */
    UINT8 password[DOT1X_SUPP_MAX_PASSWORD_LENGTH]; /* password */
    UINT8 sessionKey[DOT1X_SUPP_MAX_SESSION_KEY_LENGTH]; /* Session key */
    INT32 sessionKeySize; /* Session key size in bytes */
    UINT8 *signKey; /* Signature key */
    INT32 signKeySize; /* Signature key size in bytes */

    /* Information for the EAP "flavor specific" protocol "above" */
    DOT1X_EAP_HANDLER *authHandlerList; /* Supported handler list */
    DOT1X_SUPP_AUTH_HANDLER *authHandler; /* Current handler */

    /* Inner authentication methods */
    DOT1X_EAP_HANDLER *innerAuthHandlerList; /* Supported handler list */
    DOT1X_SUPP_AUTH_HANDLER *innerAuthHandler; /* Current handler */

    UINT8 *authResp; /* Response from authentication handler */
    UINT8 *authSpecific; /* data used by EAP type specific handler */
    UINT8 *innerAuthSpecific; 
    INT32 authWith; /* Last method used to authenticate with */

    /* Things used to make it all work */
    MSG_Q_ID eventQ; /* Queues events and messages */

    VOID *eapolCookie; /* EAPOL on the port */
    UINT8 deviceName[DOT1X_MAX_NAME_SIZE+1]; /* Device name */
    INT32 deviceUnit; /* Unit number of the network interface */
    DOT1X_SUPP_MODE mode; /* Mode of operation */
    BOOL unicastSupport; /* Unicast address support */

    /* Certificate names and location */
    UINT8 openSslDir[DOT1X_MAX_NAME_SIZE+1];
    UINT8 certDir[DOT1X_MAX_NAME_SIZE+1];
    UINT8 keyFile[DOT1X_MAX_NAME_SIZE+1];
    UINT8 keyPassword[DOT1X_SUPP_MAX_PASSWORD_LENGTH+1];
    UINT8 caCertFile[DOT1X_MAX_NAME_SIZE+1];
    UINT8 caCertPath[DOT1X_MAX_NAME_SIZE+1];
    UINT8 randFile[DOT1X_MAX_NAME_SIZE+1];
    UINT8 certFile[DOT1X_MAX_NAME_SIZE+1];
    INT32 currentTime;

    /* 802.1X statistics */
    DOT1X_SUPP_STATS stat; 

    BOOL packetTrace;
    INT32 lockKey;
    UINT32 sessionFailures;
    UINT32 sessionStarts;
    UINT32 sessionSuccess;
    UINT32 associationEvents;
    UINT32 disassociationEvents;
    UINT32 logEntries;
    UINT32 currLogEntry;
    INT32 resetTimer;
    BOOL resetFlag;
    VOID (*eventLog)();
    char logEntry[DOT1X_LOG_ENTRYS][DOT1X_LOG_ENTRY_SZ];

    UINT32 aEvents;
    UINT32 dEvents;
    UINT32 rejectedEvents;
    UINT32 wep40Keys;
    UINT32 wep104Keys;
    UINT32 physLayerPortState;
    char physLayerPortMacAddr[6];

    /* Next supplicant */
    struct supplicant_s *next;
    } DOT1X_SUPP_PAE;

/* Public APIs */

/* Create root task - Kernel mode */
STATUS dot1xSuppLibInit
    (
    VOID
    );
/* Display Supplcant Info - Kernel mode */
STATUS dot1xSuppShow
    (
    VOID
    );
/* Create and add a supplicant to the root task list - Kernel/User mode */
DOT1X_SUPP_PAE *dot1xSuppCreate
    (
    INT32 unit, /* unit number */
    char *devName, /* device name */
    DOT1X_SUPP_MODE mode /* Wireline or wireless mode */
    );
/* Destroy and delete a supplicant from root task list - Kernel/User mode */
STATUS dot1xSuppDelete
    (
    DOT1X_SUPP_PAE *pSupp /* supplicant */
    );
/* Bind an authentication handler to a supplicant - Kernel mode */
STATUS dot1xSuppAuthHandlerAdd
    (
    DOT1X_SUPP_PAE *pSupp, /* Supplicant */
    DOT1X_SUPP_AUTH_HANDLER *handler /* Handler being added */
    );
/* Unbind an authentication handler from a supplicant - Kernel mode */
STATUS dot1xSuppAuthHandlerRemove
    (
    DOT1X_SUPP_PAE *pSupp, /* Supplicant */
    DOT1X_SUPP_AUTH_HANDLER *handler /* Handler being added */
    );
/* Set the username and password for a supplicant - Kernel/User mode */
STATUS dot1xSuppLogonSet
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    char *userName, /* username logging on */
    char *password /* password */
    );
/* Get the username and password for a supplicant - Kernel/User mode */
STATUS dot1xSuppLogonGet
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    char *userName, /* username logging on */
    char *password /* password */
    );
/* Set information used by cert based authentication - Kernel/User mode */
STATUS dot1xSuppCertInfoSet
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    char *cert, /* User certificate */
    char *caCert, /* Server certificate */
    char *caCertPath, /* Server certificate Path */
    char *rand, /* Random seed file */
    char *key, /* User Private Key file */
    char *keyPwd /* User Private Key password */
    );
/* Get information used by cert based authentication - Kernel/User mode */
STATUS dot1xSuppCertInfoGet
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    char *cert, /* User certificate */
    char *caCert, /* Server certificate */
    char *caCertPath, /* Server certificate Path */
    char *rand, /* Random seed file */
    char *key, /* User Private Key file */
    char *keyPwd /* User Private Key password */
    );
/* Enable or Disable packet trace - Kernel/User mode */
BOOL dot1xSuppPacketTrace
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    BOOL state /* state 1 = ON, 0 = OFF */
    );
/* Flush supplicant event log - Kernel/User mode */
STATUS dot1xSuppEventLogFlush
    (
    DOT1X_SUPP_PAE *pSupp /* supplicant object */
    );
/* Display contents of supplicant datalog - - Kernel mode */
VOID dot1xSuppEventLogShow
    (
    DOT1X_SUPP_PAE *pSupp /* supplicant object */
    );
/* Get supplicant ID based on instance number - Kernel/User mode */
STATUS dot1xSuppIdGet
    (
    int instance, /* instance of the device */ 
    int *id /* device id */
    );
/* Force supplicant to connect to NAS - Kernel/User mode */
STATUS dot1xSuppConnect
    (
    DOT1X_SUPP_PAE *pSupp /* supplicant */
    );
/* Shutdown all supplicants and delete all tasks - Kernel mode */
STATUS dot1xSuppShutdownAll
    (
    VOID 
    );
/* Enable EAP method - Kernel/User mode */
STATUS dot1xSuppEapTypeEnable
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    UINT8 eapType /* eap type */
    );
/* Disable EAP method - Kernel/User mode */
STATUS dot1xSuppEapTypeDisable
    (
    DOT1X_SUPP_PAE *pSupp, /* supplicant */
    UINT8 eapType /* eap type */
    );
STATUS dot1xSuppHandlerCacheInstall
    (
    DOT1X_SUPP_PAE *pSupp /* supplicant */
    );
#ifdef __cplusplus
}
#endif

#endif /* __INCdot1xSuppLibh */
