/* unistd.h - unix standard library header file */

/* Copyright 1984-2004 Wind River Systems, Inc. */

/*
modification history
--------------------
02p,17mar05,dlk  Make 'place' a member of the GETOPT type.
02o,25feb05,niq  Added definitions for reentrant version of getopt
02n,06oct04,pad  Added declaration of the environ array for the user side.
02m,01oct04,yvp  Change lseek args to use off_t (for Posix).
02l,13sep04,yvp  Added prorotype for _exit().
02k,09jun04,kk   added getpid() & getppid() prototypes SPR#98109
02j,29mar04,job  Added getopt prototype. Removed non-STDC function prototypes
02i,08dec03,dat  adding const to unlink and chdir
02h,20jul00,jgn  add POSIX version information + removed old style prototypes
02g,24sep01,jkf  added const to rmdir()
02f,19aug94,ism  added getcwd() prototypes (SPR #3536)
02e,12nov93,dvs  added prototype for ftruncate.
02d,08feb93,smb  changed int to size_t in protype for read() and write()
02c,22sep92,rrr  added support for c++
02b,18sep92,smb  added the rmdir prototype.
02a,04jul92,jcf  cleaned up.
01c,26may92,rrr  the tree shuffle
01b,05dec91,rrr  added SEEK_ macros (was in ioLib.h)
01a,19nov91,rrr  written.
*/

#ifndef __INCunistdh
#define __INCunistdh

#ifdef __cplusplus
extern "C" {
#endif

#include "vxWorks.h"

#ifndef SEEK_SET
#define SEEK_SET           0       /* absolute offset, was L_SET */
#define SEEK_CUR           1       /* relative to current offset, was L_INCR */
#define SEEK_END           2       /* relative to end of file, was L_XTND */
#endif

/* POSIX defines */

#ifndef _POSIX_VERSION
#define _POSIX_VERSION 199506L
#endif

#ifndef _POSIX_C_SOURCE
#define _POSIX_C_SOURCE	199506L
#endif

extern char *optarg;
extern int optind, opterr, optopt;
extern int optreset;
#ifndef _WRS_KERNEL
extern char ** environ;
#endif

struct getopt_s
    {
    int	 opterr;       		/* if error message should be printed */
    int  optind;    		/* index into parent argv vector */
    int  optopt;		/* character checked for validity */
    int  optreset;		/* reset getopt */
    char *optarg;		/* argument associated with option */
    char *place;
    };

typedef struct getopt_s GETOPT;
typedef struct getopt_s * GETOPT_ID;

/* function declarations */

extern STATUS 		unlink (const char *name);
extern STATUS 		close (int fd);
extern int 		read (int fd, char *buffer, size_t maxbytes);
extern int 		write (int fd, char *buffer, size_t nbytes);
extern off_t 		lseek (int fd, off_t offset, int whence);
extern STATUS 		chdir (const char *pathname);
extern STATUS 		pause (void);
extern BOOL 		isatty (int fd);
extern STATUS 		rmdir (const char *dirName);
extern char *		getcwd (char *buffer, int size);
extern int    		ftruncate (int fildes, off_t length);
extern unsigned int	sleep (unsigned int);
extern unsigned int	alarm (unsigned int);
extern int              getopt (int, char * const [], const char *);
extern int 		getopt_r (int, char * const [], const char *, 
                                  GETOPT_ID pGetOpt);
extern void 		getoptInit (GETOPT_ID pArg);
extern STATUS		getOptServ (char *ParamString, const char * progName,
			  	    int *argc, char *argvloc[], int argvlen);

#ifndef _WRS_KERNEL		/* user-mode API only */
extern int		_exit (int);
#endif

#if (!defined (_WRS_KERNEL))	/* getppid() only in user space */
extern pid_t		getpid(void);
extern pid_t		getppid(void);
#else
extern RTP_ID		getpid(void); /* for porting from user to kernel */
#endif

#ifdef __cplusplus
}
#endif

#endif /* __INCunistdh */
